/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Toolbox;
using OSLib.Gadget;

[CCode (cheader_filename = "oslib/actionbutton.h")]
namespace OSLib.ActionButton
{
  [CCode (cname = "actionbutton_gadget")]
  public struct Gadget
  {
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "text_limit")]
    public unowned int TextLimit;
    [CCode (cname = "click_show")]
    public unowned Toolbox.StringReference ClickShow;
    [CCode (cname = "action")]
    public unowned bits Action;
  }

  [CCode (cname = "actionbutton_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "text_limit")]
    public unowned int TextLimit;
    [CCode (cname = "click_show")]
    public unowned Toolbox.StringReference ClickShow;
    [CCode (cname = "action")]
    public unowned bits Action;
  }

  [CCode (cname = "actionbutton_action_selected_block")]
  public enum ActionSelectedBlock
  {
    Default = 0
  }

  [CCode (cname="class_ACTION_BUTTON")]
  public const Toolbox.Class Class_ActionButton;
  [CCode (cname="actionbutton_IS_DEFAULT")]
  public const OSLib.Gadget.Flags IsDefault;
  [CCode (cname="actionbutton_IS_CANCEL")]
  public const OSLib.Gadget.Flags IsCancel;
  [CCode (cname="actionbutton_IS_LOCAL")]
  public const OSLib.Gadget.Flags IsLocal;
  [CCode (cname="actionbutton_IS_MENU")]
  public const OSLib.Gadget.Flags IsMenu;
  [CCode (cname="actionbutton_SHOW_AS_MENU")]
  public const OSLib.Gadget.Flags ShowAsMenu;
  [CCode (cname="actionbutton_SHOW_AT_CENTRE")]
  public const OSLib.Gadget.Flags ShowAtCentre;
  [CCode (cname="actionbutton_SHOW_AT_POINTER")]
  public const OSLib.Gadget.Flags ShowAtPointer;
  [CCode (cname="action_ACTION_BUTTON_SELECTED")]
  public const bits Action_ActionButtonSelected;
  [CCode (cname="actionbutton_SELECTED_ADJUST")]
  public const bits SelectedAdjust;
  [CCode (cname="actionbutton_SELECTED_SELECT")]
  public const bits SelectedSelect;
  [CCode (cname="actionbutton_SELECTED_DEFAULT")]
  public const bits SelectedDefault;
  [CCode (cname="actionbutton_SELECTED_CANCEL")]
  public const bits SelectedCancel;
  [CCode (cname="actionbutton_SELECTED_LOCAL")]
  public const bits SelectedLocal;

  [CCode (cname = "xactionbutton_set_text")]
  public static OS.Error* XSetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     string text);

  [CCode (cname = "actionbutton_set_text")]
  public static void SetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     string text);

  [CCode (cname = "xactionbutton_get_text")]
  public static OS.Error* XGetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     string text,
     int size,
     out int used);

  [CCode (cname = "actionbutton_get_text")]
  public static int GetText
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     string text,
     int size);

  [CCode (cname = "xactionbutton_set_action")]
  public static OS.Error* XSetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     bits action);

  [CCode (cname = "actionbutton_set_action")]
  public static void SetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     bits action);

  [CCode (cname = "xactionbutton_get_action")]
  public static OS.Error* XGetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     out bits action);

  [CCode (cname = "actionbutton_get_action")]
  public static bits GetAction
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button);

  [CCode (cname = "xactionbutton_set_click_show")]
  public static OS.Error* XSetClickShow
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     Toolbox.O obj,
     bits show_flags);

  [CCode (cname = "actionbutton_set_click_show")]
  public static void SetClickShow
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     Toolbox.O obj,
     bits show_flags);

  [CCode (cname = "xactionbutton_get_click_show")]
  public static OS.Error* XGetClickShow
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button,
     out Toolbox.O obj);

  [CCode (cname = "actionbutton_get_click_show")]
  public static Toolbox.O GetClickShow
    (bits flags,
     Toolbox.O window,
     Toolbox.C action_button);

}
