/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/alarm.h")]
namespace OSLib.Alarm
{
  [CCode (cname = "alarm_reason")]
  public enum Reason
  {
    [CCode (cname = "alarm_REASON_SET_LOCAL")]
    SetLocal,
    [CCode (cname = "alarm_REASON_SET_UTC")]
    SetUTC,
    [CCode (cname = "alarm_REASON_CANCEL")]
    Cancel
  }

  [CCode (cname = "alarm_date_and_time")]
  public struct DateAndTime
  {
    [CCode (cname = "year")]
    public unowned short Year;
    [CCode (cname = "month")]
    public unowned byte Month;
    [CCode (cname = "date")]
    public unowned byte Date;
    [CCode (cname = "hour")]
    public unowned byte Hour;
    [CCode (cname = "minute")]
    public unowned byte Minute;
  }

  [CCode (cname = "alarm_message_set")]
  public struct MessageSet
  {
    [CCode (cname = "set_reason")]
    public unowned Reason SetReason;
    [CCode (cname = "time.utc")]
    public unowned OS.DateAndTime Time_Utc;
    [CCode (cname = "time.local")]
    public unowned DateAndTime Time_Local;
    [CCode (cname = "sender")]
    public unowned char Sender[40];
    [CCode (cname = "alarm_name")]
    public unowned char AlarmName[40];
  }

  [CCode (cname = "alarm_message_cancel")]
  public struct MessageCancel
  {
    [CCode (cname = "cancel")]
    public unowned Reason Cancel;
    [CCode (cname = "sender")]
    public unowned char Sender[40];
    [CCode (cname = "alarm_name")]
    public unowned char AlarmName[40];
  }

  [CCode (cname = "alarm_message_gone_off")]
  public struct MessageGoneOff
  {
    [CCode (cname = "sender")]
    public unowned char Sender[40];
    [CCode (cname = "alarm_name")]
    public unowned char AlarmName[40];
  }

  [CCode (cname = "alarm_full_message_set")]
  public struct FullMessageSet : Wimp.MessageHeaderBase
  {
    [CCode (cname = "set_reason")]
    public unowned Reason SetReason;
    [CCode (cname = "time.utc")]
    public unowned OS.DateAndTime Time_Utc;
    [CCode (cname = "time.local")]
    public unowned DateAndTime Time_Local;
    [CCode (cname = "sender_name")]
    public unowned char SenderName[40];
    [CCode (cname = "alarm_name")]
    public unowned char AlarmName[40];
  }

  [CCode (cname = "alarm_full_message_cancel")]
  public struct FullMessageCancel : Wimp.MessageHeaderBase
  {
    [CCode (cname = "cancel")]
    public unowned Reason Cancel;
    [CCode (cname = "sender_name")]
    public unowned char SenderName[40];
    [CCode (cname = "alarm_name")]
    public unowned char AlarmName[40];
  }

  [CCode (cname = "alarm_full_message_gone_off")]
  public struct FullMessageGoneOff : Wimp.MessageHeaderBase
  {
    [CCode (cname = "sender_name")]
    public unowned char SenderName[40];
    [CCode (cname = "alarm_name")]
    public unowned char AlarmName[40];
  }

  [CCode (cname="alarm_NAME_LIMIT")]
  public const int NameLimit;
  [CCode (cname="message_ALARM_SET")]
  public const bits Message_AlarmSet;
  [CCode (cname="message_ALARM_GONE_OFF")]
  public const bits Message_AlarmGoneOff;

}
