/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/buffer.h")]
namespace OSLib.Buffer
{
  [CCode (cname = "buffer_no")]
  public enum No
  {
    [CCode (cname = "buffer_KEYBOARD")]
    Keyboard,
    [CCode (cname = "buffer_RS423_INPUT")]
    RS423Input,
    [CCode (cname = "buffer_RS423_OUTPUT")]
    RS423Output,
    [CCode (cname = "buffer_PRINTER")]
    Printer,
    [CCode (cname = "buffer_SOUND_CHANNEL0")]
    SoundChannel0,
    [CCode (cname = "buffer_SOUND_CHANNEL1")]
    SoundChannel1,
    [CCode (cname = "buffer_SOUND_CHANNEL2")]
    SoundChannel2,
    [CCode (cname = "buffer_SOUND_CHANNEL3")]
    SoundChannel3,
    [CCode (cname = "buffer_SPEECH")]
    Speech,
    [CCode (cname = "buffer_MOUSE")]
    Mouse
  }

  [CCode (cname = "buffer_flags")]
  public enum Flags
  {
    [CCode (cname = "buffer_DORMANT")]
    Dormant,
    [CCode (cname = "buffer_EVENT_ON_EMPTY")]
    EventOnEmpty,
    [CCode (cname = "buffer_EVENT_ON_FULL")]
    EventOnFull,
    [CCode (cname = "buffer_UP_CALL_ON_THRESHOLD")]
    UpCallOnThreshold
  }

  [CCode (cname = "buffer_b", has_type_id = false)]
  public struct B : bits
  {
  }

  [CCode (cname = "buffer_internal_id", has_type_id = false)]
  public struct InternalId : bits
  {
  }

  [CCode (cname = "buffer_service_op")]
  public enum ServiceOp
  {
    [CCode (cname = "buffer_SERVICE_INSERT_BYTE")]
    ServiceInsertByte,
    [CCode (cname = "buffer_SERVICE_INSERT_BLOCK")]
    ServiceInsertBlock,
    [CCode (cname = "buffer_SERVICE_REMOVE_BYTE")]
    ServiceRemoveByte,
    [CCode (cname = "buffer_SERVICE_REMOVE_BLOCK")]
    ServiceRemoveBlock,
    [CCode (cname = "buffer_SERVICE_EXAMINE_BYTE")]
    ServiceExamineByte,
    [CCode (cname = "buffer_SERVICE_EXAMINE_BLOCK")]
    ServiceExamineBlock,
    [CCode (cname = "buffer_SERVICE_USED_SPACE")]
    ServiceUsedSpace,
    [CCode (cname = "buffer_SERVICE_FREE_SPACE")]
    ServiceFreeSpace,
    [CCode (cname = "buffer_SERVICE_PURGE_BUFFER")]
    ServicePurgeBuffer,
    [CCode (cname = "buffer_SERVICE_NEXT_FILLED_BLOCK")]
    ServiceNextFilledBlock
  }

  [CCode (cname = "buffer_mouse_data")]
  public struct MouseData
  {
    [CCode (cname = "x")]
    public unowned short X;
    [CCode (cname = "y")]
    public unowned short Y;
    [CCode (cname = "buttons")]
    public unowned byte Buttons;
    [CCode (cname = "time")]
    public unowned byte Time[4];
  }

  [CCode (cname="error_BUFFER_MANAGER_BAD_SWI")]
  public const bits Error_BufferManagerBadSWI;
  [CCode (cname="error_BUFFER_MANAGER_TOO_MANY_BUFFERS")]
  public const bits Error_BufferManagerTooManyBuffers;
  [CCode (cname="error_BUFFER_MANAGER_BAD_BUFFER")]
  public const bits Error_BufferManagerBadBuffer;
  [CCode (cname="error_BUFFER_MANAGER_IN_USE")]
  public const bits Error_BufferManagerInUse;
  [CCode (cname="error_BUFFER_MANAGER_UNABLE_TO_DETACH")]
  public const bits Error_BufferManagerUnableToDetach;
  [CCode (cname="error_BUFFER_MANAGER_HANDLE_ALREADY_USED")]
  public const bits Error_BufferManagerHandleAlreadyUsed;
  [CCode (cname="error_BUFFER_MANAGER_BUFFER_TOO_SMALL")]
  public const bits Error_BufferManagerBufferTooSmall;
  [CCode (cname="error_BUFFER_MANAGER_BAD_PARM")]
  public const bits Error_BufferManagerBadParm;
  [CCode (cname="cnpv_COUNT_ENTRIES")]
  public const bits CnpV_CountEntries;
  [CCode (cname="cnpv_NUMBER_FREE")]
  public const bits CnpV_NumberFree;
  [CCode (cname="cnpv_PURGE")]
  public const bits CnpV_Purge;

  [CCode (cname = "xbuffer_create")]
  public static OS.Error* XCreate
    (Flags flags,
     int size,
     B b,
     out B b_out);

  [CCode (cname = "buffer_create")]
  public static B Create
    (Flags flags,
     int size,
     B b);

  [CCode (cname = "xbuffer_remove")]
  public static OS.Error* XRemove
    (B b);

  [CCode (cname = "buffer_remove")]
  public static void Remove
    (B b);

  [CCode (cname = "xbuffer_register")]
  public static OS.Error* XRegister
    (Flags flags,
     char* buffer,
     char* end,
     B b,
     out B b_out);

  [CCode (cname = "buffer_register")]
  public static B Register
    (Flags flags,
     char* buffer,
     char* end,
     B b);

  [CCode (cname = "xbuffer_deregister")]
  public static OS.Error* XDeregister
    (B b);

  [CCode (cname = "buffer_deregister")]
  public static void Deregister
    (B b);

  [CCode (cname = "xbuffer_modify_flags")]
  public static OS.Error* XModifyFlags
    (B b,
     bits eor_mask,
     bits and_mask);

  [CCode (cname = "buffer_modify_flags")]
  public static void ModifyFlags
    (B b,
     bits eor_mask,
     bits and_mask);

  [CCode (cname = "xbuffer_modify_flags2")]
  public static OS.Error* XModifyFlags2
    (B b,
     bits eor_mask,
     bits and_mask,
     out bits old_value,
     out bits new_value);

  [CCode (cname = "buffer_modify_flags2")]
  public static void ModifyFlags2
    (B b,
     bits eor_mask,
     bits and_mask,
     out bits old_value,
     out bits new_value);

  [CCode (cname = "xbuffer_link_device")]
  public static OS.Error* XLinkDevice
    (B b,
     AsmRoutine wakeup_code,
     AsmRoutine owner_change_code,
     int handle,
     char* workspace);

  [CCode (cname = "buffer_link_device")]
  public static void LinkDevice
    (B b,
     AsmRoutine wakeup_code,
     AsmRoutine owner_change_code,
     int handle,
     char* workspace);

  [CCode (cname = "xbuffer_unlink_device")]
  public static OS.Error* XUnlinkDevice
    (B b);

  [CCode (cname = "buffer_unlink_device")]
  public static void UnlinkDevice
    (B b);

  [CCode (cname = "xbuffer_get_info")]
  public static OS.Error* XGetInfo
    (B b,
     out Flags flags,
     out char* buffer,
     out char* end,
     out int insertion_point,
     out int removal_point,
     out int spare,
     out int used);

  [CCode (cname = "buffer_get_info")]
  public static void GetInfo
    (B b,
     out Flags flags,
     out char* buffer,
     out char* end,
     out int insertion_point,
     out int removal_point,
     out int spare,
     out int used);

  [CCode (cname = "xbuffer_threshold")]
  public static OS.Error* XThreshold
    (B b,
     int threshold,
     out int old_threshold);

  [CCode (cname = "buffer_threshold")]
  public static int Threshold
    (B b,
     int threshold);

  [CCode (cname = "xbuffer_internal_info")]
  public static OS.Error* XInternalInfo
    (B b,
     out InternalId id,
     out AsmRoutine service_routine,
     out char* workspace);

  [CCode (cname = "buffer_internal_info")]
  public static InternalId InternalInfo
    (B b,
     out AsmRoutine service_routine,
     out char* workspace);

  [CCode (cname = "xinsv")]
  public static OS.Error* XInsV
    (byte b,
     No buffer_no,
     out uint psr);

  [CCode (cname = "insv")]
  public static bits InsV
    (byte b,
     No buffer_no);

  [CCode (cname = "xinsv_block")]
  public static OS.Error* XInsV_Block
    (No buffer_no,
     char* data,
     int size,
     out byte* end,
     out uint psr);

  [CCode (cname = "insv_block")]
  public static bits InsV_Block
    (No buffer_no,
     char* data,
     int size,
     out byte* end);

  [CCode (cname = "xremv")]
  public static OS.Error* XRemV
    (No buffer_no,
     out byte examined_b,
     out byte removed_b,
     out uint psr);

  [CCode (cname = "remv")]
  public static bits RemV
    (No buffer_no,
     out byte examined_b,
     out byte removed_b);

  [CCode (cname = "xremv_block")]
  public static OS.Error* XRemV_Block
    (No buffer_no,
     char* buffer,
     int byte_count,
     out byte* end,
     out int spare,
     out uint psr);

  [CCode (cname = "remv_block")]
  public static bits RemV_Block
    (No buffer_no,
     char* buffer,
     int byte_count,
     out byte* end,
     out int spare);

  [CCode (cname = "xcnpv")]
  public static OS.Error* XCnpV
    (No buffer_no,
     out bits lo8bits,
     out bits hi24bits);

  [CCode (cname = "cnpv")]
  public static void CnpV
    (No buffer_no,
     out bits lo8bits,
     out bits hi24bits);

  [CCode (cname = "xservice_buffer_starting")]
  public static OS.Error* XService_BufferStarting ();

  [CCode (cname = "service_buffer_starting")]
  public static void Service_BufferStarting ();

  [CCode (cname = "xevent_output_buffer_empty")]
  public static OS.Error* XEvent_OutputBufferEmpty
    (No buffer);

  [CCode (cname = "event_output_buffer_empty")]
  public static void Event_OutputBufferEmpty
    (No buffer);

  [CCode (cname = "xevent_input_buffer_full")]
  public static OS.Error* XEvent_InputBufferFull
    (No buffer,
     byte b);

  [CCode (cname = "event_input_buffer_full")]
  public static void Event_InputBufferFull
    (No buffer,
     byte b);

  [CCode (cname = "xevent_input_buffer_full_block_op")]
  public static OS.Error* XEvent_InputBufferFullBlockOp
    (No buffer,
     char* data,
     int count);

  [CCode (cname = "event_input_buffer_full_block_op")]
  public static void Event_InputBufferFullBlockOp
    (No buffer,
     char* data,
     int count);

  [CCode (cname = "xupcall_buffer_filling")]
  public static OS.Error* XUpCall_BufferFilling
    (B b);

  [CCode (cname = "upcall_buffer_filling")]
  public static void UpCall_BufferFilling
    (B b);

  [CCode (cname = "xupcall_buffer_emptying")]
  public static OS.Error* XUpCall_BufferEmptying
    (B b);

  [CCode (cname = "upcall_buffer_emptying")]
  public static void UpCall_BufferEmptying
    (B b);

}
