/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/ddeutils.h")]
namespace OSLib.DDEUtils
{
  [CCode (cname = "ddeutils_message_processing_file")]
  public struct MessageProcessingFile
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[236];
  }

  [CCode (cname = "ddeutils_message_errors_in")]
  public struct MessageErrorsIn
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[236];
  }

  [CCode (cname = "ddeutils_message_error_details")]
  public struct MessageErrorDetails
  {
    [CCode (cname = "line_no")]
    public unowned int LineNo;
    [CCode (cname = "severity")]
    public unowned int Severity;
    [CCode (cname = "message")]
    public unowned char Message[228];
  }

  [CCode (cname = "ddeutils_message_info_for_file")]
  public struct MessageInfoForFile
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[236];
  }

  [CCode (cname = "ddeutils_message_info_details")]
  public struct MessageInfoDetails
  {
    [CCode (cname = "line_no")]
    public unowned int LineNo;
    [CCode (cname = "severity")]
    public unowned int Severity;
    [CCode (cname = "message")]
    public unowned char Message[228];
  }

  [CCode (cname = "ddeutils_full_message_processing_file")]
  public struct FullMessageProcessingFile : Wimp.MessageHeaderBase
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[236];
  }

  [CCode (cname = "ddeutils_full_message_errors_in")]
  public struct FullMessageErrorsIn : Wimp.MessageHeaderBase
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[236];
  }

  [CCode (cname = "ddeutils_full_message_error_details")]
  public struct FullMessageErrorDetails : Wimp.MessageHeaderBase
  {
    [CCode (cname = "line_no")]
    public unowned int LineNo;
    [CCode (cname = "severity")]
    public unowned int Severity;
    [CCode (cname = "message")]
    public unowned char Message[228];
  }

  [CCode (cname = "ddeutils_full_message_info_for_file")]
  public struct FullMessageInfoForFile : Wimp.MessageHeaderBase
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[236];
  }

  [CCode (cname = "ddeutils_full_message_info_details")]
  public struct FullMessageInfoDetails : Wimp.MessageHeaderBase
  {
    [CCode (cname = "line_no")]
    public unowned int LineNo;
    [CCode (cname = "severity")]
    public unowned int Severity;
    [CCode (cname = "message")]
    public unowned char Message[228];
  }

  [CCode (cname="ddeutils_SEVERITY_WARNING")]
  public const int SeverityWarning;
  [CCode (cname="ddeutils_SEVERITY_ERROR")]
  public const int SeverityError;
  [CCode (cname="ddeutils_SEVERITY_SERIOUS_ERROR")]
  public const int SeveritySeriousError;
  [CCode (cname="ddeutils_THROWBACK_PROCESSING")]
  public const int ThrowbackProcessing;
  [CCode (cname="ddeutils_THROWBACK_ERROR_DETAILS")]
  public const int ThrowbackErrorDetails;
  [CCode (cname="ddeutils_THROWBACK_INFO_DETAILS")]
  public const int ThrowbackInfoDetails;
  [CCode (cname="error_DDE_UTILS_UNKNOWN_SWI")]
  public const bits Error_DDEUtilsUnknownSWI;
  [CCode (cname="error_DDE_UTILS_NO_CLI_BUFFER")]
  public const bits Error_DDEUtilsNoCLIBuffer;
  [CCode (cname="error_DDE_UTILS_NOT_DESKTOP")]
  public const bits Error_DDEUtilsNotDesktop;
  [CCode (cname="error_DDE_UTILS_NO_TASK")]
  public const bits Error_DDEUtilsNoTask;
  [CCode (cname="error_DDE_UTILS_ALREADY_REGISTERED")]
  public const bits Error_DDEUtilsAlreadyRegistered;
  [CCode (cname="error_DDE_UTILS_NOT_REGISTERED")]
  public const bits Error_DDEUtilsNotRegistered;
  [CCode (cname="message_DDE_UTILS_THROWBACK_START")]
  public const bits Message_DDEUtilsThrowbackStart;
  [CCode (cname="message_DDE_UTILS_PROCESSING_FILE")]
  public const bits Message_DDEUtilsProcessingFile;
  [CCode (cname="message_DDE_UTILS_ERRORS_IN")]
  public const bits Message_DDEUtilsErrorsIn;
  [CCode (cname="message_DDE_UTILS_ERROR_DETAILS")]
  public const bits Message_DDEUtilsErrorDetails;
  [CCode (cname="message_DDE_UTILS_THROWBACK_END")]
  public const bits Message_DDEUtilsThrowbackEnd;
  [CCode (cname="message_DDE_UTILS_INFO_FOR_FILE")]
  public const bits Message_DDEUtilsInfoForFile;
  [CCode (cname="message_DDE_UTILS_INFO_DETAILS")]
  public const bits Message_DDEUtilsInfoDetails;

  [CCode (cname = "xddeutils_prefix")]
  public static OS.Error* XPrefix
    (string dir_name);

  [CCode (cname = "ddeutils_prefix")]
  public static void Prefix
    (string dir_name);

  [CCode (cname = "xddeutils_set_cl_size")]
  public static OS.Error* XSetCLSize
    (int size);

  [CCode (cname = "ddeutils_set_cl_size")]
  public static void SetCLSize
    (int size);

  [CCode (cname = "xddeutils_set_cl")]
  public static OS.Error* XSetCL
    (string tail);

  [CCode (cname = "ddeutils_set_cl")]
  public static void SetCL
    (string tail);

  [CCode (cname = "xddeutils_get_cl_size")]
  public static OS.Error* XGetCLSize
    (out int size);

  [CCode (cname = "ddeutils_get_cl_size")]
  public static int GetCLSize ();

  [CCode (cname = "xddeutils_get_cl")]
  public static OS.Error* XGetCl
    (string tail);

  [CCode (cname = "ddeutils_get_cl")]
  public static void GetCl
    (string tail);

  [CCode (cname = "xddeutils_throwback_register")]
  public static OS.Error* XThrowbackRegister
    (Wimp.T task);

  [CCode (cname = "ddeutils_throwback_register")]
  public static void ThrowbackRegister
    (Wimp.T task);

  [CCode (cname = "xddeutils_throwback_un_register")]
  public static OS.Error* XThrowbackUnRegister
    (Wimp.T task);

  [CCode (cname = "ddeutils_throwback_un_register")]
  public static void ThrowbackUnRegister
    (Wimp.T task);

  [CCode (cname = "xddeutils_throwback_start")]
  public static OS.Error* XThrowbackStart ();

  [CCode (cname = "ddeutils_throwback_start")]
  public static void ThrowbackStart ();

  [CCode (cname = "xddeutils_throwback_send")]
  public static OS.Error* XThrowbackSend
    (int reason,
     string file_name,
     int line_no,
     int severity,
     string message);

  [CCode (cname = "ddeutils_throwback_send")]
  public static void ThrowbackSend
    (int reason,
     string file_name,
     int line_no,
     int severity,
     string message);

  [CCode (cname = "xddeutils_throwback_end")]
  public static OS.Error* XThrowbackEnd ();

  [CCode (cname = "ddeutils_throwback_end")]
  public static void ThrowbackEnd ();

}
