/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/dhcpclient.h")]
namespace OSLib.DHCPClient
{
  [CCode (cname = "dhcpclientcontrol_reason")]
  public enum DHCPClientControl_Reason
  {
    [CCode (cname = "dhcpclientcontrolreason_ADD")]
    Add,
    [CCode (cname = "dhcpclientcontrolreason_REMOVE")]
    Remove,
    [CCode (cname = "dhcpclientcontrolreason_RENEW")]
    Renew
  }

  [CCode (cname = "dhcpclientstate_types")]
  public enum DHCPClientState_Types
  {
    [CCode (cname = "dhcpclientstatetype_END_OF_LIST")]
    EndOfList,
    [CCode (cname = "dhcpclientstatetype_STATE")]
    State,
    [CCode (cname = "dhcpclientstatetype_BOUND_ADDRESS")]
    BoundAddress,
    [CCode (cname = "dhcpclientstatetype_SERVER_ADDRESS")]
    ServerAddress,
    [CCode (cname = "dhcpclientstatetype_GATEWAY_ADDRESS")]
    GatewayAddress,
    [CCode (cname = "dhcpclientstatetype_LEASE_PERIOD")]
    LeasePeriod,
    [CCode (cname = "dhcpclientstatetype_T1_PERIOD")]
    T1Period,
    [CCode (cname = "dhcpclientstatetype_T2_PERIOD")]
    T2Period,
    [CCode (cname = "dhcpclientstatetype_DHCP_START")]
    DHCPStart,
    [CCode (cname = "dhcpclientstatetype_LEASE_START")]
    LeaseStart,
    [CCode (cname = "dhcpclientstatetype_LEASE_END")]
    LeaseEnd,
    [CCode (cname = "dhcpclientstatetype_T1_END")]
    T1End,
    [CCode (cname = "dhcpclientstatetype_T2_END")]
    T2End
  }

  [CCode (cname = "dhcpclientstate_state")]
  public enum DHCPClientState_State
  {
    [CCode (cname = "dhcpclientstatestate_SLEEPING")]
    Sleeping,
    [CCode (cname = "dhcpclientstatestate_INIT_REBOOT")]
    InitReboot,
    [CCode (cname = "dhcpclientstatestate_INIT")]
    Init,
    [CCode (cname = "dhcpclientstatestate_REBOOTING")]
    Rebooting,
    [CCode (cname = "dhcpclientstatestate_SELECTING")]
    Selecting,
    [CCode (cname = "dhcpclientstatestate_REQUESTING")]
    Requesting,
    [CCode (cname = "dhcpclientstatestate_BOUND")]
    Bound,
    [CCode (cname = "dhcpclientstatestate_RENEWING")]
    Renewing,
    [CCode (cname = "dhcpclientstatestate_REBINDING")]
    Rebinding
  }


  [CCode (cname = "xdhcpclient_control")]
  public static OS.Error* XControl
    (DHCPClientControl_Reason reason,
     char* interface_name);

  [CCode (cname = "dhcpclient_control")]
  public static void Control
    (DHCPClientControl_Reason reason,
     char* interface_name);

  [CCode (cname = "xdhcpclient_state")]
  public static OS.Error* XState
    (char* interface_name,
     DHCPClientState_Types type_list,
     char* buf,
     int buf_size,
     out DHCPClientState_Types last_type,
     out char* buf_end,
     out int space_left);

  [CCode (cname = "dhcpclient_state")]
  public static void State
    (char* interface_name,
     DHCPClientState_Types type_list,
     char* buf,
     int buf_size,
     out DHCPClientState_Types last_type,
     out char* buf_end,
     out int space_left);

  [CCode (cname = "xdhcpclient_enumerate")]
  public static OS.Error* XEnumerate
    (char* interface_name,
     char* buf,
     int buf_size,
     out int num_state_transitions,
     out char* buf_end,
     out int space_left);

  [CCode (cname = "dhcpclient_enumerate")]
  public static void Enumerate
    (char* interface_name,
     char* buf,
     int buf_size,
     out int num_state_transitions,
     out char* buf_end,
     out int space_left);

  [CCode (cname = "xservice_internet_status4")]
  public static OS.Error* XService_InternetStatus4
    (char* interface_name,
     char* dib,
     char* dhcp_reply,
     int dhcp_reply_size,
     out bool unclaimed);

  [CCode (cname = "service_internet_status4")]
  public static bool Service_InternetStatus4
    (char* interface_name,
     char* dib,
     char* dhcp_reply,
     int dhcp_reply_size);

  [CCode (cname = "xservice_internet_status5")]
  public static OS.Error* XService_InternetStatus5
    (char* interface_name,
     char* dib,
     char* dhcpoffer_buf,
     int dhcpoffer_buf_size,
     out bool unclaimed);

  [CCode (cname = "service_internet_status5")]
  public static bool Service_InternetStatus5
    (char* interface_name,
     char* dib,
     char* dhcpoffer_buf,
     int dhcpoffer_buf_size);

  [CCode (cname = "xservice_internet_status30")]
  public static OS.Error* XService_InternetStatus30
    (char* interface_name,
     int ip_address);

  [CCode (cname = "service_internet_status30")]
  public static void Service_InternetStatus30
    (char* interface_name,
     int ip_address);

  [CCode (cname = "xservice_internet_service31")]
  public static OS.Error* XService_InternetService31
    (char* interface_name,
     int ip_address);

  [CCode (cname = "service_internet_service31")]
  public static void Service_InternetService31
    (char* interface_name,
     int ip_address);

}
