/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/dma.h")]
namespace OSLib.DMA
{
  [CCode (cname = "dma_c", has_type_id = false)]
  public struct C : bits
  {
  }

  [CCode (cname = "dma_tag", has_type_id = false)]
  public struct Tag : bits
  {
  }

  [CCode (cname = "dma_routines")]
  public struct Routines
  {
    [CCode (cname = "enable")]
    public unowned AsmRoutine Enable;
    [CCode (cname = "disable")]
    public unowned AsmRoutine Disable;
    [CCode (cname = "start")]
    public unowned AsmRoutine Start;
    [CCode (cname = "completed")]
    public unowned AsmRoutine Completed;
    [CCode (cname = "sync")]
    public unowned AsmRoutine Sync;
  }

  [CCode (cname = "dma_scatter")]
  public struct Scatter
  {
    [CCode (cname = "address")]
    public unowned char* Address;
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [CCode (cname = "dma_scatter_list")]
  public struct ScatterList
  {
    [CCode (cname = "s", array_length = false)]
    public unowned Scatter[] S;
  }

  [CCode (cname = "dma_register_channel_flags")]
  public enum RegisterChannelFlags
  {
    Default = 0
  }

  [CCode (cname = "dma_queue_flags")]
  public enum QueueFlags
  {
    [CCode (cname = "dma_QUEUE_WRITE")]
    QueueWrite,
    [CCode (cname = "dma_QUEUE_CIRCULAR_LIST")]
    QueueCircularList,
    [CCode (cname = "dma_QUEUE_USE_SYNC")]
    QueueUseSync
  }

  [CCode (cname = "dma_suspend_flags")]
  public enum SuspendFlags
  {
    [CCode (cname = "dma_SUSPEND_START_NEXT")]
    SuspendStartNext
  }

  [CCode (cname = "dma_resume_flags")]
  public enum ResumeFlags
  {
    Default = 0
  }

  [CCode (cname = "dma_examine_flags")]
  public enum ExamineFlags
  {
    Default = 0
  }


  [CCode (cname = "xdma_register_channel")]
  public static OS.Error* XRegisterChannel
    (RegisterChannelFlags flags,
     int log_channel,
     int cycle_speed,
     int transfer_size,
     Routines control_routines,
     char* workspace,
     out C channel);

  [CCode (cname = "dma_register_channel")]
  public static C RegisterChannel
    (RegisterChannelFlags flags,
     int log_channel,
     int cycle_speed,
     int transfer_size,
     Routines control_routines,
     char* workspace);

  [CCode (cname = "xdma_deregister_channel")]
  public static OS.Error* XDeregisterChannel
    (C channel);

  [CCode (cname = "dma_deregister_channel")]
  public static void DeregisterChannel
    (C channel);

  [CCode (cname = "xdma_queue_transfer")]
  public static OS.Error* XQueueTransfer
    (QueueFlags flags,
     C channel,
     char* workspace,
     ScatterList scatter_list,
     int transfer_size,
     int buffer_size,
     int sync_size,
     out Tag tag);

  [CCode (cname = "dma_queue_transfer")]
  public static Tag QueueTransfer
    (QueueFlags flags,
     C channel,
     char* workspace,
     ScatterList scatter_list,
     int transfer_size,
     int buffer_size,
     int sync_size);

  [CCode (cname = "xdma_terminate_transfer")]
  public static OS.Error* XTerminateTransfer
    (OS.Error error,
     Tag tag);

  [CCode (cname = "dma_terminate_transfer")]
  public static void TerminateTransfer
    (OS.Error error,
     Tag tag);

  [CCode (cname = "xdma_suspend_transfer")]
  public static OS.Error* XSuspendTransfer
    (SuspendFlags flags,
     Tag tag);

  [CCode (cname = "dma_suspend_transfer")]
  public static void SuspendTransfer
    (SuspendFlags flags,
     Tag tag);

  [CCode (cname = "xdma_resume_transfer")]
  public static OS.Error* XResumeTransfer
    (ResumeFlags flags,
     Tag tag);

  [CCode (cname = "dma_resume_transfer")]
  public static void ResumeTransfer
    (ResumeFlags flags,
     Tag tag);

  [CCode (cname = "xdma_examine_transfer")]
  public static OS.Error* XExamineTransfer
    (ExamineFlags flags,
     Tag tag,
     out int transferred_size);

  [CCode (cname = "dma_examine_transfer")]
  public static void ExamineTransfer
    (ExamineFlags flags,
     Tag tag,
     out int transferred_size);

}
