/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileCore;

[CCode (cheader_filename = "oslib/dosfs.h")]
namespace OSLib.DOSFS
{
  [CCode (cname = "dosfs_format_specifier")]
  public enum FormatSpecifier
  {
    [CCode (cname = "dosfs_FORMAT_DOSQ")]
    FormatDOSQ,
    [CCode (cname = "dosfs_FORMAT_DOSM")]
    FormatDOSM,
    [CCode (cname = "dosfs_FORMAT_DOSH")]
    FormatDOSH,
    [CCode (cname = "dosfs_FORMAT_DOSN")]
    FormatDOSN,
    [CCode (cname = "dosfs_FORMAT_DOSP")]
    FormatDOSP,
    [CCode (cname = "dosfs_FORMAT_DOST")]
    FormatDOST,
    [CCode (cname = "dosfs_FORMAT_DOSU")]
    FormatDOSU,
    [CCode (cname = "dosfs_FORMAT_ATARIM")]
    FormatAtariM,
    [CCode (cname = "dosfs_FORMAT_ATARIN")]
    FormatAtariN,
    [CCode (cname = "dosfs_FORMAT_PCMCIA")]
    FormatPCMCIA
  }


  [CCode (cname = "xdosfs_disc_format")]
  public static OS.Error* XDiscFormat
    (FileCore.Format buffer,
     int vet_format_swi,
     int vet_format_handle,
     FormatSpecifier format);

  [CCode (cname = "dosfs_disc_format")]
  public static void DiscFormat
    (FileCore.Format buffer,
     int vet_format_swi,
     int vet_format_handle,
     FormatSpecifier format);

  [CCode (cname = "xdosfs_layout_structure")]
  public static OS.Error* XLayoutStructure
    (FormatSpecifier format,
     FileCore.DefectList defect_list,
     string disc_name,
     OS.F image_file);

  [CCode (cname = "dosfs_layout_structure")]
  public static void LayoutStructure
    (FormatSpecifier format,
     FileCore.DefectList defect_list,
     string disc_name,
     OS.F image_file);

  [CCode (cname = "xdosfs_layout_structurew")]
  public static OS.Error* XLayoutStructureW
    (FormatSpecifier format,
     FileCore.DefectList defect_list,
     string disc_name,
     OS.FW image_file);

  [CCode (cname = "dosfs_layout_structurew")]
  public static void LayoutStructureW
    (FormatSpecifier format,
     FileCore.DefectList defect_list,
     string disc_name,
     OS.FW image_file);

  [CCode (cname = "xdosfs_layout_structure_given_size")]
  public static OS.Error* XLayoutStructureGivenSize
    (FormatSpecifier format,
     FileCore.DefectList defect_list,
     string disc_name,
     OS.F image_file,
     int size);

  [CCode (cname = "dosfs_layout_structure_given_size")]
  public static void LayoutStructureGivenSize
    (FormatSpecifier format,
     FileCore.DefectList defect_list,
     string disc_name,
     OS.F image_file,
     int size);

  [CCode (cname = "xdosfs_layout_structure_given_sizew")]
  public static OS.Error* XLayoutStructureGivenSizeW
    (FormatSpecifier format,
     FileCore.DefectList defect_list,
     string disc_name,
     OS.FW image_file,
     int size);

  [CCode (cname = "dosfs_layout_structure_given_sizew")]
  public static void LayoutStructureGivenSizeW
    (FormatSpecifier format,
     FileCore.DefectList defect_list,
     string disc_name,
     OS.FW image_file,
     int size);

}
