/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/draw.h")]
namespace OSLib.Draw
{
  [CCode (cname = "draw_tag")]
  public enum Tag
  {
    [CCode (cname = "draw_END_PATH")]
    EndPath,
    [CCode (cname = "draw_CONTINUATION")]
    Continuation,
    [CCode (cname = "draw_MOVE_TO")]
    MoveTo,
    [CCode (cname = "draw_SPECIAL_MOVE_TO")]
    SpecialMoveTo,
    [CCode (cname = "draw_CLOSE_GAP")]
    CloseGap,
    [CCode (cname = "draw_CLOSE_LINE")]
    CloseLine,
    [CCode (cname = "draw_BEZIER_TO")]
    BezierTo,
    [CCode (cname = "draw_GAP_TO")]
    GapTo,
    [CCode (cname = "draw_LINE_TO")]
    LineTo
  }

  [CCode (cname = "draw_fill_style")]
  public enum FillStyle
  {
    [CCode (cname = "draw_FILL_NONZERO")]
    FillNonzero,
    [CCode (cname = "draw_FILL_NEGATIVE")]
    FillNegative,
    [CCode (cname = "draw_FILL_EVEN_ODD")]
    FillEvenOdd,
    [CCode (cname = "draw_FILL_POSITIVE")]
    FillPositive,
    [CCode (cname = "draw_FILL_WINDING_RULE")]
    FillWindingRule,
    [CCode (cname = "draw_FILL_FULL_EXTERIOR")]
    FillFullExterior,
    [CCode (cname = "draw_FILL_EXTERIOR_BOUNDARY")]
    FillExteriorBoundary,
    [CCode (cname = "draw_FILL_INTERIOR_BOUNDARY")]
    FillInteriorBoundary,
    [CCode (cname = "draw_FILL_FULL_INTERIOR")]
    FillFullInterior,
    [CCode (cname = "draw_FILL_OUTPUT_BBOX")]
    FillOutputBBox,
    [CCode (cname = "draw_FILL32_BIT_CLEAN")]
    Fill32BitClean,
    [CCode (cname = "draw_FILL_CLOSE_OPEN_SUBPATHS")]
    FillCloseOpenSubpaths,
    [CCode (cname = "draw_FILL_FLATTEN")]
    FillFlatten,
    [CCode (cname = "draw_FILL_THICKEN")]
    FillThicken,
    [CCode (cname = "draw_FILL_REFLATTEN")]
    FillReflatten,
    [CCode (cname = "draw_FILL_FLOAT")]
    FillFloat
  }

  [CCode (cname = "draw_join_style")]
  public enum JoinStyle
  {
    [CCode (cname = "draw_JOIN_MITRED")]
    JoinMitred,
    [CCode (cname = "draw_JOIN_ROUND")]
    JoinRound,
    [CCode (cname = "draw_JOIN_BEVELLED")]
    JoinBevelled
  }

  [CCode (cname = "draw_cap_style")]
  public enum CapStyle
  {
    [CCode (cname = "draw_CAP_BUTT")]
    CapButt,
    [CCode (cname = "draw_CAP_ROUND")]
    CapRound,
    [CCode (cname = "draw_CAP_SQUARE")]
    CapSquare,
    [CCode (cname = "draw_CAP_TRIANGULAR")]
    CapTriangular
  }

  [CCode (cname = "draw_output_path")]
  public enum OutputPath
  {
    [CCode (cname = "draw_SPECIAL_IN_SITU")]
    SpecialInSitu,
    [CCode (cname = "draw_SPECIAL_FILL")]
    SpecialFill,
    [CCode (cname = "draw_SPECIAL_FILL_BY_SUBPATHS")]
    SpecialFillBySubpaths,
    [CCode (cname = "draw_SPECIAL_COUNT")]
    SpecialCount,
    [CCode (cname = "draw_SPECIAL_BBOX")]
    SpecialBBox
  }

  [CCode (cname = "draw_path_element")]
  public struct PathElement
  {
    [CCode (cname = "tag")]
    public unowned Tag Tag;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "data.end_path")]
    public unowned int Data_EndPath;
    [CCode (cname = "data.continuation")]
    public unowned PathElement* Data_Continuation;
    [CCode (cname = "data.move_to")]
    public unowned OS.Coord Data_MoveTo;
    [CCode (cname = "data.special_move_to")]
    public unowned OS.Coord Data_SpecialMoveTo;
    [CCode (cname = "data.bezier_to")]
    public unowned OS.Coord Data_BezierTo[3];
    [CCode (cname = "data.gap_to")]
    public unowned OS.Coord Data_GapTo;
    [CCode (cname = "data.line_to")]
    public unowned OS.Coord Data_LineTo;
    [CCode (cname = "data.reserved")]
    public unowned byte Data_Reserved[24];
  }

  [CCode (cname = "draw_line_style")]
  public struct LineStyle
  {
    [CCode (cname = "join_style")]
    public unowned JoinStyle JoinStyle;
    [CCode (cname = "end_cap_style")]
    public unowned CapStyle EndCapStyle;
    [CCode (cname = "start_cap_style")]
    public unowned CapStyle StartCapStyle;
    [CCode (cname = "reserved")]
    public unowned byte Reserved;
    [CCode (cname = "mitre_limit")]
    public unowned int MitreLimit;
    [CCode (cname = "start_cap_width")]
    public unowned short StartCapWidth;
    [CCode (cname = "start_cap_length")]
    public unowned short StartCapLength;
    [CCode (cname = "end_cap_width")]
    public unowned short EndCapWidth;
    [CCode (cname = "end_cap_length")]
    public unowned short EndCapLength;
  }

  [CCode (cname = "draw_dash_pattern_base")]
  public struct DashPatternBase
  {
    [CCode (cname = "start")]
    public unowned int Start;
    [CCode (cname = "element_count")]
    public unowned int ElementCount;
  }

  [Compact]
  [CCode (cname = "draw_dash_pattern", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class DashPattern
  {
    public DashPattern(int numEntries);
    [CCode (cname = "start")]
    public unowned int Start;
    [CCode (cname = "element_count")]
    public unowned int ElementCount;
    [CCode (cname = "elements", array_length = false)]
    public unowned int[] Elements;
  }

  [CCode (cname = "draw_path")]
  public struct Path
  {
    [CCode (cname = "elements")]
    public unowned PathElement Elements[1];
  }

  [CCode (cname = "draw_clip_descriptor")]
  public struct ClipDescriptor
  {
    [CCode (cname = "path")]
    public unowned Path* Path;
    [CCode (cname = "fill_style")]
    public unowned FillStyle FillStyle;
    [CCode (cname = "trfm")]
    public unowned OS.Trfm Trfm;
  }

  [CCode (cname = "drawv_reason")]
  public enum DrawV_Reason
  {
    [CCode (cname = "drawv_PROCESS_PATH")]
    ProcessPath,
    [CCode (cname = "drawv_FILL")]
    Fill,
    [CCode (cname = "drawv_STROKE")]
    Stroke,
    [CCode (cname = "drawv_STROKE_PATH")]
    StrokePath,
    [CCode (cname = "drawv_FLATTEN_PATH")]
    FlattenPath,
    [CCode (cname = "drawv_TRANSFORM_PATH")]
    TransformPath,
    [CCode (cname = "drawv_FILL_CLIPPED")]
    FillClipped,
    [CCode (cname = "drawv_STROKE_CLIPPED")]
    StrokeClipped
  }

  [CCode (cname="draw_OS_UNIT")]
  public const int OSUnit;
  [CCode (cname="draw_INCH")]
  public const int Inch;
  [CCode (cname="draw_POINT")]
  public const int Point;
  [CCode (cname="draw_FILL_WINDING_RULE_SHIFT")]
  public const int FillWindingRuleShift;
  [CCode (cname="error_DRAW_NO_DRAW_IN_IRQ_MODE")]
  public const bits Error_DrawNoDrawInIRQMode;
  [CCode (cname="error_DRAW_BAD_DRAW_REASON_CODE")]
  public const bits Error_DrawBadDrawReasonCode;
  [CCode (cname="error_DRAW_RESERVED_DRAW_BITS")]
  public const bits Error_DrawReservedDrawBits;
  [CCode (cname="error_DRAW_INVALID_DRAW_ADDRESS")]
  public const bits Error_DrawInvalidDrawAddress;
  [CCode (cname="error_DRAW_BAD_PATH_ELEMENT")]
  public const bits Error_DrawBadPathElement;
  [CCode (cname="error_DRAW_BAD_PATH_SEQUENCE")]
  public const bits Error_DrawBadPathSequence;
  [CCode (cname="error_DRAW_MAY_EXPAND_PATH")]
  public const bits Error_DrawMayExpandPath;
  [CCode (cname="error_DRAW_PATH_FULL")]
  public const bits Error_DrawPathFull;
  [CCode (cname="error_DRAW_PATH_NOT_FLAT")]
  public const bits Error_DrawPathNotFlat;
  [CCode (cname="error_DRAW_BAD_CAPS_OR_JOINS")]
  public const bits Error_DrawBadCapsOrJoins;
  [CCode (cname="error_DRAW_TRANSFORM_OVERFLOW")]
  public const bits Error_DrawTransformOverflow;
  [CCode (cname="error_DRAW_DRAW_NEEDS_GRAPHICS_MODE")]
  public const bits Error_DrawDrawNeedsGraphicsMode;
  [CCode (cname="error_DRAW_UNIMPLEMENTED_DRAW")]
  public const bits Error_DrawUnimplementedDraw;

  [CCode (cname = "xdraw_process_path")]
  public static OS.Error* XProcessPath
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness,
     int thickness,
     LineStyle line_style,
     DashPattern dash_pattern,
     OutputPath processed_path,
     out char* end_or_used);

  [CCode (cname = "draw_process_path")]
  public static char* ProcessPath
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness,
     int thickness,
     LineStyle line_style,
     DashPattern dash_pattern,
     OutputPath processed_path);

  [CCode (cname = "xdraw_fill")]
  public static OS.Error* XFill
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness);

  [CCode (cname = "draw_fill")]
  public static void Fill
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness);

  [CCode (cname = "xdraw_stroke")]
  public static OS.Error* XStroke
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness,
     int thickness,
     LineStyle line_style,
     DashPattern dash_pattern);

  [CCode (cname = "draw_stroke")]
  public static void Stroke
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness,
     int thickness,
     LineStyle line_style,
     DashPattern dash_pattern);

  [CCode (cname = "xdraw_stroke_path")]
  public static OS.Error* XStrokePath
    (Path path,
     Path stroked_path,
     OS.Trfm trfm,
     int flatness,
     int thickness,
     LineStyle line_style,
     DashPattern dash_pattern,
     out char* end_or_used);

  [CCode (cname = "draw_stroke_path")]
  public static char* StrokePath
    (Path path,
     Path stroked_path,
     OS.Trfm trfm,
     int flatness,
     int thickness,
     LineStyle line_style,
     DashPattern dash_pattern);

  [CCode (cname = "xdraw_flatten_path")]
  public static OS.Error* XFlattenPath
    (Path path,
     Path flattened_path,
     int flatness,
     out char* end_or_used);

  [CCode (cname = "draw_flatten_path")]
  public static char* FlattenPath
    (Path path,
     Path flattened_path,
     int flatness);

  [CCode (cname = "xdraw_transform_path")]
  public static OS.Error* XTransformPath
    (Path path,
     Path transformed_path,
     OS.Trfm trfm,
     out char* end_or_used);

  [CCode (cname = "draw_transform_path")]
  public static char* TransformPath
    (Path path,
     Path transformed_path,
     OS.Trfm trfm);

  [CCode (cname = "xdraw_fill_clipped")]
  public static OS.Error* XFillClipped
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness,
     ClipDescriptor clip_descriptor);

  [CCode (cname = "draw_fill_clipped")]
  public static void FillClipped
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness,
     ClipDescriptor clip_descriptor);

  [CCode (cname = "xdraw_stroke_clipped")]
  public static OS.Error* XStrokeClipped
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness,
     int thickness,
     LineStyle line_style,
     DashPattern dash_pattern,
     ClipDescriptor clip_descriptor);

  [CCode (cname = "draw_stroke_clipped")]
  public static void StrokeClipped
    (Path path,
     FillStyle fill_style,
     OS.Trfm trfm,
     int flatness,
     int thickness,
     LineStyle line_style,
     DashPattern dash_pattern,
     ClipDescriptor clip_descriptor);

}
