/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/fslock.h")]
namespace OSLib.FSLock
{
  [CCode (cname = "fslock_lock_status")]
  public enum LockStatus
  {
    [CCode (cname = "fslock_STATUS_UNLOCKED")]
    StatusUnlocked,
    [CCode (cname = "fslock_STATUS_PROTECTED")]
    StatusProtected,
    [CCode (cname = "fslock_STATUS_LOCKED")]
    StatusLocked
  }

  [CCode (cname="error_FS_LOCK_UNKNOWN_SWI")]
  public const bits Error_FSLockUnknownSWI;
  [CCode (cname="error_FS_LOCK_LOCKED")]
  public const bits Error_FSLockLocked;
  [CCode (cname="error_FS_LOCK_UNKNOWN_FS")]
  public const bits Error_FSLockUnknownFS;
  [CCode (cname="error_FS_LOCK_FS_NOT_LOCKABLE")]
  public const bits Error_FSLockFSNotLockable;
  [CCode (cname="error_FS_LOCK_NO_LOCKED_FS")]
  public const bits Error_FSLockNoLockedFS;
  [CCode (cname="error_FS_LOCK_PROTECTED_DISC")]
  public const bits Error_FSLockProtectedDisc;
  [CCode (cname="error_FS_LOCK_KILLED")]
  public const bits Error_FSLockKilled;

  [CCode (cname = "xfslock_version")]
  public static OS.Error* XVersion
    (out int version,
     out char* workspace);

  [CCode (cname = "fslock_version")]
  public static int Version
    (out char* workspace);

  [CCode (cname = "xfslock_status")]
  public static OS.Error* XStatus
    (out LockStatus lock_status,
     out FileSwitch.FSNo fs_no);

  [CCode (cname = "fslock_status")]
  public static LockStatus Status
    (out FileSwitch.FSNo fs_no);

  [CCode (cname = "xfslock_change_status")]
  public static OS.Error* XChangeStatus
    (LockStatus new_lock_status,
     string password,
     string new_password,
     FileSwitch.FSNo new_fs_no);

  [CCode (cname = "fslock_change_status")]
  public static void ChangeStatus
    (LockStatus new_lock_status,
     string password,
     string new_password,
     FileSwitch.FSNo new_fs_no);

}
