/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/font.h")]
namespace OSLib.Font
{
  [CCode (cname = "font_string_flags")]
  public enum StringFlags
  {
    [CCode (cname = "font_JUSTIFY")]
    Justify,
    [CCode (cname = "font_RUBOUT")]
    Rubout,
    [CCode (cname = "font_OS_UNITS")]
    OSUnits,
    [CCode (cname = "font_GIVEN_BLOCK")]
    GivenBlock,
    [CCode (cname = "font_GIVEN_TRFM")]
    GivenTrfm,
    [CCode (cname = "font_GIVEN_LENGTH")]
    GivenLength,
    [CCode (cname = "font_GIVEN_FONT")]
    GivenFont,
    [CCode (cname = "font_KERN")]
    Kern,
    [CCode (cname = "font_RIGHT_TO_LEFT")]
    RightToLeft,
    [CCode (cname = "font_BLEND_FONT")]
    BlendFont,
    [CCode (cname = "font_GIVEN16_BIT")]
    Given16Bit,
    [CCode (cname = "font_GIVEN32_BIT")]
    Given32Bit,
    [CCode (cname = "font_RETURN_CARET_POS")]
    ReturnCaretPos,
    [CCode (cname = "font_RETURN_BBOX")]
    ReturnBBox,
    [CCode (cname = "font_RETURN_MATRIX")]
    ReturnMatrix,
    [CCode (cname = "font_RETURN_SPLIT_COUNT")]
    ReturnSplitCount
  }

  [CCode (cname = "font_output_flags")]
  public enum OutputFlags
  {
    [CCode (cname = "font_NO_OUTPUT")]
    NoOutput,
    [CCode (cname = "font_ADD_HINTS")]
    AddHints,
    [CCode (cname = "font_OUTPUT_SKELETON")]
    OutputSkeleton,
    [CCode (cname = "font_CONVERT_BITMAP")]
    ConvertBitmap,
    [CCode (cname = "font_ERROR_IF_BITMAP")]
    ErrorIfBitmap
  }

  [CCode (cname = "font_f")]
  public enum F
  {
    [CCode (cname = "font_SYSTEM")]
    System,
    [CCode (cname = "font_CURRENT")]
    Current
  }

  [CCode (cname = "font_paint_block")]
  public struct PaintBlock
  {
    [CCode (cname = "space")]
    public unowned OS.Coord Space;
    [CCode (cname = "letter")]
    public unowned OS.Coord Letter;
    [CCode (cname = "rubout")]
    public unowned OS.Box Rubout;
  }

  [CCode (cname = "font_scan_block")]
  public struct ScanBlock
  {
    [CCode (cname = "space")]
    public unowned OS.Coord Space;
    [CCode (cname = "letter")]
    public unowned OS.Coord Letter;
    [CCode (cname = "split_char")]
    public unowned int SplitChar;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
  }

  [CCode (cname = "font_thresholds")]
  public struct Thresholds
  {
    [CCode (cname = "t", array_length = false)]
    public unowned byte[] T;
  }

  [CCode (cname = "font_colour_table")]
  public struct ColourTable
  {
    [CCode (cname = "c", array_length = false)]
    public unowned byte[] C;
  }

  [CCode (cname = "font_bitmap_type")]
  public enum BitmapType
  {
    [CCode (cname = "font_BITMAP_DEEP")]
    BitmapDeep,
    [CCode (cname = "font_BITMAP_HORIZONTAL_SUB_PIXEL")]
    BitmapHorizontalSubPixel,
    [CCode (cname = "font_BITMAP_VERTICAL_SUB_PIXEL")]
    BitmapVerticalSubPixel,
    [CCode (cname = "font_BITMAP_DELETE")]
    BitmapDelete
  }

  [CCode (cname = "font_characteristic_flags")]
  public enum CharacteristicFlags
  {
    [CCode (cname = "font_CHARACTERISTIC_OLD_FORMAT")]
    CharacteristicOldFormat,
    [CCode (cname = "font_CHARACTERISTIC_ROM")]
    CharacteristicROM,
    [CCode (cname = "font_CHARACTERISTIC_MONOCHROME")]
    CharacteristicMonochrome,
    [CCode (cname = "font_CHARACTERISTIC_FILL_RULE_NON_ZERO")]
    CharacteristicFillRuleNonZero
  }

  [CCode (cname = "font_list_context")]
  public enum ListContext
  {
    [CCode (cname = "font_RETURN_FONT_NAME")]
    ReturnFontName,
    [CCode (cname = "font_RETURN_LOCAL_FONT_NAME")]
    ReturnLocalFontName,
    [CCode (cname = "font_USE_LINEFEED")]
    UseLinefeed,
    [CCode (cname = "font_RETURN_FONT_MENU")]
    ReturnFontMenu,
    [CCode (cname = "font_ALLOW_SYSTEM_FONT")]
    AllowSystemFont,
    [CCode (cname = "font_GIVEN_TICK")]
    GivenTick,
    [CCode (cname = "font_RETURN_ENCODING_MENU")]
    ReturnEncodingMenu
  }

  [CCode (cname = "font_kern_flags")]
  public enum KernFlags
  {
    [CCode (cname = "font_KERN_NO_BBOXES")]
    KernNoBBoxes,
    [CCode (cname = "font_KERN_NO_XOFFSETS")]
    KernNoXOffsets,
    [CCode (cname = "font_KERN_NO_YOFFSETS")]
    KernNoYOffsets,
    [CCode (cname = "font_KERN_SHORT_PAIRS")]
    KernShortPairs
  }

  [CCode (cname = "font_metric_flags")]
  public enum MetricFlags
  {
    [CCode (cname = "font_METRICS_KERN_YONLY")]
    MetricsKernYOnly,
    [CCode (cname = "font_METRICS_KERN_XONLY")]
    MetricsKernXOnly,
    [CCode (cname = "font_METRICS_MANY_KERN_PAIRS")]
    MetricsManyKernPairs
  }

  [CCode (cname = "font_bbox_info")]
  public struct BBoxInfo
  {
    [CCode (cname = "bboxes")]
    public unowned OS.Box Bboxes[256];
  }

  [CCode (cname = "font_width_info")]
  public struct WidthInfo
  {
    [CCode (cname = "widths")]
    public unowned int Widths[256];
  }

  [CCode (cname = "font_misc_info")]
  public struct MiscInfo
  {
    [CCode (cname = "x0")]
    public unowned short X0;
    [CCode (cname = "y0")]
    public unowned short Y0;
    [CCode (cname = "x1")]
    public unowned short X1;
    [CCode (cname = "y1")]
    public unowned short Y1;
    [CCode (cname = "xkern")]
    public unowned short Xkern;
    [CCode (cname = "ykern")]
    public unowned short Ykern;
    [CCode (cname = "italic_correction")]
    public unowned short ItalicCorrection;
    [CCode (cname = "underline_position")]
    public unowned byte UnderlinePosition;
    [CCode (cname = "underline_thickness")]
    public unowned byte UnderlineThickness;
    [CCode (cname = "cap_height")]
    public unowned short CapHeight;
    [CCode (cname = "xheight")]
    public unowned short Xheight;
    [CCode (cname = "descender")]
    public unowned short Descender;
    [CCode (cname = "ascender")]
    public unowned short Ascender;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
  }

  [CCode (cname = "font_metrics_misc_info")]
  public struct MetricsMiscInfo
  {
    [CCode (cname = "x0")]
    public unowned int X0;
    [CCode (cname = "y0")]
    public unowned int Y0;
    [CCode (cname = "x1")]
    public unowned int X1;
    [CCode (cname = "y1")]
    public unowned int Y1;
    [CCode (cname = "xkern")]
    public unowned int Xkern;
    [CCode (cname = "ykern")]
    public unowned int Ykern;
    [CCode (cname = "italic_correction")]
    public unowned int ItalicCorrection;
    [CCode (cname = "underline_position")]
    public unowned byte UnderlinePosition;
    [CCode (cname = "underline_thickness")]
    public unowned byte UnderlineThickness;
    [CCode (cname = "reserved_0")]
    public unowned byte Reserved0;
    [CCode (cname = "reserved_1")]
    public unowned byte Reserved1;
    [CCode (cname = "cap_height")]
    public unowned int CapHeight;
    [CCode (cname = "xheight")]
    public unowned int Xheight;
    [CCode (cname = "descender")]
    public unowned int Descender;
    [CCode (cname = "ascender")]
    public unowned int Ascender;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
  }

  [CCode (cname = "font_short_kern_pair")]
  public enum ShortKernPair
  {
    Default = 0
  }

  [CCode (cname = "font_long_kern_pair")]
  public struct LongKernPair
  {
    [CCode (cname = "c")]
    public unowned int C;
    [CCode (cname = "xkern")]
    public unowned int Xkern;
    [CCode (cname = "ykern")]
    public unowned int Ykern;
  }

  [CCode (cname = "font_kern_info")]
  public struct KernInfo
  {
    [CCode (cname = "long_info.table")]
    public unowned int LongInfo_Table[256];
    [CCode (cname = "long_info.size")]
    public unowned int LongInfo_Size;
    [CCode (cname = "long_info.flags")]
    public unowned KernFlags LongInfo_Flags;
    [CCode (cname = "long_info.pairs", array_length = false)]
    public unowned LongKernPair[] LongInfo_Pairs;
    [CCode (cname = "short_info.table")]
    public unowned int ShortInfo_Table[256];
    [CCode (cname = "short_info.size")]
    public unowned int ShortInfo_Size;
    [CCode (cname = "short_info.flags")]
    public unowned KernFlags ShortInfo_Flags;
    [CCode (cname = "short_info.pairs", array_length = false)]
    public unowned ShortKernPair[] ShortInfo_Pairs;
  }

  [CCode (cname = "font_short_kern_pair_ucs")]
  public struct ShortKernPairUCS
  {
    [CCode (cname = "c1")]
    public unowned char C1;
    [CCode (cname = "c2")]
    public unowned char C2;
    [CCode (cname = "kern")]
    public unowned short Kern;
  }

  [CCode (cname = "font_long_kern_pair_ucs")]
  public struct LongKernPairUCS
  {
    [CCode (cname = "c1")]
    public unowned short C1;
    [CCode (cname = "c2")]
    public unowned short C2;
    [CCode (cname = "xkern")]
    public unowned short Xkern;
    [CCode (cname = "ykern")]
    public unowned short Ykern;
  }

  [CCode (cname = "font_kern_info_ucs")]
  public struct KernInfoUCS
  {
    [CCode (cname = "long_info.table")]
    public unowned int LongInfo_Table[256];
    [CCode (cname = "long_info.size")]
    public unowned int LongInfo_Size;
    [CCode (cname = "long_info.flags")]
    public unowned KernFlags LongInfo_Flags;
    [CCode (cname = "long_info.pairs", array_length = false)]
    public unowned LongKernPairUCS[] LongInfo_Pairs;
    [CCode (cname = "short_info.table")]
    public unowned int ShortInfo_Table[256];
    [CCode (cname = "short_info.size")]
    public unowned int ShortInfo_Size;
    [CCode (cname = "short_info.flags")]
    public unowned KernFlags ShortInfo_Flags;
    [CCode (cname = "short_info.pairs", array_length = false)]
    public unowned ShortKernPairUCS[] ShortInfo_Pairs;
  }

  [CCode (cname = "font_decode_flags")]
  public enum DecodeFlags
  {
    [CCode (cname = "font_GIVEN_ENCODING_MENU")]
    GivenEncodingMenu
  }

  [CCode (cname="font_OS_UNIT")]
  public const int OSUnit;
  [CCode (cname="font_INCH")]
  public const int Inch;
  [CCode (cname="font_POINT")]
  public const int Point;
  [CCode (cname="font_COMMAND_NULL")]
  public const char CommandNull;
  [CCode (cname="font_COMMAND_XMOVE")]
  public const char CommandXMove;
  [CCode (cname="font_COMMAND_LINEFEED")]
  public const char CommandLinefeed;
  [CCode (cname="font_COMMAND_YMOVE")]
  public const char CommandYMove;
  [CCode (cname="font_COMMAND_RETURN")]
  public const char CommandReturn;
  [CCode (cname="font_COMMAND_GCOL")]
  public const char CommandGCol;
  [CCode (cname="font_COMMAND_COLOURS")]
  public const char CommandColours;
  [CCode (cname="font_COMMAND_TRUE_COLOURS")]
  public const char CommandTrueColours;
  [CCode (cname="font_COMMAND_COMMENT")]
  public const char CommandComment;
  [CCode (cname="font_COMMAND_UNDERLINE")]
  public const char CommandUnderline;
  [CCode (cname="font_COMMAND_FONT")]
  public const char CommandFont;
  [CCode (cname="font_COMMAND_HOM_TRFM")]
  public const char CommandHomTrfm;
  [CCode (cname="font_COMMAND_TRFM")]
  public const char CommandTrfm;
  [CCode (cname="error_FONT_NO_ROOM")]
  public const bits Error_FontNoRoom;
  [CCode (cname="error_FONT_CACHE_FULL")]
  public const bits Error_FontCacheFull;
  [CCode (cname="error_FONT_NO_CACHE")]
  public const bits Error_FontNoCache;
  [CCode (cname="error_FONT_TOO_LONG")]
  public const bits Error_FontTooLong;
  [CCode (cname="error_FONT64K")]
  public const bits Error_Font64K;
  [CCode (cname="error_FONT_PAL_TOO_BIG")]
  public const bits Error_FontPalTooBig;
  [CCode (cname="error_FONT_BAD_TRAN_BITS")]
  public const bits Error_FontBadTranBits;
  [CCode (cname="error_FONT_NOT_ENOUGH_BITS")]
  public const bits Error_FontNotEnoughBits;
  [CCode (cname="error_FONT_NO_FONT")]
  public const bits Error_FontNoFont;
  [CCode (cname="error_FONT_NO_PIXELS")]
  public const bits Error_FontNoPixels;
  [CCode (cname="error_FONT_BAD_FONT_NUMBER")]
  public const bits Error_FontBadFontNumber;
  [CCode (cname="error_FONT_NOT_FOUND")]
  public const bits Error_FontNotFound;
  [CCode (cname="error_FONT_BAD_FONT_FILE")]
  public const bits Error_FontBadFontFile;
  [CCode (cname="error_FONT_NO_HANDLES")]
  public const bits Error_FontNoHandles;
  [CCode (cname="error_FONT_BAD_COUNTER")]
  public const bits Error_FontBadCounter;
  [CCode (cname="error_FONT_BAD_CTRL_CHAR")]
  public const bits Error_FontBadCtrlChar;
  [CCode (cname="error_FONTS_IN_USE")]
  public const bits Error_FontsInUse;
  [CCode (cname="error_FONT_BAD_SEGMENT")]
  public const bits Error_FontBadSegment;
  [CCode (cname="error_FONT_BAD_PREFIX")]
  public const bits Error_FontBadPrefix;
  [CCode (cname="error_FONT_RESERVED")]
  public const bits Error_FontReserved;
  [CCode (cname="error_FONT_BAD_CHAR_CODE")]
  public const bits Error_FontBadCharCode;
  [CCode (cname="error_FONT_NO_BITMAPS")]
  public const bits Error_FontNoBitmaps;
  [CCode (cname="error_FONT_NO_BITMAPS2")]
  public const bits Error_FontNoBitmaps2;
  [CCode (cname="error_FONT_BAD_FONT_CACHE_FILE")]
  public const bits Error_FontBadFontCacheFile;
  [CCode (cname="error_FONT_FIELD_NOT_FOUND")]
  public const bits Error_FontFieldNotFound;
  [CCode (cname="error_FONT_BAD_MATRIX")]
  public const bits Error_FontBadMatrix;
  [CCode (cname="error_FONT_OVERFLOW")]
  public const bits Error_FontOverflow;
  [CCode (cname="error_FONT_DIVBY0")]
  public const bits Error_FontDivby0;
  [CCode (cname="error_FONT_BAD_READ_METRICS")]
  public const bits Error_FontBadReadMetrics;
  [CCode (cname="error_FONT_BAD_RGB")]
  public const bits Error_FontBadRGB;
  [CCode (cname="error_FONT_ENCODING_NOT_FOUND")]
  public const bits Error_FontEncodingNotFound;
  [CCode (cname="error_FONT_MUST_HAVE_SLASH")]
  public const bits Error_FontMustHaveSlash;
  [CCode (cname="error_FONT_BAD_ENCODING_SIZE")]
  public const bits Error_FontBadEncodingSize;
  [CCode (cname="error_FONT_TOO_MANY_IDS")]
  public const bits Error_FontTooManyIDs;
  [CCode (cname="error_FONT_TOO_FEW_IDS")]
  public const bits Error_FontTooFewIDs;
  [CCode (cname="error_FONT_NO_BASE_ENCODING")]
  public const bits Error_FontNoBaseEncoding;
  [CCode (cname="error_FONT_IDENTIFIER_NOT_FOUND")]
  public const bits Error_FontIdentifierNotFound;
  [CCode (cname="error_FONT_TOO_MANY_CHUNKS")]
  public const bits Error_FontTooManyChunks;
  [CCode (cname="error_FONT_BAD_FONT_FILE2")]
  public const bits Error_FontBadFontFile2;
  [CCode (cname="error_FONT_DATA_NOT_FOUND")]
  public const bits Error_FontDataNotFound;
  [CCode (cname="error_FONT_DATA_NOT_FOUND2")]
  public const bits Error_FontDataNotFound2;
  [CCode (cname="font_NAME_LIMIT")]
  public const int NameLimit;
  [CCode (cname="font_TICK_NONE")]
  public static char* TickNone;
  [CCode (cname="font_TICK_SYSTEM_FONT")]
  public static char* TickSystemFont;

  [CCode (cname = "xfont_cache_addr")]
  public static OS.Error* XCacheAddr
    (out int version,
     out int cache_size,
     out int cache_used);

  [CCode (cname = "font_cache_addr")]
  public static void CacheAddr
    (out int version,
     out int cache_size,
     out int cache_used);

  [CCode (cname = "xfont_find_font")]
  public static OS.Error* XFindFont
    (string font_name,
     int xsize,
     int ysize,
     int xres,
     int yres,
     out F font,
     out int xres_out,
     out int yres_out);

  [CCode (cname = "font_find_font")]
  public static F FindFont
    (string font_name,
     int xsize,
     int ysize,
     int xres,
     int yres,
     out int xres_out,
     out int yres_out);

  [CCode (cname = "xfont_lose_font")]
  public static OS.Error* XLoseFont
    (F font);

  [CCode (cname = "font_lose_font")]
  public static void LoseFont
    (F font);

  [CCode (cname = "xfont_read_defn")]
  public static OS.Error* XReadDefn
    (F font,
     char* font_id,
     out int xsize,
     out int ysize,
     out int xres,
     out int yres,
     out int age,
     out int usage_count);

  [CCode (cname = "font_read_defn")]
  public static void ReadDefn
    (F font,
     char* font_id,
     out int xsize,
     out int ysize,
     out int xres,
     out int yres,
     out int age,
     out int usage_count);

  [CCode (cname = "xfont_read_full_defn")]
  public static OS.Error* XReadFullDefn
    (F font,
     char* font_id,
     out int xsize,
     out int ysize,
     out int xres,
     out int yres,
     out int age,
     out int usage_count);

  [CCode (cname = "font_read_full_defn")]
  public static void ReadFullDefn
    (F font,
     char* font_id,
     out int xsize,
     out int ysize,
     out int xres,
     out int yres,
     out int age,
     out int usage_count);

  [CCode (cname = "xfont_read_identifier")]
  public static OS.Error* XReadIdentifier
    (F font,
     char* buffer,
     out int used);

  [CCode (cname = "font_read_identifier")]
  public static int ReadIdentifier
    (F font,
     char* buffer);

  [CCode (cname = "xfont_read_info")]
  public static OS.Error* XReadInfo
    (F font,
     out int x0,
     out int y0,
     out int x1,
     out int y1);

  [CCode (cname = "font_read_info")]
  public static void ReadInfo
    (F font,
     out int x0,
     out int y0,
     out int x1,
     out int y1);

  [CCode (cname = "xfont_string_width")]
  public static OS.Error* XStringWidth
    (string s,
     int xmax,
     int ymax,
     int split,
     int length,
     out char* split_point,
     out int x,
     out int y,
     out int length_out,
     out int index_out);

  [CCode (cname = "font_string_width")]
  public static void StringWidth
    (string s,
     int xmax,
     int ymax,
     int split,
     int length,
     out char* split_point,
     out int x,
     out int y,
     out int length_out,
     out int index_out);

  [CCode (cname = "xfont_paint")]
  public static OS.Error* XPaint
    (F font,
     string string,
     StringFlags flags,
     int xpos,
     int ypos,
     PaintBlock block,
     OS.Trfm trfm,
     int length);

  [CCode (cname = "font_paint")]
  public static void Paint
    (F font,
     string string,
     StringFlags flags,
     int xpos,
     int ypos,
     PaintBlock block,
     OS.Trfm trfm,
     int length);

  [CCode (cname = "xfont_caret")]
  public static OS.Error* XCaret
    (OS.ColourNumber colour_number,
     int height,
     StringFlags unit_flags,
     int x,
     int y);

  [CCode (cname = "font_caret")]
  public static void Caret
    (OS.ColourNumber colour_number,
     int height,
     StringFlags unit_flags,
     int x,
     int y);

  [CCode (cname = "xfont_convertto_os")]
  public static OS.Error* XConverttoOS
    (int x,
     int y,
     out int x_out,
     out int y_out);

  [CCode (cname = "font_convertto_os")]
  public static void ConverttoOS
    (int x,
     int y,
     out int x_out,
     out int y_out);

  [CCode (cname = "xfont_converttopoints")]
  public static OS.Error* XConverttopoints
    (int x,
     int y,
     out int x_out,
     out int y_out);

  [CCode (cname = "font_converttopoints")]
  public static void Converttopoints
    (int x,
     int y,
     out int x_out,
     out int y_out);

  [CCode (cname = "xfont_set_font")]
  public static OS.Error* XSetFont
    (F font);

  [CCode (cname = "font_set_font")]
  public static void SetFont
    (F font);

  [CCode (cname = "xfont_current_font")]
  public static OS.Error* XCurrentFont
    (out F font,
     out OS.GCOL bg_hint,
     out OS.GCOL fill,
     out int offset);

  [CCode (cname = "font_current_font")]
  public static F CurrentFont
    (out OS.GCOL bg_hint,
     out OS.GCOL fill,
     out int offset);

  [CCode (cname = "xfont_future_font")]
  public static OS.Error* XFutureFont
    (out F font,
     out OS.GCOL bg_hint,
     out OS.GCOL fill,
     out int offset);

  [CCode (cname = "font_future_font")]
  public static F FutureFont
    (out OS.GCOL bg_hint,
     out OS.GCOL fill,
     out int offset);

  [CCode (cname = "xfont_find_caret")]
  public static OS.Error* XFindCaret
    (string s,
     int x,
     int y,
     out char* split_point,
     out int x_out,
     out int y_out,
     out int length_out,
     out int index_out);

  [CCode (cname = "font_find_caret")]
  public static void FindCaret
    (string s,
     int x,
     int y,
     out char* split_point,
     out int x_out,
     out int y_out,
     out int length_out,
     out int index_out);

  [CCode (cname = "xfont_char_bbox")]
  public static OS.Error* XCharBBox
    (F font,
     char c,
     StringFlags unit_flags,
     out int x0,
     out int y0,
     out int x1,
     out int y1);

  [CCode (cname = "font_char_bbox")]
  public static void CharBBox
    (F font,
     char c,
     StringFlags unit_flags,
     out int x0,
     out int y0,
     out int x1,
     out int y1);

  [CCode (cname = "xfont_read_scale_factor")]
  public static OS.Error* XReadScaleFactor
    (out int xfactor,
     out int yfactor);

  [CCode (cname = "font_read_scale_factor")]
  public static void ReadScaleFactor
    (out int xfactor,
     out int yfactor);

  [CCode (cname = "xfont_set_scale_factor")]
  public static OS.Error* XSetScaleFactor
    (int xfactor,
     int yfactor);

  [CCode (cname = "font_set_scale_factor")]
  public static void SetScaleFactor
    (int xfactor,
     int yfactor);

  [CCode (cname = "xfont_list_fonts")]
  public static OS.Error* XListFonts
    (char* buffer1,
     ListContext context,
     int size1,
     char* buffer2,
     int size2,
     string tick_font,
     out ListContext context_out,
     out int used1,
     out int used2);

  [CCode (cname = "font_list_fonts")]
  public static ListContext ListFonts
    (char* buffer1,
     ListContext context,
     int size1,
     char* buffer2,
     int size2,
     string tick_font,
     out int used1,
     out int used2);

  [CCode (cname = "xfont_set_font_colours")]
  public static OS.Error* XSetFontColours
    (F font,
     OS.GCOL bg_hint,
     OS.GCOL fill,
     int offset);

  [CCode (cname = "font_set_font_colours")]
  public static void SetFontColours
    (F font,
     OS.GCOL bg_hint,
     OS.GCOL fill,
     int offset);

  [CCode (cname = "xfont_set_palette")]
  public static OS.Error* XSetPalette
    (OS.GCOL bg_hint,
     OS.GCOL fill,
     int offset,
     OS.Colour bg_hint_out,
     OS.Colour fill_out);

  [CCode (cname = "font_set_palette")]
  public static void SetPalette
    (OS.GCOL bg_hint,
     OS.GCOL fill,
     int offset,
     OS.Colour bg_hint_out,
     OS.Colour fill_out);

  [CCode (cname = "xfont_set_true_palette")]
  public static OS.Error* XSetTruePalette
    (OS.GCOL bg_hint,
     OS.GCOL fill,
     int offset,
     OS.Colour bg_hint_out,
     OS.Colour fill_out);

  [CCode (cname = "font_set_true_palette")]
  public static void SetTruePalette
    (OS.GCOL bg_hint,
     OS.GCOL fill,
     int offset,
     OS.Colour bg_hint_out,
     OS.Colour fill_out);

  [CCode (cname = "xfont_read_thresholds")]
  public static OS.Error* XReadThresholds
    (Thresholds buffer);

  [CCode (cname = "font_read_thresholds")]
  public static void ReadThresholds
    (Thresholds buffer);

  [CCode (cname = "xfont_set_thresholds")]
  public static OS.Error* XSetThresholds
    (Thresholds thresholds);

  [CCode (cname = "font_set_thresholds")]
  public static void SetThresholds
    (Thresholds thresholds);

  [CCode (cname = "xfont_find_caretj")]
  public static OS.Error* XFindCaretJ
    (string s,
     int x,
     int y,
     int xjust,
     int yjust,
     out char* split_point,
     out int x_out,
     out int y_out,
     out int length_out,
     out int index_out);

  [CCode (cname = "font_find_caretj")]
  public static void FindCaretJ
    (string s,
     int x,
     int y,
     int xjust,
     int yjust,
     out char* split_point,
     out int x_out,
     out int y_out,
     out int length_out,
     out int index_out);

  [CCode (cname = "xfont_string_bbox")]
  public static OS.Error* XStringBBox
    (string s,
     out int x0,
     out int y0,
     out int x1,
     out int y1);

  [CCode (cname = "font_string_bbox")]
  public static void StringBBox
    (string s,
     out int x0,
     out int y0,
     out int x1,
     out int y1);

  [CCode (cname = "xfont_read_colour_table")]
  public static OS.Error* XReadColourTable
    (ColourTable colour_table);

  [CCode (cname = "font_read_colour_table")]
  public static void ReadColourTable
    (ColourTable colour_table);

  [CCode (cname = "xfont_make_bitmap")]
  public static OS.Error* XMakeBitmap
    (string font_name,
     int xsize,
     int ysize,
     int xres,
     int yres,
     BitmapType flags);

  [CCode (cname = "font_make_bitmap")]
  public static void MakeBitmap
    (string font_name,
     int xsize,
     int ysize,
     int xres,
     int yres,
     BitmapType flags);

  [CCode (cname = "xfont_un_cache_file")]
  public static OS.Error* XUnCacheFile
    (string file_name,
     bool recache);

  [CCode (cname = "font_un_cache_file")]
  public static void UnCacheFile
    (string file_name,
     bool recache);

  [CCode (cname = "xfont_set_font_max")]
  public static OS.Error* XSetFontMax
    (int font_max,
     int font_max1,
     int font_max2,
     int font_max3,
     int font_max4,
     int font_max5);

  [CCode (cname = "font_set_font_max")]
  public static void SetFontMax
    (int font_max,
     int font_max1,
     int font_max2,
     int font_max3,
     int font_max4,
     int font_max5);

  [CCode (cname = "xfont_read_font_max")]
  public static OS.Error* XReadFontMax
    (out int font_max,
     out int font_max1,
     out int font_max2,
     out int font_max3,
     out int font_max4,
     out int font_max5);

  [CCode (cname = "font_read_font_max")]
  public static void ReadFontMax
    (out int font_max,
     out int font_max1,
     out int font_max2,
     out int font_max3,
     out int font_max4,
     out int font_max5);

  [CCode (cname = "xfont_read_font_prefix")]
  public static OS.Error* XReadFontPrefix
    (F font,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "font_read_font_prefix")]
  public static char ReadFontPrefix
    (F font,
     string buffer,
     int size);

  [CCode (cname = "xfont_switch_output_to_buffer")]
  public static OS.Error* XSwitchOutputToBuffer
    (OutputFlags flags,
     char* buffer,
     out char* end);

  [CCode (cname = "font_switch_output_to_buffer")]
  public static char SwitchOutputToBuffer
    (OutputFlags flags,
     char* buffer);

  [CCode (cname = "xfont_switch_output_to_buffer_flags")]
  public static OS.Error* XSwitchOutputToBufferFlags
    (out OutputFlags flags);

  [CCode (cname = "font_switch_output_to_buffer_flags")]
  public static OutputFlags SwitchOutputToBufferFlags ();

  [CCode (cname = "xfont_read_font_metrics")]
  public static OS.Error* XReadFontMetrics
    (F font,
     BBoxInfo bbox_info,
     WidthInfo xwidth_info,
     WidthInfo ywidth_info,
     MetricsMiscInfo misc_info,
     KernInfo kern_info,
     out MetricFlags flags,
     out int bbox_info_size,
     out int xwidth_info_size,
     out int ywidth_info_size,
     out int misc_info_size,
     out int kern_info_size);

  [CCode (cname = "font_read_font_metrics")]
  public static void ReadFontMetrics
    (F font,
     BBoxInfo bbox_info,
     WidthInfo xwidth_info,
     WidthInfo ywidth_info,
     MetricsMiscInfo misc_info,
     KernInfo kern_info,
     out MetricFlags flags,
     out int bbox_info_size,
     out int xwidth_info_size,
     out int ywidth_info_size,
     out int misc_info_size,
     out int kern_info_size);

  [CCode (cname = "xfont_read_font_metrics_ucs")]
  public static OS.Error* XReadFontMetricsUCS
    (F font,
     MetricsMiscInfo misc_info,
     KernInfoUCS kern_info,
     out MetricFlags flags,
     out int misc_info_size,
     out int kern_info_size);

  [CCode (cname = "font_read_font_metrics_ucs")]
  public static void ReadFontMetricsUCS
    (F font,
     MetricsMiscInfo misc_info,
     KernInfoUCS kern_info,
     out MetricFlags flags,
     out int misc_info_size,
     out int kern_info_size);

  [CCode (cname = "xfont_decode_menu")]
  public static OS.Error* XDecodeMenu
    (DecodeFlags flags,
     char* menu,
     char* block,
     char* buffer,
     int size,
     out int* next_selection,
     out int used);

  [CCode (cname = "font_decode_menu")]
  public static void DecodeMenu
    (DecodeFlags flags,
     char* menu,
     char* block,
     char* buffer,
     int size,
     out int* next_selection,
     out int used);

  [CCode (cname = "xfont_scan_string")]
  public static OS.Error* XScanString
    (F font,
     string s,
     StringFlags flags,
     int x,
     int y,
     ScanBlock block,
     OS.Trfm trfm,
     int length,
     out char* split_point,
     out int x_out,
     out int y_out,
     out int num_split_chars);

  [CCode (cname = "font_scan_string")]
  public static void ScanString
    (F font,
     string s,
     StringFlags flags,
     int x,
     int y,
     ScanBlock block,
     OS.Trfm trfm,
     int length,
     out char* split_point,
     out int x_out,
     out int y_out,
     out int num_split_chars);

  [CCode (cname = "xfont_current_rgb")]
  public static OS.Error* XCurrentRGB
    (out F font,
     out OS.Colour bg_hint,
     out OS.Colour fill,
     out int offset);

  [CCode (cname = "font_current_rgb")]
  public static F CurrentRGB
    (out OS.Colour bg_hint,
     out OS.Colour fill,
     out int offset);

  [CCode (cname = "xfont_future_rgb")]
  public static OS.Error* XFutureRGB
    (out F font,
     out OS.Colour bg_hint,
     out OS.Colour fill,
     out int offset);

  [CCode (cname = "font_future_rgb")]
  public static F FutureRGB
    (out OS.Colour bg_hint,
     out OS.Colour fill,
     out int offset);

  [CCode (cname = "xfont_read_encoding_filename")]
  public static OS.Error* XReadEncodingFilename
    (F font,
     string buffer,
     int size,
     out char* end);

  [CCode (cname = "font_read_encoding_filename")]
  public static char ReadEncodingFilename
    (F font,
     string buffer,
     int size);

  [CCode (cname = "xfont_find_field")]
  public static OS.Error* XFindField
    (string identifier,
     char qualifier,
     out string field,
     out bool found);

  [CCode (cname = "font_find_field")]
  public static bool FindField
    (string identifier,
     char qualifier,
     out string field);

  [CCode (cname = "xfont_apply_fields")]
  public static OS.Error* XApplyFields
    (string identifier,
     string fields,
     string buffer,
     int size,
     out int spare);

  [CCode (cname = "font_apply_fields")]
  public static int ApplyFields
    (string identifier,
     string fields,
     string buffer,
     int size);

  [CCode (cname = "xfont_lookup_font")]
  public static OS.Error* XLookupFont
    (F font,
     out CharacteristicFlags flags);

  [CCode (cname = "font_lookup_font")]
  public static CharacteristicFlags LookupFont
    (F font);

  [CCode (cname = "xfont_enumerate_characters")]
  public static OS.Error* XEnumerateCharacters
    (F font,
     int character,
     out int next_character,
     out int internal_character_code);

  [CCode (cname = "font_enumerate_characters")]
  public static void EnumerateCharacters
    (F font,
     int character,
     out int next_character,
     out int internal_character_code);

  [CCode (cname = "xservice_fonts_changed")]
  public static OS.Error* XService_FontsChanged ();

  [CCode (cname = "service_fonts_changed")]
  public static void Service_FontsChanged ();

}
