/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Toolbox;

[CCode (cheader_filename = "oslib/gadget.h")]
namespace OSLib.Gadget
{
  [CCode (cname = "gadget_flags")]
  public struct Flags : bits
  {
  }

  [CCode (cname = "gadget_object_base")]
  public struct ObjectBase
  {
    [CCode (cname = "flags")]
    public unowned OSLib.Gadget.Flags Flags;
    [CCode (cname = "class_no_and_size")]
    public unowned bits ClassNoAndSize;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "cmp")]
    public unowned Toolbox.C Cmp;
    [CCode (cname = "help_message")]
    public unowned Toolbox.MsgReference HelpMessage;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
  }

  [Compact]
  [CCode (cname = "gadget_object", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Object
  {
    public Object(int numEntries);
    [CCode (cname = "flags")]
    public unowned OSLib.Gadget.Flags Flags;
    [CCode (cname = "class_no_and_size")]
    public unowned bits ClassNoAndSize;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "cmp")]
    public unowned Toolbox.C Cmp;
    [CCode (cname = "help_message")]
    public unowned Toolbox.MsgReference HelpMessage;
    [CCode (cname = "help_limit")]
    public unowned int HelpLimit;
    [CCode (cname = "gadget", array_length = false)]
    public unowned int[] Gadget;
  }

  [CCode (cname = "gadget_feature")]
  public enum Feature
  {
    [CCode (cname = "gadget_FEATURE_ADD")]
    Add,
    [CCode (cname = "gadget_FEATURE_REMOVE")]
    Remove,
    [CCode (cname = "gadget_FEATURE_POST_ADD")]
    PostAdd,
    [CCode (cname = "gadget_FEATURE_METHOD")]
    Method,
    [CCode (cname = "gadget_FEATURE_CLICK")]
    Click,
    [CCode (cname = "gadget_FEATURE_PLOT")]
    Plot,
    [CCode (cname = "gadget_FEATURE_SET_FOCUS")]
    SetFocus,
    [CCode (cname = "gadget_FEATURE_MOVE")]
    Move,
    [CCode (cname = "gadget_FEATURE_FADE")]
    Fade
  }

  [CCode (cname = "gadget_extension")]
  public struct Extension
  {
    [CCode (cname = "type")]
    public unowned int Type;
    [CCode (cname = "valid_flags")]
    public unowned OSLib.Gadget.Flags ValidFlags;
    [CCode (cname = "features")]
    public unowned Feature Features;
  }

  [CCode (cname = "gadget_extension_list")]
  public struct ExtensionList
  {
    [CCode (cname = "gadget", array_length = false)]
    public unowned Extension[] Gadget;
  }

  [CCode (cname="gadget_FADED")]
  public const OSLib.Gadget.Flags Faded;
  [CCode (cname="gadget_AT_BACK")]
  public const OSLib.Gadget.Flags AtBack;
  [CCode (cname="gadget_NO_HANDLER")]
  public const bits NoHandler;
  [CCode (cname="gadget_DEFAULT_HANDLER")]
  public const bits DefaultHandler;
  [CCode (cname="gadget_PRIVATE_HANDLER")]
  public const bits PrivateHandler;
  [CCode (cname="gadget_FEATURE_ADD_SHIFT")]
  public const int FeatureAddShift;
  [CCode (cname="gadget_FEATURE_REMOVE_SHIFT")]
  public const int FeatureRemoveShift;
  [CCode (cname="gadget_FEATURE_POST_ADD_SHIFT")]
  public const int FeaturePostAddShift;
  [CCode (cname="gadget_FEATURE_METHOD_SHIFT")]
  public const int FeatureMethodShift;
  [CCode (cname="gadget_FEATURE_CLICK_SHIFT")]
  public const int FeatureClickShift;
  [CCode (cname="gadget_FEATURE_PLOT_SHIFT")]
  public const int FeaturePlotShift;
  [CCode (cname="gadget_FEATURE_SET_FOCUS_SHIFT")]
  public const int FeatureSetFocusShift;
  [CCode (cname="gadget_FEATURE_MOVE_SHIFT")]
  public const int FeatureMoveShift;
  [CCode (cname="gadget_FEATURE_FADE_SHIFT")]
  public const int FeatureFadeShift;

  [CCode (cname = "xgadget_get_flags")]
  public static OS.Error* XGetFlags
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     out OSLib.Gadget.Flags flags_out);

  [CCode (cname = "gadget_get_flags")]
  public static OSLib.Gadget.Flags GetFlags
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget);

  [CCode (cname = "xgadget_set_flags")]
  public static OS.Error* XSetFlags
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     OSLib.Gadget.Flags flags_in);

  [CCode (cname = "gadget_set_flags")]
  public static void SetFlags
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     OSLib.Gadget.Flags flags_in);

  [CCode (cname = "xgadget_set_help_message")]
  public static OS.Error* XSetHelpMessage
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     string help_message);

  [CCode (cname = "gadget_set_help_message")]
  public static void SetHelpMessage
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     string help_message);

  [CCode (cname = "xgadget_get_help_message")]
  public static OS.Error* XGetHelpMessage
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     string help_message,
     int size,
     out int used);

  [CCode (cname = "gadget_get_help_message")]
  public static int GetHelpMessage
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     string help_message,
     int size);

  [CCode (cname = "xgadget_get_icon_list")]
  public static OS.Error* XGetIconList
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     Wimp.I buffer,
     int size,
     out int used);

  [CCode (cname = "gadget_get_icon_list")]
  public static int GetIconList
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     Wimp.I buffer,
     int size);

  [CCode (cname = "xgadget_set_focus")]
  public static OS.Error* XSetFocus
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget);

  [CCode (cname = "gadget_set_focus")]
  public static void SetFocus
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget);

  [CCode (cname = "xgadget_get_type")]
  public static OS.Error* XGetType
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     out int type);

  [CCode (cname = "gadget_get_type")]
  public static int GetType
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget);

  [CCode (cname = "xgadget_move_gadget")]
  public static OS.Error* XMoveGadget
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     OS.Box bbox);

  [CCode (cname = "gadget_move_gadget")]
  public static void MoveGadget
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     OS.Box bbox);

  [CCode (cname = "xgadget_get_bbox")]
  public static OS.Error* XGetBBox
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     OS.Box bbox);

  [CCode (cname = "gadget_get_bbox")]
  public static void GetBBox
    (bits flags,
     Toolbox.O window,
     Toolbox.C gadget,
     OS.Box bbox);

}
