/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/joystick.h")]
namespace OSLib.Joystick
{
  [CCode (cname = "joystick_state")]
  public enum State
  {
    [CCode (cname = "joystick_X")]
    X,
    [CCode (cname = "joystick_Y")]
    Y,
    [CCode (cname = "joystick_SWITCHES")]
    Switches,
    [CCode (cname = "joystick_HALF_WORDX")]
    HalfWordX,
    [CCode (cname = "joystick_HALF_WORDY")]
    HalfWordY
  }

  [CCode (cname = "joystick_half_word_state")]
  public enum HalfWordState
  {
    Default = 0
  }

  [CCode (cname="joystick_XSHIFT")]
  public const int XShift;
  [CCode (cname="joystick_YSHIFT")]
  public const int YShift;
  [CCode (cname="joystick_SWITCHES_SHIFT")]
  public const int SwitchesShift;
  [CCode (cname="joystick_HALF_WORD_XSHIFT")]
  public const int HalfWordXShift;
  [CCode (cname="joystick_HALF_WORD_YSHIFT")]
  public const int HalfWordYShift;

  [CCode (cname = "xjoystick_read")]
  public static OS.Error* XRead
    (int joystick_no,
     out State state);

  [CCode (cname = "joystick_read")]
  public static State Read
    (int joystick_no);

  [CCode (cname = "xjoystick_read_half_word")]
  public static OS.Error* XReadHalfWord
    (int joystick_no,
     out HalfWordState state,
     out bits switches);

  [CCode (cname = "joystick_read_half_word")]
  public static HalfWordState ReadHalfWord
    (int joystick_no,
     out bits switches);

  [CCode (cname = "xjoystick_calibrate_top_right")]
  public static OS.Error* XCalibrateTopRight ();

  [CCode (cname = "joystick_calibrate_top_right")]
  public static void CalibrateTopRight ();

  [CCode (cname = "xjoystick_calibrate_bottom_left")]
  public static OS.Error* XCalibrateBottomLeft ();

  [CCode (cname = "joystick_calibrate_bottom_left")]
  public static void CalibrateBottomLeft ();

}
