/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/lineeditor.h")]
namespace OSLib.LineEditor
{
  [CCode (cname = "lineeditor_h", has_type_id = false)]
  public struct H : bits
  {
  }

  [CCode (cname = "lineeditor_length")]
  public enum Length
  {
    [CCode (cname = "lineeditor_UNKNOWN")]
    Unknown,
    [CCode (cname = "lineeditor_NO_EDIT")]
    NoEdit
  }

  [CCode (cname = "lineeditor_pos")]
  public enum Pos
  {
    [CCode (cname = "lineeditor_END")]
    End,
    [CCode (cname = "lineeditor_PREVIOUS_WRAP")]
    PreviousWrap,
    [CCode (cname = "lineeditor_PREVIOUS_NO_WRAP")]
    PreviousNoWrap,
    [CCode (cname = "lineeditor_NEXT_NO_WRAP")]
    NextNoWrap,
    [CCode (cname = "lineeditor_NEXT_WRAP")]
    NextWrap
  }

  [CCode (cname = "lineeditor_cursors")]
  public enum Cursors
  {
    Default = 0
  }

  [CCode (cname = "lineeditorsetoptions_flags")]
  public enum LineEditorSetOptions_Flags
  {
    [CCode (cname = "lineeditor_LOCAL_TASKWINDOWS")]
    LocalTaskwindows,
    [CCode (cname = "lineeditor_LOCAL_APPEND_DOTS")]
    LocalAppendDots,
    [CCode (cname = "lineeditor_LOCAL_OVERTYPE")]
    LocalOvertype,
    [CCode (cname = "lineeditor_LOCAL_KEEP_DUPLICATES")]
    LocalKeepDuplicates,
    [CCode (cname = "lineeditor_LOCAL_NO_TILDE")]
    LocalNoTilde,
    [CCode (cname = "lineeditor_LOCAL_NO_EVENTS")]
    LocalNoEvents,
    [CCode (cname = "lineeditor_LOCAL_NO_CIRCUMFLEX")]
    LocalNoCircumflex
  }

  [CCode (cname="lineeditor_NORMAL_SHIFT")]
  public const int NormalShift;
  [CCode (cname="lineeditor_OVERTYPE_SHIFT")]
  public const int OvertypeShift;

  [CCode (cname = "xlineeditor_set_options")]
  public static OS.Error* XSetOptions
    (int size,
     LineEditorSetOptions_Flags flags,
     int min,
     int max,
     Cursors cursors);

  [CCode (cname = "lineeditor_set_options")]
  public static void SetOptions
    (int size,
     LineEditorSetOptions_Flags flags,
     int min,
     int max,
     Cursors cursors);

  [CCode (cname = "xlineeditor_get_options")]
  public static OS.Error* XGetOptions
    (out int size,
     out LineEditorSetOptions_Flags flags,
     out int min,
     out int max,
     out Cursors cursors);

  [CCode (cname = "lineeditor_get_options")]
  public static void GetOptions
    (out int size,
     out LineEditorSetOptions_Flags flags,
     out int min,
     out int max,
     out Cursors cursors);

  [CCode (cname = "xlineeditor_create_buffer")]
  public static OS.Error* XCreateBuffer
    (int size,
     Wimp.T task,
     bits expansion,
     out H handle);

  [CCode (cname = "lineeditor_create_buffer")]
  public static H CreateBuffer
    (int size,
     Wimp.T task,
     bits expansion);

  [CCode (cname = "xlineeditor_delete_buffer")]
  public static OS.Error* XDeleteBuffer
    (H handle);

  [CCode (cname = "lineeditor_delete_buffer")]
  public static void DeleteBuffer
    (H handle);

  [CCode (cname = "xlineeditor_set_pos")]
  public static OS.Error* XSetPos
    (H handle,
     Pos position);

  [CCode (cname = "lineeditor_set_pos")]
  public static void SetPos
    (H handle,
     Pos position);

  [CCode (cname = "xlineeditor_get_pos")]
  public static OS.Error* XGetPos
    (H handle,
     out Pos position);

  [CCode (cname = "lineeditor_get_pos")]
  public static Pos GetPos
    (H handle);

  [CCode (cname = "xlineeditor_get_line")]
  public static OS.Error* XGetLine
    (H handle,
     char* buffer,
     int length,
     out int used);

  [CCode (cname = "lineeditor_get_line")]
  public static int GetLine
    (H handle,
     char* buffer,
     int length);

  [CCode (cname = "xlineeditor_append_line")]
  public static OS.Error* XAppendLine
    (H handle,
     char* line);

  [CCode (cname = "lineeditor_append_line")]
  public static void AppendLine
    (H handle,
     char* line);

  [CCode (cname = "xlineeditor_delete_line")]
  public static OS.Error* XDeleteLine
    (H handle);

  [CCode (cname = "lineeditor_delete_line")]
  public static void DeleteLine
    (H handle);

  [CCode (cname = "xlineeditor_read_info")]
  public static OS.Error* XReadInfo
    (Wimp.T task_handle,
     out Length length,
     out int pos);

  [CCode (cname = "lineeditor_read_info")]
  public static int ReadInfo
    (Wimp.T task_handle,
     out Length length);

  [CCode (cname = "xlineeditor_read_line")]
  public static OS.Error* XReadLine
    (char* buffer,
     int size,
     char min_char,
     char max_char,
     H handle,
     out int used,
     out uint psr);

  [CCode (cname = "lineeditor_read_line")]
  public static bits ReadLine
    (char* buffer,
     int size,
     char min_char,
     char max_char,
     H handle,
     out int used);

  [CCode (cname = "xlineeditor_read_line_given_echo")]
  public static OS.Error* XReadLineGivenEcho
    (char* buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     H handle,
     out int used,
     out uint psr);

  [CCode (cname = "lineeditor_read_line_given_echo")]
  public static bits ReadLineGivenEcho
    (char* buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     H handle,
     out int used);

  [CCode (cname = "xlineeditor_read_line_suppress_invalid")]
  public static OS.Error* XReadLineSuppressInvalid
    (char* buffer,
     int size,
     char min_char,
     char max_char,
     H handle,
     out int used,
     out uint psr);

  [CCode (cname = "lineeditor_read_line_suppress_invalid")]
  public static bits ReadLineSuppressInvalid
    (char* buffer,
     int size,
     char min_char,
     char max_char,
     H handle,
     out int used);

  [CCode (cname = "xlineeditor_read_line_given_echo_suppress_invalid")]
  public static OS.Error* XReadLineGivenEchoSuppressInvalid
    (char* buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     H handle,
     out int used,
     out uint psr);

  [CCode (cname = "lineeditor_read_line_given_echo_suppress_invalid")]
  public static bits ReadLineGivenEchoSuppressInvalid
    (char* buffer,
     int size,
     char min_char,
     char max_char,
     char echo,
     H handle,
     out int used);

}
