/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/netfs.h")]
namespace OSLib.NetFS
{
  [CCode (cname = "netfs_fs_op")]
  public enum FSOp
  {
    [CCode (cname = "netfs_FS_OP_CLI")]
    CLI,
    [CCode (cname = "netfs_FS_OP_SAVE")]
    Save,
    [CCode (cname = "netfs_FS_OP_LOAD")]
    Load,
    [CCode (cname = "netfs_FS_OP_CAT")]
    Cat,
    [CCode (cname = "netfs_FS_OP_LOAD_AS")]
    LoadAs,
    [CCode (cname = "netfs_FS_OP_OPEN")]
    Open,
    [CCode (cname = "netfs_FS_OP_CLOSE")]
    Close,
    [CCode (cname = "netfs_FS_OP_BGET")]
    BGet,
    [CCode (cname = "netfs_FS_OP_BPUT")]
    BPut,
    [CCode (cname = "netfs_FS_OP_GBPB_READ")]
    GBPBRead,
    [CCode (cname = "netfs_FS_OP_GBPB_WRITE")]
    GBPBWrite,
    [CCode (cname = "netfs_FS_OP_READ_ARGS")]
    ReadArgs,
    [CCode (cname = "netfs_FS_OP_SET_ARGS")]
    SetArgs,
    [CCode (cname = "netfs_FS_OP_READ_DISC_INFO")]
    ReadDiscInfo,
    [CCode (cname = "netfs_FS_OP_READ_LOGON_INFO")]
    ReadLogonInfo,
    [CCode (cname = "netfs_FS_OP_READ_DATE_AND_TIME")]
    ReadDateAndTime,
    [CCode (cname = "netfs_FS_OP_READ_EOF_STATUS")]
    ReadEOFStatus,
    [CCode (cname = "netfs_FS_OP_READ_OBJECT")]
    ReadObject,
    [CCode (cname = "netfs_FS_OP_SET_OBJECT")]
    SetObject,
    [CCode (cname = "netfs_FS_OP_DELETE_OBJECT")]
    DeleteObject,
    [CCode (cname = "netfs_FS_OP_READ_ENV")]
    ReadEnv,
    [CCode (cname = "netfs_FS_OP_OPT")]
    Opt,
    [CCode (cname = "netfs_FS_OP_LOGOFF")]
    Logoff,
    [CCode (cname = "netfs_FS_OP_READ_USER_INFO")]
    ReadUserInfo,
    [CCode (cname = "netfs_FS_OP_READ_VERSION")]
    ReadVersion,
    [CCode (cname = "netfs_FS_OP_READ_FREE")]
    ReadFree,
    [CCode (cname = "netfs_FS_OP_CREATE_DIR")]
    CreateDir,
    [CCode (cname = "netfs_FS_OP_SET_DATE_AND_TIME")]
    SetDateAndTime,
    [CCode (cname = "netfs_FS_OP_CREATE")]
    Create,
    [CCode (cname = "netfs_FS_OP_READ_USER_FREE")]
    ReadUserFree,
    [CCode (cname = "netfs_FS_OP_SET_USER_FREE")]
    SetUserFree,
    [CCode (cname = "netfs_FS_OP_READ_USER_NAME")]
    ReadUserName,
    [CCode (cname = "netfs_FS_OP_READ_LOGON_INFO_EXTENDED")]
    ReadLogonInfoExtended,
    [CCode (cname = "netfs_FS_OP_READ_USER_INFO_EXTENDED")]
    ReadUserInfoExtended,
    [CCode (cname = "netfs_FS_OP_MANAGER_OP")]
    ManagerOp
  }

  [CCode (cname = "netfs_fs_op_ex_type")]
  public enum FSOpExType
  {
    [CCode (cname = "netfs_FS_OP_EX")]
    FSOpEx,
    [CCode (cname = "netfs_FS_OP_EX_TEXT")]
    FSOpExText,
    [CCode (cname = "netfs_FS_OP_EX_TITLE")]
    FSOpExTitle,
    [CCode (cname = "netfs_FS_OP_EX_TITLE_TEXT")]
    FSOpExTitleText
  }

  [CCode (cname = "netfs_fs_op_args_type")]
  public enum FSOpArgsType
  {
    [CCode (cname = "netfs_FS_OP_ARGS_PTR")]
    FSOpArgsPtr,
    [CCode (cname = "netfs_FS_OP_ARGS_EXT")]
    FSOpArgsExt,
    [CCode (cname = "netfs_FS_OP_ARGS_SIZE")]
    FSOpArgsSize
  }

  [CCode (cname = "netfs_fs_op_read_object_type")]
  public enum FSOpReadObjectType
  {
    [CCode (cname = "netfs_FS_OP_READ_OBJECT_STAMP")]
    FSOpReadObjectStamp,
    [CCode (cname = "netfs_FS_OP_READ_OBJECT_LOAD_AND_EXEC_ADDR")]
    FSOpReadObjectLoadAndExecAddr,
    [CCode (cname = "netfs_FS_OP_READ_OBJECT_SIZE")]
    FSOpReadObjectSize,
    [CCode (cname = "netfs_FS_OP_READ_OBJECT_ATTR")]
    FSOpReadObjectAttr,
    [CCode (cname = "netfs_FS_OP_READ_OBJECT_ALL_INFO")]
    FSOpReadObjectAllInfo,
    [CCode (cname = "netfs_FS_OP_READ_OBJECT_ATTR_AND_CYCLE_NO")]
    FSOpReadObjectAttrAndCycleNo,
    [CCode (cname = "netfs_FS_OP_READ_OBJECT_SIN")]
    FSOpReadObjectSIN
  }

  [CCode (cname = "netfs_fs_op_set_object_type")]
  public enum FSOpSetObjectType
  {
    [CCode (cname = "netfs_FS_OP_SET_OBJECT_LOAD_EXEC_AND_ATTR")]
    FSOpSetObjectLoadExecAndAttr,
    [CCode (cname = "netfs_FS_OP_SET_OBJECT_LOAD_ADDR")]
    FSOpSetObjectLoadAddr,
    [CCode (cname = "netfs_FS_OP_SET_OBJECT_EXEC_ADDR")]
    FSOpSetObjectExecAddr,
    [CCode (cname = "netfs_FS_OP_SET_OBJECT_ATTR")]
    FSOpSetObjectAttr,
    [CCode (cname = "netfs_FS_OP_SET_OBJECT_STAMP")]
    FSOpSetObjectStamp
  }

  [CCode (cname = "netfs_fs_op_manager_op_type")]
  public enum FSOpManagerOpType
  {
    [CCode (cname = "netfs_FS_OP_MANAGER_OP_READ_PASSWORD_COUNT")]
    FSOpManagerOpReadPasswordCount,
    [CCode (cname = "netfs_FS_OP_MANAGER_OP_READ_PASSWORD")]
    FSOpManagerOpReadPassword,
    [CCode (cname = "netfs_FS_OP_MANAGER_OP_WRITE_PASSWORD")]
    FSOpManagerOpWritePassword,
    [CCode (cname = "netfs_FS_OP_MANAGER_OP_ADD_USER")]
    FSOpManagerOpAddUser,
    [CCode (cname = "netfs_FS_OP_MANAGER_OP_REMOVE_USER")]
    FSOpManagerOpRemoveUser,
    [CCode (cname = "netfs_FS_OP_MANAGER_OP_SET_PRIVILEGE")]
    FSOpManagerOpSetPrivilege,
    [CCode (cname = "netfs_FS_OP_MANAGER_OP_LOGOFF_USER")]
    FSOpManagerOpLogoffUser,
    [CCode (cname = "netfs_FS_OP_MANAGER_OP_SHUTDOWN")]
    FSOpManagerOpShutdown
  }

  [CCode (cname = "netfs_cli_request")]
  public struct CLIRequest
  {
    [CCode (cname = "command")]
    public unowned char Command[256];
  }

  [CCode (cname = "netfs_save_request")]
  public struct SaveRequest
  {
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned byte Size[3];
    [CCode (cname = "file_name")]
    public unowned char FileName[256];
  }

  [CCode (cname = "netfs_save_reply")]
  public struct SaveReply
  {
    [CCode (cname = "port")]
    public unowned byte Port;
    [CCode (cname = "block_limit")]
    public unowned byte BlockLimit[2];
    [CCode (cname = "leaf_name")]
    public unowned char LeafName[10];
  }

  [CCode (cname = "netfs_save_final_reply")]
  public struct SaveFinalReply
  {
    [CCode (cname = "attr")]
    public unowned byte Attr;
    [CCode (cname = "date")]
    public unowned byte Date[2];
  }

  [CCode (cname = "netfs_load_request")]
  public struct LoadRequest
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[256];
  }

  [CCode (cname = "netfs_load_reply")]
  public struct LoadReply
  {
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned byte Size[3];
    [CCode (cname = "attr")]
    public unowned byte Attr;
    [CCode (cname = "date")]
    public unowned byte Date[2];
    [CCode (cname = "file_name")]
    public unowned char FileName[256];
  }

  [CCode (cname = "netfs_ex_request")]
  public struct ExRequest
  {
    [CCode (cname = "ex_type")]
    public unowned FSOpExType ExType;
    [CCode (cname = "entry")]
    public unowned byte Entry;
    [CCode (cname = "count")]
    public unowned byte Count;
    [CCode (cname = "dir_name")]
    public unowned char DirName[256];
  }

  [CCode (cname = "netfs_cat_request")]
  public struct CatRequest
  {
    [CCode (cname = "dir_name")]
    public unowned char DirName[256];
  }

  [CCode (cname = "netfs_cat_reply")]
  public struct CatReply
  {
    [CCode (cname = "leaf_name")]
    public unowned char LeafName[10];
    [CCode (cname = "reserved1")]
    public unowned char Reserved1;
    [CCode (cname = "owner")]
    public unowned char Owner;
    [CCode (cname = "reserved2")]
    public unowned byte Reserved2[3];
    [CCode (cname = "disc_name")]
    public unowned char DiscName[16];
  }

  [CCode (cname = "netfs_open_request")]
  public struct OpenRequest
  {
    [CCode (cname = "error_if_absent")]
    public unowned byte ErrorIfAbsent;
    [CCode (cname = "read_only")]
    public unowned byte ReadOnly;
    [CCode (cname = "file_name")]
    public unowned char FileName[256];
  }

  [CCode (cname = "netfs_open_reply")]
  public struct OpenReply
  {
    [CCode (cname = "handle")]
    public unowned byte Handle;
    [CCode (cname = "leaf_name")]
    public unowned char LeafName[10];
  }

  [CCode (cname = "netfs_close_request")]
  public struct CloseRequest
  {
    [CCode (cname = "handle")]
    public unowned byte Handle;
  }

  [CCode (cname = "netfs_bget_request")]
  public struct BGetRequest
  {
    [CCode (cname = "handle")]
    public unowned byte Handle;
  }

  [CCode (cname = "netfs_bget_reply")]
  public struct BGetReply
  {
    [CCode (cname = "b")]
    public unowned byte B;
    [CCode (cname = "terminator")]
    public unowned byte Terminator;
  }

  [CCode (cname = "netfs_bput_request")]
  public struct BPutRequest
  {
    [CCode (cname = "handle")]
    public unowned byte Handle;
    [CCode (cname = "b")]
    public unowned byte B;
  }

  [CCode (cname = "netfs_gbpb_request")]
  public struct GBPBRequest
  {
    [CCode (cname = "handle")]
    public unowned byte Handle;
    [CCode (cname = "read_next")]
    public unowned byte ReadNext;
    [CCode (cname = "size")]
    public unowned byte Size[3];
    [CCode (cname = "ptr")]
    public unowned byte Ptr[3];
  }

  [CCode (cname = "netfs_gbpb_reply")]
  public struct GBPBReply
  {
    [CCode (cname = "final_block")]
    public unowned byte FinalBlock;
    [CCode (cname = "size")]
    public unowned byte Size[3];
  }

  [CCode (cname = "netfs_read_args_request")]
  public struct ReadArgsRequest
  {
    [CCode (cname = "args_type")]
    public unowned FSOpArgsType ArgsType;
  }

  [CCode (cname = "netfs_read_args_reply")]
  public struct ReadArgsReply
  {
    [CCode (cname = "arg")]
    public unowned byte Arg[3];
  }

  [CCode (cname = "netfs_set_args_request")]
  public struct SetArgsRequest
  {
    [CCode (cname = "args_type")]
    public unowned FSOpArgsType ArgsType;
    [CCode (cname = "arg")]
    public unowned byte Arg[3];
  }

  [CCode (cname = "netfs_read_disc_info_request")]
  public struct ReadDiscInfoRequest
  {
    [CCode (cname = "first")]
    public unowned byte First;
    [CCode (cname = "count")]
    public unowned byte Count;
  }

  [CCode (cname = "netfs_disc_info")]
  public struct DiscInfo
  {
    [CCode (cname = "phys_no")]
    public unowned byte PhysNo;
    [CCode (cname = "disc_name")]
    public unowned char DiscName[16];
  }

  [CCode (cname = "netfs_read_disc_info_reply_base")]
  public struct ReadDiscInfoReplyBase
  {
    [CCode (cname = "count")]
    public unowned byte Count;
  }

  [Compact]
  [CCode (cname = "netfs_read_disc_info_reply", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ReadDiscInfoReply
  {
    public ReadDiscInfoReply(int numEntries);
    [CCode (cname = "count")]
    public unowned byte Count;
    [CCode (cname = "info", array_length = false)]
    public unowned DiscInfo[] Info;
  }

  [CCode (cname = "netfs_read_logon_info_request")]
  public struct ReadLogonInfoRequest
  {
    [CCode (cname = "first")]
    public unowned byte First;
    [CCode (cname = "count")]
    public unowned byte Count;
  }

  [CCode (cname = "netfs_logon_info")]
  public struct LogonInfo
  {
    [CCode (cname = "station")]
    public unowned byte Station;
    [CCode (cname = "net")]
    public unowned byte Net;
    [CCode (cname = "user_name")]
    public unowned byte UserName[22];
    [CCode (cname = "privilege")]
    public unowned byte Privilege;
  }

  [CCode (cname = "netfs_read_logon_info_reply_base")]
  public struct ReadLogonInfoReplyBase
  {
    [CCode (cname = "count")]
    public unowned byte Count;
  }

  [Compact]
  [CCode (cname = "netfs_read_logon_info_reply", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ReadLogonInfoReply
  {
    public ReadLogonInfoReply(int numEntries);
    [CCode (cname = "count")]
    public unowned byte Count;
    [CCode (cname = "info", array_length = false)]
    public unowned LogonInfo[] Info;
  }

  [CCode (cname = "netfs_read_date_and_time_reply")]
  public struct ReadDateAndTimeReply
  {
    [CCode (cname = "date")]
    public unowned byte Date[2];
    [CCode (cname = "hr")]
    public unowned byte Hr;
    [CCode (cname = "min")]
    public unowned byte Min;
    [CCode (cname = "s")]
    public unowned byte S;
  }

  [CCode (cname = "netfs_read_eof_status_request")]
  public struct ReadEOFStatusRequest
  {
    [CCode (cname = "handle")]
    public unowned byte Handle;
  }

  [CCode (cname = "netfs_read_eof_status_reply")]
  public struct ReadEOFStatusReply
  {
    [CCode (cname = "eof_status")]
    public unowned byte EofStatus;
  }

  [CCode (cname = "netfs_read_object_request")]
  public struct ReadObjectRequest
  {
    [CCode (cname = "read_type")]
    public unowned FSOpReadObjectType ReadType;
    [CCode (cname = "file_name")]
    public unowned char FileName[256];
  }

  [CCode (cname = "netfs_delete_object_request")]
  public struct DeleteObjectRequest
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[256];
  }

  [CCode (cname = "netfs_delete_object_reply")]
  public struct DeleteObjectReply
  {
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned byte Size[3];
    [CCode (cname = "attr")]
    public unowned byte Attr;
  }

  [CCode (cname = "netfs_read_env_reply")]
  public struct ReadEnvReply
  {
    [CCode (cname = "len")]
    public unowned byte Len;
    [CCode (cname = "disc_name")]
    public unowned char DiscName[16];
    [CCode (cname = "csd_name")]
    public unowned char CsdName[10];
    [CCode (cname = "lib_name")]
    public unowned char LibName[10];
  }

  [CCode (cname = "netfs_opt_request")]
  public struct OptRequest
  {
    [CCode (cname = "opt")]
    public unowned byte Opt;
  }

  [CCode (cname = "netfs_read_user_info_request")]
  public struct ReadUserInfoRequest
  {
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
  }

  [CCode (cname = "netfs_read_user_info_reply")]
  public struct ReadUserInfoReply
  {
    [CCode (cname = "privilege")]
    public unowned byte Privilege;
    [CCode (cname = "station")]
    public unowned byte Station;
    [CCode (cname = "net")]
    public unowned byte Net;
  }

  [CCode (cname = "netfs_read_version_reply")]
  public struct ReadVersionReply
  {
    [CCode (cname = "type")]
    public unowned char Type[10];
    [CCode (cname = "version")]
    public unowned char Version[4];
  }

  [CCode (cname = "netfs_read_free_request")]
  public struct ReadFreeRequest
  {
    [CCode (cname = "disc_name")]
    public unowned char DiscName[16];
  }

  [CCode (cname = "netfs_read_free_reply")]
  public struct ReadFreeReply
  {
    [CCode (cname = "free_sector_count")]
    public unowned byte FreeSectorCount[3];
    [CCode (cname = "sector_count")]
    public unowned byte SectorCount[3];
  }

  [CCode (cname = "netfs_create_dir_request")]
  public struct CreateDirRequest
  {
    [CCode (cname = "sector_count")]
    public unowned byte SectorCount;
    [CCode (cname = "dir_name")]
    public unowned char DirName[256];
  }

  [CCode (cname = "netfs_set_date_and_time_request")]
  public struct SetDateAndTimeRequest
  {
    [CCode (cname = "date")]
    public unowned byte Date[2];
    [CCode (cname = "hr")]
    public unowned byte Hr;
    [CCode (cname = "min")]
    public unowned byte Min;
    [CCode (cname = "s")]
    public unowned byte S;
  }

  [CCode (cname = "netfs_create_request")]
  public struct CreateRequest
  {
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned byte Size[3];
    [CCode (cname = "file_name")]
    public unowned char FileName[256];
  }

  [CCode (cname = "netfs_create_reply")]
  public struct CreateReply
  {
    [CCode (cname = "attr")]
    public unowned byte Attr;
    [CCode (cname = "date")]
    public unowned byte Date[2];
  }

  [CCode (cname = "netfs_read_user_free_request")]
  public struct ReadUserFreeRequest
  {
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
  }

  [CCode (cname = "netfs_read_user_free_reply")]
  public struct ReadUserFreeReply
  {
    [CCode (cname = "user_free")]
    public unowned byte UserFree[3];
  }

  [CCode (cname = "netfs_set_user_free_request")]
  public struct SetUserFreeRequest
  {
    [CCode (cname = "user_free")]
    public unowned byte UserFree[3];
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
  }

  [CCode (cname = "netfs_read_user_name_reply")]
  public struct ReadUserNameReply
  {
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
  }

  [CCode (cname = "netfs_logon_info_extended")]
  public struct LogonInfoExtended
  {
    [CCode (cname = "station")]
    public unowned byte Station;
    [CCode (cname = "net")]
    public unowned byte Net;
    [CCode (cname = "task")]
    public unowned byte Task;
    [CCode (cname = "user_name")]
    public unowned byte UserName[22];
    [CCode (cname = "privilege")]
    public unowned byte Privilege;
  }

  [CCode (cname = "netfs_read_logon_info_extended_request")]
  public struct ReadLogonInfoExtendedRequest
  {
    [CCode (cname = "first")]
    public unowned byte First;
    [CCode (cname = "count")]
    public unowned byte Count;
  }

  [CCode (cname = "netfs_read_logon_info_extended_reply_base")]
  public struct ReadLogonInfoExtendedReplyBase
  {
    [CCode (cname = "count")]
    public unowned byte Count;
  }

  [Compact]
  [CCode (cname = "netfs_read_logon_info_extended_reply", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ReadLogonInfoExtendedReply
  {
    public ReadLogonInfoExtendedReply(int numEntries);
    [CCode (cname = "count")]
    public unowned byte Count;
    [CCode (cname = "info", array_length = false)]
    public unowned LogonInfoExtended[] Info;
  }

  [CCode (cname = "netfs_read_user_info_extended_request")]
  public struct ReadUserInfoExtendedRequest
  {
    [CCode (cname = "user_name")]
    public unowned char UserName;
  }

  [CCode (cname = "netfs_read_user_info_extended_reply")]
  public struct ReadUserInfoExtendedReply
  {
    [CCode (cname = "privilege")]
    public unowned byte Privilege;
    [CCode (cname = "station")]
    public unowned byte Station;
    [CCode (cname = "net")]
    public unowned byte Net;
    [CCode (cname = "task")]
    public unowned byte Task;
  }

  [CCode (cname = "netfs_cli_info_reply")]
  public struct CLIInfoReply
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[10];
    [CCode (cname = "reserved1")]
    public unowned char Reserved1;
    [CCode (cname = "load_addr")]
    public unowned char LoadAddr[9];
    [CCode (cname = "exec_addr")]
    public unowned char ExecAddr[9];
    [CCode (cname = "reserved2")]
    public unowned char Reserved2[2];
    [CCode (cname = "length")]
    public unowned char Length[7];
    [CCode (cname = "reserved3")]
    public unowned char Reserved3[2];
    [CCode (cname = "access")]
    public unowned char Access[7];
    [CCode (cname = "reserved4")]
    public unowned char Reserved4[4];
    [CCode (cname = "date")]
    public unowned char Date[9];
    [CCode (cname = "sin")]
    public unowned char Sin[6];
    [CCode (cname = "terminator")]
    public unowned byte Terminator;
  }

  [CCode (cname = "netfs_clii_am_reply")]
  public struct CLIIAmReply
  {
    [CCode (cname = "urd")]
    public unowned byte Urd;
    [CCode (cname = "csd")]
    public unowned byte Csd;
    [CCode (cname = "lib")]
    public unowned byte Lib;
    [CCode (cname = "opt")]
    public unowned byte Opt;
  }

  [CCode (cname = "netfs_clis_disc_reply")]
  public struct CLISDiscReply
  {
    [CCode (cname = "urd")]
    public unowned byte Urd;
    [CCode (cname = "csd")]
    public unowned byte Csd;
    [CCode (cname = "lib")]
    public unowned byte Lib;
  }

  [CCode (cname = "netfs_cli_dir_reply")]
  public struct CLIDirReply
  {
    [CCode (cname = "csd")]
    public unowned byte Csd;
  }

  [CCode (cname = "netfs_cli_unknown_reply")]
  public struct CLIUnknownReply
  {
    [CCode (cname = "command")]
    public unowned char Command[256];
  }

  [CCode (cname = "netfs_cli_lib_reply")]
  public struct CLILibReply
  {
    [CCode (cname = "lib")]
    public unowned byte Lib;
  }

  [CCode (cname = "netfs_ex_reply")]
  public struct ExReply
  {
    [CCode (cname = "ex_count")]
    public unowned byte ExCount;
    [CCode (cname = "dir_count")]
    public unowned byte DirCount;
    [CCode (cname = "name")]
    public unowned char Name[10];
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "attr")]
    public unowned byte Attr;
    [CCode (cname = "date")]
    public unowned byte Date[2];
    [CCode (cname = "sin")]
    public unowned byte Sin[3];
    [CCode (cname = "size")]
    public unowned byte Size[3];
  }

  [CCode (cname = "netfs_ex_text_reply")]
  public struct ExTextReply
  {
    [CCode (cname = "ex_count")]
    public unowned byte ExCount;
    [CCode (cname = "dir_count")]
    public unowned byte DirCount;
    [CCode (cname = "text")]
    public unowned char Text[1];
  }

  [CCode (cname = "netfs_ex_title_reply")]
  public struct ExTitleReply
  {
    [CCode (cname = "ex_count")]
    public unowned byte ExCount;
    [CCode (cname = "dir_count")]
    public unowned byte DirCount;
    [CCode (cname = "len")]
    public unowned byte Len;
    [CCode (cname = "name")]
    public unowned char Name[10];
  }

  [CCode (cname = "netfs_ex_title_text_reply")]
  public struct ExTitleTextReply
  {
    [CCode (cname = "ex_count")]
    public unowned byte ExCount;
    [CCode (cname = "dir_count")]
    public unowned byte DirCount;
    [CCode (cname = "text")]
    public unowned char Text[1];
  }

  [CCode (cname = "netfs_read_object_stamp_reply")]
  public struct ReadObjectStampReply
  {
    [CCode (cname = "date")]
    public unowned byte Date[2];
  }

  [CCode (cname = "netfs_read_object_load_and_exec_addr_reply")]
  public struct ReadObjectLoadAndExecAddrReply
  {
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
  }

  [CCode (cname = "netfs_read_object_size_reply")]
  public struct ReadObjectSizeReply
  {
    [CCode (cname = "size")]
    public unowned byte Size[3];
  }

  [CCode (cname = "netfs_read_object_attr_reply")]
  public struct ReadObjectAttrReply
  {
    [CCode (cname = "attr")]
    public unowned byte Attr;
  }

  [CCode (cname = "netfs_read_object_all_info_reply")]
  public struct ReadObjectAllInfoReply
  {
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "size")]
    public unowned byte Size[3];
    [CCode (cname = "attr")]
    public unowned byte Attr;
    [CCode (cname = "date")]
    public unowned byte Date[2];
    [CCode (cname = "access")]
    public unowned byte Access[2];
  }

  [CCode (cname = "netfs_read_object_attr_and_cycle_no_reply")]
  public struct ReadObjectAttrAndCycleNoReply
  {
    [CCode (cname = "reserved")]
    public unowned byte Reserved[2];
    [CCode (cname = "len")]
    public unowned byte Len;
    [CCode (cname = "dir_name")]
    public unowned byte DirName[10];
    [CCode (cname = "access")]
    public unowned byte Access;
    [CCode (cname = "count")]
    public unowned byte Count;
  }

  [CCode (cname = "netfs_read_object_sin_reply")]
  public struct ReadObjectSINReply
  {
    [CCode (cname = "obj_type")]
    public unowned byte ObjType;
    [CCode (cname = "sin")]
    public unowned byte Sin[6];
  }

  [CCode (cname = "netfs_set_object_load_exec_and_attr_request")]
  public struct SetObjectLoadExecAndAttrRequest
  {
    [CCode (cname = "set_type")]
    public unowned FSOpSetObjectType SetType;
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
    [CCode (cname = "attr")]
    public unowned byte Attr;
  }

  [CCode (cname = "netfs_set_object_load_addr_request")]
  public struct SetObjectLoadAddrRequest
  {
    [CCode (cname = "set_type")]
    public unowned FSOpSetObjectType SetType;
    [CCode (cname = "load_addr")]
    public unowned bits LoadAddr;
  }

  [CCode (cname = "netfs_set_object_exec_addr_request")]
  public struct SetObjectExecAddrRequest
  {
    [CCode (cname = "set_type")]
    public unowned FSOpSetObjectType SetType;
    [CCode (cname = "exec_addr")]
    public unowned bits ExecAddr;
  }

  [CCode (cname = "netfs_set_object_attr_request")]
  public struct SetObjectAttrRequest
  {
    [CCode (cname = "set_type")]
    public unowned FSOpSetObjectType SetType;
    [CCode (cname = "attr")]
    public unowned byte Attr;
  }

  [CCode (cname = "netfs_set_object_stamp_request")]
  public struct SetObjectStampRequest
  {
    [CCode (cname = "set_type")]
    public unowned FSOpSetObjectType SetType;
    [CCode (cname = "date")]
    public unowned byte Date[2];
  }

  [CCode (cname = "netfs_manager_op_read_password_count_request")]
  public struct ManagerOpReadPasswordCountRequest
  {
    [CCode (cname = "req_no")]
    public unowned FSOpManagerOpType ReqNo;
  }

  [CCode (cname = "netfs_manager_op_read_password_count_reply")]
  public struct ManagerOpReadPasswordCountReply
  {
    [CCode (cname = "user_count")]
    public unowned int UserCount;
  }

  [CCode (cname = "netfs_manager_op_read_password_request")]
  public struct ManagerOpReadPasswordRequest
  {
    [CCode (cname = "req_no")]
    public unowned FSOpManagerOpType ReqNo;
    [CCode (cname = "user_no")]
    public unowned byte UserNo[2];
  }

  [CCode (cname = "netfs_manager_op_read_password_reply")]
  public struct ManagerOpReadPasswordReply
  {
    [CCode (cname = "index")]
    public unowned int Index;
    [CCode (cname = "privilege")]
    public unowned byte Privilege;
    [CCode (cname = "opt")]
    public unowned byte Opt;
    [CCode (cname = "reserved")]
    public unowned char Reserved[4];
    [CCode (cname = "station")]
    public unowned byte Station;
    [CCode (cname = "net")]
    public unowned byte Net;
    [CCode (cname = "logon_permitted")]
    public unowned byte LogonPermitted;
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
    [CCode (cname = "password")]
    public unowned char Password[23];
    [CCode (cname = "urd_name")]
    public unowned char UrdName[10];
  }

  [CCode (cname = "netfs_manager_op_write_password_request")]
  public struct ManagerOpWritePasswordRequest
  {
    [CCode (cname = "req_no")]
    public unowned FSOpManagerOpType ReqNo;
    [CCode (cname = "reserved1")]
    public unowned char Reserved1[4];
    [CCode (cname = "privilege")]
    public unowned byte Privilege;
    [CCode (cname = "opt")]
    public unowned byte Opt;
    [CCode (cname = "reserved2")]
    public unowned byte Reserved2[6];
    [CCode (cname = "logon_permitted")]
    public unowned byte LogonPermitted;
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
    [CCode (cname = "password")]
    public unowned char Password[23];
    [CCode (cname = "urd_name")]
    public unowned char UrdName[10];
  }

  [CCode (cname = "netfs_manager_op_add_user_request")]
  public struct ManagerOpAddUserRequest
  {
    [CCode (cname = "req_no")]
    public unowned FSOpManagerOpType ReqNo;
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
  }

  [CCode (cname = "netfs_manager_op_remove_user_request")]
  public struct ManagerOpRemoveUserRequest
  {
    [CCode (cname = "req_no")]
    public unowned FSOpManagerOpType ReqNo;
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
  }

  [CCode (cname = "netfs_manager_op_set_privilege_request")]
  public struct ManagerOpSetPrivilegeRequest
  {
    [CCode (cname = "req_no")]
    public unowned FSOpManagerOpType ReqNo;
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
    [CCode (cname = "privilege")]
    public unowned byte Privilege;
  }

  [CCode (cname = "netfs_manager_op_logoff_user_request")]
  public struct ManagerOpLogoffUserRequest
  {
    [CCode (cname = "req_no")]
    public unowned FSOpManagerOpType ReqNo;
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
  }

  [CCode (cname = "netfs_cli")]
  public struct CLI
  {
    [CCode (cname = "request")]
    public unowned CLIRequest Request;
    [CCode (cname = "info_reply")]
    public unowned CLIInfoReply InfoReply;
    [CCode (cname = "i_am_reply")]
    public unowned CLIIAmReply IAmReply;
    [CCode (cname = "s_disc_reply")]
    public unowned CLISDiscReply SDiscReply;
    [CCode (cname = "dir_reply")]
    public unowned CLIDirReply DirReply;
    [CCode (cname = "unknown_reply")]
    public unowned CLIUnknownReply UnknownReply;
    [CCode (cname = "lib_reply")]
    public unowned CLILibReply LibReply;
  }

  [CCode (cname = "netfs_save")]
  public struct Save
  {
    [CCode (cname = "request")]
    public unowned SaveRequest Request;
    [CCode (cname = "reply")]
    public unowned SaveReply Reply;
    [CCode (cname = "final_reply")]
    public unowned SaveFinalReply FinalReply;
  }

  [CCode (cname = "netfs_load")]
  public struct Load
  {
    [CCode (cname = "request")]
    public unowned LoadRequest Request;
    [CCode (cname = "reply")]
    public unowned LoadReply Reply;
  }

  [CCode (cname = "netfs_ex")]
  public struct Ex
  {
    [CCode (cname = "request")]
    public unowned ExRequest Request;
    [CCode (cname = "reply")]
    public unowned ExReply Reply;
    [CCode (cname = "text_reply")]
    public unowned ExTextReply TextReply;
    [CCode (cname = "title_reply")]
    public unowned ExTitleReply TitleReply;
    [CCode (cname = "title_text_reply")]
    public unowned ExTitleTextReply TitleTextReply;
  }

  [CCode (cname = "netfs_cat")]
  public struct Cat
  {
    [CCode (cname = "request")]
    public unowned CatRequest Request;
    [CCode (cname = "reply")]
    public unowned CatReply Reply;
  }

  [CCode (cname = "netfs_open")]
  public struct Open
  {
    [CCode (cname = "request")]
    public unowned OpenRequest Request;
    [CCode (cname = "reply")]
    public unowned OpenReply Reply;
  }

  [CCode (cname = "netfs_close")]
  public struct Close
  {
    [CCode (cname = "request")]
    public unowned CloseRequest Request;
  }

  [CCode (cname = "netfs_bget")]
  public struct BGet
  {
    [CCode (cname = "request")]
    public unowned BGetRequest Request;
    [CCode (cname = "reply")]
    public unowned BGetReply Reply;
  }

  [CCode (cname = "netfs_bput")]
  public struct BPut
  {
    [CCode (cname = "request")]
    public unowned BPutRequest Request;
  }

  [CCode (cname = "netfs_gbpb")]
  public struct GBPB
  {
    [CCode (cname = "request")]
    public unowned GBPBRequest Request;
    [CCode (cname = "reply")]
    public unowned GBPBReply Reply;
  }

  [CCode (cname = "netfs_read_args")]
  public struct ReadArgs
  {
    [CCode (cname = "request")]
    public unowned ReadArgsRequest Request;
    [CCode (cname = "reply")]
    public unowned ReadArgsReply Reply;
  }

  [CCode (cname = "netfs_set_args")]
  public struct SetArgs
  {
    [CCode (cname = "request")]
    public unowned SetArgsRequest Request;
  }

  [CCode (cname = "netfs_read_disc_info")]
  public struct ReadDiscInfo
  {
    [CCode (cname = "request")]
    public unowned ReadDiscInfoRequest Request;
    [CCode (cname = "reply")]
    public unowned ReadDiscInfoReply Reply;
  }

  [CCode (cname = "netfs_read_logon_info")]
  public struct ReadLogonInfo
  {
    [CCode (cname = "request")]
    public unowned ReadLogonInfoRequest Request;
    [CCode (cname = "reply")]
    public unowned ReadLogonInfoReply Reply;
  }

  [CCode (cname = "netfs_read_date_and_time")]
  public struct ReadDateAndTime
  {
    [CCode (cname = "reply")]
    public unowned ReadDateAndTimeReply Reply;
  }

  [CCode (cname = "netfs_read_eof_status")]
  public struct ReadEOFStatus
  {
    [CCode (cname = "request")]
    public unowned ReadEOFStatusRequest Request;
    [CCode (cname = "reply")]
    public unowned ReadEOFStatusReply Reply;
  }

  [CCode (cname = "netfs_read_object")]
  public struct ReadObject
  {
    [CCode (cname = "request")]
    public unowned ReadObjectRequest Request;
    [CCode (cname = "stamp_reply")]
    public unowned ReadObjectStampReply StampReply;
    [CCode (cname = "load_and_exec_addr_reply")]
    public unowned ReadObjectLoadAndExecAddrReply LoadAndExecAddrReply;
    [CCode (cname = "size_reply")]
    public unowned ReadObjectSizeReply SizeReply;
    [CCode (cname = "attr_reply")]
    public unowned ReadObjectAttrReply AttrReply;
    [CCode (cname = "all_info_reply")]
    public unowned ReadObjectAllInfoReply AllInfoReply;
    [CCode (cname = "attr_and_cycle_no_reply")]
    public unowned ReadObjectAttrAndCycleNoReply AttrAndCycleNoReply;
    [CCode (cname = "sin_reply")]
    public unowned ReadObjectSINReply SinReply;
  }

  [CCode (cname = "netfs_set_object")]
  public struct SetObject
  {
    [CCode (cname = "load_exec_and_attr_request")]
    public unowned SetObjectLoadExecAndAttrRequest LoadExecAndAttrRequest;
    [CCode (cname = "load_addr_request")]
    public unowned SetObjectLoadAddrRequest LoadAddrRequest;
    [CCode (cname = "exec_addr_request")]
    public unowned SetObjectExecAddrRequest ExecAddrRequest;
    [CCode (cname = "attr_request")]
    public unowned SetObjectAttrRequest AttrRequest;
    [CCode (cname = "stamp_request")]
    public unowned SetObjectStampRequest StampRequest;
  }

  [CCode (cname = "netfs_delete_object")]
  public struct DeleteObject
  {
    [CCode (cname = "request")]
    public unowned DeleteObjectRequest Request;
    [CCode (cname = "reply")]
    public unowned DeleteObjectReply Reply;
  }

  [CCode (cname = "netfs_read_env")]
  public struct ReadEnv
  {
    [CCode (cname = "reply")]
    public unowned ReadEnvReply Reply;
  }

  [CCode (cname = "netfs_opt")]
  public struct Opt
  {
    [CCode (cname = "request")]
    public unowned OptRequest Request;
  }

  [CCode (cname = "netfs_read_user_info")]
  public struct ReadUserInfo
  {
    [CCode (cname = "request")]
    public unowned ReadUserInfoRequest Request;
    [CCode (cname = "reply")]
    public unowned ReadUserInfoReply Reply;
  }

  [CCode (cname = "netfs_read_version")]
  public struct ReadVersion
  {
    [CCode (cname = "reply")]
    public unowned ReadVersionReply Reply;
  }

  [CCode (cname = "netfs_read_free")]
  public struct ReadFree
  {
    [CCode (cname = "request")]
    public unowned ReadFreeRequest Request;
    [CCode (cname = "reply")]
    public unowned ReadFreeReply Reply;
  }

  [CCode (cname = "netfs_create_dir")]
  public struct CreateDir
  {
    [CCode (cname = "request")]
    public unowned CreateDirRequest Request;
  }

  [CCode (cname = "netfs_set_date_and_time")]
  public struct SetDateAndTime
  {
    [CCode (cname = "request")]
    public unowned SetDateAndTimeRequest Request;
  }

  [CCode (cname = "netfs_create")]
  public struct Create
  {
    [CCode (cname = "request")]
    public unowned CreateRequest Request;
    [CCode (cname = "reply")]
    public unowned CreateReply Reply;
  }

  [CCode (cname = "netfs_read_user_free")]
  public struct ReadUserFree
  {
    [CCode (cname = "request")]
    public unowned ReadUserFreeRequest Request;
    [CCode (cname = "reply")]
    public unowned ReadUserFreeReply Reply;
  }

  [CCode (cname = "netfs_set_user_free")]
  public struct SetUserFree
  {
    [CCode (cname = "request")]
    public unowned SetUserFreeRequest Request;
  }

  [CCode (cname = "netfs_read_user_name")]
  public struct ReadUserName
  {
    [CCode (cname = "reply")]
    public unowned ReadUserNameReply Reply;
  }

  [CCode (cname = "netfs_read_logon_info_extended")]
  public struct ReadLogonInfoExtended
  {
    [CCode (cname = "request")]
    public unowned ReadLogonInfoExtendedRequest Request;
    [CCode (cname = "reply")]
    public unowned ReadLogonInfoExtendedReply Reply;
  }

  [CCode (cname = "netfs_read_user_info_extended")]
  public struct ReadUserInfoExtended
  {
    [CCode (cname = "request")]
    public unowned ReadUserInfoExtendedRequest Request;
    [CCode (cname = "reply")]
    public unowned ReadUserInfoExtendedReply Reply;
  }

  [CCode (cname = "netfs_manager_op_read_password_count")]
  public struct ManagerOpReadPasswordCount
  {
    [CCode (cname = "request")]
    public unowned ManagerOpReadPasswordCountRequest Request;
    [CCode (cname = "reply")]
    public unowned ManagerOpReadPasswordCountReply Reply;
  }

  [CCode (cname = "netfs_manager_op_read_password")]
  public struct ManagerOpReadPassword
  {
    [CCode (cname = "request")]
    public unowned ManagerOpReadPasswordRequest Request;
    [CCode (cname = "reply")]
    public unowned ManagerOpReadPasswordReply Reply;
  }

  [CCode (cname = "netfs_manager_op_write_password")]
  public struct ManagerOpWritePassword
  {
    [CCode (cname = "request")]
    public unowned ManagerOpWritePasswordRequest Request;
  }

  [CCode (cname = "netfs_manager_op_add_user")]
  public struct ManagerOpAddUser
  {
    [CCode (cname = "request")]
    public unowned ManagerOpAddUserRequest Request;
  }

  [CCode (cname = "netfs_manager_op_remove_user")]
  public struct ManagerOpRemoveUser
  {
    [CCode (cname = "request")]
    public unowned ManagerOpRemoveUserRequest Request;
  }

  [CCode (cname = "netfs_manager_op_set_privilege")]
  public struct ManagerOpSetPrivilege
  {
    [CCode (cname = "request")]
    public unowned ManagerOpSetPrivilegeRequest Request;
  }

  [CCode (cname = "netfs_manager_op_logoff_user")]
  public struct ManagerOpLogoffUser
  {
    [CCode (cname = "request")]
    public unowned ManagerOpLogoffUserRequest Request;
  }

  [CCode (cname = "netfs_op")]
  public struct Op
  {
    [CCode (cname = "cli")]
    public unowned CLI Cli;
    [CCode (cname = "save")]
    public unowned Save Save;
    [CCode (cname = "load")]
    public unowned Load Load;
    [CCode (cname = "ex")]
    public unowned Ex Ex;
    [CCode (cname = "cat")]
    public unowned Cat Cat;
    [CCode (cname = "open")]
    public unowned Open Open;
    [CCode (cname = "close")]
    public unowned Close Close;
    [CCode (cname = "bget")]
    public unowned BGet Bget;
    [CCode (cname = "bput")]
    public unowned BPut Bput;
    [CCode (cname = "gbpb")]
    public unowned GBPB Gbpb;
    [CCode (cname = "read_args")]
    public unowned ReadArgs ReadArgs;
    [CCode (cname = "set_args")]
    public unowned SetArgs SetArgs;
    [CCode (cname = "read_disc_info")]
    public unowned ReadDiscInfo ReadDiscInfo;
    [CCode (cname = "read_logon_info")]
    public unowned ReadLogonInfo ReadLogonInfo;
    [CCode (cname = "read_date_and_time")]
    public unowned ReadDateAndTime ReadDateAndTime;
    [CCode (cname = "read_eof_status")]
    public unowned ReadEOFStatus ReadEofStatus;
    [CCode (cname = "read_object")]
    public unowned ReadObject ReadObject;
    [CCode (cname = "set_object")]
    public unowned SetObject SetObject;
    [CCode (cname = "delete_object")]
    public unowned DeleteObject DeleteObject;
    [CCode (cname = "read_env")]
    public unowned ReadEnv ReadEnv;
    [CCode (cname = "opt")]
    public unowned Opt Opt;
    [CCode (cname = "read_user_info")]
    public unowned ReadUserInfo ReadUserInfo;
    [CCode (cname = "read_version")]
    public unowned ReadVersion ReadVersion;
    [CCode (cname = "read_free")]
    public unowned ReadFree ReadFree;
    [CCode (cname = "create_dir")]
    public unowned CreateDir CreateDir;
    [CCode (cname = "set_date_and_time")]
    public unowned SetDateAndTime SetDateAndTime;
    [CCode (cname = "create")]
    public unowned Create Create;
    [CCode (cname = "read_user_free")]
    public unowned ReadUserFree ReadUserFree;
    [CCode (cname = "set_user_free")]
    public unowned SetUserFree SetUserFree;
    [CCode (cname = "read_user_name")]
    public unowned ReadUserName ReadUserName;
    [CCode (cname = "read_logon_info_extended")]
    public unowned ReadLogonInfoExtended ReadLogonInfoExtended;
    [CCode (cname = "read_user_info_extended")]
    public unowned ReadUserInfoExtended ReadUserInfoExtended;
    [CCode (cname = "manager_op_read_password_count")]
    public unowned ManagerOpReadPasswordCount ManagerOpReadPasswordCount;
    [CCode (cname = "manager_op_read_password")]
    public unowned ManagerOpReadPassword ManagerOpReadPassword;
    [CCode (cname = "manager_op_write_password")]
    public unowned ManagerOpWritePassword ManagerOpWritePassword;
    [CCode (cname = "manager_op_add_user")]
    public unowned ManagerOpAddUser ManagerOpAddUser;
    [CCode (cname = "manager_op_remove_user")]
    public unowned ManagerOpRemoveUser ManagerOpRemoveUser;
    [CCode (cname = "manager_op_set_privilege")]
    public unowned ManagerOpSetPrivilege ManagerOpSetPrivilege;
    [CCode (cname = "manager_op_logoff_user")]
    public unowned ManagerOpLogoffUser ManagerOpLogoffUser;
  }

  [CCode (cname = "netfs_file_server")]
  public struct FileServer
  {
    [CCode (cname = "station_no")]
    public unowned byte StationNo;
    [CCode (cname = "net_no")]
    public unowned byte NetNo;
    [CCode (cname = "drive")]
    public unowned byte Drive;
    [CCode (cname = "disc_name")]
    public unowned char DiscName[16];
    [CCode (cname = "reserved")]
    public unowned byte Reserved;
  }

  [CCode (cname = "netfs_date_and_time")]
  public enum DateAndTime
  {
    Default = 0
  }

  [CCode (cname = "netfs_file_server_context")]
  public struct FileServerContext
  {
    [CCode (cname = "station_no")]
    public unowned byte StationNo;
    [CCode (cname = "net_no")]
    public unowned byte NetNo;
    [CCode (cname = "reserved1")]
    public unowned byte Reserved1;
    [CCode (cname = "disc_name")]
    public unowned char DiscName[16];
    [CCode (cname = "reserved2")]
    public unowned byte Reserved2;
    [CCode (cname = "user_name")]
    public unowned char UserName[22];
    [CCode (cname = "reserved3")]
    public unowned byte Reserved3;
    [CCode (cname = "reserved4")]
    public unowned byte Reserved4;
  }

  [CCode (cname="netfs_LEAF_NAME_LIMIT")]
  public const int LeafNameLimit;
  [CCode (cname="netfs_DISC_NAME_LIMIT")]
  public const int DiscNameLimit;
  [CCode (cname="netfs_USER_NAME_LIMIT")]
  public const int UserNameLimit;
  [CCode (cname="netfs_PASSWORD_LIMIT")]
  public const int PasswordLimit;
  [CCode (cname="netfs_FILE_NAME_LIMIT")]
  public const int FileNameLimit;
  [CCode (cname="netfs_CLI_LIMIT")]
  public const int CLILimit;
  [CCode (cname="error_NET_FS_BAD_NAME")]
  public const bits Error_NetFSBadName;
  [CCode (cname="error_NET_FS_BAD_COMMAND_CODE")]
  public const bits Error_NetFSBadCommandCode;
  [CCode (cname="error_NET_FS_UNEXPECTED_COMMAND_CODE")]
  public const bits Error_NetFSUnexpectedCommandCode;
  [CCode (cname="error_NET_FS_UNKNOWN_FUNCTION_CODE")]
  public const bits Error_NetFSUnknownFunctionCode;
  [CCode (cname="error_NET_FS_UNKNOWN_STATION_NAME")]
  public const bits Error_NetFSUnknownStationName;
  [CCode (cname="error_NET_FS_UNKNOWN_STATION_NUMBER")]
  public const bits Error_NetFSUnknownStationNumber;
  [CCode (cname="error_NET_FS_STATION_NOT_FOUND")]
  public const bits Error_NetFSStationNotFound;
  [CCode (cname="error_NET_FS_FILE_SERVER_NAME_TOO_LONG")]
  public const bits Error_NetFSFileServerNameTooLong;
  [CCode (cname="error_NET_FS_BAD_FILE_SERVER_DATE")]
  public const bits Error_NetFSBadFileServerDate;
  [CCode (cname="error_NET_FS_NET_FS_INTERNAL_ERROR")]
  public const bits Error_NetFSNetFSInternalError;
  [CCode (cname="error_NET_FS_FILE_SERVER_NOT_CAPABLE")]
  public const bits Error_NetFSFileServerNotCapable;
  [CCode (cname="error_NET_FS_BROADCAST_SERVER_DEAD")]
  public const bits Error_NetFSBroadcastServerDead;
  [CCode (cname="error_NET_FS_FILE_SERVER_ONLY24_BIT")]
  public const bits Error_NetFSFileServerOnly24Bit;
  [CCode (cname="error_NET_UTILS_WRONG_VERSION")]
  public const bits Error_NetUtilsWrongVersion;
  [CCode (cname="error_NET_UTILS_NET_FS_NO_GO")]
  public const bits Error_NetUtilsNetFSNoGo;
  [CCode (cname="error_NET_UTILS_IS_THREADED")]
  public const bits Error_NetUtilsIsThreaded;
  [CCode (cname="error_NET_FS_SET_FREE_SYNTAX")]
  public const bits Error_NetFSSetFreeSyntax;
  [CCode (cname="error_NET_FS_FS_CLI_SYNTAX")]
  public const bits Error_NetFSFsCLISyntax;
  [CCode (cname="netfs_FS_OP_CLI_COMPLETE")]
  public const byte FSOpCLIComplete;
  [CCode (cname="netfs_FS_OP_CLI_INFO")]
  public const byte FSOpCLIInfo;
  [CCode (cname="netfs_FS_OP_CLII_AM")]
  public const byte FSOpCLIIAm;
  [CCode (cname="netfs_FS_OP_CLIS_DISC")]
  public const byte FSOpCLISDisc;
  [CCode (cname="netfs_FS_OP_CLI_DIR")]
  public const byte FSOpCLIDir;
  [CCode (cname="netfs_FS_OP_CLI_UNKNOWN")]
  public const byte FSOpCLIUnknown;
  [CCode (cname="netfs_FS_OP_CLI_LIB")]
  public const byte FSOpCLILib;
  [CCode (cname="netfs_FS_OP_CLI_DISC_INFO")]
  public const byte FSOpCLIDiscInfo;
  [CCode (cname="netfs_FS_OP_CLI_USER_INFO")]
  public const byte FSOpCLIUserInfo;
  [CCode (cname="netfs_NO_MORE")]
  public const int NoMore;

  [CCode (cname = "xnetfs_read_fs_number")]
  public static OS.Error* XReadFSNumber
    (out byte station_no,
     out byte net_no);

  [CCode (cname = "netfs_read_fs_number")]
  public static void ReadFSNumber
    (out byte station_no,
     out byte net_no);

  [CCode (cname = "xnetfs_set_fs_number")]
  public static OS.Error* XSetFSNumber
    (byte station_no,
     byte net_no);

  [CCode (cname = "netfs_set_fs_number")]
  public static void SetFSNumber
    (byte station_no,
     byte net_no);

  [CCode (cname = "xnetfs_read_fs_name")]
  public static OS.Error* XReadFSName
    (string buffer,
     int size,
     out char* end);

  [CCode (cname = "netfs_read_fs_name")]
  public static char ReadFSName
    (string buffer,
     int size);

  [CCode (cname = "xnetfs_set_fs_name")]
  public static OS.Error* XSetFSName
    (string fs_name);

  [CCode (cname = "netfs_set_fs_name")]
  public static void SetFSName
    (string fs_name);

  [CCode (cname = "xnetfs_read_fs_timeouts")]
  public static OS.Error* XReadFSTimeouts
    (out int transmit_count,
     out int transmit_delay,
     out int peek_count,
     out int peek_delay,
     out int receive_delay,
     out int broadcast_delay);

  [CCode (cname = "netfs_read_fs_timeouts")]
  public static void ReadFSTimeouts
    (out int transmit_count,
     out int transmit_delay,
     out int peek_count,
     out int peek_delay,
     out int receive_delay,
     out int broadcast_delay);

  [CCode (cname = "xnetfs_set_fs_timeouts")]
  public static OS.Error* XSetFSTimeouts
    (int transmit_count,
     int transmit_delay,
     int peek_count,
     int peek_delay,
     int receive_delay,
     int broadcast_delay);

  [CCode (cname = "netfs_set_fs_timeouts")]
  public static void SetFSTimeouts
    (int transmit_count,
     int transmit_delay,
     int peek_count,
     int peek_delay,
     int receive_delay,
     int broadcast_delay);

  [CCode (cname = "xnetfs_do_fs_op")]
  public static OS.Error* XDoFSOp
    (FSOp fs_op,
     Op op,
     int tx_count,
     int size,
     out int status,
     out int rx_count);

  [CCode (cname = "netfs_do_fs_op")]
  public static int DoFSOp
    (FSOp fs_op,
     Op op,
     int tx_count,
     int size,
     out int rx_count);

  [CCode (cname = "xnetfs_enumerate_fs_list")]
  public static OS.Error* XEnumerateFSList
    (int context,
     FileServer file_server,
     int size,
     int server_limit,
     out int context_out,
     out int server_count);

  [CCode (cname = "netfs_enumerate_fs_list")]
  public static int EnumerateFSList
    (int context,
     FileServer file_server,
     int size,
     int server_limit,
     out int server_count);

  [CCode (cname = "xnetfs_enumerate_fs")]
  public static OS.Error* XEnumerateFS
    (int context,
     FileServer file_server,
     int size,
     int server_limit,
     out int context_out,
     out int server_count);

  [CCode (cname = "netfs_enumerate_fs")]
  public static int EnumerateFS
    (int context,
     FileServer file_server,
     int size,
     int server_limit,
     out int server_count);

  [CCode (cname = "xnetfs_convert_date")]
  public static OS.Error* XConvertDate
    (DateAndTime n,
     OS.DateAndTime o);

  [CCode (cname = "netfs_convert_date")]
  public static void ConvertDate
    (DateAndTime n,
     OS.DateAndTime o);

  [CCode (cname = "xnetfs_do_fs_op_to_given_fs")]
  public static OS.Error* XDoFSOpToGivenFS
    (FSOp fs_op,
     Op op,
     int tx_count,
     int size,
     byte station,
     byte net,
     out int status,
     out int rx_count);

  [CCode (cname = "netfs_do_fs_op_to_given_fs")]
  public static int DoFSOpToGivenFS
    (FSOp fs_op,
     Op op,
     int tx_count,
     int size,
     byte station,
     byte net,
     out int rx_count);

  [CCode (cname = "xnetfs_update_fs_list")]
  public static OS.Error* XUpdateFSList
    (byte station_no,
     byte net_no);

  [CCode (cname = "netfs_update_fs_list")]
  public static void UpdateFSList
    (byte station_no,
     byte net_no);

  [CCode (cname = "xnetfs_enumerate_fs_contexts")]
  public static OS.Error* XEnumerateFSContexts
    (int context,
     FileServerContext file_server_context,
     int size,
     int server_limit,
     out int context_out,
     out int server_count);

  [CCode (cname = "netfs_enumerate_fs_contexts")]
  public static int EnumerateFSContexts
    (int context,
     FileServerContext file_server_context,
     int size,
     int server_limit,
     out int server_count);

  [CCode (cname = "xnetfs_read_user_id")]
  public static OS.Error* XReadUserId
    (string buffer,
     int size,
     out char* end);

  [CCode (cname = "netfs_read_user_id")]
  public static char ReadUserId
    (string buffer,
     int size);

  [CCode (cname = "xnetfs_get_object_uid")]
  public static OS.Error* XGetObjectUID
    (string path_name,
     string special,
     out FileSwitch.ObjectType obj_type,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out int uid0,
     out int uid1);

  [CCode (cname = "netfs_get_object_uid")]
  public static FileSwitch.ObjectType GetObjectUID
    (string path_name,
     string special,
     out bits load_addr,
     out bits exec_addr,
     out int size,
     out FileSwitch.Attr attr,
     out int uid0,
     out int uid1);

  [CCode (cname = "xnetfs_enable_cache")]
  public static OS.Error* XEnableCache ();

  [CCode (cname = "netfs_enable_cache")]
  public static void EnableCache ();

  [CCode (cname = "xservice_net_fs")]
  public static OS.Error* XService_NetFS ();

  [CCode (cname = "service_net_fs")]
  public static void Service_NetFS ();

  [CCode (cname = "xservice_net_fs_dying")]
  public static OS.Error* XService_NetFSDying ();

  [CCode (cname = "service_net_fs_dying")]
  public static void Service_NetFSDying ();

}
