/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/osfind.h")]
namespace OSLib.OSFind
{
  [CCode (cname = "osfind_flags")]
  public enum Flags
  {
    [CCode (cname = "osfind_PATH")]
    Path,
    [CCode (cname = "osfind_PATH_VAR")]
    PathVar,
    [CCode (cname = "osfind_NO_PATH")]
    NoPath,
    [CCode (cname = "osfind_ERROR_IF_ABSENT")]
    ErrorIfAbsent,
    [CCode (cname = "osfind_ERROR_IF_DIR")]
    ErrorIfDir
  }


  [CCode (cname = "xosfind_openin")]
  public static OS.Error* XOpenin
    (Flags flags,
     string file_name,
     string path,
     out OS.F file);

  [CCode (cname = "osfind_openin")]
  public static OS.F Openin
    (Flags flags,
     string file_name,
     string path);

  [CCode (cname = "xosfind_openinw")]
  public static OS.Error* XOpeninW
    (Flags flags,
     string file_name,
     string path,
     out OS.FW file);

  [CCode (cname = "osfind_openinw")]
  public static OS.FW OpeninW
    (Flags flags,
     string file_name,
     string path);

  [CCode (cname = "xosfind_openout")]
  public static OS.Error* XOpenout
    (Flags flags,
     string file_name,
     string path,
     out OS.F file);

  [CCode (cname = "osfind_openout")]
  public static OS.F Openout
    (Flags flags,
     string file_name,
     string path);

  [CCode (cname = "xosfind_openoutw")]
  public static OS.Error* XOpenoutW
    (Flags flags,
     string file_name,
     string path,
     out OS.FW file);

  [CCode (cname = "osfind_openoutw")]
  public static OS.FW OpenoutW
    (Flags flags,
     string file_name,
     string path);

  [CCode (cname = "xosfind_openup")]
  public static OS.Error* XOpenup
    (Flags flags,
     string file_name,
     string path,
     out OS.F file);

  [CCode (cname = "osfind_openup")]
  public static OS.F Openup
    (Flags flags,
     string file_name,
     string path);

  [CCode (cname = "xosfind_openupw")]
  public static OS.Error* XOpenupW
    (Flags flags,
     string file_name,
     string path,
     out OS.FW file);

  [CCode (cname = "osfind_openupw")]
  public static OS.FW OpenupW
    (Flags flags,
     string file_name,
     string path);

  [CCode (cname = "xosfind_close")]
  public static OS.Error* XClose
    (OS.F file);

  [CCode (cname = "osfind_close")]
  public static void Close
    (OS.F file);

  [CCode (cname = "xosfind_closew")]
  public static OS.Error* XCloseW
    (OS.FW file);

  [CCode (cname = "osfind_closew")]
  public static void CloseW
    (OS.FW file);

  [CCode (cname = "xupcallfind_create_openup")]
  public static OS.Error* XUpCallFind_CreateOpenup
    (string file_name,
     OS.F file,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfind_create_openup")]
  public static void UpCallFind_CreateOpenup
    (string file_name,
     OS.F file,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfind_create_openupw")]
  public static OS.Error* XUpCallFind_CreateOpenupW
    (string file_name,
     OS.FW file,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfind_create_openupw")]
  public static void UpCallFind_CreateOpenupW
    (string file_name,
     OS.FW file,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfind_openup")]
  public static OS.Error* XUpCallFind_Openup
    (string file_name,
     OS.F file,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfind_openup")]
  public static void UpCallFind_Openup
    (string file_name,
     OS.F file,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfind_openupw")]
  public static OS.Error* XUpCallFind_OpenupW
    (string file_name,
     OS.FW file,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfind_openupw")]
  public static void UpCallFind_OpenupW
    (string file_name,
     OS.FW file,
     string special,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfind_close")]
  public static OS.Error* XUpCallFind_Close
    (OS.F file,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfind_close")]
  public static void UpCallFind_Close
    (OS.F file,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "xupcallfind_closew")]
  public static OS.Error* XUpCallFind_CloseW
    (OS.FW file,
     FileSwitch.FSInfo fs_info);

  [CCode (cname = "upcallfind_closew")]
  public static void UpCallFind_CloseW
    (OS.FW file,
     FileSwitch.FSInfo fs_info);

}
