/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/osheap.h")]
namespace OSLib.OSHeap
{
  [CCode (cname="error_HEAP_BAD_REASON")]
  public const bits Error_HeapBadReason;
  [CCode (cname="error_HEAP_INIT")]
  public const bits Error_HeapInit;
  [CCode (cname="error_HEAP_BAD_DESC")]
  public const bits Error_HeapBadDesc;
  [CCode (cname="error_HEAP_BAD_LINK")]
  public const bits Error_HeapBadLink;
  [CCode (cname="error_HEAP_ALLOC")]
  public const bits Error_HeapAlloc;
  [CCode (cname="error_HEAP_NOT_ABLOCK")]
  public const bits Error_HeapNotABlock;
  [CCode (cname="error_HEAP_BAD_EXTEND")]
  public const bits Error_HeapBadExtend;
  [CCode (cname="error_HEAP_EXCESSIVE_SHRINK")]
  public const bits Error_HeapExcessiveShrink;
  [CCode (cname="error_HEAP_HEAP_LOCKED")]
  public const bits Error_HeapHeapLocked;

  [CCode (cname = "xosheap_resize_no_fail")]
  public static OS.Error* XResizeNoFail
    (char* heap,
     int size_increase,
     out int size_increase_out);

  [CCode (cname = "osheap_resize_no_fail")]
  public static int ResizeNoFail
    (char* heap,
     int size_increase);

  [CCode (cname = "xosheap_initialise")]
  public static OS.Error* XInitialise
    (char* heap,
     int size);

  [CCode (cname = "osheap_initialise")]
  public static void Initialise
    (char* heap,
     int size);

  [CCode (cname = "xosheap_describe")]
  public static OS.Error* XDescribe
    (char* heap,
     out int max,
     out int free);

  [CCode (cname = "osheap_describe")]
  public static int Describe
    (char* heap,
     out int max);

  [CCode (cname = "xosheap_alloc")]
  public static OS.Error* XAlloc
    (char* heap,
     int size,
     out char* blk);

  [CCode (cname = "osheap_alloc")]
  public static char* Alloc
    (char* heap,
     int size);

  [CCode (cname = "xosheap_free")]
  public static OS.Error* XFree
    (char* heap,
     char* blk);

  [CCode (cname = "osheap_free")]
  public static void Free
    (char* heap,
     char* blk);

  [CCode (cname = "xosheap_realloc")]
  public static OS.Error* XRealloc
    (char* heap,
     char* blk,
     int size_increase,
     out char* blk_out);

  [CCode (cname = "osheap_realloc")]
  public static char* Realloc
    (char* heap,
     char* blk,
     int size_increase);

  [CCode (cname = "xosheap_resize")]
  public static OS.Error* XResize
    (char* heap,
     int size_increase);

  [CCode (cname = "osheap_resize")]
  public static void Resize
    (char* heap,
     int size_increase);

  [CCode (cname = "xosheap_read_size")]
  public static OS.Error* XReadSize
    (char* heap,
     char* blk,
     out int size);

  [CCode (cname = "osheap_read_size")]
  public static int ReadSize
    (char* heap,
     char* blk);

}
