/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/pci.h")]
namespace OSLib.PCI
{
  [CCode (cname = "pci_id")]
  public enum ID
  {
    [CCode (cname = "pci_VENDOR_ID")]
    VendorID,
    [CCode (cname = "pci_DEVICE_ID")]
    DeviceID,
    [CCode (cname = "pci_ANY_ID")]
    AnyID
  }

  [CCode (cname = "pci_class")]
  public enum Class
  {
    Default = 0
  }

  [CCode (cname = "pci_function")]
  public enum Function
  {
    Default = 0
  }

  [CCode (cname = "pci_access_size", has_type_id = false)]
  public struct AccessSize : bits
  {
  }

  [CCode (cname = "pci_address", has_type_id = false)]
  public struct Address : bits
  {
  }

  [CCode (cname = "pci_configuration_address", has_type_id = false)]
  public struct ConfigurationAddress : bits
  {
  }

  [CCode (cname = "pci_memory_read_flags")]
  public enum MemoryReadFlags
  {
    [CCode (cname = "pci_MEMORY_READ_PREFETCHABLE")]
    MemoryReadPrefetchable
  }

  [CCode (cname = "pci_memory_write_flags")]
  public enum MemoryWriteFlags
  {
    Default = 0
  }

  [CCode (cname = "pci_read_info_flags")]
  public enum ReadInfoFlags
  {
    [CCode (cname = "pci_READ_INFO_DEVICE_NUMBER")]
    ReadInfoDeviceNumber,
    [CCode (cname = "pci_READ_INFO_FUNCTION_NUMBER")]
    ReadInfoFunctionNumber,
    [CCode (cname = "pci_READ_INFO_BUS_NUMBER")]
    ReadInfoBusNumber,
    [CCode (cname = "pci_READ_INFO_PARENT_BRIDGE_HANDLE")]
    ReadInfoParentBridgeHandle,
    [CCode (cname = "pci_READ_INFO_SLOT_NUMBER")]
    ReadInfoSlotNumber,
    [CCode (cname = "pci_READ_INFO_ID")]
    ReadInfoID,
    [CCode (cname = "pci_READ_INFO_REVISION_ID")]
    ReadInfoRevisionID,
    [CCode (cname = "pci_READ_INFO_SUBSYSTEM_ID")]
    ReadInfoSubsystemID,
    [CCode (cname = "pci_READ_INFO_CLASS")]
    ReadInfoClass,
    [CCode (cname = "pci_READ_INFO_CMOS_ADDRESS")]
    ReadInfoCMOSAddress,
    [CCode (cname = "pci_READ_INFO_CMOS_SIZE")]
    ReadInfoCMOSSize,
    [CCode (cname = "pci_READ_INFO_DESCRIPTION")]
    ReadInfoDescription,
    [CCode (cname = "pci_READ_INFO_DEVICE_VECTOR_NUMBER")]
    ReadInfoDeviceVectorNumber,
    [CCode (cname = "pci_READ_INFO_ETHERNET_ADDRESS_LOW")]
    ReadInfoEthernetAddressLow,
    [CCode (cname = "pci_READ_INFO_ETHERNET_ADDRESS_HIGH")]
    ReadInfoEthernetAddressHigh,
    [CCode (cname = "pci_READ_INFO_LOGICAL_DMA_CHANNEL")]
    ReadInfoLogicalDMAChannel,
    [CCode (cname = "pci_READ_INFO_VENDOR_DESCRIPTION")]
    ReadInfoVendorDescription
  }

  [CCode (cname = "pci_hardware_address_input_flags")]
  public enum HardwareAddressInputFlags
  {
    [CCode (cname = "pci_HARDWARE_ADDRESS_ACCESS_READ_WRITE")]
    HardwareAddressAccessReadWrite,
    [CCode (cname = "pci_HARDWARE_ADDRESS_ACCESS_READ_ONLY")]
    HardwareAddressAccessReadOnly,
    [CCode (cname = "pci_HARDWARE_ADDRESS_ACCESS_NONE")]
    HardwareAddressAccessNone,
    [CCode (cname = "pci_HARDWARE_ADDRESS_ACCESS")]
    HardwareAddressAccess,
    [CCode (cname = "pci_HARDWARE_ADDRESS_BUFFERABLE")]
    HardwareAddressBufferable,
    [CCode (cname = "pci_HARDWARE_ADDRESS_CACHEABLE")]
    HardwareAddressCacheable,
    [CCode (cname = "pci_HARDWARE_ADDRESS_ACCESS_SET")]
    HardwareAddressAccessSet,
    [CCode (cname = "pci_HARDWARE_ADDRESS_QUERY_ONLY")]
    HardwareAddressQueryOnly
  }

  [CCode (cname = "pci_hardware_address_output_flags")]
  public enum HardwareAddressOutputFlags
  {
    [CCode (cname = "pci_HARDWARE_ADDRESS_IS_IO")]
    HardwareAddressIsIO,
    [CCode (cname = "pci_HARDWARE_ADDRESS_PREFETCHABLE")]
    HardwareAddressPrefetchable
  }

  [CCode (cname = "pci_logical_address_flags")]
  public enum LogicalAddressFlags
  {
    [CCode (cname = "pci_LOGICAL_ADDRESS_BUFFERABLE")]
    LogicalAddressBufferable,
    [CCode (cname = "pci_LOGICAL_ADDRESS_IS_IO")]
    LogicalAddressIsIO
  }

  [CCode (cname="pci_VENDOR_ID_SHIFT")]
  public const int VendorIDShift;
  [CCode (cname="pci_DEVICE_ID_SHIFT")]
  public const int DeviceIDShift;
  [CCode (cname="pci_HARDWARE_ADDRESS_ACCESS_SHIFT")]
  public const int HardwareAddressAccessShift;

  [CCode (cname = "xpci_read_id")]
  public static OS.Error* XReadID
    (Function function,
     out ID id,
     out ID subsystem_id);

  [CCode (cname = "pci_read_id")]
  public static ID ReadID
    (Function function,
     out ID subsystem_id);

  [CCode (cname = "xpci_read_header")]
  public static OS.Error* XReadHeader
    (char* buffer,
     int size,
     Function function);

  [CCode (cname = "pci_read_header")]
  public static void ReadHeader
    (char* buffer,
     int size,
     Function function);

  [CCode (cname = "xpci_return_number")]
  public static OS.Error* XReturnNumber
    (out Function used);

  [CCode (cname = "pci_return_number")]
  public static Function ReturnNumber ();

  [CCode (cname = "xpci_enumerate_functions")]
  public static OS.Error* XEnumerateFunctions
    (Function previous_function,
     out ID id,
     out ID subsystem_id,
     out Class pciclass,
     out Function function);

  [CCode (cname = "pci_enumerate_functions")]
  public static Function EnumerateFunctions
    (Function previous_function,
     out ID id,
     out ID subsystem_id,
     out Class pciclass);

  [CCode (cname = "xpci_io_read")]
  public static OS.Error* XIORead
    (Address address,
     AccessSize access_size,
     out int value);

  [CCode (cname = "pci_io_read")]
  public static int IORead
    (Address address,
     AccessSize access_size);

  [CCode (cname = "xpci_io_write")]
  public static OS.Error* XIOWrite
    (Address address,
     int value,
     AccessSize access_size);

  [CCode (cname = "pci_io_write")]
  public static void IOWrite
    (Address address,
     int value,
     AccessSize access_size);

  [CCode (cname = "xpci_memory_read")]
  public static OS.Error* XMemoryRead
    (Address address,
     int size,
     char* buffer,
     MemoryReadFlags flags);

  [CCode (cname = "pci_memory_read")]
  public static void MemoryRead
    (Address address,
     int size,
     char* buffer,
     MemoryReadFlags flags);

  [CCode (cname = "xpci_memory_write")]
  public static OS.Error* XMemoryWrite
    (Address address,
     int size,
     char* buffer,
     MemoryWriteFlags flags);

  [CCode (cname = "pci_memory_write")]
  public static void MemoryWrite
    (Address address,
     int size,
     char* buffer,
     MemoryWriteFlags flags);

  [CCode (cname = "xpci_configuration_read")]
  public static OS.Error* XConfigurationRead
    (ConfigurationAddress address,
     AccessSize access_size,
     Function function,
     out int value);

  [CCode (cname = "pci_configuration_read")]
  public static int ConfigurationRead
    (ConfigurationAddress address,
     AccessSize access_size,
     Function function);

  [CCode (cname = "xpci_configuration_write")]
  public static OS.Error* XConfigurationWrite
    (ConfigurationAddress address,
     int value,
     AccessSize access_size,
     Function function);

  [CCode (cname = "pci_configuration_write")]
  public static void ConfigurationWrite
    (ConfigurationAddress address,
     int value,
     AccessSize access_size,
     Function function);

  [CCode (cname = "xpci_hardware_address")]
  public static OS.Error* XHardwareAddress
    (HardwareAddressInputFlags flags,
     int address_index,
     Function function,
     out HardwareAddressOutputFlags flags_out,
     out Address pci_address,
     out int length,
     out char* logical_address);

  [CCode (cname = "pci_hardware_address")]
  public static Address HardwareAddress
    (HardwareAddressInputFlags flags,
     int address_index,
     Function function,
     out HardwareAddressOutputFlags flags_out,
     out int length,
     out char* logical_address);

  [CCode (cname = "xpci_read_info")]
  public static OS.Error* XReadInfo
    (ReadInfoFlags flags,
     char* buffer,
     int size,
     Function function,
     out int length);

  [CCode (cname = "pci_read_info")]
  public static int ReadInfo
    (ReadInfoFlags flags,
     char* buffer,
     int size,
     Function function);

  [CCode (cname = "xpci_special_cycle")]
  public static OS.Error* XSpecialCycle
    (int bus,
     char* message);

  [CCode (cname = "pci_special_cycle")]
  public static void SpecialCycle
    (int bus,
     char* message);

  [CCode (cname = "xpci_find_by_location")]
  public static OS.Error* XFindByLocation
    (int bus,
     int device_or_function,
     out Function function);

  [CCode (cname = "pci_find_by_location")]
  public static Function FindByLocation
    (int bus,
     int device_or_function);

  [CCode (cname = "xpci_find_by_id")]
  public static OS.Error* XFindByID
    (ID vendor_id,
     ID device_id,
     ID subsystem_vendor_id,
     Function previous_function,
     ID subsystem_id,
     out Function function);

  [CCode (cname = "pci_find_by_id")]
  public static Function FindByID
    (ID vendor_id,
     ID device_id,
     ID subsystem_vendor_id,
     Function previous_function,
     ID subsystem_id);

  [CCode (cname = "xpci_find_by_class")]
  public static OS.Error* XFindByClass
    (Class pciclass,
     Class class_mask,
     Function previous_function,
     out Function function);

  [CCode (cname = "pci_find_by_class")]
  public static Function FindByClass
    (Class pciclass,
     Class class_mask,
     Function previous_function);

  [CCode (cname = "xpci_ram_alloc")]
  public static OS.Error* XRAMAlloc
    (int size,
     int alignment,
     int boundary_limitation,
     out char* logical_address,
     out Address pci_address);

  [CCode (cname = "pci_ram_alloc")]
  public static void RAMAlloc
    (int size,
     int alignment,
     int boundary_limitation,
     out char* logical_address,
     out Address pci_address);

  [CCode (cname = "xpci_ram_free")]
  public static OS.Error* XRAMFree
    (char* logical_address);

  [CCode (cname = "pci_ram_free")]
  public static void RAMFree
    (char* logical_address);

  [CCode (cname = "xpci_logical_address")]
  public static OS.Error* XLogicalAddress
    (LogicalAddressFlags flags,
     Address pci_address,
     int length,
     out char* logical_address);

  [CCode (cname = "pci_logical_address")]
  public static char* LogicalAddress
    (LogicalAddressFlags flags,
     Address pci_address,
     int length);

  [CCode (cname = "xservice_pci_lookup_description")]
  public static OS.Error* XService_PCILookupDescription
    (Function function,
     out bool unclaimed,
     out char* description);

  [CCode (cname = "service_pci_lookup_description")]
  public static bool Service_PCILookupDescription
    (Function function,
     out char* description);

  [CCode (cname = "xservice_pci_lookup_vendor_description")]
  public static OS.Error* XService_PCILookupVendorDescription
    (ID vendir,
     out bool unclaimed,
     out char* description);

  [CCode (cname = "service_pci_lookup_vendor_description")]
  public static bool Service_PCILookupVendorDescription
    (ID vendir,
     out char* description);

}
