/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Draw;
using OSLib.Font;
using OSLib.MessageTrans;
using OSLib.JPEG;

[CCode (cheader_filename = "oslib/pdriver.h")]
namespace OSLib.PDriver
{
  [CCode (cname = "pdriver_type")]
  public enum Type
  {
    [CCode (cname = "pdriver_TYPE_PS")]
    PS,
    [CCode (cname = "pdriver_TYPE_DM")]
    DM,
    [CCode (cname = "pdriver_TYPE_LJ")]
    LJ,
    [CCode (cname = "pdriver_TYPE_IX")]
    IX,
    [CCode (cname = "pdriver_TYPE_FX")]
    FX,
    [CCode (cname = "pdriver_TYPE_LZ")]
    LZ,
    [CCode (cname = "pdriver_TYPE_LB")]
    LB,
    [CCode (cname = "pdriver_TYPE_UF")]
    UF,
    [CCode (cname = "pdriver_TYPE_JX")]
    JX,
    [CCode (cname = "pdriver_TYPE_PJ")]
    PJ
  }

  [CCode (cname = "pdriver_features")]
  public enum Features
  {
    [CCode (cname = "pdriver_FEATURE_COLOUR")]
    FeatureColour,
    [CCode (cname = "pdriver_FEATURE_LIMITED_COLOURS")]
    FeatureLimitedColours,
    [CCode (cname = "pdriver_FEATURE_DISCRETE_COLOURS")]
    FeatureDiscreteColours,
    [CCode (cname = "pdriver_FEATURE_OUTLINES_ONLY")]
    FeatureOutlinesOnly,
    [CCode (cname = "pdriver_FEATURE_THIN_LINES_ONLY")]
    FeatureThinLinesOnly,
    [CCode (cname = "pdriver_FEATURE_NO_OVERWRITE")]
    FeatureNoOverwrite,
    [CCode (cname = "pdriver_FEATURE_TRFM_SPRITE")]
    FeatureTrfmSprite,
    [CCode (cname = "pdriver_FEATURE_TRFM_FONT")]
    FeatureTrfmFont,
    [CCode (cname = "pdriver_FEATURE_DRAW_PAGE_FLAGS")]
    FeatureDrawPageFlags,
    [CCode (cname = "pdriver_FEATURE_SCREEN_DUMP")]
    FeatureScreenDump,
    [CCode (cname = "pdriver_FEATURE_GENERAL_TRFM")]
    FeatureGeneralTrfm,
    [CCode (cname = "pdriver_FEATURE_INSERT_ILLUSTRATION")]
    FeatureInsertIllustration,
    [CCode (cname = "pdriver_FEATURE_MISC_OP")]
    FeatureMiscOp,
    [CCode (cname = "pdriver_FEATURE_SET_DRIVER")]
    FeatureSetDriver,
    [CCode (cname = "pdriver_FEATURE_DECLARE_FONT")]
    FeatureDeclareFont
  }

  [CCode (cname = "pdriver_font_flags")]
  public enum FontFlags
  {
    [CCode (cname = "pdriver_NO_DOWNLOAD")]
    NoDownload,
    [CCode (cname = "pdriver_KERNED")]
    Kerned
  }

  [CCode (cname = "pdriver_font_map_flags")]
  public enum FontMapFlags
  {
    [CCode (cname = "pdriver_FONT_RESIDENT")]
    FontResident,
    [CCode (cname = "pdriver_FONT_DOWNLOAD")]
    FontDownload,
    [CCode (cname = "pdriver_FONT_DOWNLOADED")]
    FontDownloaded
  }

  [CCode (cname = "pdriver_font_add_flags")]
  public enum FontAddFlags
  {
    [CCode (cname = "pdriver_FONT_OVERWRITE")]
    FontOverwrite
  }

  [CCode (cname = "pdriver_enumerate_flags")]
  public enum EnumerateFlags
  {
    Default = 0
  }

  [CCode (cname = "pdriver_configure_reason")]
  public enum ConfigureReason
  {
    [CCode (cname = "pdriver_CONFIGURE_PAPER_SIZES")]
    ConfigurePaperSizes
  }

  [CCode (cname = "pdriver_interrogate_reason")]
  public enum InterrogateReason
  {
    [CCode (cname = "pdriver_INTERROGATE_SETTINGS")]
    InterrogateSettings
  }

  [CCode (cname = "pdriver_message_print_error")]
  public struct MessagePrintError
  {
    [CCode (cname = "errnum")]
    public unowned int Errnum;
    [CCode (cname = "errmess")]
    public unowned char Errmess[232];
  }

  [CCode (cname = "pdriver_message_print_type_odd")]
  public struct MessagePrintTypeOdd
  {
    [CCode (cname = "pad")]
    public unowned byte Pad[20];
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "file_name")]
    public unowned char FileName[212];
  }

  [CCode (cname = "pdriver_message_print_configure")]
  public struct MessagePrintConfigure
  {
    [CCode (cname = "reason")]
    public unowned ConfigureReason Reason;
    [CCode (cname = "data.filename")]
    public unowned char* Data_Filename;
  }

  [CCode (cname = "pdriver_message_print_interrogate")]
  public struct MessagePrintInterrogate
  {
    [CCode (cname = "reason")]
    public unowned InterrogateReason Reason;
    [CCode (cname = "data.settings.printer_id")]
    public unowned int Data_Settings_PrinterId;
    [CCode (cname = "data.settings.palette_number")]
    public unowned int Data_Settings_PaletteNumber;
    [CCode (cname = "data.settings.options")]
    public unowned int Data_Settings_Options;
    [CCode (cname = "data.settings.resolution")]
    public unowned OS.Coord Data_Settings_Resolution;
    [CCode (cname = "data.settings.flags")]
    public unowned bits Data_Settings_Flags;
    [CCode (cname = "data.settings.mode_name")]
    public unowned char Data_Settings_ModeName[208];
  }

  [CCode (cname = "pdriver_message_ps_printer_query")]
  public struct MessagePSPrinterQuery
  {
    [CCode (cname = "buffer")]
    public unowned char* Buffer;
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [CCode (cname = "pdriver_full_message_print_error")]
  public struct FullMessagePrintError : Wimp.MessageHeaderBase
  {
    [CCode (cname = "errnum")]
    public unowned int Errnum;
    [CCode (cname = "errmess")]
    public unowned char Errmess[232];
  }

  [CCode (cname = "pdriver_full_message_print_type_odd")]
  public struct FullMessagePrintTypeOdd : Wimp.MessageHeaderBase
  {
    [CCode (cname = "pad")]
    public unowned byte Pad[20];
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "file_name")]
    public unowned char FileName[212];
  }

  [CCode (cname = "pdriver_full_message_print_configure")]
  public struct FullMessagePrintConfigure : Wimp.MessageHeaderBase
  {
    [CCode (cname = "reason")]
    public unowned ConfigureReason Reason;
    [CCode (cname = "data.filename")]
    public unowned char* Data_Filename;
  }

  [CCode (cname = "pdriver_full_message_print_interrogate")]
  public struct FullMessagePrintInterrogate : Wimp.MessageHeaderBase
  {
    [CCode (cname = "reason")]
    public unowned InterrogateReason Reason;
    [CCode (cname = "data.settings.printer_id")]
    public unowned int Data_Settings_PrinterId;
    [CCode (cname = "data.settings.palette_number")]
    public unowned int Data_Settings_PaletteNumber;
    [CCode (cname = "data.settings.options")]
    public unowned int Data_Settings_Options;
    [CCode (cname = "data.settings.resolution")]
    public unowned OS.Coord Data_Settings_Resolution;
    [CCode (cname = "data.settings.flags")]
    public unowned bits Data_Settings_Flags;
    [CCode (cname = "data.settings.mode_name")]
    public unowned char Data_Settings_ModeName[208];
  }

  [CCode (cname = "pdriver_full_message_ps_printer_query")]
  public struct FullMessagePSPrinterQuery : Wimp.MessageHeaderBase
  {
    [CCode (cname = "buffer")]
    public unowned char* Buffer;
    [CCode (cname = "buffer_size")]
    public unowned int BufferSize;
  }

  [CCode (cname = "pdriver_info_type")]
  public enum InfoType
  {
    Default = 0
  }

  [CCode (cname = "pdriver_draw_flags")]
  public enum DrawFlags
  {
    [CCode (cname = "pdriver_DRAW_COPY_COUNT")]
    DrawCopyCount,
    [CCode (cname = "pdriver_DRAW_MAY_PRESCAN")]
    DrawMayPrescan
  }

  [CCode (cname = "pdriver_configuration_info_base")]
  public struct ConfigurationInfoBase
  {
    [CCode (cname = "entry_count")]
    public unowned int EntryCount;
    [CCode (cname = "block_size")]
    public unowned int BlockSize;
    [CCode (cname = "entry_offsets")]
    public unowned int EntryOffsets[20];
  }

  [Compact]
  [CCode (cname = "pdriver_configuration_info", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ConfigurationInfo
  {
    public ConfigurationInfo(int numEntries);
    [CCode (cname = "entry_count")]
    public unowned int EntryCount;
    [CCode (cname = "block_size")]
    public unowned int BlockSize;
    [CCode (cname = "entry_offsets")]
    public unowned int EntryOffsets[20];
    [CCode (cname = "data", array_length = false)]
    public unowned char[] Data;
  }

  [CCode (cname="pdriver_OS_UNIT")]
  public const int OSUnit;
  [CCode (cname="pdriver_INCH")]
  public const int Inch;
  [CCode (cname="pdriver_POINT")]
  public const int Point;
  [CCode (cname="error_PDRIVER_BAD_FEATURES")]
  public const bits Error_PDriverBadFeatures;
  [CCode (cname="error_PDRIVER_NO_CURRENT_SPRITE")]
  public const bits Error_PDriverNoCurrentSprite;
  [CCode (cname="error_PDRIVER_NO_JOB_SELECTED")]
  public const bits Error_PDriverNoJobSelected;
  [CCode (cname="error_PDRIVER_NO_SUCH_JOB")]
  public const bits Error_PDriverNoSuchJob;
  [CCode (cname="error_PDRIVER_NO_CURRENT_PAGE")]
  public const bits Error_PDriverNoCurrentPage;
  [CCode (cname="error_PDRIVER_PRINTING_PAGE")]
  public const bits Error_PDriverPrintingPage;
  [CCode (cname="error_PDRIVER_INVALID_COPIES")]
  public const bits Error_PDriverInvalidCopies;
  [CCode (cname="error_PDRIVER_CANNOT_HANDLE")]
  public const bits Error_PDriverCannotHandle;
  [CCode (cname="error_PDRIVER_BAD_HALFTONE")]
  public const bits Error_PDriverBadHalftone;
  [CCode (cname="error_PDRIVER_CANCELLED")]
  public const bits Error_PDriverCancelled;
  [CCode (cname="error_PDRIVER_SINGULAR_MATRIX")]
  public const bits Error_PDriverSingularMatrix;
  [CCode (cname="error_PDRIVER_BAD_RECTANGLE")]
  public const bits Error_PDriverBadRectangle;
  [CCode (cname="error_PDRIVER_RECTANGLES_MISS")]
  public const bits Error_PDriverRectanglesMiss;
  [CCode (cname="error_PDRIVER_NO_FREE_MEMORY")]
  public const bits Error_PDriverNoFreeMemory;
  [CCode (cname="error_PDRIVER_NOT_ONE_PAGE")]
  public const bits Error_PDriverNotOnePage;
  [CCode (cname="error_PDRIVER_IN_USE")]
  public const bits Error_PDriverInUse;
  [CCode (cname="error_PDRIVER_OVERFLOW")]
  public const bits Error_PDriverOverflow;
  [CCode (cname="error_PDRIVER_BAD_MISC_OP")]
  public const bits Error_PDriverBadMiscOp;
  [CCode (cname="error_PDRIVER_NO_DUPLICATES")]
  public const bits Error_PDriverNoDuplicates;
  [CCode (cname="error_PDRIVER_NO_CURRENT_DRIVER")]
  public const bits Error_PDriverNoCurrentDriver;
  [CCode (cname="error_PDRIVER_UNKNOWN_NUMBER")]
  public const bits Error_PDriverUnknownNumber;
  [CCode (cname="error_PDRIVER_DUPLICATE_NUMBER")]
  public const bits Error_PDriverDuplicateNumber;
  [CCode (cname="error_PDRIVER_BAD_SET_PRINTER")]
  public const bits Error_PDriverBadSetPrinter;
  [CCode (cname="message_PRINT_FILE")]
  public const bits Message_PrintFile;
  [CCode (cname="message_WILL_PRINT")]
  public const bits Message_WillPrint;
  [CCode (cname="message_PRINT_SAVE")]
  public const bits Message_PrintSave;
  [CCode (cname="message_PRINT_INIT")]
  public const bits Message_PrintInit;
  [CCode (cname="message_PRINT_ERROR")]
  public const bits Message_PrintError;
  [CCode (cname="message_PRINT_TYPE_ODD")]
  public const bits Message_PrintTypeOdd;
  [CCode (cname="message_PRINT_TYPE_KNOWN")]
  public const bits Message_PrintTypeKnown;
  [CCode (cname="message_SET_PRINTER")]
  public const bits Message_SetPrinter;
  [CCode (cname="message_PRINT_INIT2")]
  public const bits Message_PrintInit2;
  [CCode (cname="message_PRINT_CONFIGURE")]
  public const bits Message_PrintConfigure;
  [CCode (cname="message_PRINT_INTERROGATE")]
  public const bits Message_PrintInterrogate;
  [CCode (cname="message_PS_PRINTER_QUERY")]
  public const bits Message_PSPrinterQuery;
  [CCode (cname="message_PS_PRINTER_ACK")]
  public const bits Message_PSPrinterAck;
  [CCode (cname="message_PS_PRINTER_MODIFIED")]
  public const bits Message_PSPrinterModified;
  [CCode (cname="message_PS_PRINTER_DEFAULTS")]
  public const bits Message_PSPrinterDefaults;
  [CCode (cname="message_PS_PRINTER_DEFAULTED")]
  public const bits Message_PSPrinterDefaulted;
  [CCode (cname="message_PS_PRINTER_NOT_PS")]
  public const bits Message_PSPrinterNotPS;
  [CCode (cname="message_RESET_PRINTER")]
  public const bits Message_ResetPrinter;
  [CCode (cname="message_PS_IS_FONTPRINT_RUNNING")]
  public const bits Message_PSIsFontprintRunning;

  [CCode (cname = "xpdriver_info")]
  public static OS.Error* XInfo
    (out InfoType type,
     out int xres,
     out int yres,
     out Features features,
     out string desc,
     out int halftone_xres,
     out int halftone_yres,
     out int printer);

  [CCode (cname = "pdriver_info")]
  public static void Info
    (out InfoType type,
     out int xres,
     out int yres,
     out Features features,
     out string desc,
     out int halftone_xres,
     out int halftone_yres,
     out int printer);

  [CCode (cname = "xpdriver_set_info")]
  public static OS.Error* XSetInfo
    (int xres,
     int yres,
     Features features,
     string printer_name,
     int halftone_xres,
     int halftone_yres,
     int printer);

  [CCode (cname = "pdriver_set_info")]
  public static void SetInfo
    (int xres,
     int yres,
     Features features,
     string printer_name,
     int halftone_xres,
     int halftone_yres,
     int printer);

  [CCode (cname = "xpdriver_check_features")]
  public static OS.Error* XCheckFeatures
    (Features features,
     Features value);

  [CCode (cname = "pdriver_check_features")]
  public static void CheckFeatures
    (Features features,
     Features value);

  [CCode (cname = "xpdriver_page_size")]
  public static OS.Error* XPageSize
    (out int xsize,
     out int ysize,
     out int left,
     out int bottom,
     out int right,
     out int top);

  [CCode (cname = "pdriver_page_size")]
  public static void PageSize
    (out int xsize,
     out int ysize,
     out int left,
     out int bottom,
     out int right,
     out int top);

  [CCode (cname = "xpdriver_set_page_size")]
  public static OS.Error* XSetPageSize
    (int xsize,
     int ysize,
     int left,
     int bottom,
     int right,
     int top);

  [CCode (cname = "pdriver_set_page_size")]
  public static void SetPageSize
    (int xsize,
     int ysize,
     int left,
     int bottom,
     int right,
     int top);

  [CCode (cname = "xpdriver_select_job")]
  public static OS.Error* XSelectJob
    (OS.F job,
     string title,
     out OS.F old_job);

  [CCode (cname = "pdriver_select_job")]
  public static OS.F SelectJob
    (OS.F job,
     string title);

  [CCode (cname = "xpdriver_select_jobw")]
  public static OS.Error* XSelectJobW
    (OS.FW job,
     string title,
     out OS.FW old_job);

  [CCode (cname = "pdriver_select_jobw")]
  public static OS.FW SelectJobW
    (OS.FW job,
     string title);

  [CCode (cname = "xpdriver_current_job")]
  public static OS.Error* XCurrentJob
    (out OS.F job);

  [CCode (cname = "pdriver_current_job")]
  public static OS.F CurrentJob ();

  [CCode (cname = "xpdriver_current_jobw")]
  public static OS.Error* XCurrentJobW
    (out OS.FW job);

  [CCode (cname = "pdriver_current_jobw")]
  public static OS.FW CurrentJobW ();

  [CCode (cname = "xpdriver_end_job")]
  public static OS.Error* XEndJob
    (OS.F job);

  [CCode (cname = "pdriver_end_job")]
  public static void EndJob
    (OS.F job);

  [CCode (cname = "xpdriver_end_jobw")]
  public static OS.Error* XEndJobW
    (OS.FW job);

  [CCode (cname = "pdriver_end_jobw")]
  public static void EndJobW
    (OS.FW job);

  [CCode (cname = "xpdriver_abort_job")]
  public static OS.Error* XAbortJob
    (OS.F job);

  [CCode (cname = "pdriver_abort_job")]
  public static void AbortJob
    (OS.F job);

  [CCode (cname = "xpdriver_abort_jobw")]
  public static OS.Error* XAbortJobW
    (OS.FW job);

  [CCode (cname = "pdriver_abort_jobw")]
  public static void AbortJobW
    (OS.FW job);

  [CCode (cname = "xpdriver_reset")]
  public static OS.Error* XReset ();

  [CCode (cname = "pdriver_reset")]
  public static void Reset ();

  [CCode (cname = "xpdriver_give_rectangle")]
  public static OS.Error* XGiveRectangle
    (int id,
     OS.Box rect,
     OS.HomTrfm trfm,
     OS.Coord pos,
     OS.Colour bg);

  [CCode (cname = "pdriver_give_rectangle")]
  public static void GiveRectangle
    (int id,
     OS.Box rect,
     OS.HomTrfm trfm,
     OS.Coord pos,
     OS.Colour bg);

  [CCode (cname = "xpdriver_draw_page")]
  public static OS.Error* XDrawPage
    (DrawFlags copies_and_flags,
     OS.Box rect,
     int page,
     string page_no,
     out bool more,
     out int id);

  [CCode (cname = "pdriver_draw_page")]
  public static bool DrawPage
    (DrawFlags copies_and_flags,
     OS.Box rect,
     int page,
     string page_no,
     out int id);

  [CCode (cname = "xpdriver_get_rectangle")]
  public static OS.Error* XGetRectangle
    (OS.Box rect,
     out bool more,
     out int id);

  [CCode (cname = "pdriver_get_rectangle")]
  public static bool GetRectangle
    (OS.Box rect,
     out int id);

  [CCode (cname = "xpdriver_cancel_job")]
  public static OS.Error* XCancelJob
    (OS.F job);

  [CCode (cname = "pdriver_cancel_job")]
  public static void CancelJob
    (OS.F job);

  [CCode (cname = "xpdriver_cancel_jobw")]
  public static OS.Error* XCancelJobW
    (OS.FW job);

  [CCode (cname = "pdriver_cancel_jobw")]
  public static void CancelJobW
    (OS.FW job);

  [CCode (cname = "xpdriver_screen_dump")]
  public static OS.Error* XScreenDump
    (OS.F job);

  [CCode (cname = "pdriver_screen_dump")]
  public static void ScreenDump
    (OS.F job);

  [CCode (cname = "xpdriver_screen_dumpw")]
  public static OS.Error* XScreenDumpW
    (OS.FW job);

  [CCode (cname = "pdriver_screen_dumpw")]
  public static void ScreenDumpW
    (OS.FW job);

  [CCode (cname = "xpdriver_enumerate_jobs")]
  public static OS.Error* XEnumerateJobs
    (int context,
     out int context_out);

  [CCode (cname = "pdriver_enumerate_jobs")]
  public static int EnumerateJobs
    (int context);

  [CCode (cname = "xpdriver_cancel_job_with_error")]
  public static OS.Error* XCancelJobWithError
    (OS.F job,
     OS.Error error);

  [CCode (cname = "pdriver_cancel_job_with_error")]
  public static void CancelJobWithError
    (OS.F job,
     OS.Error error);

  [CCode (cname = "xpdriver_cancel_job_with_errorw")]
  public static OS.Error* XCancelJobWithErrorW
    (OS.FW job,
     OS.Error error);

  [CCode (cname = "pdriver_cancel_job_with_errorw")]
  public static void CancelJobWithErrorW
    (OS.FW job,
     OS.Error error);

  [CCode (cname = "xpdriver_select_illustration")]
  public static OS.Error* XSelectIllustration
    (OS.F job,
     string title,
     out OS.F old_job);

  [CCode (cname = "pdriver_select_illustration")]
  public static OS.F SelectIllustration
    (OS.F job,
     string title);

  [CCode (cname = "xpdriver_select_illustrationw")]
  public static OS.Error* XSelectIllustrationW
    (OS.FW job,
     string title,
     out OS.FW old_job);

  [CCode (cname = "pdriver_select_illustrationw")]
  public static OS.FW SelectIllustrationW
    (OS.FW job,
     string title);

  [CCode (cname = "xpdriver_insert_illustration")]
  public static OS.Error* XInsertIllustration
    (OS.F job,
     Draw.Path clip_path,
     int x0,
     int y0,
     int x1,
     int y1,
     int x2,
     int y2);

  [CCode (cname = "pdriver_insert_illustration")]
  public static void InsertIllustration
    (OS.F job,
     Draw.Path clip_path,
     int x0,
     int y0,
     int x1,
     int y1,
     int x2,
     int y2);

  [CCode (cname = "xpdriver_insert_illustrationw")]
  public static OS.Error* XInsertIllustrationW
    (OS.FW job,
     Draw.Path clip_path,
     int x0,
     int y0,
     int x1,
     int y1,
     int x2,
     int y2);

  [CCode (cname = "pdriver_insert_illustrationw")]
  public static void InsertIllustrationW
    (OS.FW job,
     Draw.Path clip_path,
     int x0,
     int y0,
     int x1,
     int y1,
     int x2,
     int y2);

  [CCode (cname = "xpdriver_declare_font")]
  public static OS.Error* XDeclareFont
    (Font.F font,
     string font_name,
     FontFlags flags);

  [CCode (cname = "pdriver_declare_font")]
  public static void DeclareFont
    (Font.F font,
     string font_name,
     FontFlags flags);

  [CCode (cname = "xpdriver_declare_driver")]
  public static OS.Error* XDeclareDriver
    (AsmRoutine reason,
     char* workspace,
     Type type);

  [CCode (cname = "pdriver_declare_driver")]
  public static void DeclareDriver
    (AsmRoutine reason,
     char* workspace,
     Type type);

  [CCode (cname = "xpdriver_remove_driver")]
  public static OS.Error* XRemoveDriver
    (Type type);

  [CCode (cname = "pdriver_remove_driver")]
  public static void RemoveDriver
    (Type type);

  [CCode (cname = "xpdriver_select_driver")]
  public static OS.Error* XSelectDriver
    (Type type,
     out Type old_type);

  [CCode (cname = "pdriver_select_driver")]
  public static Type SelectDriver
    (Type type);

  [CCode (cname = "xpdriver_enumerate_drivers")]
  public static OS.Error* XEnumerateDrivers
    (int context,
     out int context_out,
     out Type type);

  [CCode (cname = "pdriver_enumerate_drivers")]
  public static int EnumerateDrivers
    (int context,
     out Type type);

  [CCode (cname = "xpdrivermiscop_add_font")]
  public static OS.Error* XPDriverMiscOp_AddFont
    (string font_name,
     string alien_name,
     FontMapFlags flags,
     FontAddFlags add_flags);

  [CCode (cname = "pdrivermiscop_add_font")]
  public static void PDriverMiscOp_AddFont
    (string font_name,
     string alien_name,
     FontMapFlags flags,
     FontAddFlags add_flags);

  [CCode (cname = "xpdrivermiscop_remove_font")]
  public static OS.Error* XPDriverMiscOp_RemoveFont
    (string font_name);

  [CCode (cname = "pdrivermiscop_remove_font")]
  public static void PDriverMiscOp_RemoveFont
    (string font_name);

  [CCode (cname = "xpdrivermiscop_enumerate_fonts")]
  public static OS.Error* XPDriverMiscOp_EnumerateFonts
    (string buffer,
     int size,
     int context,
     EnumerateFlags flags,
     out char* end,
     out int context_out);

  [CCode (cname = "pdrivermiscop_enumerate_fonts")]
  public static int PDriverMiscOp_EnumerateFonts
    (string buffer,
     int size,
     int context,
     EnumerateFlags flags,
     out char* end);

  [CCode (cname = "xpdrivermiscop_register_dumper")]
  public static OS.Error* XPDriverMiscOp_RegisterDumper
    (int pdumper_no,
     int dp_version_required,
     char* dumper_workspace,
     AsmRoutine dumper_code,
     bits supported_calls,
     bits supported_strips);

  [CCode (cname = "pdrivermiscop_register_dumper")]
  public static void PDriverMiscOp_RegisterDumper
    (int pdumper_no,
     int dp_version_required,
     char* dumper_workspace,
     AsmRoutine dumper_code,
     bits supported_calls,
     bits supported_strips);

  [CCode (cname = "xpdrivermiscop_deregister_dumper")]
  public static OS.Error* XPDriverMiscOp_DeregisterDumper
    (int pdumper_no);

  [CCode (cname = "pdrivermiscop_deregister_dumper")]
  public static void PDriverMiscOp_DeregisterDumper
    (int pdumper_no);

  [CCode (cname = "xpdrivermiscop_strip_types")]
  public static OS.Error* XPDriverMiscOp_StripTypes
    (int pdumper_no,
     out bits supported_strips);

  [CCode (cname = "pdrivermiscop_strip_types")]
  public static bits PDriverMiscOp_StripTypes
    (int pdumper_no);

  [CCode (cname = "xpdrivermiscopfordriver_add_font")]
  public static OS.Error* XPDriverMiscOpForDriver_AddFont
    (string font_name,
     string alien_name,
     FontMapFlags flags,
     FontAddFlags add_flags,
     Type type);

  [CCode (cname = "pdrivermiscopfordriver_add_font")]
  public static void PDriverMiscOpForDriver_AddFont
    (string font_name,
     string alien_name,
     FontMapFlags flags,
     FontAddFlags add_flags,
     Type type);

  [CCode (cname = "xpdrivermiscopfordriver_remove_font")]
  public static OS.Error* XPDriverMiscOpForDriver_RemoveFont
    (string font_name,
     Type type);

  [CCode (cname = "pdrivermiscopfordriver_remove_font")]
  public static void PDriverMiscOpForDriver_RemoveFont
    (string font_name,
     Type type);

  [CCode (cname = "xpdrivermiscopfordriver_enumerate_fonts")]
  public static OS.Error* XPDriverMiscOpForDriver_EnumerateFonts
    (string buffer,
     int size,
     int context,
     EnumerateFlags flags,
     Type type,
     out char* end,
     out int context_out);

  [CCode (cname = "pdrivermiscopfordriver_enumerate_fonts")]
  public static int PDriverMiscOpForDriver_EnumerateFonts
    (string buffer,
     int size,
     int context,
     EnumerateFlags flags,
     Type type,
     out char* end);

  [CCode (cname = "xpdrivermiscopfordriver_register_dumper")]
  public static OS.Error* XPDriverMiscOpForDriver_RegisterDumper
    (int pdumper_no,
     int dp_version_required,
     char* dumper_workspace,
     AsmRoutine dumper_code,
     bits supported_calls,
     bits supported_strips,
     Type type);

  [CCode (cname = "pdrivermiscopfordriver_register_dumper")]
  public static void PDriverMiscOpForDriver_RegisterDumper
    (int pdumper_no,
     int dp_version_required,
     char* dumper_workspace,
     AsmRoutine dumper_code,
     bits supported_calls,
     bits supported_strips,
     Type type);

  [CCode (cname = "xpdrivermiscopfordriver_deregister_dumper")]
  public static OS.Error* XPDriverMiscOpForDriver_DeregisterDumper
    (int pdumper_no,
     Type type);

  [CCode (cname = "pdrivermiscopfordriver_deregister_dumper")]
  public static void PDriverMiscOpForDriver_DeregisterDumper
    (int pdumper_no,
     Type type);

  [CCode (cname = "xpdrivermiscopfordriver_strip_types")]
  public static OS.Error* XPDriverMiscOpForDriver_StripTypes
    (int pdumper_no,
     Type type,
     out bits supported_strips);

  [CCode (cname = "pdrivermiscopfordriver_strip_types")]
  public static bits PDriverMiscOpForDriver_StripTypes
    (int pdumper_no,
     Type type);

  [CCode (cname = "xpdriver_set_driver")]
  public static OS.Error* XSetDriver
    (int pdumper_no,
     string ensure_command,
     byte* configuration_data1,
     byte* configuration_data2,
     int configuration_word);

  [CCode (cname = "pdriver_set_driver")]
  public static void SetDriver
    (int pdumper_no,
     string ensure_command,
     byte* configuration_data1,
     byte* configuration_data2,
     int configuration_word);

  [CCode (cname = "xpdriver_set_driver_extended")]
  public static OS.Error* XSetDriverExtended
    (int pdumper_no,
     string ensure_command,
     byte* configuration_data1,
     byte* configuration_data2,
     int configuration_word,
     ConfigurationInfo configuration_info);

  [CCode (cname = "pdriver_set_driver_extended")]
  public static void SetDriverExtended
    (int pdumper_no,
     string ensure_command,
     byte* configuration_data1,
     byte* configuration_data2,
     int configuration_word,
     ConfigurationInfo configuration_info);

  [CCode (cname = "xpdriverjpegswiinfo_dimensions")]
  public static OS.Error* XPDriverJPEGSWIInfo_Dimensions
    (OSLib.JPEG.Image image,
     int size,
     out JPEG.InfoFlags info_flags,
     out int width,
     out int height,
     out int xdpi,
     out int ydpi,
     out int workspace_size);

  [CCode (cname = "pdriverjpegswiinfo_dimensions")]
  public static JPEG.InfoFlags PDriverJPEGSWIInfo_Dimensions
    (OSLib.JPEG.Image image,
     int size,
     out int width,
     out int height,
     out int xdpi,
     out int ydpi,
     out int workspace_size);

  [CCode (cname = "xpdriverjpegswifileinfo_dimensions")]
  public static OS.Error* XPDriverJPEGSWIFileInfo_Dimensions
    (string file_name,
     out JPEG.InfoFlags info_flags,
     out int width,
     out int height,
     out int xdpi,
     out int ydpi,
     out int workspace_size);

  [CCode (cname = "pdriverjpegswifileinfo_dimensions")]
  public static JPEG.InfoFlags PDriverJPEGSWIFileInfo_Dimensions
    (string file_name,
     out int width,
     out int height,
     out int xdpi,
     out int ydpi,
     out int workspace_size);

  [CCode (cname = "xpdriverjpegswi_plot_scaled")]
  public static OS.Error* XPDriverJPEGSWI_PlotScaled
    (OSLib.JPEG.Image image,
     int x,
     int y,
     OS.Factors factors,
     int size,
     JPEG.ScaleFlags flags);

  [CCode (cname = "pdriverjpegswi_plot_scaled")]
  public static void PDriverJPEGSWI_PlotScaled
    (OSLib.JPEG.Image image,
     int x,
     int y,
     OS.Factors factors,
     int size,
     JPEG.ScaleFlags flags);

  [CCode (cname = "xpdriverjpegswi_plot_file_scaled")]
  public static OS.Error* XPDriverJPEGSWI_PlotFileScaled
    (string file_name,
     int x,
     int y,
     OS.Factors factors,
     JPEG.ScaleFlags flags);

  [CCode (cname = "pdriverjpegswi_plot_file_scaled")]
  public static void PDriverJPEGSWI_PlotFileScaled
    (string file_name,
     int x,
     int y,
     OS.Factors factors,
     JPEG.ScaleFlags flags);

  [CCode (cname = "xpdriverjpegswi_plot_transformed")]
  public static OS.Error* XPDriverJPEGSWI_PlotTransformed
    (OSLib.JPEG.Image image,
     JPEG.TransformFlags flags,
     OS.Trfm trfm_or_rect,
     int size);

  [CCode (cname = "pdriverjpegswi_plot_transformed")]
  public static void PDriverJPEGSWI_PlotTransformed
    (OSLib.JPEG.Image image,
     JPEG.TransformFlags flags,
     OS.Trfm trfm_or_rect,
     int size);

  [CCode (cname = "xpdriverjpegswi_plot_file_transformed")]
  public static OS.Error* XPDriverJPEGSWI_PlotFileTransformed
    (string file_name,
     JPEG.TransformFlags flags,
     OS.Trfm trfm_or_rect);

  [CCode (cname = "pdriverjpegswi_plot_file_transformed")]
  public static void PDriverJPEGSWI_PlotFileTransformed
    (string file_name,
     JPEG.TransformFlags flags,
     OS.Trfm trfm_or_rect);

  [CCode (cname = "xservice_print")]
  public static OS.Error* XService_Print ();

  [CCode (cname = "service_print")]
  public static void Service_Print ();

  [CCode (cname = "xservice_pdriver_starting")]
  public static OS.Error* XService_PDriverStarting ();

  [CCode (cname = "service_pdriver_starting")]
  public static void Service_PDriverStarting ();

  [CCode (cname = "xservice_pdriver_get_messages")]
  public static OS.Error* XService_PDriverGetMessages
    (out bool unclaimed,
     out MessageTrans.ControlBlock common_cb);

  [CCode (cname = "service_pdriver_get_messages")]
  public static bool Service_PDriverGetMessages
    (out MessageTrans.ControlBlock common_cb);

  [CCode (cname = "xservice_pdriver_changed")]
  public static OS.Error* XService_PDriverChanged ();

  [CCode (cname = "service_pdriver_changed")]
  public static void Service_PDriverChanged ();

}
