/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/portable.h")]
namespace OSLib.Portable
{
  [CCode (cname = "portable_control_flags")]
  public enum ControlFlags
  {
    [CCode (cname = "portable_CONTROL_ECONET_POWER_ENABLED")]
    ControlEconetPowerEnabled,
    [CCode (cname = "portable_CONTROL_LCD_POWER_ENABLED")]
    ControlLCDPowerEnabled,
    [CCode (cname = "portable_CONTROL_EXTERNAL_VIDEO_ENABLED")]
    ControlExternalVideoEnabled,
    [CCode (cname = "portable_CONTROL_SERIAL_ENABLED")]
    ControlSerialEnabled,
    [CCode (cname = "portable_CONTROL_DUAL_PANEL_ENABLED")]
    ControlDualPanelEnabled,
    [CCode (cname = "portable_CONTROL_VIDEO_CLOCK")]
    ControlVideoClock,
    [CCode (cname = "portable_CONTROL_INVERT_VIDEO_CLOCK")]
    ControlInvertVideoClock,
    [CCode (cname = "portable_CONTROL_BACK_LIGHT_ENABLED")]
    ControlBackLightEnabled,
    [CCode (cname = "portable_CONTROL_EXTRA_DISPLAY_LINES")]
    ControlExtraDisplayLines,
    [CCode (cname = "portable_CONTROL_EXTRA_DUAL_PANEL_DRAM")]
    ControlExtraDualPanelDRAM,
    [CCode (cname = "portable_CONTROL_SOUND_POWER_ENABLED")]
    ControlSoundPowerEnabled,
    [CCode (cname = "portable_CONTROL_PCMCIA_ENABLED")]
    ControlPCMCIAEnabled,
    [CCode (cname = "portable_CONTROL_PARALLEL_ENABLED")]
    ControlParallelEnabled,
    [CCode (cname = "portable_CONTROL_FDC_ENABLED")]
    ControlFDCEnabled,
    [CCode (cname = "portable_CONTROL_IDE_ENABLED")]
    ControlIDEEnabled,
    [CCode (cname = "portable_CONTROL_LCD_VIDEO_INVERTED")]
    ControlLCDVideoInverted,
    [CCode (cname = "portable_CONTROL_TRACKERBALL_ENABLED")]
    ControlTrackerballEnabled,
    [CCode (cname = "portable_CONTROL_MOUSE_ENABLED")]
    ControlMouseEnabled
  }

  [CCode (cname = "portable_bmu_variable")]
  public enum BMUVariable
  {
    [CCode (cname = "portable_BMU_VARIABLE_VERSION")]
    Version,
    [CCode (cname = "portable_BMU_VARIABLE_NOMINAL_CHARGE")]
    NominalCharge,
    [CCode (cname = "portable_BMU_VARIABLE_MEASURED_CHARGE")]
    MeasuredCharge,
    [CCode (cname = "portable_BMU_VARIABLE_USED_CHARGE")]
    UsedCharge,
    [CCode (cname = "portable_BMU_VARIABLE_USABLE_CHARGE")]
    UsableCharge,
    [CCode (cname = "portable_BMU_VARIABLE_CHARGE_ESTIMATE")]
    ChargeEstimate,
    [CCode (cname = "portable_BMU_VARIABLE_INST_VOLTAGE")]
    InstVoltage,
    [CCode (cname = "portable_BMU_VARIABLE_INST_CURRENT")]
    InstCurrent,
    [CCode (cname = "portable_BMU_VARIABLE_INST_TEMPERATURE")]
    InstTemperature,
    [CCode (cname = "portable_BMU_VARIABLE_FLAGS")]
    Flags,
    [CCode (cname = "portable_BMU_VARIABLE_CHARGE_RATE")]
    ChargeRate
  }

  [CCode (cname = "portable_bmu_flags")]
  public enum BMUFlags
  {
    [CCode (cname = "portable_BMU_AT_THRESHOLD3")]
    BMUAtThreshold3,
    [CCode (cname = "portable_BMU_BATTERY_CHANGED")]
    BMUBatteryChanged,
    [CCode (cname = "portable_BMU_LID_OPEN")]
    BMULidOpen,
    [CCode (cname = "portable_BMU_AT_THRESHOLD2")]
    BMUAtThreshold2,
    [CCode (cname = "portable_BMU_BATTERY_FLAT")]
    BMUBatteryFlat,
    [CCode (cname = "portable_BMU_AT_THRESHOLD1")]
    BMUAtThreshold1,
    [CCode (cname = "portable_BMU_BATTERY_LOW")]
    BMUBatteryLow,
    [CCode (cname = "portable_BMU_CHARGING_FAULT")]
    BMUChargingFault,
    [CCode (cname = "portable_BMU_CHARGE_STATE_KNOWN")]
    BMUChargeStateKnown,
    [CCode (cname = "portable_BMU_BATTERY_PRESENT")]
    BMUBatteryPresent,
    [CCode (cname = "portable_BMU_CHARGER_CONNECTED")]
    BMUChargerConnected
  }

  [CCode (cname = "portable_processor_speed")]
  public enum ProcessorSpeed
  {
    [CCode (cname = "portable_SPEED_FAST")]
    SpeedFast,
    [CCode (cname = "portable_SPEED_SLOW")]
    SpeedSlow
  }

  [CCode (cname = "portable_bmu_command")]
  public enum BMUCommand
  {
    [CCode (cname = "portable_BMU_POWER_OFF")]
    BMUPowerOff,
    [CCode (cname = "portable_BMU_SET_AUTOSTART")]
    BMUSetAutostart,
    [CCode (cname = "portable_BMU_DISCONNECT_CHARGER")]
    BMUDisconnectCharger,
    [CCode (cname = "portable_BMU_CONNECT_CHARGER")]
    BMUConnectCharger
  }

  [CCode (cname = "portable_features")]
  public enum Features
  {
    [CCode (cname = "portable_SUPPORTS_SPEED")]
    SupportsSpeed,
    [CCode (cname = "portable_SUPPORTS_IDLE")]
    SupportsIdle,
    [CCode (cname = "portable_SUPPORTS_STOP")]
    SupportsStop
  }

  [CCode (cname = "portable_restart_type")]
  public enum RestartType
  {
    [CCode (cname = "portable_RESTART_KEYBOARD")]
    RestartKeyboard,
    [CCode (cname = "portable_RESTART_CLOCK")]
    RestartClock
  }

  [CCode (cname = "portable_status_flags")]
  public enum StatusFlags
  {
    [CCode (cname = "portable_STATUS_DOCK_STATION_PRESENT")]
    StatusDockStationPresent,
    [CCode (cname = "portable_STATUS_FLOPPY_CONNECTED")]
    StatusFloppyConnected,
    [CCode (cname = "portable_STATUS_TRACKERBALL_CONNECTED")]
    StatusTrackerballConnected,
    [CCode (cname = "portable_STATUS_LCD_IN_USE")]
    StatusLCDInUse
  }

  [CCode (cname = "portable_state")]
  public enum State
  {
    [CCode (cname = "portable_POWER_DOWN")]
    PowerDown,
    [CCode (cname = "portable_POWER_UP")]
    PowerUp,
    [CCode (cname = "portable_POWER_TIDY_UP")]
    PowerTidyUp,
    [CCode (cname = "portable_POWER_ABOUT_TO_FREEZE")]
    PowerAboutToFreeze,
    [CCode (cname = "portable_POWER_AWAKENED")]
    PowerAwakened,
    [CCode (cname = "portable_POWER_WISHES_TO_FREEZE")]
    PowerWishesToFreeze
  }

  [CCode (cname="portable_PORT_ECONET")]
  public const bits PortEconet;
  [CCode (cname="portable_PORT_SERIAL")]
  public const bits PortSerial;
  [CCode (cname="portable_PORT_FDC")]
  public const bits PortFDC;
  [CCode (cname="portable_CONTROL_VIDEO_CLOCK_SHIFT")]
  public const int ControlVideoClockShift;
  [CCode (cname="portable_CONTROL_VIDEO_CLOCK_EXTERNAL")]
  public const int ControlVideoClockExternal;
  [CCode (cname="portable_CONTROL_VIDEO_CLOCK_DIVIDED_CRYSTAL")]
  public const int ControlVideoClockDividedCrystal;
  [CCode (cname="portable_CONTROL_VIDEO_CLOCK_CRYSTAL")]
  public const int ControlVideoClockCrystal;

  [CCode (cname = "xportable_speed")]
  public static OS.Error* XSpeed
    (bits eor_mask,
     bits and_mask,
     out ProcessorSpeed old_speed,
     out ProcessorSpeed new_speed);

  [CCode (cname = "portable_speed")]
  public static void Speed
    (bits eor_mask,
     bits and_mask,
     out ProcessorSpeed old_speed,
     out ProcessorSpeed new_speed);

  [CCode (cname = "xportable_control")]
  public static OS.Error* XControl
    (bits eor_mask,
     bits and_mask,
     out ControlFlags old_control,
     out ControlFlags new_control);

  [CCode (cname = "portable_control")]
  public static void Control
    (bits eor_mask,
     bits and_mask,
     out ControlFlags old_control,
     out ControlFlags new_control);

  [CCode (cname = "xportable_read_bmu_variable")]
  public static OS.Error* XReadBMUVariable
    (BMUVariable var,
     out int value);

  [CCode (cname = "portable_read_bmu_variable")]
  public static int ReadBMUVariable
    (BMUVariable var);

  [CCode (cname = "xportable_write_bmu_variable")]
  public static OS.Error* XWriteBMUVariable
    (BMUVariable var,
     int value);

  [CCode (cname = "portable_write_bmu_variable")]
  public static void WriteBMUVariable
    (BMUVariable var,
     int value);

  [CCode (cname = "xportable_read_features")]
  public static OS.Error* XReadFeatures
    (out Features features);

  [CCode (cname = "portable_read_features")]
  public static Features ReadFeatures ();

  [CCode (cname = "xportable_idle")]
  public static OS.Error* XIdle ();

  [CCode (cname = "portable_idle")]
  public static void Idle ();

  [CCode (cname = "xportable_stop")]
  public static OS.Error* XStop
    (out RestartType restart);

  [CCode (cname = "portable_stop")]
  public static RestartType Stop ();

  [CCode (cname = "xportable_status")]
  public static OS.Error* XStatus
    (out StatusFlags flags);

  [CCode (cname = "portable_status")]
  public static StatusFlags Status ();

  [CCode (cname = "xservice_portable")]
  public static OS.Error* XService_Portable
    (State state,
     bits ports,
     out bool unclaimed,
     out bits ports_to_change);

  [CCode (cname = "service_portable")]
  public static bool Service_Portable
    (State state,
     bits ports,
     out bits ports_to_change);

  [CCode (cname = "xevent_portable_bmu")]
  public static OS.Error* XEvent_PortableBMU
    (BMUFlags flags);

  [CCode (cname = "event_portable_bmu")]
  public static void Event_PortableBMU
    (BMUFlags flags);

}
