/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Window;

[CCode (cheader_filename = "oslib/printdbox.h")]
namespace OSLib.PrintDbox
{
  [CCode (cname = "printdbox_flags")]
  public enum Flags
  {
    [CCode (cname = "printdbox_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "printdbox_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted,
    [CCode (cname = "printdbox_GENERATE_SHOW_SETUP_ACTION")]
    GenerateShowSetupAction,
    [CCode (cname = "printdbox_INCLUDE_ALL_FROM_TO")]
    IncludeAllFromTo,
    [CCode (cname = "printdbox_INCLUDE_COPIES")]
    IncludeCopies,
    [CCode (cname = "printdbox_INCLUDE_SCALE")]
    IncludeScale,
    [CCode (cname = "printdbox_INCLUDE_ORIENTATION")]
    IncludeOrientation,
    [CCode (cname = "printdbox_INCLUDE_SAVE")]
    IncludeSave,
    [CCode (cname = "printdbox_INCLUDE_SET_UP")]
    IncludeSetUp,
    [CCode (cname = "printdbox_INCLUDE_DRAFT")]
    IncludeDraft,
    [CCode (cname = "printdbox_SELECT_FROM_TO")]
    SelectFromTo,
    [CCode (cname = "printdbox_SELECT_SIDEWAYS")]
    SelectSideways,
    [CCode (cname = "printdbox_SELECT_DRAFT")]
    SelectDraft
  }

  [CCode (cname = "printdbox_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "from")]
    public unowned int From;
    [CCode (cname = "to")]
    public unowned int To;
    [CCode (cname = "copies")]
    public unowned int Copies;
    [CCode (cname = "scale")]
    public unowned int Scale;
    [CCode (cname = "further_options")]
    public unowned Toolbox.StringReference FurtherOptions;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "printdbox_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "from")]
    public unowned int From;
    [CCode (cname = "to")]
    public unowned int To;
    [CCode (cname = "copies")]
    public unowned int Copies;
    [CCode (cname = "scale")]
    public unowned int Scale;
    [CCode (cname = "further_options")]
    public unowned Toolbox.StringReference FurtherOptions;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "printdbox_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "printdbox_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "printdbox_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "printdbox_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "printdbox_action_setup_about_to_be_shown")]
  public struct ActionSetupAboutToBeShown
  {
    [CCode (cname = "setup")]
    public unowned Toolbox.O Setup;
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "printdbox_action_setup_about_to_be_shown_block")]
  public struct ActionSetupAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "setup")]
    public unowned Toolbox.O Setup;
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "printdbox_action_save")]
  public struct ActionSave
  {
    [CCode (cname = "start")]
    public unowned int Start;
    [CCode (cname = "finish")]
    public unowned int Finish;
    [CCode (cname = "copies")]
    public unowned int Copies;
    [CCode (cname = "scale_factor")]
    public unowned int ScaleFactor;
  }

  [CCode (cname = "printdbox_action_save_block")]
  public struct ActionSaveBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "start")]
    public unowned int Start;
    [CCode (cname = "finish")]
    public unowned int Finish;
    [CCode (cname = "copies")]
    public unowned int Copies;
    [CCode (cname = "scale_factor")]
    public unowned int ScaleFactor;
  }

  [CCode (cname = "printdbox_action_print")]
  public struct ActionPrint
  {
    [CCode (cname = "start")]
    public unowned int Start;
    [CCode (cname = "finish")]
    public unowned int Finish;
    [CCode (cname = "copies")]
    public unowned int Copies;
    [CCode (cname = "scale_factor")]
    public unowned int ScaleFactor;
  }

  [CCode (cname = "printdbox_action_print_block")]
  public struct ActionPrintBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "start")]
    public unowned int Start;
    [CCode (cname = "finish")]
    public unowned int Finish;
    [CCode (cname = "copies")]
    public unowned int Copies;
    [CCode (cname = "scale_factor")]
    public unowned int ScaleFactor;
  }

  [CCode (cname="class_PRINT_DBOX")]
  public const Toolbox.Class Class_PrintDbox;
  [CCode (cname="action_PRINT_DBOX_ABOUT_TO_BE_SHOWN")]
  public const bits Action_PrintDboxAboutToBeShown;
  [CCode (cname="action_PRINT_DBOX_DIALOGUE_COMPLETED")]
  public const bits Action_PrintDboxDialogueCompleted;
  [CCode (cname="action_PRINT_DBOX_SET_UP_ABOUT_TO_BE_SHOWN")]
  public const bits Action_PrintDboxSetUpAboutToBeShown;
  [CCode (cname="action_PRINT_DBOX_SAVE")]
  public const bits Action_PrintDboxSave;
  [CCode (cname="action_PRINT_DBOX_SET_UP")]
  public const bits Action_PrintDboxSetUp;
  [CCode (cname="action_PRINT_DBOX_PRINT")]
  public const bits Action_PrintDboxPrint;
  [CCode (cname="printdbox_PRINT_SIDEWAYS")]
  public const bits PrintSideways;
  [CCode (cname="printdbox_PRINT_DRAFT")]
  public const bits PrintDraft;
  [CCode (cname="error_PRINT_DBOX_TASKS_ACTIVE")]
  public const bits Error_PrintDboxTasksActive;
  [CCode (cname="error_PRINT_DBOX_ALLOC_FAILED")]
  public const bits Error_PrintDboxAllocFailed;
  [CCode (cname="error_PRINT_DBOX_SHORT_BUFFER")]
  public const bits Error_PrintDboxShortBuffer;
  [CCode (cname="error_PRINT_DBOX_NO_SUCH_TASK")]
  public const bits Error_PrintDboxNoSuchTask;
  [CCode (cname="error_PRINT_DBOX_NO_SUCH_METHOD")]
  public const bits Error_PrintDboxNoSuchMethod;
  [CCode (cname="error_PRINT_DBOX_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_PrintDboxNoSuchMiscOpMethod;
  [CCode (cname="printdbox_PAGE_RANGE_ALL")]
  public const int PageRangeAll;

  [CCode (cname = "xprintdbox_get_window_id")]
  public static OS.Error* XGetWindowId
    (bits flags,
     Toolbox.O print_dbox,
     out Toolbox.O window);

  [CCode (cname = "printdbox_get_window_id")]
  public static Toolbox.O GetWindowId
    (bits flags,
     Toolbox.O print_dbox);

  [CCode (cname = "xprintdbox_set_page_range")]
  public static OS.Error* XSetPageRange
    (bits flags,
     Toolbox.O print_dbox,
     int start,
     int end);

  [CCode (cname = "printdbox_set_page_range")]
  public static void SetPageRange
    (bits flags,
     Toolbox.O print_dbox,
     int start,
     int end);

  [CCode (cname = "xprintdbox_get_page_range")]
  public static OS.Error* XGetPageRange
    (bits flags,
     Toolbox.O print_dbox,
     out int start,
     out int end);

  [CCode (cname = "printdbox_get_page_range")]
  public static void GetPageRange
    (bits flags,
     Toolbox.O print_dbox,
     out int start,
     out int end);

  [CCode (cname = "xprintdbox_set_copies")]
  public static OS.Error* XSetCopies
    (bits flags,
     Toolbox.O print_dbox,
     int copy_count);

  [CCode (cname = "printdbox_set_copies")]
  public static void SetCopies
    (bits flags,
     Toolbox.O print_dbox,
     int copy_count);

  [CCode (cname = "xprintdbox_get_copies")]
  public static OS.Error* XGetCopies
    (bits flags,
     Toolbox.O print_dbox,
     out int copy_count);

  [CCode (cname = "printdbox_get_copies")]
  public static int GetCopies
    (bits flags,
     Toolbox.O print_dbox);

  [CCode (cname = "xprintdbox_set_scale")]
  public static OS.Error* XSetScale
    (bits flags,
     Toolbox.O print_dbox,
     int scale);

  [CCode (cname = "printdbox_set_scale")]
  public static void SetScale
    (bits flags,
     Toolbox.O print_dbox,
     int scale);

  [CCode (cname = "xprintdbox_get_scale")]
  public static OS.Error* XGetScale
    (bits flags,
     Toolbox.O print_dbox,
     out int scale);

  [CCode (cname = "printdbox_get_scale")]
  public static int GetScale
    (bits flags,
     Toolbox.O print_dbox);

  [CCode (cname = "xprintdbox_set_orientation")]
  public static OS.Error* XSetOrientation
    (bits flags,
     Toolbox.O print_dbox,
     bool landscape);

  [CCode (cname = "printdbox_set_orientation")]
  public static void SetOrientation
    (bits flags,
     Toolbox.O print_dbox,
     bool landscape);

  [CCode (cname = "xprintdbox_get_orientation")]
  public static OS.Error* XGetOrientation
    (bits flags,
     Toolbox.O print_dbox,
     out bool landscape);

  [CCode (cname = "printdbox_get_orientation")]
  public static bool GetOrientation
    (bits flags,
     Toolbox.O print_dbox);

  [CCode (cname = "xprintdbox_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O print_dbox,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "printdbox_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O print_dbox,
     string buffer,
     int size);

  [CCode (cname = "xprintdbox_set_draft")]
  public static OS.Error* XSetDraft
    (bits flags,
     Toolbox.O print_dbox,
     bool draft);

  [CCode (cname = "printdbox_set_draft")]
  public static void SetDraft
    (bits flags,
     Toolbox.O print_dbox,
     bool draft);

  [CCode (cname = "xprintdbox_get_draft")]
  public static OS.Error* XGetDraft
    (bits flags,
     Toolbox.O print_dbox,
     out bool draft);

  [CCode (cname = "printdbox_get_draft")]
  public static bool GetDraft
    (bits flags,
     Toolbox.O print_dbox);

}
