/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/report.h")]
namespace OSLib.Report
{
  [CCode (cname = "report_registers_save_area")]
  public struct RegistersSaveArea
  {
    [CCode (cname = "reg")]
    public unowned int Reg[16];
  }

  [CCode (cname = "report_registers_flags")]
  public enum RegistersFlags
  {
    [CCode (cname = "report_REGISTERS_DISPLAY_R0")]
    RegistersDisplayR0,
    [CCode (cname = "report_REGISTERS_DISPLAY_R1")]
    RegistersDisplayR1,
    [CCode (cname = "report_REGISTERS_DISPLAY_R2")]
    RegistersDisplayR2,
    [CCode (cname = "report_REGISTERS_DISPLAY_R3")]
    RegistersDisplayR3,
    [CCode (cname = "report_REGISTERS_DISPLAY_R4")]
    RegistersDisplayR4,
    [CCode (cname = "report_REGISTERS_DISPLAY_R5")]
    RegistersDisplayR5,
    [CCode (cname = "report_REGISTERS_DISPLAY_R6")]
    RegistersDisplayR6,
    [CCode (cname = "report_REGISTERS_DISPLAY_R7")]
    RegistersDisplayR7,
    [CCode (cname = "report_REGISTERS_DISPLAY_R8")]
    RegistersDisplayR8,
    [CCode (cname = "report_REGISTERS_DISPLAY_R9")]
    RegistersDisplayR9,
    [CCode (cname = "report_REGISTERS_DISPLAY_R10")]
    RegistersDisplayR10,
    [CCode (cname = "report_REGISTERS_DISPLAY_R11")]
    RegistersDisplayR11,
    [CCode (cname = "report_REGISTERS_DISPLAY_R12")]
    RegistersDisplayR12,
    [CCode (cname = "report_REGISTERS_DISPLAY_R13")]
    RegistersDisplayR13,
    [CCode (cname = "report_REGISTERS_DISPLAY_R14")]
    RegistersDisplayR14,
    [CCode (cname = "report_REGISTERS_DISPLAY_R15")]
    RegistersDisplayR15,
    [CCode (cname = "report_REGISTERS_REGISTERS_ARE_STRINGS")]
    RegistersRegistersAreStrings
  }


  [CCode (cname = "xreport_text0")]
  public static OS.Error* XText0
    (string text);

  [CCode (cname = "report_text0")]
  public static void Text0
    (string text);

  [CCode (cname = "xreport_regs")]
  public static OS.Error* XRegs ();

  [CCode (cname = "report_regs")]
  public static void Regs ();

  [CCode (cname = "xreport_registers")]
  public static OS.Error* XRegisters
    (string heading,
     RegistersSaveArea registers,
     RegistersFlags flags);

  [CCode (cname = "report_registers")]
  public static void Registers
    (string heading,
     RegistersSaveArea registers,
     RegistersFlags flags);

  [CCode (cname = "xreport_where")]
  public static OS.Error* XWhere ();

  [CCode (cname = "report_where")]
  public static void Where ();

  [CCode (cname = "xreport_poll")]
  public static OS.Error* XPoll
    (Wimp.EventNo event);

  [CCode (cname = "report_poll")]
  public static void Poll
    (Wimp.EventNo event);

  [CCode (cname = "xreport_dump")]
  public static OS.Error* XDump
    (char* address,
     int length,
     int width);

  [CCode (cname = "report_dump")]
  public static void Dump
    (char* address,
     int length,
     int width);

  [CCode (cname = "xreport_quit")]
  public static OS.Error* XQuit ();

  [CCode (cname = "report_quit")]
  public static void Quit ();

  [CCode (cname = "xreport_clear")]
  public static OS.Error* XClear ();

  [CCode (cname = "report_clear")]
  public static void Clear ();

  [CCode (cname = "xreport_open")]
  public static OS.Error* XOpen ();

  [CCode (cname = "report_open")]
  public static void Open ();

  [CCode (cname = "xreport_close")]
  public static OS.Error* XClose ();

  [CCode (cname = "report_close")]
  public static void Close ();

  [CCode (cname = "xreport_on")]
  public static OS.Error* XOn ();

  [CCode (cname = "report_on")]
  public static void On ();

  [CCode (cname = "xreport_off")]
  public static OS.Error* XOff ();

  [CCode (cname = "report_off")]
  public static void Off ();

  [CCode (cname = "xreport_cmd_on")]
  public static OS.Error* XCmdOn ();

  [CCode (cname = "report_cmd_on")]
  public static void CmdOn ();

  [CCode (cname = "xreport_cmd_off")]
  public static OS.Error* XCmdOff ();

  [CCode (cname = "report_cmd_off")]
  public static void CmdOff ();

  [CCode (cname = "xreport_hide")]
  public static OS.Error* XHide ();

  [CCode (cname = "report_hide")]
  public static void Hide ();

  [CCode (cname = "xreport_un_hide")]
  public static OS.Error* XUnHide ();

  [CCode (cname = "report_un_hide")]
  public static void UnHide ();

  [CCode (cname = "xreport_err_on")]
  public static OS.Error* XErrOn ();

  [CCode (cname = "report_err_on")]
  public static void ErrOn ();

  [CCode (cname = "xreport_err_off")]
  public static OS.Error* XErrOff ();

  [CCode (cname = "report_err_off")]
  public static void ErrOff ();

  [CCode (cname = "xreport_task_on")]
  public static OS.Error* XTaskOn ();

  [CCode (cname = "report_task_on")]
  public static void TaskOn ();

  [CCode (cname = "xreport_task_off")]
  public static OS.Error* XTaskOff ();

  [CCode (cname = "report_task_off")]
  public static void TaskOff ();

  [CCode (cname = "xreport_vdu4_on")]
  public static OS.Error* XVdu4On ();

  [CCode (cname = "report_vdu4_on")]
  public static void Vdu4On ();

  [CCode (cname = "xreport_vdu4_off")]
  public static OS.Error* XVdu4Off ();

  [CCode (cname = "report_vdu4_off")]
  public static void Vdu4Off ();

  [CCode (cname = "xreport_rma_on")]
  public static OS.Error* XRmaOn ();

  [CCode (cname = "report_rma_on")]
  public static void RmaOn ();

  [CCode (cname = "xreport_rma_off")]
  public static OS.Error* XRmaOff ();

  [CCode (cname = "report_rma_off")]
  public static void RmaOff ();

  [CCode (cname = "xreport_time_on")]
  public static OS.Error* XTimeOn ();

  [CCode (cname = "report_time_on")]
  public static void TimeOn ();

  [CCode (cname = "xreport_time_off")]
  public static OS.Error* XTimeOff ();

  [CCode (cname = "report_time_off")]
  public static void TimeOff ();

  [CCode (cname = "xreport_srce_on")]
  public static OS.Error* XSrceOn ();

  [CCode (cname = "report_srce_on")]
  public static void SrceOn ();

  [CCode (cname = "xreport_srce_off")]
  public static OS.Error* XSrceOff ();

  [CCode (cname = "report_srce_off")]
  public static void SrceOff ();

  [CCode (cname = "xreport_obey_on")]
  public static OS.Error* XObeyOn ();

  [CCode (cname = "report_obey_on")]
  public static void ObeyOn ();

  [CCode (cname = "xreport_obey_off")]
  public static OS.Error* XObeyOff ();

  [CCode (cname = "report_obey_off")]
  public static void ObeyOff ();

  [CCode (cname = "xreport_push")]
  public static OS.Error* XPush ();

  [CCode (cname = "report_push")]
  public static void Push ();

  [CCode (cname = "xreport_pull")]
  public static OS.Error* XPull ();

  [CCode (cname = "report_pull")]
  public static void Pull ();

}
