/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.Toolbox;

[CCode (cheader_filename = "oslib/resed.h")]
namespace OSLib.ResEd
{
  [CCode (cname = "resed_d", has_type_id = false)]
  public struct D : bits
  {
  }

  [CCode (cname = "resed_o", has_type_id = false)]
  public struct O : bits
  {
  }

  [CCode (cname = "resed_object_load_flags")]
  public enum ObjectLoadFlags
  {
    [CCode (cname = "resed_FORCE_LOAD")]
    ForceLoad
  }

  [CCode (cname = "resed_object_loaded_flags")]
  public enum ObjectLoadedFlags
  {
    [CCode (cname = "resed_LOAD_FAILED")]
    LoadFailed
  }

  [CCode (cname = "resed_object_sending_flags")]
  public enum ObjectSendingFlags
  {
    [CCode (cname = "resed_CANNOT_SEND")]
    CannotSend
  }

  [CCode (cname = "resed_object_send_flags")]
  public enum ObjectSendFlags
  {
    [CCode (cname = "resed_DELETE_AFTER_SEND")]
    DeleteAfterSend
  }

  [CCode (cname = "resed_keycut_details_flags")]
  public enum KeycutDetailsFlags
  {
    [CCode (cname = "resed_KEYCUT_RAISES_EVENT")]
    KeycutRaisesEvent,
    [CCode (cname = "resed_KEYCUT_SHOWS_OBJECT")]
    KeycutShowsObject,
    [CCode (cname = "resed_KEYCUT_SHOWS_TRANSIENT")]
    KeycutShowsTransient
  }

  [CCode (cname = "resed_error")]
  public enum Error
  {
    [CCode (cname = "resed_ERROR_NO_MEMORY")]
    NoMemory,
    [CCode (cname = "resed_ERROR_UK_VERSION")]
    UkVersion,
    [CCode (cname = "resed_ERROR_UK_OBJECT")]
    UkObject,
    [CCode (cname = "resed_ERROR_CORRUPT_DATA")]
    CorruptData,
    [CCode (cname = "resed_ERROR_NON_FATAL_INTERNAL_ERROR")]
    NonFatalInternalError,
    [CCode (cname = "resed_ERROR_FATAL_INTERNAL_ERROR")]
    FatalInternalError
  }

  [CCode (cname = "resed_message_object_load")]
  public struct MessageObjectLoad
  {
    [CCode (cname = "flags")]
    public unowned ObjectLoadFlags Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "obj_class")]
    public unowned Toolbox.Class ObjClass;
    [CCode (cname = "version")]
    public unowned int Version;
    [CCode (cname = "address")]
    public unowned byte* Address;
    [CCode (cname = "obj_size")]
    public unowned int ObjSize;
    [CCode (cname = "name")]
    public unowned char Name[208];
  }

  [CCode (cname = "resed_message_object_loaded")]
  public struct MessageObjectLoaded
  {
    [CCode (cname = "flags")]
    public unowned ObjectLoadedFlags Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "error")]
    public unowned Error Error;
  }

  [CCode (cname = "resed_message_object_sending")]
  public struct MessageObjectSending
  {
    [CCode (cname = "flags")]
    public unowned ObjectSendingFlags Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "address")]
    public unowned byte* Address;
    [CCode (cname = "obj_size")]
    public unowned int ObjSize;
    [CCode (cname = "error")]
    public unowned Error Error;
  }

  [CCode (cname = "resed_message_object_send")]
  public struct MessageObjectSend
  {
    [CCode (cname = "flags")]
    public unowned ObjectSendFlags Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
  }

  [CCode (cname = "resed_message_object_renamed")]
  public struct MessageObjectRenamed
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "new_name")]
    public unowned char NewName[224];
  }

  [CCode (cname = "resed_message_object_deleted")]
  public struct MessageObjectDeleted
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
  }

  [CCode (cname = "resed_message_object_modified")]
  public struct MessageObjectModified
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
  }

  [CCode (cname = "resed_message_object_closed")]
  public struct MessageObjectClosed
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
  }

  [CCode (cname = "resed_message_sprites_changed")]
  public struct MessageSpritesChanged
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
  }

  [CCode (cname = "resed_message_object_name_request")]
  public struct MessageObjectNameRequest
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "window")]
    public unowned Wimp.W Window;
    [CCode (cname = "icon")]
    public unowned Wimp.I Icon;
  }

  [CCode (cname = "resed_message_object_name")]
  public struct MessageObjectName
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "window")]
    public unowned Wimp.W Window;
    [CCode (cname = "icon")]
    public unowned Wimp.I Icon;
    [CCode (cname = "obj_class")]
    public unowned Toolbox.Class ObjClass;
    [CCode (cname = "name")]
    public unowned char Name[212];
  }

  [CCode (cname = "resed_message_keycut_details")]
  public struct MessageKeycutDetails
  {
    [CCode (cname = "flags")]
    public unowned KeycutDetailsFlags Flags;
    [CCode (cname = "shell")]
    public unowned Wimp.T Shell;
    [CCode (cname = "window")]
    public unowned Wimp.W Window;
    [CCode (cname = "key")]
    public unowned Wimp.KeyNo Key;
    [CCode (cname = "key_action")]
    public unowned Toolbox.Action KeyAction;
    [CCode (cname = "names")]
    public unowned char Names[216];
  }

  [CCode (cname = "resed_full_message_object_load")]
  public struct FullMessageObjectLoad : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ObjectLoadFlags Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "obj_class")]
    public unowned Toolbox.Class ObjClass;
    [CCode (cname = "version")]
    public unowned int Version;
    [CCode (cname = "address")]
    public unowned byte* Address;
    [CCode (cname = "obj_size")]
    public unowned int ObjSize;
    [CCode (cname = "name")]
    public unowned char Name[208];
  }

  [CCode (cname = "resed_full_message_object_loaded")]
  public struct FullMessageObjectLoaded : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ObjectLoadedFlags Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "error")]
    public unowned Error Error;
  }

  [CCode (cname = "resed_full_message_object_sending")]
  public struct FullMessageObjectSending : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ObjectSendingFlags Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "address")]
    public unowned byte* Address;
    [CCode (cname = "obj_size")]
    public unowned int ObjSize;
    [CCode (cname = "error")]
    public unowned Error Error;
  }

  [CCode (cname = "resed_full_message_object_send")]
  public struct FullMessageObjectSend : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ObjectSendFlags Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
  }

  [CCode (cname = "resed_full_message_object_renamed")]
  public struct FullMessageObjectRenamed : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "new_name")]
    public unowned char NewName[224];
  }

  [CCode (cname = "resed_full_message_object_deleted")]
  public struct FullMessageObjectDeleted : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
  }

  [CCode (cname = "resed_full_message_object_modified")]
  public struct FullMessageObjectModified : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
  }

  [CCode (cname = "resed_full_message_object_closed")]
  public struct FullMessageObjectClosed : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
  }

  [CCode (cname = "resed_full_message_sprites_changed")]
  public struct FullMessageSpritesChanged : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
  }

  [CCode (cname = "resed_full_message_object_name_request")]
  public struct FullMessageObjectNameRequest : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "window")]
    public unowned Wimp.W Window;
    [CCode (cname = "icon")]
    public unowned Wimp.I Icon;
  }

  [CCode (cname = "resed_full_message_object_name")]
  public struct FullMessageObjectName : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "doc_id")]
    public unowned D DocId;
    [CCode (cname = "obj_id")]
    public unowned O ObjId;
    [CCode (cname = "window")]
    public unowned Wimp.W Window;
    [CCode (cname = "icon")]
    public unowned Wimp.I Icon;
    [CCode (cname = "obj_class")]
    public unowned Toolbox.Class ObjClass;
    [CCode (cname = "name")]
    public unowned char Name[212];
  }

  [CCode (cname = "resed_full_message_keycut_details")]
  public struct FullMessageKeycutDetails : Wimp.MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned KeycutDetailsFlags Flags;
    [CCode (cname = "shell")]
    public unowned Wimp.T Shell;
    [CCode (cname = "window")]
    public unowned Wimp.W Window;
    [CCode (cname = "key")]
    public unowned Wimp.KeyNo Key;
    [CCode (cname = "key_action")]
    public unowned Toolbox.Action KeyAction;
    [CCode (cname = "names")]
    public unowned char Names[216];
  }

  [CCode (cname="message_RESED_OBJECT_LOAD")]
  public const bits Message_ResedObjectLoad;
  [CCode (cname="message_RESED_OBJECT_LOADED")]
  public const bits Message_ResedObjectLoaded;
  [CCode (cname="message_RESED_OBJECT_SEND")]
  public const bits Message_ResedObjectSend;
  [CCode (cname="message_RESED_OBJECT_SENDING")]
  public const bits Message_ResedObjectSending;
  [CCode (cname="message_RESED_OBJECT_RENAMED")]
  public const bits Message_ResedObjectRenamed;
  [CCode (cname="message_RESED_OBJECT_DELETED")]
  public const bits Message_ResedObjectDeleted;
  [CCode (cname="message_RESED_OBJECT_MODIFIED")]
  public const bits Message_ResedObjectModified;
  [CCode (cname="message_RESED_OBJECT_CLOSED")]
  public const bits Message_ResedObjectClosed;
  [CCode (cname="message_RESED_SPRITES_CHANGED")]
  public const bits Message_ResedSpritesChanged;
  [CCode (cname="message_RESED_OBJECT_NAME_REQUEST")]
  public const bits Message_ResedObjectNameRequest;
  [CCode (cname="message_RESED_OBJECT_NAME")]
  public const bits Message_ResedObjectName;
  [CCode (cname="message_RESED_KEYCUT_DETAILS")]
  public const bits Message_ResedKeycutDetails;

}
