/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/routerdiscovery.h")]
namespace OSLib.RouterDiscovery
{
  [CCode (cname = "routerdiscoverycontrol_operation")]
  public enum RouterDiscoveryControl_Operation
  {
    [CCode (cname = "routerdiscoverycontroloperation_ACTIVATE_HOST_MODE")]
    ActivateHostMode,
    [CCode (cname = "routerdiscoverycontroloperation_ACTIVATE_ROUTER_MODE")]
    ActivateRouterMode,
    [CCode (cname = "routerdiscoverycontroloperation_DEACTIVATE")]
    Deactivate
  }

  [CCode (cname = "routerdiscovery_router_preference")]
  public struct RouterPreference
  {
    [CCode (cname = "router_address")]
    public unowned int RouterAddress;
    [CCode (cname = "preference")]
    public unowned int Preference;
  }

  [CCode (cname = "serviceinternetservice40_state")]
  public enum ServiceInternetService40_State
  {
    [CCode (cname = "serviceinternetservice40state_END_MONITORING")]
    EndMonitoring,
    [CCode (cname = "serviceinternetservice40state_START_SOLICITING")]
    StartSoliciting,
    [CCode (cname = "serviceinternetservice40state_START_MONITORING")]
    StartMonitoring
  }

  [CCode (cname = "serviceinternetservice41_state")]
  public enum ServiceInternetService41_State
  {
    [CCode (cname = "serviceinternetservice41state_END_ADVERTISEMENTS")]
    EndAdvertisements,
    [CCode (cname = "serviceinternetservice41state_START_ADVERTISEMENTS")]
    StartAdvertisements
  }


  [CCode (cname = "xrouterdiscoverycontrol_activate_host_mode")]
  public static OS.Error* XRouterDiscoveryControl_ActivateHostMode
    (char* interface,
     int solicitations_address);

  [CCode (cname = "routerdiscoverycontrol_activate_host_mode")]
  public static void RouterDiscoveryControl_ActivateHostMode
    (char* interface,
     int solicitations_address);

  [CCode (cname = "xrouterdiscoverycontrol_activate_router_mode")]
  public static OS.Error* XRouterDiscoveryControl_ActivateRouterMode
    (char* interface,
     int advertisements_address,
     int min_adv_interval,
     int max_adv_interval,
     RouterPreference preference);

  [CCode (cname = "routerdiscoverycontrol_activate_router_mode")]
  public static void RouterDiscoveryControl_ActivateRouterMode
    (char* interface,
     int advertisements_address,
     int min_adv_interval,
     int max_adv_interval,
     RouterPreference preference);

  [CCode (cname = "xrouterdiscoverycontrol_deactivate")]
  public static OS.Error* XRouterDiscoveryControl_Deactivate
    (char* interface);

  [CCode (cname = "routerdiscoverycontrol_deactivate")]
  public static void RouterDiscoveryControl_Deactivate
    (char* interface);

  [CCode (cname = "xservice_internet_service40")]
  public static OS.Error* XService_InternetService40
    (ServiceInternetService40_State state,
     char* interface);

  [CCode (cname = "service_internet_service40")]
  public static void Service_InternetService40
    (ServiceInternetService40_State state,
     char* interface);

  [CCode (cname = "xservice_internet_service41")]
  public static OS.Error* XService_InternetService41
    (ServiceInternetService41_State state,
     char* interface,
     int num_routers,
     RouterPreference preference);

  [CCode (cname = "service_internet_service41")]
  public static void Service_InternetService41
    (ServiceInternetService41_State state,
     char* interface,
     int num_routers,
     RouterPreference preference);

  [CCode (cname = "xservice_internet_service42")]
  public static OS.Error* XService_InternetService42
    (char* interface,
     int gateway);

  [CCode (cname = "service_internet_service42")]
  public static void Service_InternetService42
    (char* interface,
     int gateway);

}
