/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Toolbox;
using OSLib.Window;

[CCode (cheader_filename = "oslib/saveas.h")]
namespace OSLib.SaveAs
{
  [CCode (cname = "saveas_flags")]
  public enum Flags
  {
    [CCode (cname = "saveas_GENERATE_ABOUT_TO_BE_SHOWN")]
    GenerateAboutToBeShown,
    [CCode (cname = "saveas_GENERATE_DIALOGUE_COMPLETED")]
    GenerateDialogueCompleted,
    [CCode (cname = "saveas_NO_SELECTION_BUTTON")]
    NoSelectionButton,
    [CCode (cname = "saveas_GIVEN_DATA")]
    GivenData,
    [CCode (cname = "saveas_CLIENT_SUPPORTS_RAM_TRANSFER")]
    ClientSupportsRAMTransfer
  }

  [CCode (cname = "saveas_object")]
  public struct Object
  {
    [CCode (cname = "flags")]
    public unowned Flags Flags;
    [CCode (cname = "file_name")]
    public unowned Toolbox.MsgReference FileName;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "saveas_object_template")]
  public struct ObjectTemplate : Toolbox.ResourceFileObjectBase
  {
    [CCode (cname = "objectflags")]
    public unowned Flags Objectflags;
    [CCode (cname = "file_name")]
    public unowned Toolbox.MsgReference FileName;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "title")]
    public unowned Toolbox.MsgReference Title;
    [CCode (cname = "title_limit")]
    public unowned int TitleLimit;
    [CCode (cname = "alternative_window_name")]
    public unowned Toolbox.StringReference AlternativeWindowName;
  }

  [CCode (cname = "saveas_full")]
  public enum Full
  {
    Default = 0
  }

  [CCode (cname = "saveas_action_about_to_be_shown")]
  public struct ActionAboutToBeShown
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "saveas_action_save_to_file")]
  public struct ActionSaveToFile
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[212];
  }

  [CCode (cname = "saveas_action_fill_buffer")]
  public struct ActionFillBuffer
  {
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "buffer")]
    public unowned byte* Buffer;
    [CCode (cname = "size_saved")]
    public unowned int SizeSaved;
  }

  [CCode (cname = "saveas_action_save_completed")]
  public struct ActionSaveCompleted
  {
    [CCode (cname = "ref")]
    public unowned bits Ref;
    [CCode (cname = "file_name")]
    public unowned char FileName[208];
  }

  [CCode (cname = "saveas_action_about_to_be_shown_block")]
  public struct ActionAboutToBeShownBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "tag")]
    public unowned Toolbox.PositionTag Tag;
    [CCode (cname = "position.top_left")]
    public unowned OS.Coord Position_TopLeft;
    [CCode (cname = "position.full")]
    public unowned Full Position_Full;
  }

  [CCode (cname = "saveas_action_dialogue_completed_block")]
  public enum ActionDialogueCompletedBlock
  {
    Default = 0
  }

  [CCode (cname = "saveas_action_save_to_file_block")]
  public struct ActionSaveToFileBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "file_name")]
    public unowned char FileName[212];
  }

  [CCode (cname = "saveas_action_fill_buffer_block")]
  public struct ActionFillBufferBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "buffer_size")]
    public unowned int BufferSize;
    [CCode (cname = "buffer")]
    public unowned byte* Buffer;
    [CCode (cname = "size_saved")]
    public unowned int SizeSaved;
  }

  [CCode (cname = "saveas_action_save_completed_block")]
  public struct ActionSaveCompletedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "ref")]
    public unowned bits Ref;
    [CCode (cname = "file_name")]
    public unowned char FileName[208];
  }

  [CCode (cname="class_SAVE_AS")]
  public const Toolbox.Class Class_SaveAs;
  [CCode (cname="saveas_SAVE_SUCCESSFUL")]
  public const bits SaveSuccessful;
  [CCode (cname="saveas_SAVE_SELECTION")]
  public const bits SaveSelection;
  [CCode (cname="saveas_SAVE_SAFE")]
  public const bits SaveSafe;
  [CCode (cname="action_SAVE_AS_ABOUT_TO_BE_SHOWN")]
  public const bits Action_SaveAsAboutToBeShown;
  [CCode (cname="action_SAVE_AS_DIALOGUE_COMPLETED")]
  public const bits Action_SaveAsDialogueCompleted;
  [CCode (cname="action_SAVE_AS_SAVE_TO_FILE")]
  public const bits Action_SaveAsSaveToFile;
  [CCode (cname="action_SAVE_AS_FILL_BUFFER")]
  public const bits Action_SaveAsFillBuffer;
  [CCode (cname="action_SAVE_AS_SAVE_COMPLETED")]
  public const bits Action_SaveAsSaveCompleted;
  [CCode (cname="error_SAVE_AS_TASKS_ACTIVE")]
  public const bits Error_SaveAsTasksActive;
  [CCode (cname="error_SAVE_AS_ALLOC_FAILED")]
  public const bits Error_SaveAsAllocFailed;
  [CCode (cname="error_SAVE_AS_SHORT_BUFFER")]
  public const bits Error_SaveAsShortBuffer;
  [CCode (cname="error_SAVE_AS_FILE_NAME_TOO_LONG")]
  public const bits Error_SaveAsFileNameTooLong;
  [CCode (cname="error_SAVE_AS_NO_SUCH_TASK")]
  public const bits Error_SaveAsNoSuchTask;
  [CCode (cname="error_SAVE_AS_NO_SUCH_METHOD")]
  public const bits Error_SaveAsNoSuchMethod;
  [CCode (cname="error_SAVE_AS_NO_SUCH_MISC_OP_METHOD")]
  public const bits Error_SaveAsNoSuchMiscOpMethod;
  [CCode (cname="error_SAVE_AS_NOT_TYPE1")]
  public const bits Error_SaveAsNotType1;
  [CCode (cname="error_SAVE_AS_NOT_TYPE3")]
  public const bits Error_SaveAsNotType3;
  [CCode (cname="error_SAVE_AS_BUFFER_EXCEEDED")]
  public const bits Error_SaveAsBufferExceeded;
  [CCode (cname="error_SAVE_AS_DATA_ADDRESS_UNSET")]
  public const bits Error_SaveAsDataAddressUnset;
  [CCode (cname="error_SAVE_AS_NOT_FULL_PATH")]
  public const bits Error_SaveAsNotFullPath;

  [CCode (cname = "xsaveas_get_window_id")]
  public static OS.Error* XGetWindowID
    (bits flags,
     Toolbox.O save_as,
     out Toolbox.O window);

  [CCode (cname = "saveas_get_window_id")]
  public static Toolbox.O GetWindowID
    (bits flags,
     Toolbox.O save_as);

  [CCode (cname = "xsaveas_set_title")]
  public static OS.Error* XSetTitle
    (bits flags,
     Toolbox.O save_as,
     string title);

  [CCode (cname = "saveas_set_title")]
  public static void SetTitle
    (bits flags,
     Toolbox.O save_as,
     string title);

  [CCode (cname = "xsaveas_get_title")]
  public static OS.Error* XGetTitle
    (bits flags,
     Toolbox.O save_as,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "saveas_get_title")]
  public static int GetTitle
    (bits flags,
     Toolbox.O save_as,
     string buffer,
     int size);

  [CCode (cname = "xsaveas_set_file_name")]
  public static OS.Error* XSetFileName
    (bits flags,
     Toolbox.O save_as,
     string file_name);

  [CCode (cname = "saveas_set_file_name")]
  public static void SetFileName
    (bits flags,
     Toolbox.O save_as,
     string file_name);

  [CCode (cname = "xsaveas_get_file_name")]
  public static OS.Error* XGetFileName
    (bits flags,
     Toolbox.O save_as,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "saveas_get_file_name")]
  public static int GetFileName
    (bits flags,
     Toolbox.O save_as,
     string buffer,
     int size);

  [CCode (cname = "xsaveas_set_file_type")]
  public static OS.Error* XSetFileType
    (bits flags,
     Toolbox.O save_as,
     bits file_type);

  [CCode (cname = "saveas_set_file_type")]
  public static void SetFileType
    (bits flags,
     Toolbox.O save_as,
     bits file_type);

  [CCode (cname = "xsaveas_get_file_type")]
  public static OS.Error* XGetFileType
    (bits flags,
     Toolbox.O save_as,
     out bits file_type);

  [CCode (cname = "saveas_get_file_type")]
  public static bits GetFileType
    (bits flags,
     Toolbox.O save_as);

  [CCode (cname = "xsaveas_set_file_size")]
  public static OS.Error* XSetFileSize
    (bits flags,
     Toolbox.O save_as,
     int file_size);

  [CCode (cname = "saveas_set_file_size")]
  public static void SetFileSize
    (bits flags,
     Toolbox.O save_as,
     int file_size);

  [CCode (cname = "xsaveas_get_file_size")]
  public static OS.Error* XGetFileSize
    (bits flags,
     Toolbox.O save_as,
     out int file_size);

  [CCode (cname = "saveas_get_file_size")]
  public static int GetFileSize
    (bits flags,
     Toolbox.O save_as);

  [CCode (cname = "xsaveas_selection_available")]
  public static OS.Error* XSelectionAvailable
    (bits flags,
     Toolbox.O save_as,
     bool selection_available);

  [CCode (cname = "saveas_selection_available")]
  public static void SelectionAvailable
    (bits flags,
     Toolbox.O save_as,
     bool selection_available);

  [CCode (cname = "xsaveas_set_data_address")]
  public static OS.Error* XSetDataAddress
    (bits flags,
     Toolbox.O save_as,
     char* data,
     int size,
     char* selection,
     int selection_size);

  [CCode (cname = "saveas_set_data_address")]
  public static void SetDataAddress
    (bits flags,
     Toolbox.O save_as,
     char* data,
     int size,
     char* selection,
     int selection_size);

  [CCode (cname = "xsaveas_buffer_filled")]
  public static OS.Error* XBufferFilled
    (bits flags,
     Toolbox.O save_as,
     char* buffer,
     int size);

  [CCode (cname = "saveas_buffer_filled")]
  public static void BufferFilled
    (bits flags,
     Toolbox.O save_as,
     char* buffer,
     int size);

  [CCode (cname = "xsaveas_file_save_completed")]
  public static OS.Error* XFileSaveCompleted
    (bits flags,
     Toolbox.O save_as,
     string file_name);

  [CCode (cname = "saveas_file_save_completed")]
  public static void FileSaveCompleted
    (bits flags,
     Toolbox.O save_as,
     string file_name);

}
