/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;

[CCode (cheader_filename = "oslib/taskmanager.h")]
namespace OSLib.TaskManager
{
  [CCode (cname = "taskmanager_flags")]
  public enum Flags
  {
    [CCode (cname = "taskmanager_MODULE_TASK")]
    ModuleTask,
    [CCode (cname = "taskmanager_DRAGGABLE_SLOT")]
    DraggableSlot
  }

  [CCode (cname = "taskmanager_task")]
  public struct Task
  {
    [CCode (cname = "task")]
    public unowned Wimp.T Task;
    [CCode (cname = "name")]
    public unowned char* Name;
    [CCode (cname = "slot_size")]
    public unowned int SlotSize;
    [CCode (cname = "flags")]
    public unowned Flags Flags;
  }

  [CCode (cname = "taskmanager_shutdown_flags")]
  public enum ShutdownFlags
  {
    [CCode (cname = "taskmanager_SHUTDOWN_EXIT_ONLY")]
    ShutdownExitOnly,
    [CCode (cname = "taskmanager_SHUTDOWN_NO_PRE_QUIT")]
    ShutdownNoPreQuit,
    [CCode (cname = "taskmanager_SHUTDOWN_PORTABLE_POWER_DOWN")]
    ShutdownPortablePowerDown,
    [CCode (cname = "taskmanager_SHUTDOWN_SEND_MESSAGE")]
    ShutdownSendMessage,
    [CCode (cname = "taskmanager_SHUTDOWN_IGNORE_UP_CALLS")]
    ShutdownIgnoreUpCalls,
    [CCode (cname = "taskmanager_SHUTDOWN_REBOOT")]
    ShutdownReboot,
    [CCode (cname = "taskmanager_SHUTDOWN_POWER_OFF")]
    ShutdownPowerOff
  }

  [CCode (cname = "taskmanager_group_id")]
  public enum GroupID
  {
    [CCode (cname = "taskmanager_GROUP_OVERVIEW")]
    GroupOverview,
    [CCode (cname = "taskmanager_GROUP_REQUIRED")]
    GroupRequired,
    [CCode (cname = "taskmanager_GROUP_SUBSIDIARY")]
    GroupSubsidiary,
    [CCode (cname = "taskmanager_GROUP_COMMERCIAL")]
    GroupCommercial,
    [CCode (cname = "taskmanager_GROUP_FREEWARE")]
    GroupFreeware,
    [CCode (cname = "taskmanager_GROUP_USER")]
    GroupUser,
    [CCode (cname = "taskmanager_GROUP_MASK")]
    GroupMask
  }

  [CCode (cname="taskmanager_NAME_LIMIT")]
  public const int NameLimit;

  [CCode (cname = "xtaskmanager_task_name_from_handle")]
  public static OS.Error* XTaskNameFromHandle
    (Wimp.T task,
     out string name);

  [CCode (cname = "taskmanager_task_name_from_handle")]
  public static string TaskNameFromHandle
    (Wimp.T task);

  [CCode (cname = "xtaskmanager_enumerate_tasks")]
  public static OS.Error* XEnumerateTasks
    (int context,
     Task task,
     int size,
     out int context_out,
     out char* end);

  [CCode (cname = "taskmanager_enumerate_tasks")]
  public static int EnumerateTasks
    (int context,
     Task task,
     int size,
     out char* end);

  [CCode (cname = "xtaskmanager_shutdown")]
  public static OS.Error* XShutdown
    (ShutdownFlags flags);

  [CCode (cname = "taskmanager_shutdown")]
  public static void Shutdown
    (ShutdownFlags flags);

  [CCode (cname = "xtaskmanager_start_task")]
  public static OS.Error* XStartTask
    (char* path_name);

  [CCode (cname = "taskmanager_start_task")]
  public static void StartTask
    (char* path_name);

  [CCode (cname = "xservice_task_manager_acknowledgements")]
  public static OS.Error* XService_TaskManagerAcknowledgements
    (AsmRoutine add_message);

  [CCode (cname = "service_task_manager_acknowledgements")]
  public static void Service_TaskManagerAcknowledgements
    (AsmRoutine add_message);

}
