/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.Wimp;
using OSLib.MessageTrans;
using OSLib.OSSpriteOp;

[CCode (cheader_filename = "oslib/toolbox.h")]
namespace OSLib.Toolbox
{
  [CCode (cname = "toolbox_o", has_type_id = false)]
  public struct O : bits
  {
  }

  [CCode (cname = "toolbox_c")]
  public enum C
  {
    [CCode (cname = "toolbox_NULL_COMPONENT")]
    NullComponent,
    [CCode (cname = "toolbox_WINDOW_FOCUS")]
    WindowFocus
  }

  [CCode (cname = "toolbox_class")]
  public struct Class : bits
  {
  }

  [CCode (cname = "toolbox_id", has_type_id = false)]
  public struct Id : bits
  {
  }

  [CCode (cname = "toolbox_position_tag")]
  public enum PositionTag
  {
    [CCode (cname = "toolbox_POSITION_DEFAULT")]
    PositionDefault,
    [CCode (cname = "toolbox_POSITION_FULL")]
    PositionFull,
    [CCode (cname = "toolbox_POSITION_TOP_LEFT")]
    PositionTopLeft,
    [CCode (cname = "toolbox_POSITION_CENTRED")]
    PositionCentred,
    [CCode (cname = "toolbox_POSITION_AT_POINTER")]
    PositionAtPointer
  }

  [CCode (cname = "toolbox_full")]
  public struct Full
  {
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned Wimp.W Next;
  }

  [CCode (cname = "toolbox_child")]
  public struct Child
  {
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned Wimp.W Next;
    [CCode (cname = "flags")]
    public unowned Wimp.WindowFlags Flags;
    [CCode (cname = "parent")]
    public unowned Wimp.W Parent;
    [CCode (cname = "linkage")]
    public unowned Wimp.WindowNestingFlags Linkage;
  }

  [CCode (cname = "toolbox_position")]
  public struct Position
  {
    [CCode (cname = "top_left")]
    public unowned OS.Coord TopLeft;
    [CCode (cname = "full")]
    public unowned Full Full;
    [CCode (cname = "child")]
    public unowned Child Child;
  }

  [CCode (cname = "toolbox_object_flags")]
  public enum ObjectFlags
  {
    [CCode (cname = "toolbox_OBJECT_CREATE_ON_LOAD")]
    ObjectCreateOnLoad,
    [CCode (cname = "toolbox_OBJECT_SHOW_ON_CREATE")]
    ObjectShowOnCreate,
    [CCode (cname = "toolbox_OBJECT_SHARED")]
    ObjectShared,
    [CCode (cname = "toolbox_OBJECT_ANCESTOR")]
    ObjectAncestor
  }

  [CCode (cname = "toolbox_block")]
  public struct Block
  {
    [CCode (cname = "ancestor_obj")]
    public unowned O AncestorObj;
    [CCode (cname = "ancestor_cmp")]
    public unowned C AncestorCmp;
    [CCode (cname = "parent_obj")]
    public unowned O ParentObj;
    [CCode (cname = "parent_cmp")]
    public unowned C ParentCmp;
    [CCode (cname = "this_obj")]
    public unowned O ThisObj;
    [CCode (cname = "this_cmp")]
    public unowned C ThisCmp;
  }

  [CCode (cname = "toolbox_resource_file_object_base")]
  public struct ResourceFileObjectBase
  {
    [CCode (cname = "class_no")]
    public unowned Class ClassNo;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "version")]
    public unowned int Version;
    [CCode (cname = "name")]
    public unowned char Name[12];
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "header_size")]
    public unowned int HeaderSize;
    [CCode (cname = "body_size")]
    public unowned int BodySize;
  }

  [Compact]
  [CCode (cname = "toolbox_resource_file_object", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ResourceFileObject
  {
    public ResourceFileObject(int numEntries);
    [CCode (cname = "class_no")]
    public unowned Class ClassNo;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "version")]
    public unowned int Version;
    [CCode (cname = "name")]
    public unowned char Name[12];
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "header_size")]
    public unowned int HeaderSize;
    [CCode (cname = "body_size")]
    public unowned int BodySize;
    [CCode (cname = "object", array_length = false)]
    public unowned int[] Object;
  }

  [CCode (cname = "toolbox_object_template_header")]
  public enum ObjectTemplateHeader
  {
    Default = 0
  }

  [CCode (cname = "toolbox_object_descriptor", has_type_id = false)]
  public struct ObjectDescriptor : bits
  {
  }

  [CCode (cname = "toolbox_template")]
  public struct Template
  {
    [CCode (cname = "class_no")]
    public unowned Class ClassNo;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "version")]
    public unowned int Version;
    [CCode (cname = "name")]
    public unowned char Name[12];
    [CCode (cname = "reserved")]
    public unowned int Reserved;
    [CCode (cname = "class_template")]
    public unowned ObjectDescriptor ClassTemplate;
    [CCode (cname = "body_size")]
    public unowned int BodySize;
  }

  [CCode (cname = "toolbox_string_reference")]
  public enum StringReference
  {
    Default = 0
  }

  [CCode (cname = "toolbox_msg_reference")]
  public enum MsgReference
  {
    Default = 0
  }

  [CCode (cname = "toolbox_sprite_area_reference")]
  public enum SpriteAreaReference
  {
    Default = 0
  }

  [CCode (cname = "toolbox_object_offset")]
  public enum ObjectOffset
  {
    Default = 0
  }

  [CCode (cname = "toolbox_relocation_directive")]
  public enum RelocationDirective
  {
    [CCode (cname = "toolbox_RELOCATE_STRING_REFERENCE")]
    RelocateStringReference,
    [CCode (cname = "toolbox_RELOCATE_MSG_REFERENCE")]
    RelocateMsgReference,
    [CCode (cname = "toolbox_RELOCATE_SPRITE_AREA_REFERENCE")]
    RelocateSpriteAreaReference,
    [CCode (cname = "toolbox_RELOCATE_OBJECT_OFFSET")]
    RelocateObjectOffset
  }

  [CCode (cname = "toolbox_relocation")]
  public struct Relocation
  {
    [CCode (cname = "word")]
    public unowned int Word;
    [CCode (cname = "relocation")]
    public unowned RelocationDirective Relocation;
  }

  [CCode (cname = "toolbox_relocation_table_base")]
  public struct RelocationTableBase
  {
    [CCode (cname = "relocation_count")]
    public unowned int RelocationCount;
  }

  [Compact]
  [CCode (cname = "toolbox_relocation_table", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class RelocationTable
  {
    public RelocationTable(int numEntries);
    [CCode (cname = "relocation_count")]
    public unowned int RelocationCount;
    [CCode (cname = "relocations", array_length = false)]
    public unowned Relocation[] Relocations;
  }

  [CCode (cname = "toolbox_relocatable_object")]
  public struct RelocatableObject
  {
    [CCode (cname = "string_table_offset")]
    public unowned int StringTableOffset;
    [CCode (cname = "message_table_offset")]
    public unowned int MessageTableOffset;
    [CCode (cname = "relocation_table_offset")]
    public unowned int RelocationTableOffset;
    [CCode (cname = "rf_obj")]
    public unowned ResourceFileObject RfObj;
  }

  [CCode (cname = "toolbox_resource_file_base")]
  public struct ResourceFileBase
  {
    [CCode (cname = "file_id")]
    public unowned int FileId;
    [CCode (cname = "version")]
    public unowned int Version;
    [CCode (cname = "header_size")]
    public unowned int HeaderSize;
  }

  [Compact]
  [CCode (cname = "toolbox_resource_file", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ResourceFile
  {
    public ResourceFile(int numEntries);
    [CCode (cname = "file_id")]
    public unowned int FileId;
    [CCode (cname = "version")]
    public unowned int Version;
    [CCode (cname = "header_size")]
    public unowned int HeaderSize;
    [CCode (cname = "rel_obj", array_length = false)]
    public unowned RelocatableObject[] RelObj;
  }

  [CCode (cname = "toolbox_action_list")]
  public struct ActionList
  {
    [CCode (cname = "action_nos", array_length = false)]
    public unowned bits[] ActionNos;
  }

  [CCode (cname = "toolbox_create_flags")]
  public enum CreateFlags
  {
    [CCode (cname = "toolbox_CREATE_GIVEN_OBJECT")]
    CreateGivenObject
  }

  [CCode (cname = "toolbox_delete_flags")]
  public enum DeleteFlags
  {
    [CCode (cname = "toolbox_DELETE_NO_RECURSE")]
    DeleteNoRecurse
  }

  [CCode (cname = "toolbox_show_flags")]
  public enum ShowFlags
  {
    [CCode (cname = "toolbox_SHOW_AS_MENU")]
    ShowAsMenu,
    [CCode (cname = "toolbox_SHOW_AS_SUB_MENU")]
    ShowAsSubMenu,
    [CCode (cname = "toolbox_SHOW_AS_CHILD_WINDOW")]
    ShowAsChildWindow
  }

  [CCode (cname = "toolbox_hide_flags")]
  public enum HideFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_get_info_flags")]
  public enum GetInfoFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_info")]
  public enum Info
  {
    [CCode (cname = "toolbox_INFO_SHOWING")]
    Showing
  }

  [CCode (cname = "toolbox_misc_op_flags")]
  public enum MiscOpFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_handle_flags")]
  public enum HandleFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_get_class_flags")]
  public enum GetClassFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_get_parent_flags")]
  public enum GetParentFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_get_ancestor_flags")]
  public enum GetAncestorFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_get_template_flags")]
  public enum GetTemplateFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_event_flags")]
  public enum EventFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_initialise_flags")]
  public enum InitialiseFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_load_flags")]
  public enum LoadFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_look_up_flags")]
  public enum LookUpFlags
  {
    Default = 0
  }

  [CCode (cname = "toolbox_filter_type")]
  public enum FilterType
  {
    [CCode (cname = "toolbox_REGISTER_EVENT_FILTER")]
    RegisterEventFilter,
    [CCode (cname = "toolbox_REGISTER_MESSAGE_FILTER")]
    RegisterMessageFilter,
    [CCode (cname = "toolbox_REGISTER_ACTION_FILTER")]
    RegisterActionFilter
  }

  [CCode (cname = "toolbox_event_pair")]
  public struct EventPair
  {
    [CCode (cname = "event")]
    public unowned int Event;
    [CCode (cname = "class_no")]
    public unowned Class ClassNo;
  }

  [CCode (cname = "toolbox_event_pair_list")]
  public struct EventPairList
  {
    [CCode (cname = "pair", array_length = false)]
    public unowned EventPair[] Pair;
  }

  [CCode (cname = "toolbox_message_pair")]
  public struct MessagePair
  {
    [CCode (cname = "message")]
    public unowned bits Message;
    [CCode (cname = "class_no")]
    public unowned Class ClassNo;
  }

  [CCode (cname = "toolbox_message_pair_list")]
  public struct MessagePairList
  {
    [CCode (cname = "pair", array_length = false)]
    public unowned MessagePair[] Pair;
  }

  [CCode (cname = "toolbox_action_pair")]
  public struct ActionPair
  {
    [CCode (cname = "action_no")]
    public unowned bits ActionNo;
    [CCode (cname = "class_no")]
    public unowned Class ClassNo;
  }

  [CCode (cname = "toolbox_action_pair_list")]
  public struct ActionPairList
  {
    [CCode (cname = "pair", array_length = false)]
    public unowned ActionPair[] Pair;
  }

  [CCode (cname = "toolbox_pair_list")]
  public struct PairList
  {
    [CCode (cname = "events")]
    public unowned EventPairList Events;
    [CCode (cname = "messages")]
    public unowned MessagePairList Messages;
    [CCode (cname = "action_pairs")]
    public unowned ActionPairList ActionPairs;
  }

  [CCode (cname = "toolbox_register_flags")]
  public enum RegisterFlags
  {
    [CCode (cname = "toolbox_DE_REGISTER_FILTER")]
    DeRegisterFilter
  }

  [CCode (cname = "toolbox_action_interest")]
  public struct ActionInterest
  {
    [CCode (cname = "action_no")]
    public unowned bits ActionNo;
    [CCode (cname = "class_no")]
    public unowned Class ClassNo;
  }

  [CCode (cname = "toolbox_action_header_base")]
  public struct ActionHeaderBase
  {
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "ref_no")]
    public unowned int RefNo;
    [CCode (cname = "action_no")]
    public unowned bits ActionNo;
    [CCode (cname = "flags")]
    public unowned bits Flags;
  }

  [Compact]
  [CCode (cname = "toolbox_action_header", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class ActionHeader
  {
    public ActionHeader(int numEntries);
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "ref_no")]
    public unowned int RefNo;
    [CCode (cname = "action_no")]
    public unowned bits ActionNo;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "action", array_length = false)]
    public unowned byte[] Action;
  }

  [CCode (cname = "toolbox_action_error")]
  public struct ActionError
  {
    [CCode (cname = "errnum")]
    public unowned bits Errnum;
    [CCode (cname = "errmess")]
    public unowned char Errmess[208];
  }

  [CCode (cname = "toolbox_action_error_block")]
  public struct ActionErrorBlock : ActionHeaderBase
  {
    [CCode (cname = "errnum")]
    public unowned bits Errnum;
    [CCode (cname = "errmess")]
    public unowned char Errmess[208];
  }

  [CCode (cname = "toolbox_action_object_auto_created")]
  public struct ActionObjectAutoCreated
  {
    [CCode (cname = "name")]
    public unowned char Name[212];
  }

  [CCode (cname = "toolbox_action_object_auto_created_block")]
  public struct ActionObjectAutoCreatedBlock : ActionHeaderBase
  {
    [CCode (cname = "name")]
    public unowned char Name[212];
  }

  [CCode (cname = "toolbox_action_object_deleted")]
  public struct ActionObjectDeleted
  {
    [CCode (cname = "class_no")]
    public unowned Class ClassNo;
    [CCode (cname = "client_handle")]
    public unowned char** ClientHandle;
  }

  [CCode (cname = "toolbox_action_object_deleted_block")]
  public struct ActionObjectDeletedBlock : ActionHeaderBase
  {
    [CCode (cname = "class_no")]
    public unowned Class ClassNo;
    [CCode (cname = "client_handle")]
    public unowned char** ClientHandle;
  }

  [CCode (cname = "toolbox_action")]
  public struct Action
  {
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "ref_no")]
    public unowned int RefNo;
    [CCode (cname = "action_no")]
    public unowned bits ActionNo;
    [CCode (cname = "flags")]
    public unowned bits Flags;
    [CCode (cname = "data.error")]
    public unowned ActionError Data_Error;
    [CCode (cname = "data.created")]
    public unowned ActionObjectAutoCreated Data_Created;
    [CCode (cname = "data.deleted")]
    public unowned ActionObjectDeleted Data_Deleted;
    [CCode (cname = "data.reserved")]
    public unowned byte Data_Reserved[212];
  }

  [CCode (cname = "toolbox_message")]
  public struct Message
  {
    [CCode (cname = "o")]
    public unowned O O;
    [CCode (cname = "c")]
    public unowned C C;
    [CCode (cname = "action_no")]
    public unowned Action ActionNo;
  }

  [CCode (cname = "toolbox_full_message")]
  public struct FullMessage : Wimp.MessageHeaderBase
  {
    [CCode (cname = "o")]
    public unowned O O;
    [CCode (cname = "c")]
    public unowned C C;
    [CCode (cname = "action_no")]
    public unowned Action ActionNo;
  }

  [CCode (cname="toolbox_NAME_LIMIT")]
  public const int NameLimit;
  [CCode (cname="toolbox_WIMP_OBJECTS")]
  public const Class WimpObjects;
  [CCode (cname="toolbox_ALL")]
  public const Class All;
  [CCode (cname="toolbox_EVENT")]
  public const bits Event;
  [CCode (cname="error_TOOLBOX_NO_MEM")]
  public const bits Error_ToolboxNoMem;
  [CCode (cname="error_TOOLBOX_BAD_SWI")]
  public const bits Error_ToolboxBadSWI;
  [CCode (cname="error_TOOLBOX_INVALID_OBJECT_ID")]
  public const bits Error_ToolboxInvalidObjectID;
  [CCode (cname="error_TOOLBOX_NOT_ATOOLBOX_TASK")]
  public const bits Error_ToolboxNotAToolboxTask;
  [CCode (cname="error_TOOLBOX_NO_DIR_NAME")]
  public const bits Error_ToolboxNoDirName;
  [CCode (cname="error_TOOLBOX_NO_MSGS_FD")]
  public const bits Error_ToolboxNoMsgsFD;
  [CCode (cname="error_TOOLBOX_NO_ID_BLOCK")]
  public const bits Error_ToolboxNoIDBlock;
  [CCode (cname="error_TOOLBOX_BAD_RES_FILE")]
  public const bits Error_ToolboxBadResFile;
  [CCode (cname="error_TOOLBOX_TASKS_ACTIVE")]
  public const bits Error_ToolboxTasksActive;
  [CCode (cname="error_TOOLBOX_TEMPLATE_NOT_FOUND")]
  public const bits Error_ToolboxTemplateNotFound;
  [CCode (cname="error_TOOLBOX_NO_SUCH_PRE_FILTER")]
  public const bits Error_ToolboxNoSuchPreFilter;
  [CCode (cname="error_TOOLBOX_NOT_ARES_FILE")]
  public const bits Error_ToolboxNotAResFile;
  [CCode (cname="error_TOOLBOX_BAD_RES_FILE_VERSION")]
  public const bits Error_ToolboxBadResFileVersion;
  [CCode (cname="error_TOOLBOX_BAD_FLAGS")]
  public const bits Error_ToolboxBadFlags;
  [CCode (cname="toolbox_RESOURCE_FILE_VERSION")]
  public const int ResourceFileVersion;
  [CCode (cname="toolbox_WIMP_OBJECT_POST_FILTER")]
  public const Class WimpObjectPostFilter;
  [CCode (cname="toolbox_ANY_POST_FILTER")]
  public const Class AnyPostFilter;
  [CCode (cname="action_ERROR")]
  public const bits Action_Error;
  [CCode (cname="action_OBJECT_AUTO_CREATED")]
  public const bits Action_ObjectAutoCreated;
  [CCode (cname="action_OBJECT_DELETED")]
  public const bits Action_ObjectDeleted;
  [CCode (cname="toolbox_ACTION_OBJECT_DELETED_EXTENDED_INFORMATION")]
  public const bits ActionObjectDeletedExtendedInformation;

  [CCode (cname = "xtoolbox_create_object")]
  public static OS.Error* XCreateObject
    (CreateFlags flags,
     Id id,
     out O obj);

  [CCode (cname = "toolbox_create_object")]
  public static O CreateObject
    (CreateFlags flags,
     Id id);

  [CCode (cname = "xtoolbox_create_object_from_template")]
  public static OS.Error* XCreateObjectFromTemplate
    (CreateFlags flags,
     Template toolbox_template,
     out O obj);

  [CCode (cname = "toolbox_create_object_from_template")]
  public static O CreateObjectFromTemplate
    (CreateFlags flags,
     Template toolbox_template);

  [CCode (cname = "xtoolbox_delete_object")]
  public static OS.Error* XDeleteObject
    (DeleteFlags flags,
     O obj);

  [CCode (cname = "toolbox_delete_object")]
  public static void DeleteObject
    (DeleteFlags flags,
     O obj);

  [CCode (cname = "xtoolbox_show_object")]
  public static OS.Error* XShowObject
    (ShowFlags flags,
     O obj,
     PositionTag tag,
     Position position,
     O parent_obj,
     C parent_cmp);

  [CCode (cname = "toolbox_show_object")]
  public static void ShowObject
    (ShowFlags flags,
     O obj,
     PositionTag tag,
     Position position,
     O parent_obj,
     C parent_cmp);

  [CCode (cname = "xtoolbox_hide_object")]
  public static OS.Error* XHideObject
    (HideFlags flags,
     O obj);

  [CCode (cname = "toolbox_hide_object")]
  public static void HideObject
    (HideFlags flags,
     O obj);

  [CCode (cname = "xtoolbox_get_object_info")]
  public static OS.Error* XGetObjectInfo
    (GetInfoFlags flags,
     O obj,
     out Info info);

  [CCode (cname = "toolbox_get_object_info")]
  public static Info GetObjectInfo
    (GetInfoFlags flags,
     O obj);

  [CCode (cname = "xtoolbox_set_client_handle")]
  public static OS.Error* XSetClientHandle
    (HandleFlags flags,
     O obj,
     char* handle);

  [CCode (cname = "toolbox_set_client_handle")]
  public static void SetClientHandle
    (HandleFlags flags,
     O obj,
     char* handle);

  [CCode (cname = "xtoolbox_get_client_handle")]
  public static OS.Error* XGetClientHandle
    (HandleFlags flags,
     O obj,
     out char* handle);

  [CCode (cname = "toolbox_get_client_handle")]
  public static char* GetClientHandle
    (HandleFlags flags,
     O obj);

  [CCode (cname = "xtoolbox_get_object_class")]
  public static OS.Error* XGetObjectClass
    (GetClassFlags flags,
     O obj,
     out Class class_no);

  [CCode (cname = "toolbox_get_object_class")]
  public static Class GetObjectClass
    (GetClassFlags flags,
     O obj);

  [CCode (cname = "xtoolbox_get_parent")]
  public static OS.Error* XGetParent
    (GetParentFlags flags,
     O obj,
     out O parent_obj,
     out C parent_cmp);

  [CCode (cname = "toolbox_get_parent")]
  public static void GetParent
    (GetParentFlags flags,
     O obj,
     out O parent_obj,
     out C parent_cmp);

  [CCode (cname = "xtoolbox_get_ancestor")]
  public static OS.Error* XGetAncestor
    (GetAncestorFlags flags,
     O obj,
     out O ancestor_obj,
     out C ancestor_cmp);

  [CCode (cname = "toolbox_get_ancestor")]
  public static void GetAncestor
    (GetAncestorFlags flags,
     O obj,
     out O ancestor_obj,
     out C ancestor_cmp);

  [CCode (cname = "xtoolbox_get_template_name")]
  public static OS.Error* XGetTemplateName
    (GetTemplateFlags flags,
     O obj,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "toolbox_get_template_name")]
  public static int GetTemplateName
    (GetTemplateFlags flags,
     O obj,
     string buffer,
     int size);

  [CCode (cname = "xtoolbox_raise_toolbox_event")]
  public static OS.Error* XRaiseToolboxEvent
    (EventFlags flags,
     O obj,
     C cmp,
     Action action);

  [CCode (cname = "toolbox_raise_toolbox_event")]
  public static void RaiseToolboxEvent
    (EventFlags flags,
     O obj,
     C cmp,
     Action action);

  [CCode (cname = "xtoolboxgetsysinfo_task_name")]
  public static OS.Error* XToolboxGetSysInfo_TaskName
    (string buffer,
     int size,
     out int used);

  [CCode (cname = "toolboxgetsysinfo_task_name")]
  public static int ToolboxGetSysInfo_TaskName
    (string buffer,
     int size);

  [CCode (cname = "xtoolboxgetsysinfo_message_trans_control_block")]
  public static OS.Error* XToolboxGetSysInfo_MessageTransControlBlock
    (MessageTrans.ControlBlock cb);

  [CCode (cname = "toolboxgetsysinfo_message_trans_control_block")]
  public static void ToolboxGetSysInfo_MessageTransControlBlock
    (MessageTrans.ControlBlock cb);

  [CCode (cname = "xtoolboxgetsysinfo_dir_name")]
  public static OS.Error* XToolboxGetSysInfo_DirName
    (string buffer,
     int size,
     out int used);

  [CCode (cname = "toolboxgetsysinfo_dir_name")]
  public static int ToolboxGetSysInfo_DirName
    (string buffer,
     int size);

  [CCode (cname = "xtoolboxgetsysinfo_task")]
  public static OS.Error* XToolboxGetSysInfo_Task
    (out Wimp.T task);

  [CCode (cname = "toolboxgetsysinfo_task")]
  public static Wimp.T ToolboxGetSysInfo_Task ();

  [CCode (cname = "xtoolboxgetsysinfo_sprite_area")]
  public static OS.Error* XToolboxGetSysInfo_SpriteArea
    (out OSSpriteOp.Area area);

  [CCode (cname = "toolboxgetsysinfo_sprite_area")]
  public static OSSpriteOp.Area ToolboxGetSysInfo_SpriteArea ();

  [CCode (cname = "xtoolbox_initialise")]
  public static OS.Error* XInitialise
    (InitialiseFlags flags,
     int version,
     Wimp.MessageList messages,
     ActionList action_nos,
     string resource_dir_name,
     MessageTrans.ControlBlock cb,
     Block block,
     out int version_out,
     out Wimp.T task_out,
     out OSSpriteOp.Area area);

  [CCode (cname = "toolbox_initialise")]
  public static Wimp.T Initialise
    (InitialiseFlags flags,
     int version,
     Wimp.MessageList messages,
     ActionList action_nos,
     string resource_dir_name,
     MessageTrans.ControlBlock cb,
     Block block,
     out int version_out,
     out OSSpriteOp.Area area);

  [CCode (cname = "xtoolbox_load_resources")]
  public static OS.Error* XLoadResources
    (LoadFlags flags,
     string resource_file_name);

  [CCode (cname = "toolbox_load_resources")]
  public static void LoadResources
    (LoadFlags flags,
     string resource_file_name);

  [CCode (cname = "xtoolboxmemory_allocate")]
  public static OS.Error* XToolboxMemory_Allocate
    (int size,
     out char* blk);

  [CCode (cname = "toolboxmemory_allocate")]
  public static char* ToolboxMemory_Allocate
    (int size);

  [CCode (cname = "xtoolboxmemory_free")]
  public static OS.Error* XToolboxMemory_Free
    (char* blk);

  [CCode (cname = "toolboxmemory_free")]
  public static void ToolboxMemory_Free
    (char* blk);

  [CCode (cname = "xtoolboxmemory_extend")]
  public static OS.Error* XToolboxMemory_Extend
    (char* blk,
     int change_by,
     out char* blk_out);

  [CCode (cname = "toolboxmemory_extend")]
  public static char* ToolboxMemory_Extend
    (char* blk,
     int change_by);

  [CCode (cname = "xtoolbox_de_register_object_module")]
  public static OS.Error* XDeRegisterObjectModule
    (bits flags,
     Class class_no);

  [CCode (cname = "toolbox_de_register_object_module")]
  public static void DeRegisterObjectModule
    (bits flags,
     Class class_no);

  [CCode (cname = "xtoolbox_template_look_up")]
  public static OS.Error* XTemplateLookUp
    (LookUpFlags flags,
     string object_name,
     out Id id);

  [CCode (cname = "toolbox_template_look_up")]
  public static Id TemplateLookUp
    (LookUpFlags flags,
     string object_name);

  [CCode (cname = "xtoolbox_register_object_module")]
  public static OS.Error* XRegisterObjectModule
    (bits flags,
     Class class_no,
     bits swi_base,
     string resource_file_name);

  [CCode (cname = "toolbox_register_object_module")]
  public static void RegisterObjectModule
    (bits flags,
     Class class_no,
     bits swi_base,
     string resource_file_name);

  [CCode (cname = "xtoolbox_register_pre_filter")]
  public static OS.Error* XRegisterPreFilter
    (RegisterFlags flags,
     bits swi_no,
     char* handle);

  [CCode (cname = "toolbox_register_pre_filter")]
  public static void RegisterPreFilter
    (RegisterFlags flags,
     bits swi_no,
     char* handle);

  [CCode (cname = "xtoolbox_register_post_filter")]
  public static OS.Error* XRegisterPostFilter
    (RegisterFlags flags,
     bits swi_no,
     char* handle,
     FilterType filter_type,
     PairList pairs);

  [CCode (cname = "toolbox_register_post_filter")]
  public static void RegisterPostFilter
    (RegisterFlags flags,
     bits swi_no,
     char* handle,
     FilterType filter_type,
     PairList pairs);

  [CCode (cname = "xservice_toolbox_starting")]
  public static OS.Error* XService_ToolboxStarting ();

  [CCode (cname = "service_toolbox_starting")]
  public static void Service_ToolboxStarting ();

  [CCode (cname = "xservice_toolbox_task_born")]
  public static OS.Error* XService_ToolboxTaskBorn
    (Wimp.T task);

  [CCode (cname = "service_toolbox_task_born")]
  public static void Service_ToolboxTaskBorn
    (Wimp.T task);

  [CCode (cname = "xservice_toolbox_task_died")]
  public static OS.Error* XService_ToolboxTaskDied
    (Wimp.T task);

  [CCode (cname = "service_toolbox_task_died")]
  public static void Service_ToolboxTaskDied
    (Wimp.T task);

  [CCode (cname = "xservice_toolbox_dying")]
  public static OS.Error* XService_ToolboxDying ();

  [CCode (cname = "service_toolbox_dying")]
  public static void Service_ToolboxDying ();

  [CCode (cname = "xservice_window_deleted")]
  public static OS.Error* XService_WindowDeleted
    (Wimp.W w,
     Wimp.T task);

  [CCode (cname = "service_window_deleted")]
  public static void Service_WindowDeleted
    (Wimp.W w,
     Wimp.T task);

  [CCode (cname = "xservice_redrawing_window")]
  public static OS.Error* XService_RedrawingWindow
    (int type);

  [CCode (cname = "service_redrawing_window")]
  public static void Service_RedrawingWindow
    (int type);

}
