/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;

[CCode (cheader_filename = "oslib/url.h")]
namespace OSLib.URL
{
  [CCode (cname = "url_s", has_type_id = false)]
  public struct S : bits
  {
  }

  [CCode (cname = "url_get_url_flags")]
  public enum GetURLFlags
  {
    [CCode (cname = "url_GET_URL_USER_AGENT_SUPPLIED")]
    GetURLUserAgentSupplied,
    [CCode (cname = "url_GET_URL_DATA_LENGTH_SUPPLIED")]
    GetURLDataLengthSupplied
  }

  [CCode (cname = "url_parse_url_flags")]
  public enum ParseURLFlags
  {
    [CCode (cname = "url_PARSE_URL_DATA_LENGTH_SUPPLIED")]
    ParseURLDataLengthSupplied,
    [CCode (cname = "url_PARSE_URL_ESCAPE_CONTROL_CHARS")]
    ParseURLEscapeControlChars
  }

  [CCode (cname = "url_method")]
  public enum Method
  {
    [CCode (cname = "url_METHOD_CODE")]
    CODE,
    [CCode (cname = "url_METHOD_FLAGS")]
    FLAGS
  }

  [CCode (cname = "url_method_code")]
  public enum MethodCode
  {
    [CCode (cname = "url_GET")]
    GET,
    [CCode (cname = "url_HEAD")]
    HEAD,
    [CCode (cname = "url_OPTIONS")]
    OPTIONS,
    [CCode (cname = "url_POST")]
    POST,
    [CCode (cname = "url_TRACE")]
    TRACE,
    [CCode (cname = "url_PUT")]
    PUT,
    [CCode (cname = "url_DELETE")]
    DELETE,
    [CCode (cname = "url_RETR")]
    RETR,
    [CCode (cname = "url_LIST")]
    LIST,
    [CCode (cname = "url_STOR")]
    STOR,
    [CCode (cname = "url_MKD")]
    MKD,
    [CCode (cname = "url_RMD")]
    RMD,
    [CCode (cname = "url_RNFR")]
    RNFR,
    [CCode (cname = "url_RNTO")]
    RNTO,
    [CCode (cname = "url_DELE")]
    DELE,
    [CCode (cname = "url_STOU")]
    STOU
  }

  [CCode (cname = "url_method_flags")]
  public enum MethodFlags
  {
    Default = 0
  }

  [CCode (cname = "url_fetch_status")]
  public enum FetchStatus
  {
    [CCode (cname = "url_CONNECTED")]
    Connected,
    [CCode (cname = "url_SENT_REQUEST")]
    SentRequest,
    [CCode (cname = "url_SENT_DATA")]
    SentData,
    [CCode (cname = "url_RESPONSE_RECEIVED")]
    ResponseReceived,
    [CCode (cname = "url_TRANSFER_IN_PROGRESS")]
    TransferInProgress,
    [CCode (cname = "url_ALL_DATA_RECEIVED")]
    AllDataReceived,
    [CCode (cname = "url_TRANSFER_ABORTED")]
    TransferAborted
  }

  [CCode (cname = "url_components")]
  public struct Components
  {
    [CCode (cname = "full_url")]
    public unowned string FullUrl;
    [CCode (cname = "scheme")]
    public unowned string Scheme;
    [CCode (cname = "host")]
    public unowned string Host;
    [CCode (cname = "port")]
    public unowned string Port;
    [CCode (cname = "user")]
    public unowned string User;
    [CCode (cname = "password")]
    public unowned string Password;
    [CCode (cname = "account")]
    public unowned string Account;
    [CCode (cname = "path")]
    public unowned string Path;
    [CCode (cname = "query")]
    public unowned string Query;
    [CCode (cname = "fragment")]
    public unowned string Fragment;
  }

  [CCode (cname = "url_component_sizes")]
  public struct ComponentSizes
  {
    [CCode (cname = "full_url")]
    public unowned int FullUrl;
    [CCode (cname = "scheme")]
    public unowned int Scheme;
    [CCode (cname = "host")]
    public unowned int Host;
    [CCode (cname = "port")]
    public unowned int Port;
    [CCode (cname = "user")]
    public unowned int User;
    [CCode (cname = "password")]
    public unowned int Password;
    [CCode (cname = "account")]
    public unowned int Account;
    [CCode (cname = "path")]
    public unowned int Path;
    [CCode (cname = "query")]
    public unowned int Query;
    [CCode (cname = "fragment")]
    public unowned int Fragment;
  }

  [CCode (cname = "url_protocol_register_flags")]
  public enum ProtocolRegisterFlags
  {
    [CCode (cname = "url_HAS_PROTOCOL_FLAGS")]
    HasProtocolFlags,
    [CCode (cname = "url_HAS_DEFAULT_PORT")]
    HasDefaultPort
  }

  [CCode (cname = "url_protocol_flags")]
  public enum ProtocolFlags
  {
    [CCode (cname = "url_NON_UNIX_PATH")]
    NonUnixPath,
    [CCode (cname = "url_DONT_PARSE")]
    DontParse,
    [CCode (cname = "url_ALLOW_USERNAME")]
    AllowUsername,
    [CCode (cname = "url_ALLOW_HASH_IN_HOSTNAME")]
    AllowHashInHostname,
    [CCode (cname = "url_NO_HOSTNAME")]
    NoHostname,
    [CCode (cname = "url_REMOVE_LEADING_UP_COMPONENTS")]
    RemoveLeadingUpComponents
  }

  [CCode (cname="url_METHOD_CODE_SHIFT")]
  public const int MethodCodeShift;
  [CCode (cname="url_METHOD_FLAGS_SHIFT")]
  public const int MethodFlagsShift;
  [CCode (cname="error_URL_BAD_SESSION")]
  public const bits Error_URLBadSession;
  [CCode (cname="error_URL_NO_MEMORY")]
  public const bits Error_URLNoMemory;
  [CCode (cname="error_URL_NO_FETCHER")]
  public const bits Error_URLNoFetcher;
  [CCode (cname="error_URL_UNKNOWN_SWI")]
  public const bits Error_URLUnknownSWI;
  [CCode (cname="error_URL_BAD_FETCH")]
  public const bits Error_URLBadFetch;
  [CCode (cname="error_URL_FETCH_FINISHED")]
  public const bits Error_URLFetchFinished;
  [CCode (cname="error_URL_PROTOCOL_CLAIMED")]
  public const bits Error_URLProtocolClaimed;
  [CCode (cname="error_URL_NO_FETCH")]
  public const bits Error_URLNoFetch;
  [CCode (cname="error_URL_BAD_MESSAGE")]
  public const bits Error_URLBadMessage;
  [CCode (cname="error_URL_NOT_PARSEABLE")]
  public const bits Error_URLNotParseable;

  [CCode (cname = "xurl_register")]
  public static OS.Error* XRegister
    (bits flags,
     out S session);

  [CCode (cname = "url_register")]
  public static S Register
    (bits flags);

  [CCode (cname = "xurl_get_url")]
  public static OS.Error* XGetURL
    (GetURLFlags flags,
     S session,
     Method method,
     string url,
     char* data,
     int data_length,
     string user_agent,
     out FetchStatus status);

  [CCode (cname = "url_get_url")]
  public static FetchStatus GetURL
    (GetURLFlags flags,
     S session,
     Method method,
     string url,
     char* data,
     int data_length,
     string user_agent);

  [CCode (cname = "xurl_status")]
  public static OS.Error* XStatus
    (bits flags,
     S session,
     out FetchStatus status,
     out bits response_code,
     out int bytes_read,
     out int total_size);

  [CCode (cname = "url_status")]
  public static FetchStatus Status
    (bits flags,
     S session,
     out bits response_code,
     out int bytes_read,
     out int total_size);

  [CCode (cname = "xurl_read_data")]
  public static OS.Error* XReadData
    (bits flags,
     S session,
     char* buffer,
     int buffer_size,
     out FetchStatus status,
     out int bytes_read,
     out int bytes_remaining);

  [CCode (cname = "url_read_data")]
  public static int ReadData
    (bits flags,
     S session,
     char* buffer,
     int buffer_size,
     out FetchStatus status,
     out int bytes_remaining);

  [CCode (cname = "xurl_set_proxy__enable")]
  public static OS.Error* XSetProxy_Enable
    (bits flags,
     S session,
     string proxy_url,
     string scheme,
     out FetchStatus status);

  [CCode (cname = "url_set_proxy__enable")]
  public static FetchStatus SetProxy_Enable
    (bits flags,
     S session,
     string proxy_url,
     string scheme);

  [CCode (cname = "xurl_set_proxy__disable")]
  public static OS.Error* XSetProxy_Disable
    (bits flags,
     S session,
     string proxy_url,
     out FetchStatus status);

  [CCode (cname = "url_set_proxy__disable")]
  public static FetchStatus SetProxy_Disable
    (bits flags,
     S session,
     string proxy_url);

  [CCode (cname = "xurl_stop")]
  public static OS.Error* XStop
    (bits flags,
     S session,
     out FetchStatus status);

  [CCode (cname = "url_stop")]
  public static FetchStatus Stop
    (bits flags,
     S session);

  [CCode (cname = "xurl_deregister")]
  public static OS.Error* XDeregister
    (bits flags,
     S session,
     out FetchStatus status);

  [CCode (cname = "url_deregister")]
  public static FetchStatus Deregister
    (bits flags,
     S session);

  [CCode (cname = "xurlparseurl_return_lengths")]
  public static OS.Error* XURLParseURL_ReturnLengths
    (ParseURLFlags flags,
     string base_url,
     string relative_url,
     ComponentSizes block,
     int block_size,
     out bits flags_out);

  [CCode (cname = "urlparseurl_return_lengths")]
  public static bits URLParseURL_ReturnLengths
    (ParseURLFlags flags,
     string base_url,
     string relative_url,
     ComponentSizes block,
     int block_size);

  [CCode (cname = "xurlparseurl_return_data")]
  public static OS.Error* XURLParseURL_ReturnData
    (ParseURLFlags flags,
     string base_url,
     string relative_url,
     Components block,
     int block_size,
     out bits flags_out);

  [CCode (cname = "urlparseurl_return_data")]
  public static bits URLParseURL_ReturnData
    (ParseURLFlags flags,
     string base_url,
     string relative_url,
     Components block,
     int block_size);

  [CCode (cname = "xurlparseurl_compose_from_components")]
  public static OS.Error* XURLParseURL_ComposeFromComponents
    (ParseURLFlags flags,
     Components block,
     int block_size,
     out bits flags_out);

  [CCode (cname = "urlparseurl_compose_from_components")]
  public static bits URLParseURL_ComposeFromComponents
    (ParseURLFlags flags,
     Components block,
     int block_size);

  [CCode (cname = "xurlparseurl_quick_resolve")]
  public static OS.Error* XURLParseURL_QuickResolve
    (ParseURLFlags flags,
     string base_url,
     string relative_url,
     string url_buffer,
     int url_buffer_size,
     out bits flags_out);

  [CCode (cname = "urlparseurl_quick_resolve")]
  public static bits URLParseURL_QuickResolve
    (ParseURLFlags flags,
     string base_url,
     string relative_url,
     string url_buffer,
     int url_buffer_size);

  [CCode (cname = "xurl_enumerate_schemes")]
  public static OS.Error* XEnumerateSchemes
    (bits flags,
     bits context,
     out bits status,
     out bits context_out,
     out string scheme,
     out string description,
     out bits swi_base,
     out bits version);

  [CCode (cname = "url_enumerate_schemes")]
  public static bits EnumerateSchemes
    (bits flags,
     bits context,
     out bits status,
     out string scheme,
     out string description,
     out bits swi_base,
     out bits version);

  [CCode (cname = "xurl_enumerate_proxies")]
  public static OS.Error* XEnumerateProxies
    (bits flags,
     S session,
     bits context,
     out bits status,
     out bits context_out,
     out string scheme,
     out string proxy_url);

  [CCode (cname = "url_enumerate_proxies")]
  public static bits EnumerateProxies
    (bits flags,
     S session,
     bits context,
     out bits status,
     out string scheme,
     out string proxy_url);

  [CCode (cname = "xurl_protocol_register")]
  public static OS.Error* XProtocolRegister
    (ProtocolRegisterFlags flags,
     bits swi_base,
     string scheme,
     bits version,
     string name,
     ProtocolFlags protocol_flags,
     bits default_port,
     out int flags_out);

  [CCode (cname = "url_protocol_register")]
  public static int ProtocolRegister
    (ProtocolRegisterFlags flags,
     bits swi_base,
     string scheme,
     bits version,
     string name,
     ProtocolFlags protocol_flags,
     bits default_port);

  [CCode (cname = "xurl_protocol_deregister")]
  public static OS.Error* XProtocolDeregister
    (bits flags,
     bits swi_base,
     out bits flags_out,
     out bits sessions_affected);

  [CCode (cname = "url_protocol_deregister")]
  public static bits ProtocolDeregister
    (bits flags,
     bits swi_base,
     out bits flags_out);

  [CCode (cname = "xservice_url_module")]
  public static OS.Error* XService_URLModule
    (bits reason_code,
     bits version);

  [CCode (cname = "service_url_module")]
  public static void Service_URLModule
    (bits reason_code,
     bits version);

  [CCode (cname = "xservice_url_module_started")]
  public static OS.Error* XService_URLModuleStarted
    (bits version);

  [CCode (cname = "service_url_module_started")]
  public static void Service_URLModuleStarted
    (bits version);

  [CCode (cname = "xservice_url_module_dying")]
  public static OS.Error* XService_URLModuleDying
    (bits version);

  [CCode (cname = "service_url_module_dying")]
  public static void Service_URLModuleDying
    (bits version);

  [CCode (cname = "xservice_url_protocol_module")]
  public static OS.Error* XService_URLProtocolModule
    (bits reason_code,
     string scheme,
     bits swi_base,
     string description);

  [CCode (cname = "service_url_protocol_module")]
  public static void Service_URLProtocolModule
    (bits reason_code,
     string scheme,
     bits swi_base,
     string description);

  [CCode (cname = "xservice_url_protocol_module_started")]
  public static OS.Error* XService_URLProtocolModuleStarted
    (string scheme,
     bits swi_base,
     string description);

  [CCode (cname = "service_url_protocol_module_started")]
  public static void Service_URLProtocolModuleStarted
    (string scheme,
     bits swi_base,
     string description);

  [CCode (cname = "xservice_url_protocol_module_dying")]
  public static OS.Error* XService_URLProtocolModuleDying
    (string scheme,
     bits swi_base,
     string description);

  [CCode (cname = "service_url_protocol_module_dying")]
  public static void Service_URLProtocolModuleDying
    (string scheme,
     bits swi_base,
     string description);

}
