/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.OS;
using OSLib.OSSpriteOp;
using OSLib.Font;

[CCode (cheader_filename = "oslib/wimp.h")]
namespace OSLib.Wimp
{
  [CCode (cname = "wimp_w", has_type_id = false)]
  public struct W : bits
  {
  }

  [CCode (cname = "wimp_i")]
  public enum I
  {
    [CCode (cname = "wimp_ICON_WINDOW")]
    IconWindow,
    [CCode (cname = "wimp_ICON_BACK")]
    IconBack,
    [CCode (cname = "wimp_ICON_CLOSE")]
    IconClose,
    [CCode (cname = "wimp_ICON_TITLE")]
    IconTitle,
    [CCode (cname = "wimp_ICON_TOGGLE")]
    IconToggle,
    [CCode (cname = "wimp_ICON_SCROLL_UP")]
    IconScrollUp,
    [CCode (cname = "wimp_ICON_VSCROLL")]
    IconVScroll,
    [CCode (cname = "wimp_ICON_SCROLL_DOWN")]
    IconScrollDown,
    [CCode (cname = "wimp_ICON_SIZE")]
    IconSize,
    [CCode (cname = "wimp_ICON_SCROLL_LEFT")]
    IconScrollLeft,
    [CCode (cname = "wimp_ICON_HSCROLL")]
    IconHScroll,
    [CCode (cname = "wimp_ICON_SCROLL_RIGHT")]
    IconScrollRight,
    [CCode (cname = "wimp_ICON_FRAME")]
    IconFrame,
    [CCode (cname = "wimp_ICON_ICONISE")]
    IconIconise
  }

  [CCode (cname = "wimp_t", has_type_id = false)]
  public struct T : bits
  {
  }

  [CCode (cname = "wimp_colour")]
  public enum Colour
  {
    [CCode (cname = "wimp_COLOUR_WHITE")]
    White,
    [CCode (cname = "wimp_COLOUR_VERY_LIGHT_GREY")]
    VeryLightGrey,
    [CCode (cname = "wimp_COLOUR_LIGHT_GREY")]
    LightGrey,
    [CCode (cname = "wimp_COLOUR_MID_LIGHT_GREY")]
    MidLightGrey,
    [CCode (cname = "wimp_COLOUR_MID_DARK_GREY")]
    MidDarkGrey,
    [CCode (cname = "wimp_COLOUR_DARK_GREY")]
    DarkGrey,
    [CCode (cname = "wimp_COLOUR_VERY_DARK_GREY")]
    VeryDarkGrey,
    [CCode (cname = "wimp_COLOUR_BLACK")]
    Black,
    [CCode (cname = "wimp_COLOUR_DARK_BLUE")]
    DarkBlue,
    [CCode (cname = "wimp_COLOUR_YELLOW")]
    Yellow,
    [CCode (cname = "wimp_COLOUR_LIGHT_GREEN")]
    LightGreen,
    [CCode (cname = "wimp_COLOUR_RED")]
    Red,
    [CCode (cname = "wimp_COLOUR_CREAM")]
    Cream,
    [CCode (cname = "wimp_COLOUR_DARK_GREEN")]
    DarkGreen,
    [CCode (cname = "wimp_COLOUR_ORANGE")]
    Orange,
    [CCode (cname = "wimp_COLOUR_LIGHT_BLUE")]
    LightBlue,
    [CCode (cname = "wimp_COLOUR_TRANSPARENT")]
    Transparent
  }

  [CCode (cname = "wimp_version_no")]
  public enum VersionNo
  {
    [CCode (cname = "wimp_VERSION_RO2")]
    VersionRO2,
    [CCode (cname = "wimp_VERSION_RO30")]
    VersionRO30,
    [CCode (cname = "wimp_VERSION_RO3")]
    VersionRO3,
    [CCode (cname = "wimp_VERSION_RO35")]
    VersionRO35,
    [CCode (cname = "wimp_VERSION_RO36")]
    VersionRO36,
    [CCode (cname = "wimp_VERSION_RO37")]
    VersionRO37,
    [CCode (cname = "wimp_VERSION_RO38")]
    VersionRO38,
    [CCode (cname = "wimp_VERSION_RO40")]
    VersionRO40
  }

  [CCode (cname = "wimp_event_no")]
  public enum EventNo
  {
    [CCode (cname = "wimp_NULL_REASON_CODE")]
    NullReasonCode,
    [CCode (cname = "wimp_REDRAW_WINDOW_REQUEST")]
    RedrawWindowRequest,
    [CCode (cname = "wimp_OPEN_WINDOW_REQUEST")]
    OpenWindowRequest,
    [CCode (cname = "wimp_CLOSE_WINDOW_REQUEST")]
    CloseWindowRequest,
    [CCode (cname = "wimp_POINTER_LEAVING_WINDOW")]
    PointerLeavingWindow,
    [CCode (cname = "wimp_POINTER_ENTERING_WINDOW")]
    PointerEnteringWindow,
    [CCode (cname = "wimp_MOUSE_CLICK")]
    MouseClick,
    [CCode (cname = "wimp_USER_DRAG_BOX")]
    UserDragBox,
    [CCode (cname = "wimp_KEY_PRESSED")]
    KeyPressed,
    [CCode (cname = "wimp_MENU_SELECTION")]
    MenuSelection,
    [CCode (cname = "wimp_SCROLL_REQUEST")]
    ScrollRequest,
    [CCode (cname = "wimp_LOSE_CARET")]
    LoseCaret,
    [CCode (cname = "wimp_GAIN_CARET")]
    GainCaret,
    [CCode (cname = "wimp_POLLWORD_NON_ZERO")]
    PollwordNonZero,
    [CCode (cname = "wimp_USER_MESSAGE")]
    UserMessage,
    [CCode (cname = "wimp_USER_MESSAGE_RECORDED")]
    UserMessageRecorded,
    [CCode (cname = "wimp_USER_MESSAGE_ACKNOWLEDGE")]
    UserMessageAcknowledge
  }

  [CCode (cname = "wimp_poll_flags")]
  public enum PollFlags
  {
    [CCode (cname = "wimp_MASK_NULL")]
    MaskNull,
    [CCode (cname = "wimp_QUEUE_REDRAW")]
    QueueRedraw,
    [CCode (cname = "wimp_MASK_LEAVING")]
    MaskLeaving,
    [CCode (cname = "wimp_MASK_ENTERING")]
    MaskEntering,
    [CCode (cname = "wimp_QUEUE_MOUSE")]
    QueueMouse,
    [CCode (cname = "wimp_QUEUE_KEY")]
    QueueKey,
    [CCode (cname = "wimp_MASK_LOSE")]
    MaskLose,
    [CCode (cname = "wimp_MASK_GAIN")]
    MaskGain,
    [CCode (cname = "wimp_MASK_POLLWORD")]
    MaskPollword,
    [CCode (cname = "wimp_MASK_ICON_LEAVING")]
    MaskIconLeaving,
    [CCode (cname = "wimp_MASK_ICON_ENTERING")]
    MaskIconEntering,
    [CCode (cname = "wimp_MASK_MESSAGE")]
    MaskMessage,
    [CCode (cname = "wimp_MASK_RECORDED")]
    MaskRecorded,
    [CCode (cname = "wimp_MASK_ACKNOWLEDGE")]
    MaskAcknowledge,
    [CCode (cname = "wimp_GIVEN_POLLWORD")]
    GivenPollword,
    [CCode (cname = "wimp_POLL_HIGH_PRIORITY")]
    PollHighPriority,
    [CCode (cname = "wimp_SAVE_FP")]
    SaveFP,
    [CCode (cname = "wimp_ISSUE_POLL14_AND15")]
    IssuePoll14And15
  }

  [CCode (cname = "wimp_furniture")]
  public enum Furniture
  {
    [CCode (cname = "wimp_FURNITURE_WHOLE")]
    Whole,
    [CCode (cname = "wimp_FURNITURE_BACK")]
    Back,
    [CCode (cname = "wimp_FURNITURE_CLOSE")]
    Close,
    [CCode (cname = "wimp_FURNITURE_TITLE")]
    Title,
    [CCode (cname = "wimp_FURNITURE_TOGGLE")]
    Toggle,
    [CCode (cname = "wimp_FURNITURE_VSCROLL")]
    VScroll,
    [CCode (cname = "wimp_FURNITURE_SIZE")]
    Size,
    [CCode (cname = "wimp_FURNITURE_HSCROLL")]
    HScroll,
    [CCode (cname = "wimp_FURNITURE_ICONISE")]
    Iconise
  }

  [CCode (cname = "wimp_window_nesting_flags")]
  public enum WindowNestingFlags
  {
    [CCode (cname = "wimp_CHILD_LINKS_PARENT_WORK_AREA")]
    ChildLinksParentWorkArea,
    [CCode (cname = "wimp_CHILD_LINKS_PARENT_VISIBLE_BOTTOM_OR_LEFT")]
    ChildLinksParentVisibleBottomOrLeft,
    [CCode (cname = "wimp_CHILD_LINKS_PARENT_VISIBLE_TOP_OR_RIGHT")]
    ChildLinksParentVisibleTopOrRight,
    [CCode (cname = "wimp_CHILD_LINKS_PARENT")]
    ChildLinksParent
  }

  [CCode (cname = "wimp_window_flags")]
  public enum WindowFlags
  {
    [CCode (cname = "wimp_WINDOW_MOVEABLE")]
    WindowMoveable,
    [CCode (cname = "wimp_WINDOW_AUTO_REDRAW")]
    WindowAutoRedraw,
    [CCode (cname = "wimp_WINDOW_PANE")]
    WindowPane,
    [CCode (cname = "wimp_WINDOW_NO_BOUNDS")]
    WindowNoBounds,
    [CCode (cname = "wimp_WINDOW_SCROLL_REPEAT")]
    WindowScrollRepeat,
    [CCode (cname = "wimp_WINDOW_SCROLL")]
    WindowScroll,
    [CCode (cname = "wimp_WINDOW_REAL_COLOURS")]
    WindowRealColours,
    [CCode (cname = "wimp_WINDOW_BACK")]
    WindowBack,
    [CCode (cname = "wimp_WINDOW_HOT_KEYS")]
    WindowHotKeys,
    [CCode (cname = "wimp_WINDOW_BOUNDED")]
    WindowBounded,
    [CCode (cname = "wimp_WINDOW_IGNORE_XEXTENT")]
    WindowIgnoreXExtent,
    [CCode (cname = "wimp_WINDOW_IGNORE_YEXTENT")]
    WindowIgnoreYExtent,
    [CCode (cname = "wimp_WINDOW_OPEN")]
    WindowOpen,
    [CCode (cname = "wimp_WINDOW_NOT_COVERED")]
    WindowNotCovered,
    [CCode (cname = "wimp_WINDOW_FULL_SIZE")]
    WindowFullSize,
    [CCode (cname = "wimp_WINDOW_TOGGLED")]
    WindowToggled,
    [CCode (cname = "wimp_WINDOW_HAS_FOCUS")]
    WindowHasFocus,
    [CCode (cname = "wimp_WINDOW_BOUNDED_ONCE")]
    WindowBoundedOnce,
    [CCode (cname = "wimp_WINDOW_PARTIAL_SIZE")]
    WindowPartialSize,
    [CCode (cname = "wimp_WINDOW_FURNITURE_WINDOW")]
    WindowFurnitureWindow,
    [CCode (cname = "wimp_WINDOW_FOREGROUND_WINDOW")]
    WindowForegroundWindow,
    [CCode (cname = "wimp_WINDOW_BACK_ICON")]
    WindowBackIcon,
    [CCode (cname = "wimp_WINDOW_CLOSE_ICON")]
    WindowCloseIcon,
    [CCode (cname = "wimp_WINDOW_TITLE_ICON")]
    WindowTitleIcon,
    [CCode (cname = "wimp_WINDOW_TOGGLE_ICON")]
    WindowToggleIcon,
    [CCode (cname = "wimp_WINDOW_VSCROLL")]
    WindowVScroll,
    [CCode (cname = "wimp_WINDOW_SIZE_ICON")]
    WindowSizeIcon,
    [CCode (cname = "wimp_WINDOW_HSCROLL")]
    WindowHScroll,
    [CCode (cname = "wimp_WINDOW_NEW_FORMAT")]
    WindowNewFormat
  }

  [CCode (cname = "wimp_extra_window_flags")]
  public enum ExtraWindowFlags
  {
    [CCode (cname = "wimp_WINDOW_USE_TITLE_VALIDATION_COLOURS")]
    WindowUseTitleValidationColours,
    [CCode (cname = "wimp_WINDOW_USE_TITLE_VALIDATION_STRING")]
    WindowUseTitleValidationString,
    [CCode (cname = "wimp_WINDOW_USE_EXTENDED_SCROLL_REQUEST")]
    WindowUseExtendedScrollRequest,
    [CCode (cname = "wimp_WINDOW_NEVER3D")]
    WindowNever3D,
    [CCode (cname = "wimp_WINDOW_ALWAYS3D")]
    WindowAlways3D,
    [CCode (cname = "wimp_WINDOW_GIVE_SHADED_ICON_INFO")]
    WindowGiveShadedIconInfo
  }

  [CCode (cname = "wimp_icon_flags")]
  public enum IconFlags
  {
    [CCode (cname = "wimp_ICON_TEXT")]
    IconText,
    [CCode (cname = "wimp_ICON_SPRITE")]
    IconSprite,
    [CCode (cname = "wimp_ICON_BORDER")]
    IconBorder,
    [CCode (cname = "wimp_ICON_HCENTRED")]
    IconHCentred,
    [CCode (cname = "wimp_ICON_VCENTRED")]
    IconVCentred,
    [CCode (cname = "wimp_ICON_FILLED")]
    IconFilled,
    [CCode (cname = "wimp_ICON_ANTI_ALIASED")]
    IconAntiAliased,
    [CCode (cname = "wimp_ICON_NEEDS_HELP")]
    IconNeedsHelp,
    [CCode (cname = "wimp_ICON_INDIRECTED")]
    IconIndirected,
    [CCode (cname = "wimp_ICON_RJUSTIFIED")]
    IconRJustified,
    [CCode (cname = "wimp_ICON_ALLOW_ADJUST")]
    IconAllowAdjust,
    [CCode (cname = "wimp_ICON_HALF_SIZE")]
    IconHalfSize,
    [CCode (cname = "wimp_ICON_BUTTON_TYPE")]
    IconButtonType,
    [CCode (cname = "wimp_ICON_ESG")]
    IconESG,
    [CCode (cname = "wimp_ICON_SELECTED")]
    IconSelected,
    [CCode (cname = "wimp_ICON_SHADED")]
    IconShaded,
    [CCode (cname = "wimp_ICON_DELETED")]
    IconDeleted,
    [CCode (cname = "wimp_ICON_FG_COLOUR")]
    IconFGColour,
    [CCode (cname = "wimp_ICON_BG_COLOUR")]
    IconBGColour,
    [CCode (cname = "wimp_ICON_FONT_HANDLE")]
    IconFontHandle,
    [CCode (cname = "wimp_BUTTON_NEVER")]
    ButtonNever,
    [CCode (cname = "wimp_BUTTON_ALWAYS")]
    ButtonAlways,
    [CCode (cname = "wimp_BUTTON_REPEAT")]
    ButtonRepeat,
    [CCode (cname = "wimp_BUTTON_CLICK")]
    ButtonClick,
    [CCode (cname = "wimp_BUTTON_RELEASE")]
    ButtonRelease,
    [CCode (cname = "wimp_BUTTON_DOUBLE_CLICK")]
    ButtonDoubleClick,
    [CCode (cname = "wimp_BUTTON_CLICK_DRAG")]
    ButtonClickDrag,
    [CCode (cname = "wimp_BUTTON_RELEASE_DRAG")]
    ButtonReleaseDrag,
    [CCode (cname = "wimp_BUTTON_DOUBLE_DRAG")]
    ButtonDoubleDrag,
    [CCode (cname = "wimp_BUTTON_MENU_ICON")]
    ButtonMenuIcon,
    [CCode (cname = "wimp_BUTTON_DOUBLE_CLICK_DRAG")]
    ButtonDoubleClickDrag,
    [CCode (cname = "wimp_BUTTON_RADIO")]
    ButtonRadio,
    [CCode (cname = "wimp_BUTTON_WRITE_CLICK_DRAG")]
    ButtonWriteClickDrag,
    [CCode (cname = "wimp_BUTTON_WRITABLE")]
    ButtonWritable
  }

  [CCode (cname = "wimp_menu_flags")]
  public enum MenuFlags
  {
    [CCode (cname = "wimp_MENU_TICKED")]
    MenuTicked,
    [CCode (cname = "wimp_MENU_SEPARATE")]
    MenuSeparate,
    [CCode (cname = "wimp_MENU_WRITABLE")]
    MenuWritable,
    [CCode (cname = "wimp_MENU_GIVE_WARNING")]
    MenuGiveWarning,
    [CCode (cname = "wimp_MENU_SUB_MENU_WHEN_SHADED")]
    MenuSubMenuWhenShaded,
    [CCode (cname = "wimp_MENU_LAST")]
    MenuLast,
    [CCode (cname = "wimp_MENU_TITLE_INDIRECTED")]
    MenuTitleIndirected
  }

  [CCode (cname = "wimp_mouse_state")]
  public enum MouseState
  {
    [CCode (cname = "wimp_CLICK_ADJUST")]
    ClickAdjust,
    [CCode (cname = "wimp_DOUBLE_ADJUST")]
    DoubleAdjust,
    [CCode (cname = "wimp_CLICK_MENU")]
    ClickMenu,
    [CCode (cname = "wimp_CLICK_SELECT")]
    ClickSelect,
    [CCode (cname = "wimp_DOUBLE_SELECT")]
    DoubleSelect,
    [CCode (cname = "wimp_DRAG_ADJUST")]
    DragAdjust,
    [CCode (cname = "wimp_DRAG_SELECT")]
    DragSelect,
    [CCode (cname = "wimp_SINGLE_ADJUST")]
    SingleAdjust,
    [CCode (cname = "wimp_SINGLE_SELECT")]
    SingleSelect
  }

  [CCode (cname = "wimp_drag_type")]
  public enum DragType
  {
    [CCode (cname = "wimp_DRAG_SYSTEM_POSITION")]
    DragSystemPosition,
    [CCode (cname = "wimp_DRAG_SYSTEM_SIZE")]
    DragSystemSize,
    [CCode (cname = "wimp_DRAG_SYSTEM_HSCROLL")]
    DragSystemHScroll,
    [CCode (cname = "wimp_DRAG_SYSTEM_VSCROLL")]
    DragSystemVScroll,
    [CCode (cname = "wimp_DRAG_USER_FIXED")]
    DragUserFixed,
    [CCode (cname = "wimp_DRAG_USER_RUBBER")]
    DragUserRubber,
    [CCode (cname = "wimp_DRAG_USER_POINT")]
    DragUserPoint,
    [CCode (cname = "wimp_DRAG_ASM_FIXED")]
    DragAsmFixed,
    [CCode (cname = "wimp_DRAG_ASM_RUBBER")]
    DragAsmRubber,
    [CCode (cname = "wimp_DRAG_ASM_FIXED_INDEFINITE")]
    DragAsmFixedIndefinite,
    [CCode (cname = "wimp_DRAG_ASM_RUBBER_INDEFINITE")]
    DragAsmRubberIndefinite,
    [CCode (cname = "wimp_DRAG_SYSTEM_SCROLL_BOTH")]
    DragSystemScrollBoth,
    [CCode (cname = "wimp_DRAG_START_TEXT_SELECTION")]
    DragStartTextSelection
  }

  [CCode (cname = "wimp_key_no")]
  public enum KeyNo
  {
    [CCode (cname = "wimp_KEY_BACKSPACE")]
    KeyBackspace,
    [CCode (cname = "wimp_KEY_RETURN")]
    KeyReturn,
    [CCode (cname = "wimp_KEY_ESCAPE")]
    KeyEscape,
    [CCode (cname = "wimp_KEY_HOME")]
    KeyHome,
    [CCode (cname = "wimp_KEY_DELETE")]
    KeyDelete,
    [CCode (cname = "wimp_KEY_SHIFT")]
    KeyShift,
    [CCode (cname = "wimp_KEY_CONTROL")]
    KeyControl,
    [CCode (cname = "wimp_KEY_PRINT")]
    KeyPrint,
    [CCode (cname = "wimp_KEY_F1")]
    KeyF1,
    [CCode (cname = "wimp_KEY_F2")]
    KeyF2,
    [CCode (cname = "wimp_KEY_F3")]
    KeyF3,
    [CCode (cname = "wimp_KEY_F4")]
    KeyF4,
    [CCode (cname = "wimp_KEY_F5")]
    KeyF5,
    [CCode (cname = "wimp_KEY_F6")]
    KeyF6,
    [CCode (cname = "wimp_KEY_F7")]
    KeyF7,
    [CCode (cname = "wimp_KEY_F8")]
    KeyF8,
    [CCode (cname = "wimp_KEY_F9")]
    KeyF9,
    [CCode (cname = "wimp_KEY_TAB")]
    KeyTab,
    [CCode (cname = "wimp_KEY_COPY")]
    KeyCopy,
    [CCode (cname = "wimp_KEY_LEFT")]
    KeyLeft,
    [CCode (cname = "wimp_KEY_RIGHT")]
    KeyRight,
    [CCode (cname = "wimp_KEY_LOGO")]
    KeyLogo,
    [CCode (cname = "wimp_KEY_MENU")]
    KeyMenu,
    [CCode (cname = "wimp_KEY_F10")]
    KeyF10,
    [CCode (cname = "wimp_KEY_F11")]
    KeyF11,
    [CCode (cname = "wimp_KEY_F12")]
    KeyF12,
    [CCode (cname = "wimp_KEY_INSERT")]
    KeyInsert,
    [CCode (cname = "wimp_KEY_DOWN")]
    KeyDown,
    [CCode (cname = "wimp_KEY_UP")]
    KeyUp,
    [CCode (cname = "wimp_KEY_PAGE_DOWN")]
    KeyPageDown,
    [CCode (cname = "wimp_KEY_PAGE_UP")]
    KeyPageUp
  }

  [CCode (cname = "wimp_drag_box_flags")]
  public enum DragBoxFlags
  {
    [CCode (cname = "wimp_DRAG_BOX_KEEP_IN_LINE")]
    DragBoxKeepInLine,
    [CCode (cname = "wimp_DRAG_BOX_CLIP")]
    DragBoxClip
  }

  [CCode (cname = "wimp_auto_scroll_flags")]
  public enum AutoScrollFlags
  {
    [CCode (cname = "wimp_AUTO_SCROLL_ENABLE_HORIZONTAL")]
    AutoScrollEnableHorizontal,
    [CCode (cname = "wimp_AUTO_SCROLL_ENABLE_VERTICAL")]
    AutoScrollEnableVertical,
    [CCode (cname = "wimp_AUTO_SCROLL_DELEGATE_TO_TASK")]
    AutoScrollDelegateToTask,
    [CCode (cname = "wimp_AUTO_SCROLL_READ_STATE")]
    AutoScrollReadState
  }

  [CCode (cname = "wimp_error_box_flags")]
  public enum ErrorBoxFlags
  {
    [CCode (cname = "wimp_ERROR_BOX_OK_ICON")]
    ErrorBoxOKIcon,
    [CCode (cname = "wimp_ERROR_BOX_CANCEL_ICON")]
    ErrorBoxCancelIcon,
    [CCode (cname = "wimp_ERROR_BOX_HIGHLIGHT_CANCEL")]
    ErrorBoxHighlightCancel,
    [CCode (cname = "wimp_ERROR_BOX_NO_PROMPT")]
    ErrorBoxNoPrompt,
    [CCode (cname = "wimp_ERROR_BOX_SHORT_TITLE")]
    ErrorBoxShortTitle,
    [CCode (cname = "wimp_ERROR_BOX_LEAVE_OPEN")]
    ErrorBoxLeaveOpen,
    [CCode (cname = "wimp_ERROR_BOX_CLOSE")]
    ErrorBoxClose,
    [CCode (cname = "wimp_ERROR_BOX_NO_BEEP")]
    ErrorBoxNoBeep,
    [CCode (cname = "wimp_ERROR_BOX_GIVEN_CATEGORY")]
    ErrorBoxGivenCategory,
    [CCode (cname = "wimp_ERROR_BOX_CATEGORY")]
    ErrorBoxCategory,
    [CCode (cname = "wimp_ERROR_BOX_CATEGORY_INFO")]
    ErrorBoxCategoryInfo,
    [CCode (cname = "wimp_ERROR_BOX_CATEGORY_ERROR")]
    ErrorBoxCategoryError,
    [CCode (cname = "wimp_ERROR_BOX_CATEGORY_PROGRAM")]
    ErrorBoxCategoryProgram,
    [CCode (cname = "wimp_ERROR_BOX_CATEGORY_QUESTION")]
    ErrorBoxCategoryQuestion
  }

  [CCode (cname = "wimp_error_box_selection")]
  public enum ErrorBoxSelection
  {
    [CCode (cname = "wimp_ERROR_BOX_SELECTED_NOTHING")]
    ErrorBoxSelectedNothing,
    [CCode (cname = "wimp_ERROR_BOX_SELECTED_OK")]
    ErrorBoxSelectedOK,
    [CCode (cname = "wimp_ERROR_BOX_SELECTED_CANCEL")]
    ErrorBoxSelectedCancel
  }

  [CCode (cname = "wimp_scroll_direction")]
  public enum ScrollDirection
  {
    [CCode (cname = "wimp_SCROLL_DOUBLE_EXTENDED_LEFT")]
    ScrollDoubleExtendedLeft,
    [CCode (cname = "wimp_SCROLL_DOUBLE_EXTENDED_DOWN")]
    ScrollDoubleExtendedDown,
    [CCode (cname = "wimp_SCROLL_SINGLE_EXTENDED_LEFT")]
    ScrollSingleExtendedLeft,
    [CCode (cname = "wimp_SCROLL_SINGLE_EXTENDED_DOWN")]
    ScrollSingleExtendedDown,
    [CCode (cname = "wimp_SCROLL_AUTO_LEFT")]
    ScrollAutoLeft,
    [CCode (cname = "wimp_SCROLL_AUTO_DOWN")]
    ScrollAutoDown,
    [CCode (cname = "wimp_SCROLL_PAGE_LEFT")]
    ScrollPageLeft,
    [CCode (cname = "wimp_SCROLL_PAGE_DOWN")]
    ScrollPageDown,
    [CCode (cname = "wimp_SCROLL_COLUMN_LEFT")]
    ScrollColumnLeft,
    [CCode (cname = "wimp_SCROLL_LINE_DOWN")]
    ScrollLineDown,
    [CCode (cname = "wimp_SCROLL_NONE")]
    ScrollNone,
    [CCode (cname = "wimp_SCROLL_LINE_UP")]
    ScrollLineUp,
    [CCode (cname = "wimp_SCROLL_COLUMN_RIGHT")]
    ScrollColumnRight,
    [CCode (cname = "wimp_SCROLL_PAGE_UP")]
    ScrollPageUp,
    [CCode (cname = "wimp_SCROLL_PAGE_RIGHT")]
    ScrollPageRight,
    [CCode (cname = "wimp_SCROLL_AUTO_UP")]
    ScrollAutoUp,
    [CCode (cname = "wimp_SCROLL_AUTO_RIGHT")]
    ScrollAutoRight,
    [CCode (cname = "wimp_SCROLL_SINGLE_EXTENDED_UP")]
    ScrollSingleExtendedUp,
    [CCode (cname = "wimp_SCROLL_SINGLE_EXTENDED_RIGHT")]
    ScrollSingleExtendedRight,
    [CCode (cname = "wimp_SCROLL_DOUBLE_EXTENDED_UP")]
    ScrollDoubleExtendedUp,
    [CCode (cname = "wimp_SCROLL_DOUBLE_EXTENDED_RIGHT")]
    ScrollDoubleExtendedRight
  }

  [CCode (cname = "wimp_icon_data")]
  public struct IconData
  {
    [CCode (cname = "text")]
    public unowned char Text[12];
    [CCode (cname = "sprite")]
    public unowned char Sprite[12];
    [CCode (cname = "text_and_sprite")]
    public unowned char TextAndSprite[12];
    [CCode (cname = "indirected_text.text")]
    public unowned string IndirectedText_Text;
    [CCode (cname = "indirected_text.validation")]
    public unowned string IndirectedText_Validation;
    [CCode (cname = "indirected_text.size")]
    public unowned int IndirectedText_Size;
    [CCode (cname = "indirected_sprite.id")]
    public unowned OSSpriteOp.Id IndirectedSprite_Id;
    [CCode (cname = "indirected_sprite.area")]
    public unowned OSSpriteOp.Area* IndirectedSprite_Area;
    [CCode (cname = "indirected_sprite.size")]
    public unowned int IndirectedSprite_Size;
    [CCode (cname = "indirected_text_and_sprite.text")]
    public unowned string IndirectedTextAndSprite_Text;
    [CCode (cname = "indirected_text_and_sprite.validation")]
    public unowned string IndirectedTextAndSprite_Validation;
    [CCode (cname = "indirected_text_and_sprite.size")]
    public unowned int IndirectedTextAndSprite_Size;
  }

  [CCode (cname = "wimp_icon")]
  public struct Icon
  {
    [CCode (cname = "extent")]
    public unowned OS.Box Extent;
    [CCode (cname = "flags")]
    public unowned IconFlags Flags;
    [CCode (cname = "data")]
    public unowned IconData Data;
  }

  [CCode (cname = "wimp_icon_create")]
  public struct IconCreate
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "icon")]
    public unowned Icon Icon;
  }

  [CCode (cname = "wimp_window_base")]
  public struct WindowBase
  {
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned W Next;
    [CCode (cname = "flags")]
    public unowned WindowFlags Flags;
    [CCode (cname = "title_fg")]
    public unowned Colour TitleFg;
    [CCode (cname = "title_bg")]
    public unowned Colour TitleBg;
    [CCode (cname = "work_fg")]
    public unowned Colour WorkFg;
    [CCode (cname = "work_bg")]
    public unowned Colour WorkBg;
    [CCode (cname = "scroll_outer")]
    public unowned Colour ScrollOuter;
    [CCode (cname = "scroll_inner")]
    public unowned Colour ScrollInner;
    [CCode (cname = "highlight_bg")]
    public unowned Colour HighlightBg;
    [CCode (cname = "extra_flags")]
    public unowned ExtraWindowFlags ExtraFlags;
    [CCode (cname = "extent")]
    public unowned OS.Box Extent;
    [CCode (cname = "title_flags")]
    public unowned IconFlags TitleFlags;
    [CCode (cname = "work_flags")]
    public unowned IconFlags WorkFlags;
    [CCode (cname = "sprite_area")]
    public unowned OSSpriteOp.Area* SpriteArea;
    [CCode (cname = "xmin")]
    public unowned short Xmin;
    [CCode (cname = "ymin")]
    public unowned short Ymin;
    [CCode (cname = "title_data")]
    public unowned IconData TitleData;
    [CCode (cname = "icon_count")]
    public unowned int IconCount;
  }

  [Compact]
  [CCode (cname = "wimp_window", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Window
  {
    public Window(int numEntries);
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned W Next;
    [CCode (cname = "flags")]
    public unowned WindowFlags Flags;
    [CCode (cname = "title_fg")]
    public unowned Colour TitleFg;
    [CCode (cname = "title_bg")]
    public unowned Colour TitleBg;
    [CCode (cname = "work_fg")]
    public unowned Colour WorkFg;
    [CCode (cname = "work_bg")]
    public unowned Colour WorkBg;
    [CCode (cname = "scroll_outer")]
    public unowned Colour ScrollOuter;
    [CCode (cname = "scroll_inner")]
    public unowned Colour ScrollInner;
    [CCode (cname = "highlight_bg")]
    public unowned Colour HighlightBg;
    [CCode (cname = "extra_flags")]
    public unowned ExtraWindowFlags ExtraFlags;
    [CCode (cname = "extent")]
    public unowned OS.Box Extent;
    [CCode (cname = "title_flags")]
    public unowned IconFlags TitleFlags;
    [CCode (cname = "work_flags")]
    public unowned IconFlags WorkFlags;
    [CCode (cname = "sprite_area")]
    public unowned OSSpriteOp.Area* SpriteArea;
    [CCode (cname = "xmin")]
    public unowned short Xmin;
    [CCode (cname = "ymin")]
    public unowned short Ymin;
    [CCode (cname = "title_data")]
    public unowned IconData TitleData;
    [CCode (cname = "icon_count")]
    public unowned int IconCount;
    [CCode (cname = "icons", array_length = false)]
    public unowned Icon[] Icons;
  }

  [CCode (cname = "wimp_window_state")]
  public struct WindowState
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned W Next;
    [CCode (cname = "flags")]
    public unowned WindowFlags Flags;
  }

  [CCode (cname = "wimp_icon_state")]
  public struct IconState
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "icon")]
    public unowned Icon Icon;
  }

  [CCode (cname = "wimp_set_icon_state_block")]
  public struct SetIconStateBlock
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "eor_bits")]
    public unowned IconFlags EorBits;
    [CCode (cname = "clear_bits")]
    public unowned IconFlags ClearBits;
  }

  [CCode (cname = "wimp_window_info_base")]
  public struct WindowInfoBase
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned W Next;
    [CCode (cname = "flags")]
    public unowned WindowFlags Flags;
    [CCode (cname = "title_fg")]
    public unowned Colour TitleFg;
    [CCode (cname = "title_bg")]
    public unowned Colour TitleBg;
    [CCode (cname = "work_fg")]
    public unowned Colour WorkFg;
    [CCode (cname = "work_bg")]
    public unowned Colour WorkBg;
    [CCode (cname = "scroll_outer")]
    public unowned Colour ScrollOuter;
    [CCode (cname = "scroll_inner")]
    public unowned Colour ScrollInner;
    [CCode (cname = "highlight_bg")]
    public unowned Colour HighlightBg;
    [CCode (cname = "extra_flags")]
    public unowned ExtraWindowFlags ExtraFlags;
    [CCode (cname = "extent")]
    public unowned OS.Box Extent;
    [CCode (cname = "title_flags")]
    public unowned IconFlags TitleFlags;
    [CCode (cname = "work_flags")]
    public unowned IconFlags WorkFlags;
    [CCode (cname = "sprite_area")]
    public unowned OSSpriteOp.Area* SpriteArea;
    [CCode (cname = "xmin")]
    public unowned short Xmin;
    [CCode (cname = "ymin")]
    public unowned short Ymin;
    [CCode (cname = "title_data")]
    public unowned IconData TitleData;
    [CCode (cname = "icon_count")]
    public unowned int IconCount;
  }

  [Compact]
  [CCode (cname = "wimp_window_info", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class WindowInfo
  {
    public WindowInfo(int numEntries);
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned W Next;
    [CCode (cname = "flags")]
    public unowned WindowFlags Flags;
    [CCode (cname = "title_fg")]
    public unowned Colour TitleFg;
    [CCode (cname = "title_bg")]
    public unowned Colour TitleBg;
    [CCode (cname = "work_fg")]
    public unowned Colour WorkFg;
    [CCode (cname = "work_bg")]
    public unowned Colour WorkBg;
    [CCode (cname = "scroll_outer")]
    public unowned Colour ScrollOuter;
    [CCode (cname = "scroll_inner")]
    public unowned Colour ScrollInner;
    [CCode (cname = "highlight_bg")]
    public unowned Colour HighlightBg;
    [CCode (cname = "extra_flags")]
    public unowned ExtraWindowFlags ExtraFlags;
    [CCode (cname = "extent")]
    public unowned OS.Box Extent;
    [CCode (cname = "title_flags")]
    public unowned IconFlags TitleFlags;
    [CCode (cname = "work_flags")]
    public unowned IconFlags WorkFlags;
    [CCode (cname = "sprite_area")]
    public unowned OSSpriteOp.Area* SpriteArea;
    [CCode (cname = "xmin")]
    public unowned short Xmin;
    [CCode (cname = "ymin")]
    public unowned short Ymin;
    [CCode (cname = "title_data")]
    public unowned IconData TitleData;
    [CCode (cname = "icon_count")]
    public unowned int IconCount;
    [CCode (cname = "icons", array_length = false)]
    public unowned Icon[] Icons;
  }

  [CCode (cname = "wimp_outline")]
  public struct Outline
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "outline")]
    public unowned OS.Box Outline;
  }

  [CCode (cname = "wimp_draw")]
  public struct Draw
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "box")]
    public unowned OS.Box Box;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "clip")]
    public unowned OS.Box Clip;
  }

  [CCode (cname = "wimp_open")]
  public struct Open
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned W Next;
  }

  [CCode (cname = "wimp_close")]
  public struct Close
  {
    [CCode (cname = "w")]
    public unowned W W;
  }

  [CCode (cname = "wimp_leaving")]
  public struct Leaving
  {
    [CCode (cname = "w")]
    public unowned W W;
  }

  [CCode (cname = "wimp_entering")]
  public struct Entering
  {
    [CCode (cname = "w")]
    public unowned W W;
  }

  [CCode (cname = "wimp_pointer")]
  public struct Pointer
  {
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "buttons")]
    public unowned MouseState Buttons;
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
  }

  [CCode (cname = "wimp_dragged")]
  public struct Dragged
  {
    [CCode (cname = "final")]
    public unowned OS.Box Final;
  }

  [CCode (cname = "wimp_key")]
  public struct Key
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "index")]
    public unowned int Index;
    [CCode (cname = "c")]
    public unowned KeyNo C;
  }

  [CCode (cname = "wimp_selection")]
  public struct Selection
  {
    [CCode (cname = "items")]
    public unowned int Items[9];
  }

  [CCode (cname = "wimp_scroll")]
  public struct Scroll
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "visible")]
    public unowned OS.Box Visible;
    [CCode (cname = "xscroll")]
    public unowned int Xscroll;
    [CCode (cname = "yscroll")]
    public unowned int Yscroll;
    [CCode (cname = "next")]
    public unowned W Next;
    [CCode (cname = "xmin")]
    public unowned ScrollDirection Xmin;
    [CCode (cname = "ymin")]
    public unowned ScrollDirection Ymin;
    [CCode (cname = "i")]
    public unowned I I;
  }

  [CCode (cname = "wimp_pollword")]
  public struct Pollword
  {
    [CCode (cname = "addr")]
    public unowned char* Addr;
    [CCode (cname = "word")]
    public unowned int Word;
  }

  [CCode (cname = "wimp_drag")]
  public struct Drag
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "type")]
    public unowned DragType Type;
    [CCode (cname = "initial")]
    public unowned OS.Box Initial;
    [CCode (cname = "bbox")]
    public unowned OS.Box Bbox;
    [CCode (cname = "handle")]
    public unowned char* Handle;
    [CCode (cname = "draw")]
    public unowned AsmRoutine Draw;
    [CCode (cname = "undraw")]
    public unowned AsmRoutine Undraw;
    [CCode (cname = "redraw")]
    public unowned AsmRoutine Redraw;
  }

  [CCode (cname = "wimp_auto_scroll_info")]
  public struct AutoScrollInfo
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "pause_zone_sizes")]
    public unowned OS.Box PauseZoneSizes;
    [CCode (cname = "pause_duration")]
    public unowned int PauseDuration;
    [CCode (cname = "state_change")]
    public unowned AsmRoutine StateChange;
    [CCode (cname = "handle")]
    public unowned char* Handle;
  }

  [CCode (cname = "wimp_caret")]
  public struct Caret
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "index")]
    public unowned int Index;
  }

  [CCode (cname = "wimp_text_selection")]
  public struct TextSelection
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "start_index")]
    public unowned int StartIndex;
    [CCode (cname = "end_index")]
    public unowned int EndIndex;
  }

  [CCode (cname = "wimp_menu_entry")]
  public struct MenuEntry
  {
    [CCode (cname = "menu_flags")]
    public unowned MenuFlags MenuFlags;
    [CCode (cname = "sub_menu")]
    public unowned Menu* SubMenu;
    [CCode (cname = "icon_flags")]
    public unowned IconFlags IconFlags;
    [CCode (cname = "data")]
    public unowned IconData Data;
  }

  [CCode (cname = "wimp_menu_base")]
  public struct MenuBase
  {
    [CCode (cname = "title_data.text")]
    public unowned char TitleData_Text[12];
    [CCode (cname = "title_data.indirected_text.text")]
    public unowned string TitleData_IndirectedText_Text;
    [CCode (cname = "title_data.indirected_text.reserved")]
    public unowned byte TitleData_IndirectedText_Reserved[8];
    [CCode (cname = "title_fg")]
    public unowned Colour TitleFg;
    [CCode (cname = "title_bg")]
    public unowned Colour TitleBg;
    [CCode (cname = "work_fg")]
    public unowned Colour WorkFg;
    [CCode (cname = "work_bg")]
    public unowned Colour WorkBg;
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "gap")]
    public unowned int Gap;
  }

  [Compact]
  [CCode (cname = "wimp_menu", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class Menu
  {
    public Menu(int numEntries);
    [CCode (cname = "title_data.text")]
    public unowned char TitleData_Text[12];
    [CCode (cname = "title_data.indirected_text.text")]
    public unowned string TitleData_IndirectedText_Text;
    [CCode (cname = "title_data.indirected_text.reserved")]
    public unowned byte TitleData_IndirectedText_Reserved[8];
    [CCode (cname = "title_fg")]
    public unowned Colour TitleFg;
    [CCode (cname = "title_bg")]
    public unowned Colour TitleBg;
    [CCode (cname = "work_fg")]
    public unowned Colour WorkFg;
    [CCode (cname = "work_bg")]
    public unowned Colour WorkBg;
    [CCode (cname = "width")]
    public unowned int Width;
    [CCode (cname = "height")]
    public unowned int Height;
    [CCode (cname = "gap")]
    public unowned int Gap;
    [CCode (cname = "entries", array_length = false)]
    public unowned MenuEntry[] Entries;
  }

  [CCode (cname = "wimp_pre_quit_flags")]
  public enum PreQuitFlags
  {
    [CCode (cname = "wimp_PRE_QUIT_TASK_ONLY")]
    PreQuitTaskOnly
  }

  [CCode (cname = "wimp_save_desktop_flags")]
  public enum SaveDesktopFlags
  {
    Default = 0
  }

  [CCode (cname = "wimp_shutdown_flags")]
  public enum ShutdownFlags
  {
    Default = 0
  }

  [CCode (cname = "wimp_claim_flags")]
  public enum ClaimFlags
  {
    [CCode (cname = "wimp_CLAIM_CARET_OR_SELECTION")]
    ClaimCaretOrSelection,
    [CCode (cname = "wimp_CLAIM_CLIPBOARD")]
    ClaimClipboard
  }

  [CCode (cname = "wimp_data_request_flags")]
  public enum DataRequestFlags
  {
    [CCode (cname = "wimp_DATA_REQUEST_CLIPBOARD")]
    DataRequestClipboard
  }

  [CCode (cname = "wimp_dragging_flags")]
  public enum DraggingFlags
  {
    [CCode (cname = "wimp_DRAGGING_FROM_SELECTION")]
    DraggingFromSelection,
    [CCode (cname = "wimp_DRAGGING_FROM_CLIPBOARD")]
    DraggingFromClipboard,
    [CCode (cname = "wimp_DRAGGING_DELETE_SOURCE")]
    DraggingDeleteSource,
    [CCode (cname = "wimp_DRAGGING_TERMINATE_DRAG")]
    DraggingTerminateDrag
  }

  [CCode (cname = "wimp_drag_claim_flags")]
  public enum DragClaimFlags
  {
    [CCode (cname = "wimp_DRAG_CLAIM_POINTER_CHANGED")]
    DragClaimPointerChanged,
    [CCode (cname = "wimp_DRAG_CLAIM_SUPRESS_DRAGBOX")]
    DragClaimSupressDragbox,
    [CCode (cname = "wimp_DRAG_CLAIM_DELETE_SOURCE")]
    DragClaimDeleteSource
  }

  [CCode (cname = "wimp_release_flags")]
  public enum ReleaseFlags
  {
    [CCode (cname = "wimp_RELEASE_CLIPBOARD")]
    ReleaseClipboard
  }

  [CCode (cname = "wimp_app_control_flags")]
  public enum AppControlFlags
  {
    [CCode (cname = "wimp_APP_CONTROL_IMMEDIATE")]
    AppControlImmediate
  }

  [CCode (cname = "wimp_app_control_operation")]
  public enum AppControlOperation
  {
    [CCode (cname = "wimp_APP_CONTROL_STOP")]
    AppControlStop,
    [CCode (cname = "wimp_APP_CONTROL_WAKE_UP")]
    AppControlWakeUp,
    [CCode (cname = "wimp_APP_CONTROL_PAUSE")]
    AppControlPause,
    [CCode (cname = "wimp_APP_CONTROL_MINIMISE_MEMORY")]
    AppControlMinimiseMemory,
    [CCode (cname = "wimp_APP_CONTROL_OPEN_CONFIGURATION")]
    AppControlOpenConfiguration
  }

  [CCode (cname = "wimp_message_header_base")]
  public struct MessageHeaderBase
  {
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "sender")]
    public unowned T Sender;
    [CCode (cname = "my_ref")]
    public unowned int MyRef;
    [CCode (cname = "your_ref")]
    public unowned int YourRef;
    [CCode (cname = "action")]
    public unowned bits Action;
  }

  [Compact]
  [CCode (cname = "wimp_message_header", cheader_filename = "oslib/valavapi.h", free_function = "free")]
  public class MessageHeader
  {
    public MessageHeader(int numEntries);
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "sender")]
    public unowned T Sender;
    [CCode (cname = "my_ref")]
    public unowned int MyRef;
    [CCode (cname = "your_ref")]
    public unowned int YourRef;
    [CCode (cname = "action")]
    public unowned bits Action;
    [CCode (cname = "data", array_length = false)]
    public unowned byte[] Data;
  }

  [CCode (cname = "wimp_message_data_xfer")]
  public struct MessageDataXfer
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "est_size")]
    public unowned int EstSize;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "file_name")]
    public unowned char FileName[212];
  }

  [CCode (cname = "wimp_message_ram_xfer")]
  public struct MessageRAMXfer
  {
    [CCode (cname = "addr")]
    public unowned char* Addr;
    [CCode (cname = "size")]
    public unowned int Size;
  }

  [CCode (cname = "wimp_message_prequit")]
  public struct MessagePrequit
  {
    [CCode (cname = "flags")]
    public unowned PreQuitFlags Flags;
  }

  [CCode (cname = "wimp_message_save_desktop")]
  public struct MessageSaveDesktop
  {
    [CCode (cname = "file")]
    public unowned OS.F File;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[3];
    [CCode (cname = "flags")]
    public unowned SaveDesktopFlags Flags;
  }

  [CCode (cname = "wimp_message_save_desktopw")]
  public struct MessageSaveDesktopW
  {
    [CCode (cname = "file")]
    public unowned OS.FW File;
    [CCode (cname = "flags")]
    public unowned SaveDesktopFlags Flags;
  }

  [CCode (cname = "wimp_message_device")]
  public struct MessageDevice
  {
    [CCode (cname = "major")]
    public unowned int Major;
    [CCode (cname = "minor")]
    public unowned int Minor;
    [CCode (cname = "info")]
    public unowned char Info[228];
  }

  [CCode (cname = "wimp_message_data_saved")]
  public struct MessageDataSaved
  {
    [CCode (cname = "ref")]
    public unowned int Ref;
  }

  [CCode (cname = "wimp_message_shutdown")]
  public struct MessageShutdown
  {
    [CCode (cname = "flags")]
    public unowned ShutdownFlags Flags;
  }

  [CCode (cname = "wimp_message_claim_entity")]
  public struct MessageClaimEntity
  {
    [CCode (cname = "flags")]
    public unowned ClaimFlags Flags;
  }

  [CCode (cname = "wimp_message_data_request")]
  public struct MessageDataRequest
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "flags")]
    public unowned DataRequestFlags Flags;
    [CCode (cname = "file_types")]
    public unowned bits FileTypes[54];
  }

  [CCode (cname = "wimp_message_dragging")]
  public struct MessageDragging
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "flags")]
    public unowned DraggingFlags Flags;
    [CCode (cname = "box")]
    public unowned OS.Box Box;
    [CCode (cname = "file_types")]
    public unowned bits FileTypes[50];
  }

  [CCode (cname = "wimp_message_drag_claim")]
  public struct MessageDragClaim
  {
    [CCode (cname = "flags")]
    public unowned DragClaimFlags Flags;
    [CCode (cname = "file_types")]
    public unowned bits FileTypes[58];
  }

  [CCode (cname = "wimp_message_release_entity")]
  public struct MessageReleaseEntity
  {
    [CCode (cname = "flags")]
    public unowned ReleaseFlags Flags;
  }

  [CCode (cname = "wimp_message_app_control")]
  public struct MessageAppControl
  {
    [CCode (cname = "protocol_version")]
    public unowned int ProtocolVersion;
    [CCode (cname = "flags")]
    public unowned AppControlFlags Flags;
    [CCode (cname = "operation")]
    public unowned AppControlOperation Operation;
  }

  [CCode (cname = "wimp_full_message_data_xfer")]
  public struct FullMessageDataXfer : MessageHeaderBase
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "est_size")]
    public unowned int EstSize;
    [CCode (cname = "file_type")]
    public unowned bits FileType;
    [CCode (cname = "file_name")]
    public unowned char FileName[212];
  }

  [CCode (cname = "wimp_full_message_ram_xfer")]
  public struct FullMessageRAMXfer : MessageHeaderBase
  {
    [CCode (cname = "addr")]
    public unowned char* Addr;
    [CCode (cname = "xfer_size")]
    public unowned int XferSize;
  }

  [CCode (cname = "wimp_full_message_prequit")]
  public struct FullMessagePrequit : MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned PreQuitFlags Flags;
  }

  [CCode (cname = "wimp_full_message_save_desktopw")]
  public struct FullMessageSaveDesktopW : MessageHeaderBase
  {
    [CCode (cname = "file")]
    public unowned OS.FW File;
    [CCode (cname = "flags")]
    public unowned SaveDesktopFlags Flags;
  }

  [CCode (cname = "wimp_full_message_device")]
  public struct FullMessageDevice : MessageHeaderBase
  {
    [CCode (cname = "major")]
    public unowned int Major;
    [CCode (cname = "minor")]
    public unowned int Minor;
    [CCode (cname = "info")]
    public unowned char Info[228];
  }

  [CCode (cname = "wimp_full_message_data_saved")]
  public struct FullMessageDataSaved : MessageHeaderBase
  {
    [CCode (cname = "ref")]
    public unowned int Ref;
  }

  [CCode (cname = "wimp_full_message_shutdown")]
  public struct FullMessageShutdown : MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ShutdownFlags Flags;
  }

  [CCode (cname = "wimp_full_message_claim_entity")]
  public struct FullMessageClaimEntity : MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ClaimFlags Flags;
  }

  [CCode (cname = "wimp_full_message_data_request")]
  public struct FullMessageDataRequest : MessageHeaderBase
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "flags")]
    public unowned DataRequestFlags Flags;
    [CCode (cname = "file_types")]
    public unowned bits FileTypes[54];
  }

  [CCode (cname = "wimp_full_message_dragging")]
  public struct FullMessageDragging : MessageHeaderBase
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "i")]
    public unowned I I;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "flags")]
    public unowned DraggingFlags Flags;
    [CCode (cname = "box")]
    public unowned OS.Box Box;
    [CCode (cname = "file_types")]
    public unowned bits FileTypes[50];
  }

  [CCode (cname = "wimp_full_message_drag_claim")]
  public struct FullMessageDragClaim : MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned DragClaimFlags Flags;
    [CCode (cname = "file_types")]
    public unowned bits FileTypes[58];
  }

  [CCode (cname = "wimp_full_message_release_entity")]
  public struct FullMessageReleaseEntity : MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ReleaseFlags Flags;
  }

  [CCode (cname = "wimp_full_message_app_control")]
  public struct FullMessageAppControl : MessageHeaderBase
  {
    [CCode (cname = "protocol_version")]
    public unowned int ProtocolVersion;
    [CCode (cname = "flags")]
    public unowned AppControlFlags Flags;
    [CCode (cname = "operation")]
    public unowned AppControlOperation Operation;
  }

  [CCode (cname = "wimp_iconise_flags")]
  public enum IconiseFlags
  {
    [CCode (cname = "wimp_ICONISE_WITH_SHIFT_CLOSE")]
    IconiseWithShiftClose
  }

  [CCode (cname = "wimp_toggle_backdrop_flags")]
  public enum ToggleBackdropFlags
  {
    [CCode (cname = "wimp_TOGGLE_BACKDROP_TYPE")]
    ToggleBackdropType,
    [CCode (cname = "wimp_TOGGLE_BACKDROP")]
    ToggleBackdrop,
    [CCode (cname = "wimp_TOGGLE_BACKDROP_TO_FRONT")]
    ToggleBackdropToFront,
    [CCode (cname = "wimp_TOGGLE_BACKDROP_TO_BACK")]
    ToggleBackdropToBack,
    [CCode (cname = "wimp_TOGGLE_BACKDROP_GET_POSITION")]
    ToggleBackdropGetPosition
  }

  [CCode (cname = "wimp_screen_edge_notification_flags")]
  public enum ScreenEdgeNotificationFlags
  {
    [CCode (cname = "wimp_SCREEN_EDGE_NOTIFICATION_LEFT")]
    ScreenEdgeNotificationLeft,
    [CCode (cname = "wimp_SCREEN_EDGE_NOTIFICATION_BOTTOM")]
    ScreenEdgeNotificationBottom,
    [CCode (cname = "wimp_SCREEN_EDGE_NOTIFICATION_RIGHT")]
    ScreenEdgeNotificationRight,
    [CCode (cname = "wimp_SCREEN_EDGE_NOTIFICATION_TOP")]
    ScreenEdgeNotificationTop
  }

  [CCode (cname = "wimp_message_menu_warning")]
  public struct MessageMenuWarning
  {
    [CCode (cname = "sub_menu")]
    public unowned Menu* SubMenu;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "selection")]
    public unowned Selection Selection;
  }

  [CCode (cname = "wimp_message_task_initialise")]
  public struct MessageTaskInitialise
  {
    [CCode (cname = "cao")]
    public unowned char* Cao;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "task_name")]
    public unowned char TaskName[228];
  }

  [CCode (cname = "wimp_message_slot_size")]
  public struct MessageSlotSize
  {
    [CCode (cname = "new_curr")]
    public unowned int NewCurr;
    [CCode (cname = "new_next")]
    public unowned int NewNext;
  }

  [CCode (cname = "wimp_message_set_slot")]
  public struct MessageSetSlot
  {
    [CCode (cname = "new_curr")]
    public unowned int NewCurr;
    [CCode (cname = "task")]
    public unowned T Task;
  }

  [CCode (cname = "wimp_message_task_name_rq")]
  public struct MessageTaskNameRq
  {
    [CCode (cname = "task")]
    public unowned T Task;
  }

  [CCode (cname = "wimp_message_task_name_is")]
  public struct MessageTaskNameIs
  {
    [CCode (cname = "task")]
    public unowned T Task;
    [CCode (cname = "size")]
    public unowned int Size;
    [CCode (cname = "task_name")]
    public unowned char TaskName[228];
  }

  [CCode (cname = "wimp_message_menus_deleted")]
  public struct MessageMenusDeleted
  {
    [CCode (cname = "menu")]
    public unowned Menu* Menu;
  }

  [CCode (cname = "wimp_message_iconise")]
  public struct MessageIconise
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "task")]
    public unowned T Task;
    [CCode (cname = "title")]
    public unowned char Title[20];
  }

  [CCode (cname = "wimp_message_window_closed")]
  public struct MessageWindowClosed
  {
    [CCode (cname = "w")]
    public unowned W W;
  }

  [CCode (cname = "wimp_message_window_info")]
  public struct MessageWindowInfo
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
    [CCode (cname = "sprite_name")]
    public unowned char SpriteName[8];
    [CCode (cname = "title")]
    public unowned char Title[20];
  }

  [CCode (cname = "wimp_message_font_changed")]
  public struct MessageFontChanged
  {
    [CCode (cname = "font")]
    public unowned Font.F Font;
  }

  [CCode (cname = "wimp_message_iconise_at")]
  public struct MessageIconiseAt
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "task")]
    public unowned T Task;
    [CCode (cname = "at")]
    public unowned OS.Coord At;
    [CCode (cname = "flags")]
    public unowned IconiseFlags Flags;
  }

  [CCode (cname = "wimp_message_toggle_backdrop")]
  public struct MessageToggleBackdrop
  {
    [CCode (cname = "flags")]
    public unowned ToggleBackdropFlags Flags;
  }

  [CCode (cname = "wimp_message_screen_edge_notification")]
  public struct MessageScreenEdgeNotification
  {
    [CCode (cname = "flags")]
    public unowned ScreenEdgeNotificationFlags Flags;
  }

  [CCode (cname = "wimp_full_message_menu_warning")]
  public struct FullMessageMenuWarning : MessageHeaderBase
  {
    [CCode (cname = "sub_menu")]
    public unowned Menu* SubMenu;
    [CCode (cname = "pos")]
    public unowned OS.Coord Pos;
    [CCode (cname = "selection")]
    public unowned Selection Selection;
  }

  [CCode (cname = "wimp_full_message_mode_change")]
  public enum FullMessageModeChange
  {
    Default = 0
  }

  [CCode (cname = "wimp_full_message_task_initialise")]
  public struct FullMessageTaskInitialise : MessageHeaderBase
  {
    [CCode (cname = "cao")]
    public unowned char* Cao;
    [CCode (cname = "memory_size")]
    public unowned int MemorySize;
    [CCode (cname = "task_name")]
    public unowned char TaskName[228];
  }

  [CCode (cname = "wimp_full_message_task_close_down")]
  public enum FullMessageTaskCloseDown
  {
    Default = 0
  }

  [CCode (cname = "wimp_full_message_slot_size")]
  public struct FullMessageSlotSize : MessageHeaderBase
  {
    [CCode (cname = "new_curr")]
    public unowned int NewCurr;
    [CCode (cname = "new_next")]
    public unowned int NewNext;
  }

  [CCode (cname = "wimp_full_message_set_slot")]
  public struct FullMessageSetSlot : MessageHeaderBase
  {
    [CCode (cname = "new_curr")]
    public unowned int NewCurr;
    [CCode (cname = "task")]
    public unowned T Task;
  }

  [CCode (cname = "wimp_full_message_task_name_rq")]
  public struct FullMessageTaskNameRq : MessageHeaderBase
  {
    [CCode (cname = "task")]
    public unowned T Task;
  }

  [CCode (cname = "wimp_full_message_task_name_is")]
  public struct FullMessageTaskNameIs : MessageHeaderBase
  {
    [CCode (cname = "task")]
    public unowned T Task;
    [CCode (cname = "slot_size")]
    public unowned int SlotSize;
    [CCode (cname = "task_name")]
    public unowned char TaskName[228];
  }

  [CCode (cname = "wimp_full_message_task_started")]
  public enum FullMessageTaskStarted
  {
    Default = 0
  }

  [CCode (cname = "wimp_full_message_menus_deleted")]
  public struct FullMessageMenusDeleted : MessageHeaderBase
  {
    [CCode (cname = "menu")]
    public unowned Menu* Menu;
  }

  [CCode (cname = "wimp_full_message_iconise")]
  public struct FullMessageIconise : MessageHeaderBase
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "task")]
    public unowned T Task;
    [CCode (cname = "title")]
    public unowned char Title[20];
  }

  [CCode (cname = "wimp_full_message_window_closed")]
  public struct FullMessageWindowClosed : MessageHeaderBase
  {
    [CCode (cname = "w")]
    public unowned W W;
  }

  [CCode (cname = "wimp_full_message_window_info")]
  public struct FullMessageWindowInfo : MessageHeaderBase
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
    [CCode (cname = "sprite_name")]
    public unowned char SpriteName[8];
    [CCode (cname = "title")]
    public unowned char Title[20];
  }

  [CCode (cname = "wimp_full_message_swap")]
  public enum FullMessageSwap
  {
    Default = 0
  }

  [CCode (cname = "wimp_full_message_tools_changed")]
  public enum FullMessageToolsChanged
  {
    Default = 0
  }

  [CCode (cname = "wimp_full_message_font_changed")]
  public struct FullMessageFontChanged : MessageHeaderBase
  {
    [CCode (cname = "font")]
    public unowned Font.F Font;
  }

  [CCode (cname = "wimp_full_message_iconise_at")]
  public struct FullMessageIconiseAt : MessageHeaderBase
  {
    [CCode (cname = "w")]
    public unowned W W;
    [CCode (cname = "task")]
    public unowned T Task;
    [CCode (cname = "at")]
    public unowned OS.Coord At;
    [CCode (cname = "flags")]
    public unowned IconiseFlags Flags;
  }

  [CCode (cname = "wimp_full_message_toggle_backdrop")]
  public struct FullMessageToggleBackdrop : MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ToggleBackdropFlags Flags;
  }

  [CCode (cname = "wimp_full_message_screen_edge_notification")]
  public struct FullMessageScreenEdgeNotification : MessageHeaderBase
  {
    [CCode (cname = "flags")]
    public unowned ScreenEdgeNotificationFlags Flags;
  }

  [CCode (cname = "wimp_message")]
  public struct Message : MessageHeaderBase
  {
    [CCode (cname = "data.data_xfer")]
    public unowned MessageDataXfer Data_DataXfer;
    [CCode (cname = "data.ram_xfer")]
    public unowned MessageRAMXfer Data_RamXfer;
    [CCode (cname = "data.prequit")]
    public unowned MessagePrequit Data_Prequit;
    [CCode (cname = "data.save_desktop")]
    public unowned MessageSaveDesktop Data_SaveDesktop;
    [CCode (cname = "data.save_desktopw")]
    public unowned MessageSaveDesktopW Data_SaveDesktopw;
    [CCode (cname = "data.device")]
    public unowned MessageDevice Data_Device;
    [CCode (cname = "data.data_saved")]
    public unowned MessageDataSaved Data_DataSaved;
    [CCode (cname = "data.shutdown")]
    public unowned MessageShutdown Data_Shutdown;
    [CCode (cname = "data.claim_entity")]
    public unowned MessageClaimEntity Data_ClaimEntity;
    [CCode (cname = "data.data_request")]
    public unowned MessageDataRequest Data_DataRequest;
    [CCode (cname = "data.dragging")]
    public unowned MessageDragging Data_Dragging;
    [CCode (cname = "data.drag_claim")]
    public unowned MessageDragClaim Data_DragClaim;
    [CCode (cname = "data.release_entity")]
    public unowned MessageReleaseEntity Data_ReleaseEntity;
    [CCode (cname = "data.app_control")]
    public unowned MessageAppControl Data_AppControl;
    [CCode (cname = "data.reserved")]
    public unowned byte Data_Reserved[236];
  }

  [CCode (cname = "wimp_block")]
  public struct Block
  {
    [CCode (cname = "redraw")]
    public unowned Draw Redraw;
    [CCode (cname = "open")]
    public unowned Open Open;
    [CCode (cname = "close")]
    public unowned Close Close;
    [CCode (cname = "leaving")]
    public unowned Leaving Leaving;
    [CCode (cname = "entering")]
    public unowned Entering Entering;
    [CCode (cname = "pointer")]
    public unowned Pointer Pointer;
    [CCode (cname = "dragged")]
    public unowned Dragged Dragged;
    [CCode (cname = "key")]
    public unowned Key Key;
    [CCode (cname = "selection")]
    public unowned Selection Selection;
    [CCode (cname = "scroll")]
    public unowned Scroll Scroll;
    [CCode (cname = "caret")]
    public unowned Caret Caret;
    [CCode (cname = "pollword")]
    public unowned Pollword Pollword;
    [CCode (cname = "message")]
    public unowned Message Message;
    [CCode (cname = "reserved")]
    public unowned byte Reserved[256];
  }

  [CCode (cname = "wimp_message_list")]
  public struct MessageList
  {
    [CCode (cname = "messages", array_length = false)]
    public unowned int[] Messages;
  }

  [CCode (cname = "wimp_priority")]
  public enum Priority
  {
    [CCode (cname = "wimp_PRIORITY_TASK_MANAGER")]
    TaskManager,
    [CCode (cname = "wimp_PRIORITY_PALETTE_UTILITY")]
    PaletteUtility,
    [CCode (cname = "wimp_PRIORITY_ADFS_FILER_HARD_DISCS")]
    ADFSFilerHardDiscs,
    [CCode (cname = "wimp_PRIORITY_ADFS_FILER_FLOPPY_DISCS")]
    ADFSFilerFloppyDiscs,
    [CCode (cname = "wimp_PRIORITY_RESOURCE_FILER")]
    ResourceFiler,
    [CCode (cname = "wimp_PRIORITY_RAMFS_FILER")]
    RAMFSFiler,
    [CCode (cname = "wimp_PRIORITY_NFS_FILER")]
    NFSFiler,
    [CCode (cname = "wimp_PRIORITY_NET_FILER")]
    NetFiler,
    [CCode (cname = "wimp_PRIORITY_PRINTERS")]
    Printers,
    [CCode (cname = "wimp_PRIORITY_TINY_DIRS")]
    TinyDirs
  }

  [CCode (cname = "wimp_plot_icon_flags")]
  public enum PlotIconFlags
  {
    Default = 0
  }

  [CCode (cname = "wimp_command_window_state")]
  public enum CommandWindowState
  {
    [CCode (cname = "wimp_COMMAND_WINDOW_INACTIVE")]
    CommandWindowInactive,
    [CCode (cname = "wimp_COMMAND_WINDOW_PENDING")]
    CommandWindowPending,
    [CCode (cname = "wimp_COMMAND_WINDOW_OPEN")]
    CommandWindowOpen
  }

  [CCode (cname = "wimp_menu_state_flags")]
  public enum MenuStateFlags
  {
    [CCode (cname = "wimp_GIVEN_WINDOW_AND_ICON")]
    GivenWindowAndIcon
  }

  [CCode (cname = "wimp_filter_action")]
  public enum FilterAction
  {
    [CCode (cname = "wimp_FILTER_PRE_POLL")]
    FilterPrePoll,
    [CCode (cname = "wimp_FILTER_POST_POLL")]
    FilterPostPoll,
    [CCode (cname = "wimp_FILTER_COPY")]
    FilterCopy,
    [CCode (cname = "wimp_FILTER_RECT")]
    FilterRect,
    [CCode (cname = "wimp_FILTER_POST_RECT")]
    FilterPostRect,
    [CCode (cname = "wimp_FILTER_POST_ICON")]
    FilterPostIcon
  }

  [CCode (cname = "wimptextop_string_width_flags")]
  public enum WimpTextOp_StringWidthFlags
  {
    [CCode (cname = "wimptextop_SIZE_COVERED")]
    SizeCovered
  }

  [CCode (cname = "wimptextop_paint_flags")]
  public enum WimpTextOp_PaintFlags
  {
    [CCode (cname = "wimptextop_VJUSTIFY")]
    VJustify,
    [CCode (cname = "wimptextop_OFFSETS_FOR_REDRAW")]
    OffsetsForRedraw,
    [CCode (cname = "wimptextop_GIVEN_BASELINE")]
    GivenBaseline,
    [CCode (cname = "wimptextop_RJUSTIFY")]
    RJustify
  }

  [CCode (cname="wimp_ICON_NAME_LIMIT")]
  public const int IconNameLimit;
  [CCode (cname="wimp_TEMPLATE_NAME_LIMIT")]
  public const int TemplateNameLimit;
  [CCode (cname="wimp_MENU_NAME_LIMIT")]
  public const int MenuNameLimit;
  [CCode (cname="wimp_MENU_DEPTH_LIMIT")]
  public const int MenuDepthLimit;
  [CCode (cname="wimp_CHAR_XSIZE")]
  public const int CharXSize;
  [CCode (cname="wimp_CHAR_YSIZE")]
  public const int CharYSize;
  [CCode (cname="wimp_BACKGROUND")]
  public const W Background;
  [CCode (cname="wimp_ICON_BAR")]
  public const W IconBar;
  [CCode (cname="wimp_ICON_BAR_RIGHT")]
  public const W IconBarRight;
  [CCode (cname="wimp_ICON_BAR_LEFT")]
  public const W IconBarLeft;
  [CCode (cname="wimp_ICON_BAR_LEFT_RELATIVE")]
  public const W IconBarLeftRelative;
  [CCode (cname="wimp_ICON_BAR_RIGHT_RELATIVE")]
  public const W IconBarRightRelative;
  [CCode (cname="wimp_ICON_BAR_LEFT_HIGH_PRIORITY")]
  public const W IconBarLeftHighPriority;
  [CCode (cname="wimp_ICON_BAR_LEFT_LOW_PRIORITY")]
  public const W IconBarLeftLowPriority;
  [CCode (cname="wimp_ICON_BAR_RIGHT_LOW_PRIORITY")]
  public const W IconBarRightLowPriority;
  [CCode (cname="wimp_ICON_BAR_RIGHT_HIGH_PRIORITY")]
  public const W IconBarRightHighPriority;
  [CCode (cname="wimp_TOP")]
  public const W Top;
  [CCode (cname="wimp_BOTTOM")]
  public const W Bottom;
  [CCode (cname="wimp_HIDDEN")]
  public const W Hidden;
  [CCode (cname="wimp_NO_ICON")]
  public const W NoIcon;
  [CCode (cname="wimp_CHILD_LS_EDGE_SHIFT")]
  public const int ChildLSEdgeShift;
  [CCode (cname="wimp_CHILD_BS_EDGE_SHIFT")]
  public const int ChildBSEdgeShift;
  [CCode (cname="wimp_CHILD_RS_EDGE_SHIFT")]
  public const int ChildRSEdgeShift;
  [CCode (cname="wimp_CHILD_TS_EDGE_SHIFT")]
  public const int ChildTSEdgeShift;
  [CCode (cname="wimp_CHILD_XORIGIN_SHIFT")]
  public const int ChildXOriginShift;
  [CCode (cname="wimp_CHILD_YORIGIN_SHIFT")]
  public const int ChildYOriginShift;
  [CCode (cname="wimp_ICON_BUTTON_TYPE_SHIFT")]
  public const int IconButtonTypeShift;
  [CCode (cname="wimp_ICON_ESG_SHIFT")]
  public const int IconESGShift;
  [CCode (cname="wimp_ICON_FG_COLOUR_SHIFT")]
  public const int IconFGColourShift;
  [CCode (cname="wimp_ICON_BG_COLOUR_SHIFT")]
  public const int IconBGColourShift;
  [CCode (cname="wimp_ICON_FONT_HANDLE_SHIFT")]
  public const int IconFontHandleShift;
  [CCode (cname="wimp_CLOSE_MENU")]
  public static Menu* CloseMenu;
  [CCode (cname="wimp_NO_SUB_MENU")]
  public static Menu* NoSubMenu;
  [CCode (cname="wimp_DEFER_SUB_MENU")]
  public static Menu* DeferSubMenu;
  [CCode (cname="wimp_NO_FONTS")]
  public static byte* NoFonts;
  [CCode (cname="wimp_GET_SIZE")]
  public static Window* GetSize;
  [CCode (cname="wimp_MENU_ITEM_HEIGHT")]
  public const int MenuItemHeight;
  [CCode (cname="wimp_MENU_ITEM_GAP")]
  public const int MenuItemGap;
  [CCode (cname="wimp_MENU_ITEM_SEPARATION")]
  public const int MenuItemSeparation;
  [CCode (cname="message_QUIT")]
  public const bits Message_Quit;
  [CCode (cname="message_DATA_SAVE")]
  public const bits Message_DataSave;
  [CCode (cname="message_DATA_SAVE_ACK")]
  public const bits Message_DataSaveAck;
  [CCode (cname="message_DATA_LOAD")]
  public const bits Message_DataLoad;
  [CCode (cname="message_DATA_LOAD_ACK")]
  public const bits Message_DataLoadAck;
  [CCode (cname="message_DATA_OPEN")]
  public const bits Message_DataOpen;
  [CCode (cname="message_RAM_FETCH")]
  public const bits Message_RAMFetch;
  [CCode (cname="message_RAM_TRANSMIT")]
  public const bits Message_RAMTransmit;
  [CCode (cname="message_PREQUIT")]
  public const bits Message_Prequit;
  [CCode (cname="message_PRE_QUIT")]
  public const bits Message_PreQuit;
  [CCode (cname="message_PALETTE_CHANGE")]
  public const bits Message_PaletteChange;
  [CCode (cname="message_SAVE_DESKTOP")]
  public const bits Message_SaveDesktop;
  [CCode (cname="message_DEVICE_CLAIM")]
  public const bits Message_DeviceClaim;
  [CCode (cname="message_DEVICE_IN_USE")]
  public const bits Message_DeviceInUse;
  [CCode (cname="message_DATA_SAVED")]
  public const bits Message_DataSaved;
  [CCode (cname="message_SHUTDOWN")]
  public const bits Message_Shutdown;
  [CCode (cname="message_CLAIM_ENTITY")]
  public const bits Message_ClaimEntity;
  [CCode (cname="message_DATA_REQUEST")]
  public const bits Message_DataRequest;
  [CCode (cname="message_DRAGGING")]
  public const bits Message_Dragging;
  [CCode (cname="message_DRAG_CLAIM")]
  public const bits Message_DragClaim;
  [CCode (cname="message_RELEASE_ENTITY")]
  public const bits Message_ReleaseEntity;
  [CCode (cname="message_APP_CONTROL")]
  public const bits Message_AppControl;
  [CCode (cname="message_MENU_WARNING")]
  public const bits Message_MenuWarning;
  [CCode (cname="message_MODE_CHANGE")]
  public const bits Message_ModeChange;
  [CCode (cname="message_TASK_INITIALISE")]
  public const bits Message_TaskInitialise;
  [CCode (cname="message_TASK_CLOSE_DOWN")]
  public const bits Message_TaskCloseDown;
  [CCode (cname="message_SLOT_SIZE")]
  public const bits Message_SlotSize;
  [CCode (cname="message_SET_SLOT")]
  public const bits Message_SetSlot;
  [CCode (cname="message_TASK_NAME_RQ")]
  public const bits Message_TaskNameRq;
  [CCode (cname="message_TASK_NAME_IS")]
  public const bits Message_TaskNameIs;
  [CCode (cname="message_TASK_STARTED")]
  public const bits Message_TaskStarted;
  [CCode (cname="message_MENUS_DELETED")]
  public const bits Message_MenusDeleted;
  [CCode (cname="message_ICONISE")]
  public const bits Message_Iconise;
  [CCode (cname="message_WINDOW_CLOSED")]
  public const bits Message_WindowClosed;
  [CCode (cname="message_WINDOW_INFO")]
  public const bits Message_WindowInfo;
  [CCode (cname="message_SWAP")]
  public const bits Message_Swap;
  [CCode (cname="message_TOOLS_CHANGED")]
  public const bits Message_ToolsChanged;
  [CCode (cname="message_FONT_CHANGED")]
  public const bits Message_FontChanged;
  [CCode (cname="message_ICONISE_AT")]
  public const bits Message_IconiseAt;
  [CCode (cname="message_TOGGLE_BACKDROP")]
  public const bits Message_ToggleBackdrop;
  [CCode (cname="message_SCREEN_EDGE_NOTIFICATION")]
  public const bits Message_ScreenEdgeNotification;
  [CCode (cname="wimp_ERROR_BOX_CATEGORY_SHIFT")]
  public const int ErrorBoxCategoryShift;
  [CCode (cname="wimp_ERROR_BOX_CONTINUE_RESPONSE_SHIFT")]
  public const int ErrorBoxContinueResponseShift;
  [CCode (cname="wimp_ERROR_BOX_QUIT_RESPONSE_SHIFT")]
  public const int ErrorBoxQuitResponseShift;
  [CCode (cname="error_WIMP_NO_CLAIM")]
  public const bits Error_WimpNoClaim;
  [CCode (cname="error_WIMP_BAD_OP")]
  public const bits Error_WimpBadOp;
  [CCode (cname="error_WIMP_RECT_FULL")]
  public const bits Error_WimpRectFull;
  [CCode (cname="error_WIMP_TOO_MANY")]
  public const bits Error_WimpTooMany;
  [CCode (cname="error_WIMP_TOO_BIG")]
  public const bits Error_WimpTooBig;
  [CCode (cname="error_WIMP_DIV_BY0")]
  public const bits Error_WimpDivBy0;
  [CCode (cname="error_WIMP_GET_RECT")]
  public const bits Error_WimpGetRect;
  [CCode (cname="error_WIMP_FOCUS")]
  public const bits Error_WimpFocus;
  [CCode (cname="error_WIMP_BAD_HANDLE")]
  public const bits Error_WimpBadHandle;
  [CCode (cname="error_WIMP_TOO_MENUS")]
  public const bits Error_WimpTooMenus;
  [CCode (cname="error_WIMP_BAD_EXTENT")]
  public const bits Error_WimpBadExtent;
  [CCode (cname="error_WIMP_BAD_POINTER")]
  public const bits Error_WimpBadPointer;
  [CCode (cname="error_WIMP_NO_TEMPLATE_FILE")]
  public const bits Error_WimpNoTemplateFile;
  [CCode (cname="error_WIMP_BAD_TEMPLATE")]
  public const bits Error_WimpBadTemplate;
  [CCode (cname="error_WIMP_BAD_FONTS")]
  public const bits Error_WimpBadFonts;
  [CCode (cname="error_WIMP_BAD_SYNTAX")]
  public const bits Error_WimpBadSyntax;
  [CCode (cname="error_WIMP_NO_TASKS")]
  public const bits Error_WimpNoTasks;
  [CCode (cname="error_WIMP_NO_TEMPLATE")]
  public const bits Error_WimpNoTemplate;
  [CCode (cname="error_WIMP_NOT_PRESENT")]
  public const bits Error_WimpNotPresent;
  [CCode (cname="error_WIMP_IN_USE")]
  public const bits Error_WimpInUse;
  [CCode (cname="error_WIMP_BAD_PAL_FILE")]
  public const bits Error_WimpBadPalFile;
  [CCode (cname="error_WIMP_BAD_VERSION")]
  public const bits Error_WimpBadVersion;
  [CCode (cname="error_WIMP_BAD_MESSAGE_SIZE")]
  public const bits Error_WimpBadMessageSize;
  [CCode (cname="error_WIMP_BAD_REASON_CODE")]
  public const bits Error_WimpBadReasonCode;
  [CCode (cname="error_WIMP_BAD_TASK_HANDLE")]
  public const bits Error_WimpBadTaskHandle;
  [CCode (cname="error_WIMP_CANT_TASK")]
  public const bits Error_WimpCantTask;
  [CCode (cname="error_WIMP_BAD_SUB_MENU")]
  public const bits Error_WimpBadSubMenu;
  [CCode (cname="error_WIMP_OWNER_WINDOW")]
  public const bits Error_WimpOwnerWindow;
  [CCode (cname="error_WIMP_BAD_MODE")]
  public const bits Error_WimpBadMode;
  [CCode (cname="error_WIMP_BAD_TRANSFER")]
  public const bits Error_WimpBadTransfer;
  [CCode (cname="error_WIMP_BAD_SYS_INFO")]
  public const bits Error_WimpBadSysInfo;
  [CCode (cname="error_WIMP_BAD_PTR_IN_R1")]
  public const bits Error_WimpBadPtrInR1;
  [CCode (cname="error_WIMP_BAD_ESCAPE_STATE")]
  public const bits Error_WimpBadEscapeState;
  [CCode (cname="error_WIMP_BAD_ICON_HANDLE")]
  public const bits Error_WimpBadIconHandle;
  [CCode (cname="error_WIMP_BAD_R3")]
  public const bits Error_WimpBadR3;
  [CCode (cname="error_WIMP_NO_DOT_IN_FILENAME")]
  public const bits Error_WimpNoDotInFilename;
  [CCode (cname="error_WIMP_TEMPLATE_EOF")]
  public const bits Error_WimpTemplateEOF;
  [CCode (cname="error_WIMP_BAD_SLOT")]
  public const bits Error_WimpBadSlot;
  [CCode (cname="error_WIMP_BAD_CONF_FONT")]
  public const bits Error_WimpBadConfFont;
  [CCode (cname="error_WIMP_NO_SPRITES")]
  public const bits Error_WimpNoSprites;
  [CCode (cname="error_WIMP_BAD_PAL_FILE2")]
  public const bits Error_WimpBadPalFile2;
  [CCode (cname="error_WIMP_BAD_SPRITES")]
  public const bits Error_WimpBadSprites;
  [CCode (cname="error_WIMP_BAD_PARENT")]
  public const bits Error_WimpBadParent;
  [CCode (cname="wimp_DEFAULT_PALETTE")]
  public static OS.Palette* DefaultPalette;
  [CCode (cname="wimp_PALETTE")]
  public static OS.Palette* Palette;

  [CCode (cname = "xwimp_initialise")]
  public static OS.Error* XInitialise
    (VersionNo version,
     string name,
     MessageList messages,
     out VersionNo version_out,
     out T task_out);

  [CCode (cname = "wimp_initialise")]
  public static T Initialise
    (VersionNo version,
     string name,
     MessageList messages,
     out VersionNo version_out);

  [CCode (cname = "xwimp_init")]
  public static OS.Error* XInit
    (VersionNo version,
     bits r1,
     string name,
     MessageList messages,
     out VersionNo version_out,
     out T task_out);

  [CCode (cname = "wimp_init")]
  public static T Init
    (VersionNo version,
     bits r1,
     string name,
     MessageList messages,
     out VersionNo version_out);

  [CCode (cname = "xwimp_create_window")]
  public static OS.Error* XCreateWindow
    (Window window,
     out W w);

  [CCode (cname = "wimp_create_window")]
  public static W CreateWindow
    (Window window);

  [CCode (cname = "xwimp_create_icon")]
  public static OS.Error* XCreateIcon
    (IconCreate icon,
     out I i);

  [CCode (cname = "wimp_create_icon")]
  public static I CreateIcon
    (IconCreate icon);

  [CCode (cname = "xwimp_create_icon_relative")]
  public static OS.Error* XCreateIconRelative
    (I neighbour,
     IconCreate icon,
     out I i);

  [CCode (cname = "wimp_create_icon_relative")]
  public static I CreateIconRelative
    (I neighbour,
     IconCreate icon);

  [CCode (cname = "xwimp_create_icon_prioritised")]
  public static OS.Error* XCreateIconPrioritised
    (Priority priority,
     IconCreate icon,
     out I i);

  [CCode (cname = "wimp_create_icon_prioritised")]
  public static I CreateIconPrioritised
    (Priority priority,
     IconCreate icon);

  [CCode (cname = "xwimp_delete_window")]
  public static OS.Error* XDeleteWindow
    (W w);

  [CCode (cname = "wimp_delete_window")]
  public static void DeleteWindow
    (W w);

  [CCode (cname = "xwimp_delete_icon")]
  public static OS.Error* XDeleteIcon
    (W w,
     I i);

  [CCode (cname = "wimp_delete_icon")]
  public static void DeleteIcon
    (W w,
     I i);

  [CCode (cname = "xwimp_open_window")]
  public static OS.Error* XOpenWindow
    (Open open);

  [CCode (cname = "wimp_open_window")]
  public static void OpenWindow
    (Open open);

  [CCode (cname = "xwimp_open_window_nested")]
  public static OS.Error* XOpenWindowNested
    (Open open,
     W parent,
     WindowNestingFlags linkage);

  [CCode (cname = "wimp_open_window_nested")]
  public static void OpenWindowNested
    (Open open,
     W parent,
     WindowNestingFlags linkage);

  [CCode (cname = "xwimp_open_window_nested_with_flags")]
  public static OS.Error* XOpenWindowNestedWithFlags
    (WindowState open,
     W parent,
     WindowNestingFlags linkage);

  [CCode (cname = "wimp_open_window_nested_with_flags")]
  public static void OpenWindowNestedWithFlags
    (WindowState open,
     W parent,
     WindowNestingFlags linkage);

  [CCode (cname = "xwimp_close_window")]
  public static OS.Error* XCloseWindow
    (W w);

  [CCode (cname = "wimp_close_window")]
  public static void CloseWindow
    (W w);

  [CCode (cname = "xwimp_poll")]
  public static OS.Error* XPoll
    (PollFlags mask,
     Block block,
     int* pollword,
     out EventNo event);

  [CCode (cname = "wimp_poll")]
  public static EventNo Poll
    (PollFlags mask,
     Block block,
     int* pollword);

  [CCode (cname = "xwimp_redraw_window")]
  public static OS.Error* XRedrawWindow
    (Draw redraw,
     out bool more);

  [CCode (cname = "wimp_redraw_window")]
  public static bool RedrawWindow
    (Draw redraw);

  [CCode (cname = "xwimp_update_window")]
  public static OS.Error* XUpdateWindow
    (Draw update,
     out bool more);

  [CCode (cname = "wimp_update_window")]
  public static bool UpdateWindow
    (Draw update);

  [CCode (cname = "xwimp_get_rectangle")]
  public static OS.Error* XGetRectangle
    (Draw draw,
     out bool more);

  [CCode (cname = "wimp_get_rectangle")]
  public static bool GetRectangle
    (Draw draw);

  [CCode (cname = "xwimp_get_window_state")]
  public static OS.Error* XGetWindowState
    (WindowState state);

  [CCode (cname = "wimp_get_window_state")]
  public static void GetWindowState
    (WindowState state);

  [CCode (cname = "xwimp_get_window_state_and_nesting")]
  public static OS.Error* XGetWindowStateAndNesting
    (WindowState state,
     out W parent,
     out bits linkage);

  [CCode (cname = "wimp_get_window_state_and_nesting")]
  public static W GetWindowStateAndNesting
    (WindowState state,
     out bits linkage);

  [CCode (cname = "xwimp_get_window_info")]
  public static OS.Error* XGetWindowInfo
    (WindowInfo info);

  [CCode (cname = "wimp_get_window_info")]
  public static void GetWindowInfo
    (WindowInfo info);

  [CCode (cname = "xwimp_get_window_info_header_only")]
  public static OS.Error* XGetWindowInfoHeaderOnly
    (WindowInfo info);

  [CCode (cname = "wimp_get_window_info_header_only")]
  public static void GetWindowInfoHeaderOnly
    (WindowInfo info);

  [CCode (cname = "xwimp_set_icon_state")]
  public static OS.Error* XSetIconState
    (W w,
     I i,
     IconFlags eor_bits,
     IconFlags clear_bits);

  [CCode (cname = "wimp_set_icon_state")]
  public static void SetIconState
    (W w,
     I i,
     IconFlags eor_bits,
     IconFlags clear_bits);

  [CCode (cname = "xwimp_set_icon_state_from_block")]
  public static OS.Error* XSetIconStateFromBlock
    (SetIconStateBlock block);

  [CCode (cname = "wimp_set_icon_state_from_block")]
  public static void SetIconStateFromBlock
    (SetIconStateBlock block);

  [CCode (cname = "xwimp_get_icon_state")]
  public static OS.Error* XGetIconState
    (IconState icon_state);

  [CCode (cname = "wimp_get_icon_state")]
  public static void GetIconState
    (IconState icon_state);

  [CCode (cname = "xwimp_get_pointer_info")]
  public static OS.Error* XGetPointerInfo
    (Pointer pointer);

  [CCode (cname = "wimp_get_pointer_info")]
  public static void GetPointerInfo
    (Pointer pointer);

  [CCode (cname = "xwimp_drag_box")]
  public static OS.Error* XDragBox
    (Drag drag);

  [CCode (cname = "wimp_drag_box")]
  public static void DragBox
    (Drag drag);

  [CCode (cname = "xwimp_drag_box_with_flags")]
  public static OS.Error* XDragBoxWithFlags
    (Drag drag,
     DragBoxFlags flags);

  [CCode (cname = "wimp_drag_box_with_flags")]
  public static void DragBoxWithFlags
    (Drag drag,
     DragBoxFlags flags);

  [CCode (cname = "xwimp_force_redraw")]
  public static OS.Error* XForceRedraw
    (W w,
     int x0,
     int y0,
     int x1,
     int y1);

  [CCode (cname = "wimp_force_redraw")]
  public static void ForceRedraw
    (W w,
     int x0,
     int y0,
     int x1,
     int y1);

  [CCode (cname = "xwimp_force_redraw_furniture")]
  public static OS.Error* XForceRedrawFurniture
    (W w,
     Furniture item);

  [CCode (cname = "wimp_force_redraw_furniture")]
  public static void ForceRedrawFurniture
    (W w,
     Furniture item);

  [CCode (cname = "xwimp_force_redraw_title")]
  public static OS.Error* XForceRedrawTitle
    (W w);

  [CCode (cname = "wimp_force_redraw_title")]
  public static void ForceRedrawTitle
    (W w);

  [CCode (cname = "xwimp_set_caret_position")]
  public static OS.Error* XSetCaretPosition
    (W w,
     I i,
     int x,
     int y,
     int height,
     int index);

  [CCode (cname = "wimp_set_caret_position")]
  public static void SetCaretPosition
    (W w,
     I i,
     int x,
     int y,
     int height,
     int index);

  [CCode (cname = "xwimpsetcaretpositionextended_caret")]
  public static OS.Error* XWimpSetCaretPositionExtended_Caret
    (Caret caret);

  [CCode (cname = "wimpsetcaretpositionextended_caret")]
  public static void WimpSetCaretPositionExtended_Caret
    (Caret caret);

  [CCode (cname = "xwimpsetcaretpositionextended_selection")]
  public static OS.Error* XWimpSetCaretPositionExtended_Selection
    (TextSelection selection);

  [CCode (cname = "wimpsetcaretpositionextended_selection")]
  public static void WimpSetCaretPositionExtended_Selection
    (TextSelection selection);

  [CCode (cname = "xwimp_get_caret_position")]
  public static OS.Error* XGetCaretPosition
    (Caret caret);

  [CCode (cname = "wimp_get_caret_position")]
  public static void GetCaretPosition
    (Caret caret);

  [CCode (cname = "xwimpgetcaretpositionextended_caret")]
  public static OS.Error* XWimpGetCaretPositionExtended_Caret
    (Caret caret);

  [CCode (cname = "wimpgetcaretpositionextended_caret")]
  public static void WimpGetCaretPositionExtended_Caret
    (Caret caret);

  [CCode (cname = "xwimpgetcaretpositionextended_selection")]
  public static OS.Error* XWimpGetCaretPositionExtended_Selection
    (TextSelection selection);

  [CCode (cname = "wimpgetcaretpositionextended_selection")]
  public static void WimpGetCaretPositionExtended_Selection
    (TextSelection selection);

  [CCode (cname = "xwimp_create_menu")]
  public static OS.Error* XCreateMenu
    (Menu menu,
     int x,
     int y);

  [CCode (cname = "wimp_create_menu")]
  public static void CreateMenu
    (Menu menu,
     int x,
     int y);

  [CCode (cname = "xwimp_decode_menu")]
  public static OS.Error* XDecodeMenu
    (Menu menu,
     Selection selection,
     string decode);

  [CCode (cname = "wimp_decode_menu")]
  public static void DecodeMenu
    (Menu menu,
     Selection selection,
     string decode);

  [CCode (cname = "xwimp_which_icon")]
  public static OS.Error* XWhichIcon
    (W w,
     I which,
     IconFlags mask,
     IconFlags match);

  [CCode (cname = "wimp_which_icon")]
  public static void WhichIcon
    (W w,
     I which,
     IconFlags mask,
     IconFlags match);

  [CCode (cname = "xwimp_set_extent")]
  public static OS.Error* XSetExtent
    (W w,
     OS.Box box);

  [CCode (cname = "wimp_set_extent")]
  public static void SetExtent
    (W w,
     OS.Box box);

  [CCode (cname = "xwimp_set_pointer_shape")]
  public static OS.Error* XSetPointerShape
    (int shape_no,
     char* shape_data,
     int width,
     int height,
     int xactive,
     int yactive);

  [CCode (cname = "wimp_set_pointer_shape")]
  public static void SetPointerShape
    (int shape_no,
     char* shape_data,
     int width,
     int height,
     int xactive,
     int yactive);

  [CCode (cname = "xwimp_open_template")]
  public static OS.Error* XOpenTemplate
    (string file_name);

  [CCode (cname = "wimp_open_template")]
  public static void OpenTemplate
    (string file_name);

  [CCode (cname = "xwimp_close_template")]
  public static OS.Error* XCloseTemplate ();

  [CCode (cname = "wimp_close_template")]
  public static void CloseTemplate ();

  [CCode (cname = "xwimp_load_template")]
  public static OS.Error* XLoadTemplate
    (Window window,
     string data,
     char* end,
     byte* font_ref,
     string name,
     int context,
     out int used,
     out int data_used,
     out int context_out);

  [CCode (cname = "wimp_load_template")]
  public static int LoadTemplate
    (Window window,
     string data,
     char* end,
     byte* font_ref,
     string name,
     int context,
     out int used,
     out int data_used);

  [CCode (cname = "xwimp_process_key")]
  public static OS.Error* XProcessKey
    (KeyNo c);

  [CCode (cname = "wimp_process_key")]
  public static void ProcessKey
    (KeyNo c);

  [CCode (cname = "xwimp_close_down")]
  public static OS.Error* XCloseDown
    (T t);

  [CCode (cname = "wimp_close_down")]
  public static void CloseDown
    (T t);

  [CCode (cname = "xwimp_start_task")]
  public static OS.Error* XStartTask
    (string command,
     out T t);

  [CCode (cname = "wimp_start_task")]
  public static T StartTask
    (string command);

  [CCode (cname = "xwimp_report_error")]
  public static OS.Error* XReportError
    (OS.Error error,
     ErrorBoxFlags flags,
     string name,
     out ErrorBoxSelection click);

  [CCode (cname = "wimp_report_error")]
  public static ErrorBoxSelection ReportError
    (OS.Error error,
     ErrorBoxFlags flags,
     string name);

  [CCode (cname = "xwimp_report_error_by_category")]
  public static OS.Error* XReportErrorByCategory
    (OS.Error error,
     ErrorBoxFlags flags,
     string name,
     string sprite_name,
     OSSpriteOp.Area area,
     string buttons,
     out ErrorBoxSelection click);

  [CCode (cname = "wimp_report_error_by_category")]
  public static ErrorBoxSelection ReportErrorByCategory
    (OS.Error error,
     ErrorBoxFlags flags,
     string name,
     string sprite_name,
     OSSpriteOp.Area area,
     string buttons);

  [CCode (cname = "xwimp_get_window_outline")]
  public static OS.Error* XGetWindowOutline
    (Outline outline);

  [CCode (cname = "wimp_get_window_outline")]
  public static void GetWindowOutline
    (Outline outline);

  [CCode (cname = "xwimp_poll_idle")]
  public static OS.Error* XPollIdle
    (PollFlags mask,
     Block block,
     OS.T t,
     int* pollword,
     out EventNo event);

  [CCode (cname = "wimp_poll_idle")]
  public static EventNo PollIdle
    (PollFlags mask,
     Block block,
     OS.T t,
     int* pollword);

  [CCode (cname = "xwimp_plot_icon")]
  public static OS.Error* XPlotIcon
    (Icon icon);

  [CCode (cname = "wimp_plot_icon")]
  public static void PlotIcon
    (Icon icon);

  [CCode (cname = "xwimp_plot_icon_with_window")]
  public static OS.Error* XPlotIconWithWindow
    (Icon icon,
     int x_origin,
     int y_origin,
     PlotIconFlags flags,
     W window);

  [CCode (cname = "wimp_plot_icon_with_window")]
  public static void PlotIconWithWindow
    (Icon icon,
     int x_origin,
     int y_origin,
     PlotIconFlags flags,
     W window);

  [CCode (cname = "xwimp_plot_icon_with_sprite_area")]
  public static OS.Error* XPlotIconWithSpriteArea
    (Icon icon,
     int x_origin,
     int y_origin,
     PlotIconFlags flags,
     Colour background_colour,
     OSSpriteOp.Area sprite_area);

  [CCode (cname = "wimp_plot_icon_with_sprite_area")]
  public static void PlotIconWithSpriteArea
    (Icon icon,
     int x_origin,
     int y_origin,
     PlotIconFlags flags,
     Colour background_colour,
     OSSpriteOp.Area sprite_area);

  [CCode (cname = "xwimp_plot_icon_with_window_sprite_area")]
  public static OS.Error* XPlotIconWithWindowSpriteArea
    (Icon icon,
     PlotIconFlags flags,
     int x_origin,
     int y_origin,
     W window,
     OSSpriteOp.Area sprite_area);

  [CCode (cname = "wimp_plot_icon_with_window_sprite_area")]
  public static void PlotIconWithWindowSpriteArea
    (Icon icon,
     PlotIconFlags flags,
     int x_origin,
     int y_origin,
     W window,
     OSSpriteOp.Area sprite_area);

  [CCode (cname = "xwimp_plot_icon_with_background_sprite_area")]
  public static OS.Error* XPlotIconWithBackgroundSpriteArea
    (Icon icon,
     PlotIconFlags flags,
     int x_origin,
     int y_origin,
     Colour background_colour,
     OSSpriteOp.Area sprite_area);

  [CCode (cname = "wimp_plot_icon_with_background_sprite_area")]
  public static void PlotIconWithBackgroundSpriteArea
    (Icon icon,
     PlotIconFlags flags,
     int x_origin,
     int y_origin,
     Colour background_colour,
     OSSpriteOp.Area sprite_area);

  [CCode (cname = "xwimp_set_mode")]
  public static OS.Error* XSetMode
    (OS.Mode mode);

  [CCode (cname = "wimp_set_mode")]
  public static void SetMode
    (OS.Mode mode);

  [CCode (cname = "xwimp_set_palette")]
  public static OS.Error* XSetPalette
    (OS.Palette palette);

  [CCode (cname = "wimp_set_palette")]
  public static void SetPalette
    (OS.Palette palette);

  [CCode (cname = "xwimp_read_palette")]
  public static OS.Error* XReadPalette
    (OS.Palette palette);

  [CCode (cname = "wimp_read_palette")]
  public static void ReadPalette
    (OS.Palette palette);

  [CCode (cname = "xwimp_read_true_palette")]
  public static OS.Error* XReadTruePalette
    (OS.Palette palette);

  [CCode (cname = "wimp_read_true_palette")]
  public static void ReadTruePalette
    (OS.Palette palette);

  [CCode (cname = "xwimp_set_colour")]
  public static OS.Error* XSetColour
    (Colour colour);

  [CCode (cname = "wimp_set_colour")]
  public static void SetColour
    (Colour colour);

  [CCode (cname = "xwimp_send_message")]
  public static OS.Error* XSendMessage
    (EventNo event,
     Message message,
     T to);

  [CCode (cname = "wimp_send_message")]
  public static void SendMessage
    (EventNo event,
     Message message,
     T to);

  [CCode (cname = "xwimp_send_message_to_window")]
  public static OS.Error* XSendMessageToWindow
    (EventNo event,
     Message message,
     W to_w,
     I to_i,
     out T to_t);

  [CCode (cname = "wimp_send_message_to_window")]
  public static T SendMessageToWindow
    (EventNo event,
     Message message,
     W to_w,
     I to_i);

  [CCode (cname = "xwimp_create_sub_menu")]
  public static OS.Error* XCreateSubMenu
    (Menu sub_menu,
     int x,
     int y);

  [CCode (cname = "wimp_create_sub_menu")]
  public static void CreateSubMenu
    (Menu sub_menu,
     int x,
     int y);

  [CCode (cname = "xwimp_base_of_sprites")]
  public static OS.Error* XBaseOfSprites
    (out OSSpriteOp.Area rom,
     out OSSpriteOp.Area ram);

  [CCode (cname = "wimp_base_of_sprites")]
  public static void BaseOfSprites
    (out OSSpriteOp.Area rom,
     out OSSpriteOp.Area ram);

  [CCode (cname = "xwimp_block_copy")]
  public static OS.Error* XBlockCopy
    (W w,
     int x0,
     int y0,
     int x1,
     int y1,
     int X0,
     int Y0);

  [CCode (cname = "wimp_block_copy")]
  public static void BlockCopy
    (W w,
     int x0,
     int y0,
     int x1,
     int y1,
     int X0,
     int Y0);

  [CCode (cname = "xwimp_slot_size")]
  public static OS.Error* XSlotSize
    (int new_curr_slot,
     int new_next_slot,
     out int curr_slot,
     out int next_slot,
     out int free_slot);

  [CCode (cname = "wimp_slot_size")]
  public static void SlotSize
    (int new_curr_slot,
     int new_next_slot,
     out int curr_slot,
     out int next_slot,
     out int free_slot);

  [CCode (cname = "xwimp_read_pix_trans")]
  public static OS.Error* XReadPixTrans
    (OSSpriteOp.Flags flags,
     OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     OS.Factors factors,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "wimp_read_pix_trans")]
  public static void ReadPixTrans
    (OSSpriteOp.Flags flags,
     OSSpriteOp.Area area,
     OSSpriteOp.Id id,
     OS.Factors factors,
     OSSpriteOp.TransTab trans_tab);

  [CCode (cname = "xwimpclaimfreememory_alloc")]
  public static OS.Error* XWimpClaimFreeMemory_Alloc
    (int size,
     out int size_out,
     out char* blk);

  [CCode (cname = "wimpclaimfreememory_alloc")]
  public static char* WimpClaimFreeMemory_Alloc
    (int size,
     out int size_out);

  [CCode (cname = "xwimpclaimfreememory_free")]
  public static OS.Error* XWimpClaimFreeMemory_Free ();

  [CCode (cname = "wimpclaimfreememory_free")]
  public static void WimpClaimFreeMemory_Free ();

  [CCode (cname = "xwimp_command_window")]
  public static OS.Error* XCommandWindow
    (string op);

  [CCode (cname = "wimp_command_window")]
  public static void CommandWindow
    (string op);

  [CCode (cname = "xwimp_command_window_activate")]
  public static OS.Error* XCommandWindowActivate ();

  [CCode (cname = "wimp_command_window_activate")]
  public static void CommandWindowActivate ();

  [CCode (cname = "xwimp_command_window_close_with_prompt")]
  public static OS.Error* XCommandWindowCloseWithPrompt ();

  [CCode (cname = "wimp_command_window_close_with_prompt")]
  public static void CommandWindowCloseWithPrompt ();

  [CCode (cname = "xwimp_command_window_close")]
  public static OS.Error* XCommandWindowClose ();

  [CCode (cname = "wimp_command_window_close")]
  public static void CommandWindowClose ();

  [CCode (cname = "xwimp_command_window_read_state")]
  public static OS.Error* XCommandWindowReadState
    (out CommandWindowState state);

  [CCode (cname = "wimp_command_window_read_state")]
  public static CommandWindowState CommandWindowReadState ();

  [CCode (cname = "xwimp_text_colour")]
  public static OS.Error* XTextColour
    (Colour colour);

  [CCode (cname = "wimp_text_colour")]
  public static void TextColour
    (Colour colour);

  [CCode (cname = "xwimp_transfer_block")]
  public static OS.Error* XTransferBlock
    (T from_t,
     char* from,
     T to_t,
     char* to,
     int len);

  [CCode (cname = "wimp_transfer_block")]
  public static void TransferBlock
    (T from_t,
     char* from,
     T to_t,
     char* to,
     int len);

  [CCode (cname = "xwimp_set_font_colours")]
  public static OS.Error* XSetFontColours
    (Colour fg,
     Colour bg);

  [CCode (cname = "wimp_set_font_colours")]
  public static void SetFontColours
    (Colour fg,
     Colour bg);

  [CCode (cname = "xwimp_get_menu_state")]
  public static OS.Error* XGetMenuState
    (MenuStateFlags flags,
     Selection selection,
     W w,
     I i);

  [CCode (cname = "wimp_get_menu_state")]
  public static void GetMenuState
    (MenuStateFlags flags,
     Selection selection,
     W w,
     I i);

  [CCode (cname = "xwimp_register_filter")]
  public static OS.Error* XRegisterFilter
    (FilterAction type,
     AsmRoutine filter_code,
     char* handle);

  [CCode (cname = "wimp_register_filter")]
  public static void RegisterFilter
    (FilterAction type,
     AsmRoutine filter_code,
     char* handle);

  [CCode (cname = "xwimp_add_messages")]
  public static OS.Error* XAddMessages
    (MessageList messages);

  [CCode (cname = "wimp_add_messages")]
  public static void AddMessages
    (MessageList messages);

  [CCode (cname = "xwimp_remove_messages")]
  public static OS.Error* XRemoveMessages
    (MessageList messages);

  [CCode (cname = "wimp_remove_messages")]
  public static void RemoveMessages
    (MessageList messages);

  [CCode (cname = "xwimp_set_colour_mapping")]
  public static OS.Error* XSetColourMapping
    (OS.Palette palette,
     byte* map2,
     byte* map4,
     byte* map16);

  [CCode (cname = "wimp_set_colour_mapping")]
  public static void SetColourMapping
    (OS.Palette palette,
     byte* map2,
     byte* map4,
     byte* map16);

  [CCode (cname = "xwimptextop_set_colour")]
  public static OS.Error* XWimpTextOp_SetColour
    (OS.Colour fg,
     OS.Colour bg);

  [CCode (cname = "wimptextop_set_colour")]
  public static void WimpTextOp_SetColour
    (OS.Colour fg,
     OS.Colour bg);

  [CCode (cname = "xwimptextop_string_width")]
  public static OS.Error* XWimpTextOp_StringWidth
    (string s,
     int len,
     out int width);

  [CCode (cname = "wimptextop_string_width")]
  public static int WimpTextOp_StringWidth
    (string s,
     int len);

  [CCode (cname = "xwimptextop_string_width_with_flags")]
  public static OS.Error* XWimpTextOp_StringWidthWithFlags
    (WimpTextOp_StringWidthFlags flags,
     string s,
     int len,
     out int width);

  [CCode (cname = "wimptextop_string_width_with_flags")]
  public static int WimpTextOp_StringWidthWithFlags
    (WimpTextOp_StringWidthFlags flags,
     string s,
     int len);

  [CCode (cname = "xwimptextop_paint")]
  public static OS.Error* XWimpTextOp_Paint
    (WimpTextOp_PaintFlags flags,
     string s,
     int x0,
     int y0);

  [CCode (cname = "wimptextop_paint")]
  public static void WimpTextOp_Paint
    (WimpTextOp_PaintFlags flags,
     string s,
     int x0,
     int y0);

  [CCode (cname = "xwimptextop_get_split_point")]
  public static OS.Error* XWimpTextOp_GetSplitPoint
    (string s,
     int width,
     int split_char,
     out string split_point);

  [CCode (cname = "wimptextop_get_split_point")]
  public static string WimpTextOp_GetSplitPoint
    (string s,
     int width,
     int split_char);

  [CCode (cname = "xwimptextop_truncate_with_ellipsis")]
  public static OS.Error* XWimpTextOp_TruncateWithEllipsis
    (string s,
     string out_buf,
     int buf_size,
     int width,
     out int buf_size_required);

  [CCode (cname = "wimptextop_truncate_with_ellipsis")]
  public static int WimpTextOp_TruncateWithEllipsis
    (string s,
     string out_buf,
     int buf_size,
     int width);

  [CCode (cname = "xwimp_set_watchdog_state")]
  public static OS.Error* XSetWatchdogState
    (bool enable,
     int code_word);

  [CCode (cname = "wimp_set_watchdog_state")]
  public static void SetWatchdogState
    (bool enable,
     int code_word);

  [CCode (cname = "xwimp_resize_icon")]
  public static OS.Error* XResizeIcon
    (W w,
     I i,
     int x0,
     int y0,
     int x1,
     int y1);

  [CCode (cname = "wimp_resize_icon")]
  public static void ResizeIcon
    (W w,
     I i,
     int x0,
     int y0,
     int x1,
     int y1);

  [CCode (cname = "xwimp_auto_scroll")]
  public static OS.Error* XAutoScroll
    (AutoScrollFlags flags,
     AutoScrollInfo info,
     out AutoScrollFlags flags_out);

  [CCode (cname = "wimp_auto_scroll")]
  public static AutoScrollFlags AutoScroll
    (AutoScrollFlags flags,
     AutoScrollInfo info);

  [CCode (cname = "xservice_memory")]
  public static OS.Error* XService_Memory
    (int change,
     char* cao);

  [CCode (cname = "service_memory")]
  public static void Service_Memory
    (int change,
     char* cao);

  [CCode (cname = "xservice_start_wimp")]
  public static OS.Error* XService_StartWimp ();

  [CCode (cname = "service_start_wimp")]
  public static void Service_StartWimp ();

  [CCode (cname = "xservice_started_wimp")]
  public static OS.Error* XService_StartedWimp ();

  [CCode (cname = "service_started_wimp")]
  public static void Service_StartedWimp ();

  [CCode (cname = "xservice_mouse_trap")]
  public static OS.Error* XService_MouseTrap ();

  [CCode (cname = "service_mouse_trap")]
  public static void Service_MouseTrap ();

  [CCode (cname = "xservice_wimp_close_down")]
  public static OS.Error* XService_WimpCloseDown ();

  [CCode (cname = "service_wimp_close_down")]
  public static void Service_WimpCloseDown ();

  [CCode (cname = "xservice_wimp_report_error")]
  public static OS.Error* XService_WimpReportError
    (bool opening);

  [CCode (cname = "service_wimp_report_error")]
  public static void Service_WimpReportError
    (bool opening);

  [CCode (cname = "xservice_wimp_save_desktop")]
  public static OS.Error* XService_WimpSaveDesktop
    (SaveDesktopFlags flags,
     OS.F file,
     out OS.Error error,
     out bool unclaimed);

  [CCode (cname = "service_wimp_save_desktop")]
  public static bool Service_WimpSaveDesktop
    (SaveDesktopFlags flags,
     OS.F file,
     out OS.Error error);

  [CCode (cname = "xservice_wimp_save_desktopw")]
  public static OS.Error* XService_WimpSaveDesktopW
    (SaveDesktopFlags flags,
     OS.FW file,
     out OS.Error error,
     out bool unclaimed);

  [CCode (cname = "service_wimp_save_desktopw")]
  public static bool Service_WimpSaveDesktopW
    (SaveDesktopFlags flags,
     OS.FW file,
     out OS.Error error);

  [CCode (cname = "xservice_wimp_palette")]
  public static OS.Error* XService_WimpPalette ();

  [CCode (cname = "service_wimp_palette")]
  public static void Service_WimpPalette ();

  [CCode (cname = "xservice_desktop_welcome")]
  public static OS.Error* XService_DesktopWelcome ();

  [CCode (cname = "service_desktop_welcome")]
  public static void Service_DesktopWelcome ();

  [CCode (cname = "xservice_shut_down")]
  public static OS.Error* XService_ShutDown ();

  [CCode (cname = "service_shut_down")]
  public static void Service_ShutDown ();

  [CCode (cname = "xservice_shut_down_complete")]
  public static OS.Error* XService_ShutDownComplete ();

  [CCode (cname = "service_shut_down_complete")]
  public static void Service_ShutDownComplete ();

  [CCode (cname = "xservice_wimp_sprites_moved")]
  public static OS.Error* XService_WimpSpritesMoved ();

  [CCode (cname = "service_wimp_sprites_moved")]
  public static void Service_WimpSpritesMoved ();

  [CCode (cname = "xservice_wimp_register_filters")]
  public static OS.Error* XService_WimpRegisterFilters ();

  [CCode (cname = "service_wimp_register_filters")]
  public static void Service_WimpRegisterFilters ();

  [CCode (cname = "xservice_error_starting")]
  public static OS.Error* XService_ErrorStarting
    (OS.Error error,
     ErrorBoxFlags flags,
     string name,
     string sprite_name,
     OSSpriteOp.Area area,
     string buttons,
     out OS.Error error_out,
     out ErrorBoxFlags flags_out,
     out string name_out,
     out string sprite_name_out,
     out OSSpriteOp.Area area_out,
     out string buttons_out);

  [CCode (cname = "service_error_starting")]
  public static void Service_ErrorStarting
    (OS.Error error,
     ErrorBoxFlags flags,
     string name,
     string sprite_name,
     OSSpriteOp.Area area,
     string buttons,
     out OS.Error error_out,
     out ErrorBoxFlags flags_out,
     out string name_out,
     out string sprite_name_out,
     out OSSpriteOp.Area area_out,
     out string buttons_out);

  [CCode (cname = "xservice_error_button_pressed")]
  public static OS.Error* XService_ErrorButtonPressed
    (ErrorBoxSelection button,
     string button_list,
     out bool redisplay,
     out bool unclaimed,
     out ErrorBoxSelection button_out);

  [CCode (cname = "service_error_button_pressed")]
  public static bool Service_ErrorButtonPressed
    (ErrorBoxSelection button,
     string button_list,
     out bool redisplay,
     out ErrorBoxSelection button_out);

  [CCode (cname = "xservice_error_ending")]
  public static OS.Error* XService_ErrorEnding
    (ErrorBoxSelection button,
     out bool unclaimed,
     out ErrorBoxSelection button_out);

  [CCode (cname = "service_error_ending")]
  public static bool Service_ErrorEnding
    (ErrorBoxSelection button,
     out ErrorBoxSelection button_out);

  [CCode (cname = "xservice_wimp_tool_sprites_changed")]
  public static OS.Error* XService_WimpToolSpritesChanged
    (OSSpriteOp.Area tool_area);

  [CCode (cname = "service_wimp_tool_sprites_changed")]
  public static void Service_WimpToolSpritesChanged
    (OSSpriteOp.Area tool_area);

}
