/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.Toolbox;
using OSLib.Gadget;
using OSLib.Font;

[CCode (cheader_filename = "oslib/writablefield.h")]
namespace OSLib.WritableField
{
  [CCode (cname = "writablefield_gadget")]
  public struct Gadget
  {
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "text_limit")]
    public unowned int TextLimit;
    [CCode (cname = "allowable")]
    public unowned Toolbox.MsgReference Allowable;
    [CCode (cname = "allowable_limit")]
    public unowned int AllowableLimit;
    [CCode (cname = "prev")]
    public unowned Toolbox.C Prev;
    [CCode (cname = "next")]
    public unowned Toolbox.C Next;
  }

  [CCode (cname = "writablefield_object")]
  public struct Object : OSLib.Gadget.ObjectBase
  {
    [CCode (cname = "text")]
    public unowned Toolbox.MsgReference Text;
    [CCode (cname = "text_limit")]
    public unowned int TextLimit;
    [CCode (cname = "allowable")]
    public unowned Toolbox.MsgReference Allowable;
    [CCode (cname = "allowable_limit")]
    public unowned int AllowableLimit;
    [CCode (cname = "prev")]
    public unowned Toolbox.C Prev;
    [CCode (cname = "next")]
    public unowned Toolbox.C Next;
  }

  [CCode (cname = "writablefield_action_value_changed")]
  public struct ActionValueChanged
  {
    [CCode (cname = "string")]
    public unowned char String[212];
  }

  [CCode (cname = "writablefield_action_value_changed_block")]
  public struct ActionValueChangedBlock : Toolbox.ActionHeaderBase
  {
    [CCode (cname = "string")]
    public unowned char String[212];
  }

  [CCode (cname="class_WRITABLE_FIELD")]
  public const Toolbox.Class Class_WritableField;
  [CCode (cname="writablefield_GENERATE_USER_VALUE_CHANGED")]
  public const bits GenerateUserValueChanged;
  [CCode (cname="writablefield_GENERATE_SET_VALUE_CHANGED")]
  public const bits GenerateSetValueChanged;
  [CCode (cname="writablefield_RJUSTIFIED")]
  public const bits RJustified;
  [CCode (cname="writablefield_HCENTRED")]
  public const bits HCentred;
  [CCode (cname="writablefield_CONCEAL_TEXT")]
  public const bits ConcealText;
  [CCode (cname="action_WRITABLE_FIELD_VALUE_CHANGED")]
  public const bits Action_WritableFieldValueChanged;
  [CCode (cname="writablefield_VALUE_TOO_LONG")]
  public const bits ValueTooLong;

  [CCode (cname = "xwritablefield_set_value")]
  public static OS.Error* XSetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C writable_field,
     string value);

  [CCode (cname = "writablefield_set_value")]
  public static void SetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C writable_field,
     string value);

  [CCode (cname = "xwritablefield_get_value")]
  public static OS.Error* XGetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C writable_field,
     string buffer,
     int size,
     out int used);

  [CCode (cname = "writablefield_get_value")]
  public static int GetValue
    (bits flags,
     Toolbox.O window,
     Toolbox.C writable_field,
     string buffer,
     int size);

  [CCode (cname = "xwritablefield_set_allowable")]
  public static OS.Error* XSetAllowable
    (bits flags,
     Toolbox.O window,
     Toolbox.C writable_field,
     string allowed);

  [CCode (cname = "writablefield_set_allowable")]
  public static void SetAllowable
    (bits flags,
     Toolbox.O window,
     Toolbox.C writable_field,
     string allowed);

  [CCode (cname = "xwritablefield_set_font")]
  public static OS.Error* XSetFont
    (bits flags,
     Toolbox.O window,
     Toolbox.C writable_field,
     string font_name,
     int xsize,
     int ysize);

  [CCode (cname = "writablefield_set_font")]
  public static void SetFont
    (bits flags,
     Toolbox.O window,
     Toolbox.C writable_field,
     string font_name,
     int xsize,
     int ysize);

}
