/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/zlib.h")]
namespace OSLib.ZLib
{
  [CCode (cname = "zlib_data_type")]
  public enum DataType
  {
    [CCode (cname = "zlib_BINARY_DATA")]
    BinaryData,
    [CCode (cname = "zlib_ASCII_DATA")]
    ASCIIData,
    [CCode (cname = "zlib_UNKNOWN_DATA")]
    UnknownData
  }

  [CCode (cname = "zlib_flush_type")]
  public enum FlushType
  {
    [CCode (cname = "zlib_NO_FLUSH")]
    NoFlush,
    [CCode (cname = "zlib_PARTIAL_FLUSH")]
    PartialFlush,
    [CCode (cname = "zlib_SYNC_FLUSH")]
    SyncFlush,
    [CCode (cname = "zlib_FULL_FLUSH")]
    FullFlush,
    [CCode (cname = "zlib_FINISH")]
    Finish
  }

  [CCode (cname = "zlib_compression_level")]
  public enum CompressionLevel
  {
    [CCode (cname = "zlib_NO_COMPRESSION")]
    NoCompression,
    [CCode (cname = "zlib_BEST_SPEED")]
    BestSpeed,
    [CCode (cname = "zlib_BEST_COMPRESSION")]
    BestCompression,
    [CCode (cname = "zlib_DEFAULT_COMPRESSION")]
    DefaultCompression
  }

  [CCode (cname = "zlib_compression_strategy")]
  public enum CompressionStrategy
  {
    [CCode (cname = "zlib_DEFAULT_STRATEGY")]
    DefaultStrategy,
    [CCode (cname = "zlib_FILTERED")]
    Filtered,
    [CCode (cname = "zlib_HUFFMAN_ONLY")]
    HuffmanOnly
  }

  [CCode (cname = "zlib_compression_method")]
  public enum CompressionMethod
  {
    [CCode (cname = "zlib_DEFLATED")]
    Deflated
  }

  [CCode (cname = "zlib_memory_level")]
  public enum MemoryLevel
  {
    Default = 0
  }

  [CCode (cname = "zlib_window_bits")]
  public enum WindowBits
  {
    Default = 0
  }

  [CCode (cname = "zlib_return_code")]
  public enum ReturnCode
  {
    [CCode (cname = "zlib_OK")]
    OK,
    [CCode (cname = "zlib_STREAM_END")]
    StreamEnd,
    [CCode (cname = "zlib_NEED_DICT")]
    NeedDict,
    [CCode (cname = "zlib_ERRNO")]
    Errno,
    [CCode (cname = "zlib_STREAM_ERROR")]
    StreamError,
    [CCode (cname = "zlib_DATA_ERROR")]
    DataError,
    [CCode (cname = "zlib_MEM_ERROR")]
    MemError,
    [CCode (cname = "zlib_BUF_ERROR")]
    BufError,
    [CCode (cname = "zlib_VERSION_ERROR")]
    VersionError
  }

  [CCode (cname = "zlib_seek_type")]
  public enum SeekType
  {
    [CCode (cname = "zlib_SEEK_ABSOLUTE")]
    SeekAbsolute,
    [CCode (cname = "zlib_SEEK_RELATIVE")]
    SeekRelative
  }

  [CCode (cname = "zlib_stream_control_block")]
  public struct StreamControlBlock
  {
    [CCode (cname = "next_in")]
    public unowned int NextIn;
    [CCode (cname = "avail_in")]
    public unowned int AvailIn;
    [CCode (cname = "total_in")]
    public unowned int TotalIn;
    [CCode (cname = "next_out")]
    public unowned int NextOut;
    [CCode (cname = "avail_out")]
    public unowned int AvailOut;
    [CCode (cname = "total_out")]
    public unowned int TotalOut;
    [CCode (cname = "msg")]
    public unowned char* Msg;
    [CCode (cname = "state")]
    public unowned int State;
    [CCode (cname = "zalloc")]
    public unowned char* Zalloc;
    [CCode (cname = "zfree")]
    public unowned char* Zfree;
    [CCode (cname = "opaque")]
    public unowned char* Opaque;
    [CCode (cname = "data_type")]
    public unowned DataType DataType;
    [CCode (cname = "adler")]
    public unowned int Adler;
    [CCode (cname = "reserved")]
    public unowned int Reserved;
  }

  [CCode (cname = "zlib_gzip_handle", has_type_id = false)]
  public struct GZipHandle : bits
  {
  }


  [CCode (cname = "xzlib_compress_return_sizes")]
  public static OS.Error* XCompressReturnSizes
    (int input_size,
     out int workspace_size,
     out int output_size);

  [CCode (cname = "zlib_compress_return_sizes")]
  public static void CompressReturnSizes
    (int input_size,
     out int workspace_size,
     out int output_size);

  [CCode (cname = "xzlib_compress")]
  public static OS.Error* XCompress
    (bits flags,
     char* workspace,
     char* input,
     int input_size,
     char* output,
     int output_size,
     out bits status,
     out char* unused_input,
     out int unused_input_size,
     out char* unused_output,
     out int unused_output_size);

  [CCode (cname = "zlib_compress")]
  public static bits Compress
    (bits flags,
     char* workspace,
     char* input,
     int input_size,
     char* output,
     int output_size,
     out char* unused_input,
     out int unused_input_size,
     out char* unused_output,
     out int unused_output_size);

  [CCode (cname = "xzlib_decompress_return_sizes")]
  public static OS.Error* XDecompressReturnSizes
    (int input_size,
     out int workspace_size,
     out int output_size);

  [CCode (cname = "zlib_decompress_return_sizes")]
  public static void DecompressReturnSizes
    (int input_size,
     out int workspace_size,
     out int output_size);

  [CCode (cname = "xzlib_decompress")]
  public static OS.Error* XDecompress
    (bits flags,
     char* workspace,
     char* input,
     int input_size,
     char* output,
     int output_size,
     out bits status,
     out char* unused_input,
     out int unused_input_size,
     out char* unused_output,
     out int unused_output_size);

  [CCode (cname = "zlib_decompress")]
  public static bits Decompress
    (bits flags,
     char* workspace,
     char* input,
     int input_size,
     char* output,
     int output_size,
     out char* unused_input,
     out int unused_input_size,
     out char* unused_output,
     out int unused_output_size);

  [CCode (cname = "xzlib_crc32")]
  public static OS.Error* XCRC32
    (int continuation_value,
     char* data_start,
     char* data_end,
     out int crc32);

  [CCode (cname = "zlib_crc32")]
  public static int CRC32
    (int continuation_value,
     char* data_start,
     char* data_end);

  [CCode (cname = "xzlib_adler32")]
  public static OS.Error* XAdler32
    (int continuation_value,
     char* data_start,
     char* data_end,
     out int crc32);

  [CCode (cname = "zlib_adler32")]
  public static int Adler32
    (int continuation_value,
     char* data_start,
     char* data_end);

  [CCode (cname = "xzlib_version")]
  public static OS.Error* XVersion
    (out char* version);

  [CCode (cname = "zlib_version")]
  public static void Version
    (out char* version);

  [CCode (cname = "xzlib_zcompress")]
  public static OS.Error* XZCompress
    (char* output,
     int output_size,
     char* input,
     int input_size,
     out ReturnCode return_code,
     out int output_used);

  [CCode (cname = "zlib_zcompress")]
  public static ReturnCode ZCompress
    (char* output,
     int output_size,
     char* input,
     int input_size,
     out int output_used);

  [CCode (cname = "xzlib_zcompress2")]
  public static OS.Error* XZCompress2
    (char* output,
     int output_size,
     char* input,
     int input_size,
     CompressionLevel compression_level,
     out ReturnCode return_code,
     out int output_used);

  [CCode (cname = "zlib_zcompress2")]
  public static ReturnCode ZCompress2
    (char* output,
     int output_size,
     char* input,
     int input_size,
     CompressionLevel compression_level,
     out int output_used);

  [CCode (cname = "xzlib_zuncompress")]
  public static OS.Error* XZUncompress
    (char* output,
     int output_size,
     char* input,
     int input_size,
     out ReturnCode return_code,
     out int output_used);

  [CCode (cname = "zlib_zuncompress")]
  public static ReturnCode ZUncompress
    (char* output,
     int output_size,
     char* input,
     int input_size,
     out int output_used);

  [CCode (cname = "xzlib_deflate_init")]
  public static OS.Error* XDeflateInit
    (StreamControlBlock scb,
     CompressionLevel compression_level,
     char* version_expected,
     int scb_size,
     out ReturnCode return_code);

  [CCode (cname = "zlib_deflate_init")]
  public static ReturnCode DeflateInit
    (StreamControlBlock scb,
     CompressionLevel compression_level,
     char* version_expected,
     int scb_size);

  [CCode (cname = "xzlib_inflate_init")]
  public static OS.Error* XInflateInit
    (StreamControlBlock scb,
     char* version_expected,
     int scb_size,
     out ReturnCode return_code);

  [CCode (cname = "zlib_inflate_init")]
  public static ReturnCode InflateInit
    (StreamControlBlock scb,
     char* version_expected,
     int scb_size);

  [CCode (cname = "xzlib_deflate_init2")]
  public static OS.Error* XDeflateInit2
    (StreamControlBlock scb,
     CompressionLevel compression_level,
     CompressionMethod compression_method,
     WindowBits window_bits,
     MemoryLevel memory_level,
     CompressionStrategy compression_strategy,
     char* version_expected,
     int scb_size,
     out ReturnCode return_code);

  [CCode (cname = "zlib_deflate_init2")]
  public static ReturnCode DeflateInit2
    (StreamControlBlock scb,
     CompressionLevel compression_level,
     CompressionMethod compression_method,
     WindowBits window_bits,
     MemoryLevel memory_level,
     CompressionStrategy compression_strategy,
     char* version_expected,
     int scb_size);

  [CCode (cname = "xzlib_inflate_init2")]
  public static OS.Error* XInflateInit2
    (StreamControlBlock scb,
     WindowBits window_bits,
     char* version_expected,
     int scb_size,
     out ReturnCode return_code);

  [CCode (cname = "zlib_inflate_init2")]
  public static ReturnCode InflateInit2
    (StreamControlBlock scb,
     WindowBits window_bits,
     char* version_expected,
     int scb_size);

  [CCode (cname = "xzlib_deflate")]
  public static OS.Error* XDeflate
    (StreamControlBlock scb,
     FlushType flush_type,
     out ReturnCode return_code);

  [CCode (cname = "zlib_deflate")]
  public static ReturnCode Deflate
    (StreamControlBlock scb,
     FlushType flush_type);

  [CCode (cname = "xzlib_deflate_end")]
  public static OS.Error* XDeflateEnd
    (StreamControlBlock scb,
     out ReturnCode return_code);

  [CCode (cname = "zlib_deflate_end")]
  public static ReturnCode DeflateEnd
    (StreamControlBlock scb);

  [CCode (cname = "xzlib_inflate")]
  public static OS.Error* XInflate
    (StreamControlBlock scb,
     FlushType flush_type,
     out ReturnCode return_code);

  [CCode (cname = "zlib_inflate")]
  public static ReturnCode Inflate
    (StreamControlBlock scb,
     FlushType flush_type);

  [CCode (cname = "xzlib_inflate_end")]
  public static OS.Error* XInflateEnd
    (StreamControlBlock scb,
     out ReturnCode return_code);

  [CCode (cname = "zlib_inflate_end")]
  public static ReturnCode InflateEnd
    (StreamControlBlock scb);

  [CCode (cname = "xzlib_deflate_set_dictionary")]
  public static OS.Error* XDeflateSetDictionary
    (StreamControlBlock scb,
     char* dictionary,
     int dictionary_size,
     out ReturnCode return_code);

  [CCode (cname = "zlib_deflate_set_dictionary")]
  public static ReturnCode DeflateSetDictionary
    (StreamControlBlock scb,
     char* dictionary,
     int dictionary_size);

  [CCode (cname = "xzlib_deflate_copy")]
  public static OS.Error* XDeflateCopy
    (StreamControlBlock destination,
     StreamControlBlock source,
     out ReturnCode return_code);

  [CCode (cname = "zlib_deflate_copy")]
  public static ReturnCode DeflateCopy
    (StreamControlBlock destination,
     StreamControlBlock source);

  [CCode (cname = "xzlib_deflate_reset")]
  public static OS.Error* XDeflateReset
    (StreamControlBlock scb,
     out ReturnCode return_code);

  [CCode (cname = "zlib_deflate_reset")]
  public static ReturnCode DeflateReset
    (StreamControlBlock scb);

  [CCode (cname = "xzlib_deflate_params")]
  public static OS.Error* XDeflateParams
    (StreamControlBlock scb,
     CompressionLevel compression_level,
     CompressionStrategy compression_strategy,
     out ReturnCode return_code);

  [CCode (cname = "zlib_deflate_params")]
  public static ReturnCode DeflateParams
    (StreamControlBlock scb,
     CompressionLevel compression_level,
     CompressionStrategy compression_strategy);

  [CCode (cname = "xzlib_inflate_set_dictionary")]
  public static OS.Error* XInflateSetDictionary
    (StreamControlBlock scb,
     char* dictionary,
     int dictionary_size,
     out ReturnCode return_code);

  [CCode (cname = "zlib_inflate_set_dictionary")]
  public static ReturnCode InflateSetDictionary
    (StreamControlBlock scb,
     char* dictionary,
     int dictionary_size);

  [CCode (cname = "xzlib_inflate_sync")]
  public static OS.Error* XInflateSync
    (StreamControlBlock scb,
     out ReturnCode return_code);

  [CCode (cname = "zlib_inflate_sync")]
  public static ReturnCode InflateSync
    (StreamControlBlock scb);

  [CCode (cname = "xzlib_inflate_reset")]
  public static OS.Error* XInflateReset
    (StreamControlBlock scb,
     out ReturnCode return_code);

  [CCode (cname = "zlib_inflate_reset")]
  public static ReturnCode InflateReset
    (StreamControlBlock scb);

  [CCode (cname = "xzlib_gz_open")]
  public static OS.Error* XGZOpen
    (char* filename,
     char* mode,
     bits load_address_in,
     bits exec_address_in,
     int size_in,
     FileSwitch.Attr attr_in,
     out GZipHandle handle,
     out bits load_address_out,
     out bits exec_address_out,
     out int size_out,
     out FileSwitch.Attr attr_out);

  [CCode (cname = "zlib_gz_open")]
  public static GZipHandle GZOpen
    (char* filename,
     char* mode,
     bits load_address_in,
     bits exec_address_in,
     int size_in,
     FileSwitch.Attr attr_in,
     out bits load_address_out,
     out bits exec_address_out,
     out int size_out,
     out FileSwitch.Attr attr_out);

  [CCode (cname = "xzlib_gz_read")]
  public static OS.Error* XGZRead
    (GZipHandle handle,
     byte* data,
     int size,
     out int read);

  [CCode (cname = "zlib_gz_read")]
  public static int GZRead
    (GZipHandle handle,
     byte* data,
     int size);

  [CCode (cname = "xzlib_gz_write")]
  public static OS.Error* XGZWrite
    (GZipHandle handle,
     byte* data,
     int size,
     out int written);

  [CCode (cname = "zlib_gz_write")]
  public static int GZWrite
    (GZipHandle handle,
     byte* data,
     int size);

  [CCode (cname = "xzlib_gz_flush")]
  public static OS.Error* XGZFlush
    (GZipHandle handle,
     FlushType flush_type,
     out ReturnCode return_code);

  [CCode (cname = "zlib_gz_flush")]
  public static ReturnCode GZFlush
    (GZipHandle handle,
     FlushType flush_type);

  [CCode (cname = "xzlib_gz_close")]
  public static OS.Error* XGZClose
    (GZipHandle handle,
     out ReturnCode return_code);

  [CCode (cname = "zlib_gz_close")]
  public static ReturnCode GZClose
    (GZipHandle handle);

  [CCode (cname = "xzlib_gz_error")]
  public static OS.Error* XGZError
    (GZipHandle handle,
     out char* message,
     out ReturnCode return_code);

  [CCode (cname = "zlib_gz_error")]
  public static char GZError
    (GZipHandle handle,
     out ReturnCode return_code);

  [CCode (cname = "xzlib_gz_seek")]
  public static OS.Error* XGZSeek
    (GZipHandle handle,
     int position,
     SeekType seek_type,
     out int new_position);

  [CCode (cname = "zlib_gz_seek")]
  public static int GZSeek
    (GZipHandle handle,
     int position,
     SeekType seek_type);

  [CCode (cname = "xzlib_gz_tell")]
  public static OS.Error* XGZTell
    (GZipHandle handle,
     out int position);

  [CCode (cname = "zlib_gz_tell")]
  public static int GZTell
    (GZipHandle handle);

  [CCode (cname = "xzlib_gzeof")]
  public static OS.Error* XGZEOF
    (GZipHandle handle,
     out bool eof);

  [CCode (cname = "zlib_gzeof")]
  public static bool GZEOF
    (GZipHandle handle);

  [CCode (cname = "xzlib_task_associate")]
  public static OS.Error* XTaskAssociate
    (StreamControlBlock scb);

  [CCode (cname = "zlib_task_associate")]
  public static void TaskAssociate
    (StreamControlBlock scb);

}
