/* OSLib---efficient, type-safe, transparent, extensible,
 * register-safe API coverage of RISC OS
 * 
 * Copyright (c) 1994-2012 Jonathan Coxhead and OSLib maintainers
 * 
 * Licence:
 * 
 *    OSLib is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 1, or (at your option)
 * any later version - and this with the following clarification and
 * special exception:
 * 
 *    Linking this library statically or dynamically with other modules
 * is making a combined work based on this library. Thus, the terms
 * and conditions of the GNU General Public License cover the whole
 * combination.
 * 
 *    As a special exception, the copyright holders of this library
 * give you permission to link this library with independent modules
 * to produce an executable, regardless of the license terms of these
 * independent modules, and to copy and distribute the resulting
 * executable under terms of your choice, provided that you also meet,
 * for each linked independent module, the terms and conditions of the
 * license of that module. An independent module is a module which is
 * not derived from or based on this library. If you modify this
 * library, you may extend this exception to your version of the library,
 * but you are not obliged to do so. If you do not wish to do so, delete
 * this exception statement from your version.
 * 
 *    OSLib is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 *    You should have received a copy of the GNU General Public License
 * along with this programme; if not, write to the Free Software
 * Foundation, Inc, 675 Mass Ave, Cambridge, MA 02139, USA.
 */

using OSLib;
using OSLib.FileSwitch;

[CCode (cheader_filename = "oslib/zipper.h")]
namespace OSLib.Zipper
{
  [CCode (cname = "zipper_un_zip_handle", has_type_id = false)]
  public struct UnZipHandle : bits
  {
  }

  [CCode (cname = "zipper_zip_handle", has_type_id = false)]
  public struct ZipHandle : bits
  {
  }

  [CCode (cname = "zipper_open_flags")]
  public enum OpenFlags
  {
    [CCode (cname = "zipperopenflags_APPEND")]
    Append,
    [CCode (cname = "zipperopenflags_NO_DISCARD")]
    NoDiscard
  }

  [CCode (cname = "zipper_close_flags")]
  public enum CloseFlags
  {
    Default = 0
  }

  [CCode (cname = "zipper_info_flags")]
  public enum InfoFlags
  {
    Default = 0
  }

  [CCode (cname = "zipper_enumerate_flags")]
  public enum EnumerateFlags
  {
    [CCode (cname = "zipperenumerateflags_CREATOR_VERSION_NUMBER")]
    CreatorVersionNumber,
    [CCode (cname = "zipperenumerateflags_GENERAL_PURPOSE_FLAGS")]
    GeneralPurposeFlags,
    [CCode (cname = "zipperenumerateflags_COMPRESSION_METHOD")]
    CompressionMethod,
    [CCode (cname = "zipperenumerateflags_COMPRESSED_SIZE")]
    CompressedSize,
    [CCode (cname = "zipperenumerateflags_EXPANDED_SIZE")]
    ExpandedSize,
    [CCode (cname = "zipperenumerateflags_INTERNAL_ATTRIBUTES")]
    InternalAttributes,
    [CCode (cname = "zipperenumerateflags_EXTERNAL_ATTRIBUTES")]
    ExternalAttributes,
    [CCode (cname = "zipperenumerateflags_TIMESTAMP")]
    Timestamp,
    [CCode (cname = "zipperenumerateflags_LOAD_AND_EXEC_ADDRESS")]
    LoadAndExecAddress,
    [CCode (cname = "zipperenumerateflags_RO_EXPANDED_SIZE")]
    ROExpandedSize,
    [CCode (cname = "zipperenumerateflags_RO_ATTRIBUTES")]
    ROAttributes,
    [CCode (cname = "zipperenumerateflags_RO_OBJECT_TYPE")]
    ROObjectType,
    [CCode (cname = "zipperenumerateflags_RO_FILE_TYPE")]
    ROFileType,
    [CCode (cname = "zipperenumerateflags_GBPB_FILENAME_LEN")]
    GBPBFilenameLen,
    [CCode (cname = "zipperenumerateflags_FILENAME_LEN")]
    FilenameLen,
    [CCode (cname = "zipperenumerateflags_EXTRA_INFO_LEN")]
    ExtraInfoLen,
    [CCode (cname = "zipperenumerateflags_FILE_COMMENT_LEN")]
    FileCommentLen
  }

  [CCode (cname = "zipper_un_zip_file_open_flags")]
  public enum UnZipFileOpenFlags
  {
    Default = 0
  }

  [CCode (cname = "zipper_un_zip_file_close_flags")]
  public enum UnZipFileCloseFlags
  {
    Default = 0
  }

  [CCode (cname = "zipper_un_zip_file_read_flags")]
  public enum UnZipFileReadFlags
  {
    Default = 0
  }

  [CCode (cname = "zipper_un_zip_file_eof_flags")]
  public enum UnZipFileEOFFlags
  {
    Default = 0
  }

  [CCode (cname = "zipper_zip_file_open_flags")]
  public enum ZipFileOpenFlags
  {
    [CCode (cname = "zipperfileopenflags_RO_STYLE_TIME_SPECIFIER")]
    ROStyleTimeSpecifier,
    [CCode (cname = "zipperfileopenflags_OBJ_IS_DIR")]
    ObjIsDir,
    [CCode (cname = "zipperfileopenflags_OBJ_IS_TEXT")]
    ObjIsText,
    [CCode (cname = "zipperfileopenflags_EXTRA_FIELD")]
    ExtraField,
    [CCode (cname = "zipperfileopenflags_COMMENT_FIELD")]
    CommentField,
    [CCode (cname = "zipperfileopenflags_FILE_ATTRIBUTES")]
    FileAttributes
  }

  [CCode (cname = "zipper_zip_file_close_flags")]
  public enum ZipFileCloseFlags
  {
    Default = 0
  }

  [CCode (cname = "zipper_zip_file_write_flags")]
  public enum ZipFileWriteFlags
  {
    Default = 0
  }

  [CCode (cname = "zipper_compression_details")]
  public enum CompressionDetails
  {
    [CCode (cname = "zipper_COMPRESSION_METHOD")]
    CompressionMethod,
    [CCode (cname = "zipper_COMPRESSION_LEVEL")]
    CompressionLevel,
    [CCode (cname = "zipper_COMPRESSION_METHOD_STORE")]
    CompressionMethodStore,
    [CCode (cname = "zipper_COMPRESSION_METHOD_DEFLATE")]
    CompressionMethodDeflate,
    [CCode (cname = "zipper_COMPRESSION_LEVEL_MINIMUM")]
    CompressionLevelMinimum,
    [CCode (cname = "zipper_COMPRESSION_LEVEL_MAXIMUM")]
    CompressionLevelMaximum
  }

  [CCode (cname = "zipper_time_specifier")]
  public struct TimeSpecifier
  {
    [CCode (cname = "riscos_style.load_addr")]
    public unowned bits RiscosStyle_LoadAddr;
    [CCode (cname = "riscos_style.exec_addr")]
    public unowned bits RiscosStyle_ExecAddr;
    [CCode (cname = "riscos_style.attr")]
    public unowned FileSwitch.Attr RiscosStyle_Attr;
    [CCode (cname = "zip_style.centiseconds")]
    public unowned int ZipStyle_Centiseconds;
    [CCode (cname = "zip_style.seconds")]
    public unowned int ZipStyle_Seconds;
    [CCode (cname = "zip_style.minutes")]
    public unowned int ZipStyle_Minutes;
    [CCode (cname = "zip_style.hours")]
    public unowned int ZipStyle_Hours;
    [CCode (cname = "zip_style.date")]
    public unowned int ZipStyle_Date;
    [CCode (cname = "zip_style.month")]
    public unowned int ZipStyle_Month;
    [CCode (cname = "zip_style.year")]
    public unowned int ZipStyle_Year;
  }

  [CCode (cname = "zipper_file_attributes")]
  public struct FileAttributes
  {
    [CCode (cname = "internal")]
    public unowned bits Internal;
    [CCode (cname = "external")]
    public unowned bits External;
  }

  [CCode (cname="zipper_COMPRESSION_METHOD_SHIFT")]
  public const int CompressionMethodShift;
  [CCode (cname="zipper_COMPRESSION_LEVEL_SHIFT")]
  public const int CompressionLevelShift;

  [CCode (cname = "xzipper_un_zip_open")]
  public static OS.Error* XUnZipOpen
    (OpenFlags flags,
     char* filename,
     out UnZipHandle handle);

  [CCode (cname = "zipper_un_zip_open")]
  public static UnZipHandle UnZipOpen
    (OpenFlags flags,
     char* filename);

  [CCode (cname = "xzipper_un_zip_close")]
  public static OS.Error* XUnZipClose
    (CloseFlags flags,
     UnZipHandle handle);

  [CCode (cname = "zipper_un_zip_close")]
  public static void UnZipClose
    (CloseFlags flags,
     UnZipHandle handle);

  [CCode (cname = "xzipper_un_zip_info_object_count")]
  public static OS.Error* XUnZipInfoObjectCount
    (InfoFlags flags,
     UnZipHandle handle,
     out int objects);

  [CCode (cname = "zipper_un_zip_info_object_count")]
  public static int UnZipInfoObjectCount
    (InfoFlags flags,
     UnZipHandle handle);

  [CCode (cname = "xzipper_un_zip_info_archive_comment")]
  public static OS.Error* XUnZipInfoArchiveComment
    (InfoFlags flags,
     UnZipHandle handle,
     char* buffer,
     int buffer_size,
     out int read);

  [CCode (cname = "zipper_un_zip_info_archive_comment")]
  public static int UnZipInfoArchiveComment
    (InfoFlags flags,
     UnZipHandle handle,
     char* buffer,
     int buffer_size);

  [CCode (cname = "xzipper_un_zip_enumerate")]
  public static OS.Error* XUnZipEnumerate
    (EnumerateFlags flags,
     UnZipHandle handle,
     char* buffer,
     int objects,
     int offset,
     int buffer_size,
     out int objects_read,
     out int next_offset,
     out int buffer_unused);

  [CCode (cname = "zipper_un_zip_enumerate")]
  public static int UnZipEnumerate
    (EnumerateFlags flags,
     UnZipHandle handle,
     char* buffer,
     int objects,
     int offset,
     int buffer_size,
     out int next_offset,
     out int buffer_unused);

  [CCode (cname = "xzipper_un_zip_file_info_to_buffer")]
  public static OS.Error* XUnZipFileInfoToBuffer
    (EnumerateFlags flags,
     UnZipHandle handle,
     char* filename,
     char* data,
     int data_size,
     out char* data_end,
     out int data_unused);

  [CCode (cname = "zipper_un_zip_file_info_to_buffer")]
  public static void UnZipFileInfoToBuffer
    (EnumerateFlags flags,
     UnZipHandle handle,
     char* filename,
     char* data,
     int data_size,
     out char* data_end,
     out int data_unused);

  [CCode (cname = "xzipper_un_zip_file_info_to_registers")]
  public static OS.Error* XUnZipFileInfoToRegisters
    (UnZipHandle handle,
     char* filename,
     out FileSwitch.ObjectType obj_type,
     out bits load_address,
     out bits exec_address,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "zipper_un_zip_file_info_to_registers")]
  public static void UnZipFileInfoToRegisters
    (UnZipHandle handle,
     char* filename,
     out FileSwitch.ObjectType obj_type,
     out bits load_address,
     out bits exec_address,
     out int size,
     out FileSwitch.Attr attr);

  [CCode (cname = "xzipper_un_zip_file_open")]
  public static OS.Error* XUnZipFileOpen
    (UnZipFileOpenFlags flags,
     UnZipHandle handle,
     char* filename);

  [CCode (cname = "zipper_un_zip_file_open")]
  public static void UnZipFileOpen
    (UnZipFileOpenFlags flags,
     UnZipHandle handle,
     char* filename);

  [CCode (cname = "xzipper_un_zip_file_close")]
  public static OS.Error* XUnZipFileClose
    (UnZipFileCloseFlags flags,
     UnZipHandle handle);

  [CCode (cname = "zipper_un_zip_file_close")]
  public static void UnZipFileClose
    (UnZipFileCloseFlags flags,
     UnZipHandle handle);

  [CCode (cname = "xzipper_un_zip_file_read")]
  public static OS.Error* XUnZipFileRead
    (UnZipFileReadFlags flags,
     UnZipHandle handle,
     char* data,
     int size,
     out int read);

  [CCode (cname = "zipper_un_zip_file_read")]
  public static int UnZipFileRead
    (UnZipFileReadFlags flags,
     UnZipHandle handle,
     char* data,
     int size);

  [CCode (cname = "xzipper_un_zip_file_eof")]
  public static OS.Error* XUnZipFileEOF
    (UnZipFileEOFFlags flags,
     UnZipHandle handle,
     out bool eof);

  [CCode (cname = "zipper_un_zip_file_eof")]
  public static bool UnZipFileEOF
    (UnZipFileEOFFlags flags,
     UnZipHandle handle);

  [CCode (cname = "xzipper_zip_open")]
  public static OS.Error* XZipOpen
    (OpenFlags flags,
     char* filename,
     out ZipHandle handle);

  [CCode (cname = "zipper_zip_open")]
  public static ZipHandle ZipOpen
    (OpenFlags flags,
     char* filename);

  [CCode (cname = "xzipper_zip_close")]
  public static OS.Error* XZipClose
    (CloseFlags flags,
     ZipHandle handle);

  [CCode (cname = "zipper_zip_close")]
  public static void ZipClose
    (CloseFlags flags,
     ZipHandle handle);

  [CCode (cname = "xzipper_zip_file_open")]
  public static OS.Error* XZipFileOpen
    (ZipFileOpenFlags flags,
     ZipHandle handle,
     char* filename,
     CompressionDetails compression_details,
     TimeSpecifier time_specifier,
     char* extra,
     int extra_size,
     char* comment,
     FileAttributes attr);

  [CCode (cname = "zipper_zip_file_open")]
  public static void ZipFileOpen
    (ZipFileOpenFlags flags,
     ZipHandle handle,
     char* filename,
     CompressionDetails compression_details,
     TimeSpecifier time_specifier,
     char* extra,
     int extra_size,
     char* comment,
     FileAttributes attr);

  [CCode (cname = "xzipper_zip_file_close")]
  public static OS.Error* XZipFileClose
    (ZipFileCloseFlags flags,
     ZipHandle handle,
     out int uncompressed_size,
     out int compressed_size);

  [CCode (cname = "zipper_zip_file_close")]
  public static void ZipFileClose
    (ZipFileCloseFlags flags,
     ZipHandle handle,
     out int uncompressed_size,
     out int compressed_size);

  [CCode (cname = "xzipper_zip_file_write")]
  public static OS.Error* XZipFileWrite
    (ZipFileWriteFlags flags,
     ZipHandle handle,
     char* data,
     int size);

  [CCode (cname = "zipper_zip_file_write")]
  public static void ZipFileWrite
    (ZipFileWriteFlags flags,
     ZipHandle handle,
     char* data,
     int size);

}
